@@ -9,9 +9,12 @@
 
 #include <qstyle.h>
 #include <qevent.h>
+#include <qtimer.h>
 #include "kstlinestyle.h"
 #include "kstsettings.h"
 
+#include "kstbasecurve.h"
+
 bool CurveAppearanceWidget::showLines()
 {
   return _showLines->isChecked();
@@ -23,6 +26,10 @@ bool CurveAppearanceWidget::showPoints()
   return _showPoints->isChecked();
 }
 
+bool CurveAppearanceWidget::showBars()
+{
+  return _showBars->isChecked();
+}
 
 void CurveAppearanceWidget::init()
 {
@@ -80,15 +87,29 @@ QColor CurveAppearanceWidget::color()
 
 void CurveAppearanceWidget::drawLine()
 {
-
   QPixmap pix(_label->contentsRect().height()*7, _label->contentsRect().height());
   QPainter p(&pix);
   QPen pen(color(),lineWidth(),KstLineStyle[lineStyle()]);
   KstPoint tmppoint;
 
   p.fillRect(p.window(), QColor("white"));
-  p.setPen(pen);
 
+  if (showBars()) {
+    QRect rectBar((pix.width()-pix.height())/2,
+                  pix.height()/2,
+                  pix.height(),
+                  (pix.height()/2)+1);
+
+    if (barStyle() == 1) {
+      p.fillRect(rectBar,QBrush(QColor(color())));
+      p.setPen(QPen(QColor("black"),lineWidth(),KstLineStyle[lineStyle()]));
+    } else {
+      p.setPen(pen);
+    }
+    p.drawRect(rectBar);
+  }
+  
+  p.setPen(pen);
   if (_showLines->isChecked()) {
     p.drawLine(1,pix.height()/2,pix.width()-1, pix.height()/2);
   }
@@ -114,6 +135,8 @@ void CurveAppearanceWidget::reset(QColor newColor)
 {
   _showLines->setChecked(true);
   _showPoints->setChecked(false);
+  _showBars->setChecked(false);
+  _barStyle->setCurrentItem(1);
   _color->setColor(newColor);
   _spinBoxLineWidth->setValue(0);
   fillLineStyleCombo();
@@ -220,16 +243,17 @@ int CurveAppearanceWidget::lineWidth()
   return _spinBoxLineWidth->value();
 }
 
-
-void CurveAppearanceWidget::setValue( bool hasLines, bool hasPoints, const QColor & c, int pointType, int lineWidth, int lineStyle )
+void CurveAppearanceWidget::setValue( bool hasLines, bool hasPoints, bool hasBars, const QColor & c, int pointType, int lineWidth, int lineStyle, int barStyle )
 {
   _showLines->setChecked(hasLines);
   _showPoints->setChecked(hasPoints);
+  _showBars->setChecked(hasBars);
   _color->setColor(c);
   fillCombo();
   fillLineStyleCombo();
   _spinBoxLineWidth->setValue(lineWidth);
   _combo->setCurrentItem(pointType);
+  _barStyle->setCurrentItem(barStyle);
   if (lineStyle < 0 || lineStyle >= KSTLINESTYLE_MAXTYPE) {
     lineStyle = 0;
   }
@@ -243,3 +267,10 @@ void CurveAppearanceWidget::resizeEvent( QResizeEvent * pEvent )
   redrawCombo();
   QWidget::resizeEvent(pEvent);
 }
+
+
+
+int CurveAppearanceWidget::barStyle()
+{
+    return _barStyle->currentItem();
+}
