@@ -128,8 +128,9 @@ void CurvePlacementWidget::updateGrid()
   KstApp *app = KstApp::inst();
   if (app) {
     KMdiChildView *c = app->findWindow(_plotWindow->currentText());
-    if (c) {
-      KstTopLevelViewPtr view = static_cast<KstViewWindow*>(c)->view();
+    KstViewWindow *w = dynamic_cast<KstViewWindow*>(c);
+    if (w) {
+      KstTopLevelViewPtr view = w->view();
       _reGrid->setChecked(view->onGrid());
       _plotColumns->setValue(view->columns());
     }
