@@ -61,6 +61,8 @@ void DIALOGTYPE::show_Edit(const QString &field) {
   setCaption(i18n("Edit %1").arg(DTYPE));
   show();
   raise();
+  _OK->setEnabled(true);
+  _cancel->setEnabled(true);
 }
 
 /** show_new:
@@ -70,17 +72,31 @@ void DIALOGTYPE::show_New() {
   DP = 0L;
 
   _fillFieldsForNew();
-  
+
   setCaption(i18n("New %1").arg(DTYPE));
   show();
   raise();
+  _OK->setEnabled(true);
+  _cancel->setEnabled(true);
 }
 
 void DIALOGTYPE::OK() {
+  _OK->setEnabled(false);
+  _cancel->setEnabled(false);
   if (_newDialog || (DP == 0L)) {
-    if (new_I()) close();
+    if (new_I()) {
+      close();
+    } else {
+      _OK->setEnabled(true);
+      _cancel->setEnabled(true);
+    }
   } else {
-    if (edit_I()) close();
+    if (edit_I()) {
+      close();
+    } else {
+      _OK->setEnabled(true);
+      _cancel->setEnabled(true);
+    }
   }
 }
 
