@@ -16,26 +16,33 @@
  ***************************************************************************/
 
 #include "ascii.h"
+#include "asciiconfig.h"
 
 #include <kdebug.h>
 
 #include <qfile.h>
+#include <qlayout.h>
+#include <qlineedit.h>
 #include <qregexp.h>
 
+#include <assert.h>
 #include <ctype.h>
 #include <stdlib.h>
 
 
-AsciiSource::AsciiSource(const QString& filename, const QString& type)
-  : KstDataSource(filename, type) {
-    if (!type.isEmpty() && type != "ASCII") {
-      return;
-    }
+AsciiSource::AsciiSource(KConfig *cfg, const QString& filename, const QString& type)
+: KstDataSource(cfg, filename, type) {
+  if (!type.isEmpty() && type != "ASCII") {
+    return;
+  }
 
-    _rowIndex = 0L;
-    if (initFile()) {
-      _valid = true;
-    }
+  _rowIndex = 0L;
+  if (initFile()) {
+    _valid = true;
+  }
+
+  cfg->setGroup("ASCII General");
+  _delimiters = QRegExp::escape(cfg->readEntry("Comment Delimiters", "#/c!;"));
 }
 
 
@@ -130,8 +137,7 @@ KstObject::UpdateType AsciiSource::update(int u) {
         }
         _rowIndex[_numFrames] = bufstart + i + 1;
         has_dat = is_comment = false;
-      } else if (tmpbuf[i] == '#' || tmpbuf[i] == '!' || tmpbuf[i] == '/' ||
-                 tmpbuf[i] == ';' || tmpbuf[i] == 'c') {
+      } else if (_delimiters.contains(tmpbuf[i])) {
         is_comment = true;
       }
     }
@@ -252,11 +258,9 @@ bool AsciiSource::isEmpty() const {
 }
 
 
-// FIXME: refactor to cache the number of columns and just update it in
-//        ::update() as needed
-QStringList AsciiSource::fieldList() const {
+static QStringList fieldListFor(const QString& filename) {
   QStringList rc;
-  QFile file(_filename);
+  QFile file(filename);
   bool done = false;
   QString line;
 
@@ -284,14 +288,48 @@ QStringList AsciiSource::fieldList() const {
 }
 
 
-void AsciiSource::save(QTextStream &ts) {
-  KstDataSource::save(ts);
+// FIXME: refactor to cache the number of columns and just update it in
+//        ::update() as needed
+QStringList AsciiSource::fieldList() const {
+  return fieldListFor(_filename);
 }
 
 
+void AsciiSource::save(QTextStream &ts, const QString& indent) {
+  KstDataSource::save(ts, indent);
+}
+
+
+class ConfigWidgetAscii : public KstDataSourceConfigWidget {
+  public:
+    ConfigWidgetAscii() : KstDataSourceConfigWidget() {
+      QGridLayout *layout = new QGridLayout(this, 1, 1);
+      _ac = new AsciiConfig(this);
+      layout->addWidget(_ac, 0, 0);
+      layout->activate();
+    }
+
+    virtual ~ConfigWidgetAscii() {}
+
+    virtual void setConfig(KConfig *cfg) {
+      KstDataSourceConfigWidget::setConfig(cfg);
+      cfg->setGroup("ASCII General");
+      _ac->_delimiters->setText(cfg->readEntry("Comment Delimiters", "#/c!;"));
+    }
+
+    virtual void save() {
+      assert(_cfg);
+      _cfg->setGroup("ASCII General");
+      _cfg->writeEntry("Comment Delimiters", _ac->_delimiters->text());
+    }
+
+    AsciiConfig *_ac;
+};
+
+
 extern "C" {
-KstDataSource *create_ascii(const QString& filename, const QString& type) {
-  return new AsciiSource(filename, type);
+KstDataSource *create_ascii(KConfig *cfg, const QString& filename, const QString& type) {
+  return new AsciiSource(cfg, filename, type);
 }
 
 QStringList provides_ascii() {
@@ -301,14 +339,15 @@ QStringList provides_ascii() {
 }
 
 /** understands_ascii: returns true iff:
-    the first line which is neither empty nor a comment
-    (a comment starts with #, /, c, or !)
-    is a number.  **/
+    the first line which is neither empty nor a comment is a number.  **/
 // FIXME: might it be better to be more lenient and ignore lines with
 //        ascii free text?
-int understands_ascii(const QString& filename) {
+int understands_ascii(KConfig *cfg, const QString& filename) {
   QFile f(filename);
 
+  cfg->setGroup("ASCII General");
+  QString delimiters = QRegExp::escape(cfg->readEntry("Comment Delimiters", "#/c!;"));
+
   if (f.open(IO_ReadOnly)) {
     QString s;
     Q_LONG rc = 0;
@@ -320,7 +359,7 @@ int understands_ascii(const QString& filename) {
         done = true;
       } else if (rc == 1) {
         // empty line; do nothing
-      } else if (QRegExp("^\\s*[\\#\\/\\c\\!].*").exactMatch(s)) {
+      } else if (QRegExp(QString("^\\s*[%1].*").arg(delimiters)).exactMatch(s)) {
         // comment; do nothing
       } else if (QRegExp("^[\\s]*(([Nn][Aa][Nn]|(\\-\\+)?[Ii][Nn][Ff]|[0-9\\+\\-\\.eE]+)[\\s]*)+").exactMatch(s)) {
         // a number - this may be an ascii file - assume that it is
@@ -332,6 +371,31 @@ int understands_ascii(const QString& filename) {
   }
   return 0;
 }
+
+
+QStringList fieldList_ascii(KConfig *cfg, const QString& filename, const QString& type, QString *typeSuggestion, bool *complete) {
+  if ((!type.isEmpty() && !provides_ascii().contains(type)) ||
+      !understands_ascii(cfg, filename)) {
+    return QStringList();
+  }
+
+  if (complete) {
+    *complete = true;
+  }
+
+  if (typeSuggestion) {
+    *typeSuggestion = "ASCII";
+  }
+
+  return fieldListFor(filename);
+}
+
+
+QWidget *widget_ascii(const QString& filename) {
+  Q_UNUSED(filename)
+  return new ConfigWidgetAscii;
+}
+
 }
 
 // vim: ts=2 sw=2 et
