@@ -20,8 +20,8 @@
 #include "getdata_struct.h"
 
 
-DirFileSource::DirFileSource(const QString& filename, const QString& type)
-: KstDataSource(filename, type) {
+DirFileSource::DirFileSource(KConfig *cfg, const QString& filename, const QString& type)
+: KstDataSource(cfg, filename, type) {
   if (init()) {
     _valid = true;
   }
@@ -121,15 +121,15 @@ QString DirFileSource::fileType() const {
 }
 
 
-void DirFileSource::save(QTextStream &ts) {
-  KstDataSource::save(ts);
+void DirFileSource::save(QTextStream &ts, const QString& indent) {
+  KstDataSource::save(ts, indent);
 }
 
 //#include <kdebug.h>
 
 extern "C" {
-KstDataSource *create_dirfile(const QString& filename, const QString& type) {
-  return new DirFileSource(filename, type);
+KstDataSource *create_dirfile(KConfig *cfg, const QString& filename, const QString& type) {
+  return new DirFileSource(cfg, filename, type);
 }
 
 QStringList provides_dirfile() {
@@ -138,7 +138,7 @@ QStringList provides_dirfile() {
   return rc;
 }
 
-int understands_dirfile(const QString& filename) {
+int understands_dirfile(KConfig*, const QString& filename) {
   // FIXME: GetNFrames causes a memory error here.  I think it is due to
   // the lfilename parameter.
   int err = 0;
@@ -151,6 +151,42 @@ int understands_dirfile(const QString& filename) {
   return 0;
 }
 
+
+QStringList fieldList_dirfile(KConfig*, const QString& filename, const QString& type, QString *typeSuggestion, bool *complete) {
+  Q_UNUSED(type)
+  int err = 0;
+  struct FormatType *ft = GetFormat(filename.latin1(), &err);
+  QStringList fieldList;
+
+  if (complete) {
+    *complete = true;
+  }
+
+  if (typeSuggestion) {
+    *typeSuggestion = "Directory of Binary Files";
+  }
+
+  if (err == GD_E_OK) {
+    fieldList.append("INDEX");
+    for (int i = 0; i < ft->n_lincom; i++) {
+      fieldList.append(ft->lincomEntries[i].field);
+    }
+
+    for (int i = 0; i < ft->n_linterp; i++) {
+      fieldList.append(ft->linterpEntries[i].field);
+    }
+
+    for (int i = 0; i < ft->n_bit; i++) {
+      fieldList.append(ft->bitEntries[i].field);
+    }
+
+    for (int i = 0; i < ft->n_raw; i++) {
+      fieldList.append(ft->rawEntries[i].field);
+    }
+  }
+  return fieldList;
+}
+
 }
 
 // vim: ts=2 sw=2 et
