@@ -44,7 +44,7 @@ static struct {
 int recurse_level = 0;
 int first_time = 1;
 
-int DoField(struct FormatType *F, const char *field_code,
+static int DoField(struct FormatType *F, const char *field_code,
 	    int first_frame, int first_samp,
 	    int num_frames, int num_samp, 
 	    char return_type, void *data_out,
@@ -60,7 +60,8 @@ int DoField(struct FormatType *F, const char *field_code,
 static int GetLine(FILE *fp, char *line) {
   char *ret_val;
   int first_char;
-
+  int i, len;
+  
   do {
     ret_val=fgets(line, MAX_LINE_LENGTH, fp);
     first_char=0;
@@ -68,7 +69,14 @@ static int GetLine(FILE *fp, char *line) {
     line += first_char;
   } while (((line[0] =='#') || (strlen(line)<2)) && (ret_val!=NULL));
 
+  
   if (ret_val!=NULL) {
+    /* truncate comments from end of lines */
+    len = strlen(line);
+    for (i=0; i<len; i++) {
+      if (line[i]=='#') line[i] = '\0';
+    }
+    
     return (1); /* a line was read */
   } else {
     return(0);  /* there were no valid lines */
@@ -189,13 +197,26 @@ static void ParseMplex(char in_cols[15][MAX_LINE_LENGTH],
 /*                                                                         */
 /***************************************************************************/
 static void ParseBit(char in_cols[15][MAX_LINE_LENGTH],
+                     int n_cols,
 		     struct BitEntryType *B,
 		     int *error_code) {
+  int error = 0;
+  
   strcpy(B->field, in_cols[0]); /* field */
   strncpy(B->raw_field, in_cols[2], FIELD_LENGTH); /* field */
 
   B->bitnum=atoi(in_cols[3]);
-  if ((B->bitnum<0) || B->bitnum>31) {
+  if (n_cols>4) {
+    B->numbits=atoi(in_cols[4]);
+  } else {
+    B->numbits=1;
+  }
+  
+  if (B->numbits<1) error = 1;
+  if (B->bitnum<0) error = 1;
+  if (B->bitnum + B->numbits - 1 > 31) error = 1;
+  
+  if (error) {
     *error_code=GD_E_FORMAT;
     return;
   }
@@ -274,6 +295,7 @@ struct FormatType *GetFormat(const char *filedir, int *error_code) {
 
   strcpy(F->FileDirName, filedir);
   F->n_raw = F->n_lincom = F->n_linterp = F->n_mplex = F->n_bit = 0;
+  F->frame_offset = 0;
   F->rawEntries = NULL;
   F->lincomEntries = NULL;
   F->linterpEntries = NULL;
@@ -288,7 +310,7 @@ struct FormatType *GetFormat(const char *filedir, int *error_code) {
                     in_cols[4], in_cols[5], in_cols[6], in_cols[7], 
                     in_cols[8], in_cols[9], in_cols[10], in_cols[11], 
                     in_cols[12], in_cols[13], in_cols[14]);
-    if (n_cols<4) {
+    if (n_cols<2) {
       *error_code = GD_E_FORMAT;
       FreeF(F);
       return(NULL);
@@ -328,8 +350,10 @@ struct FormatType *GetFormat(const char *filedir, int *error_code) {
       F->bitEntries =
 	realloc(F->bitEntries,
 		F->n_bit*sizeof(struct BitEntryType));
-      ParseBit(in_cols, F->bitEntries+F->n_bit - 1,
-		  error_code); 
+      ParseBit(in_cols, n_cols, F->bitEntries+F->n_bit - 1,
+		  error_code);
+    } else if (strcmp(in_cols[0], "FRAMEOFFSET")==0) {
+      F->frame_offset = atoi(in_cols[1]);
     } else {
       FreeF(F);
       *error_code = GD_E_FORMAT;
@@ -640,6 +664,45 @@ static int ConvertType(unsigned char *data_in, char in_type,
   return(GD_E_OK);
 }
 
+
+/***************************************************************************/
+/*                                                                         */
+/*      FillZero: fill data buffer with zero of the appropriate type       */
+/*        used if s0<0 - fill up to 0, or up to ns+s0, whichever is less   */
+/*                                                                         */
+/***************************************************************************/
+static int FillZero(char *databuffer, char type, int s0, int ns) {
+  int nz;
+
+  if (s0>=0) return 0;
+  
+  if (s0+ns>0) nz = -s0;
+  else nz = ns;
+
+  switch (type) {
+  case 'c':
+    memset(databuffer, 0, nz);
+    break;
+  case 's':
+  case 'u':
+    memset(databuffer, 0, nz*sizeof(short));
+    break;
+  case 'i':
+  case 'S':
+  case 'U':
+    memset(databuffer, 0, nz*sizeof(int));
+    break;
+  case 'f':
+    memset(databuffer, 0, nz*sizeof(float));
+    break;
+  case 'd':
+    memset(databuffer, 0, nz*sizeof(double));    
+    break;
+  }
+
+  return (nz);
+  
+}
 /***************************************************************************/
 /*                                                                         */
 /*   Get samples per frame for field, given FormatType *F                  */
@@ -771,13 +834,22 @@ static int DoIfRaw(struct FormatType *F, const char *field_code,
       return(1);
     }
   }
-  
+
   databuffer = (unsigned char *)malloc(ns*R->size);
 
-  lseek(R->fp, s0*R->size, SEEK_SET);
-  bytes_read = read(R->fp, databuffer, ns*R->size);
-  *n_read = bytes_read/R->size;
+  *n_read = 0;
+  if (s0 < 0) {
+    *n_read = FillZero(databuffer, R->type, s0, ns);
+    ns -= *n_read;
+    s0 = 0;
+  }
 
+  if (ns>0) {
+    lseek(R->fp, s0*R->size, SEEK_SET);
+    bytes_read = read(R->fp, databuffer + *n_read*R->size, ns*R->size);
+    *n_read += bytes_read/R->size;
+  }
+  
   *error_code =
     ConvertType(databuffer, R->type, data_out, return_type, *n_read);
 
@@ -988,7 +1060,7 @@ static int DoIfLincom(struct FormatType *F, const char *field_code,
 
   if (L->n_infields > 1) {
     tmpbuf = AllocTmpbuff(return_type, *n_read);
-    for (i=0; i<L->n_infields; i++) {
+    for (i=1; i<L->n_infields; i++) {
       recurse_level++;
       if (DoField(F, L->in_fields[i],
 		  first_frame, first_samp,
@@ -1027,7 +1099,8 @@ static int DoIfBit(struct FormatType *F, const char *field_code,
   int i;
   int spf;
   int ns;
-
+  unsigned mask;
+  
   /******* binary search for the field *******/
   /* make a BitEntry we can compare to */
   strncpy(tB.field, field_code, FIELD_LENGTH); 
@@ -1041,8 +1114,11 @@ static int DoIfBit(struct FormatType *F, const char *field_code,
   recurse_level++;
   spf = GetSPF(B->raw_field, F, error_code);
   recurse_level--;
-  if (*error_code!=GD_E_OK) return(1);
-
+  if (*error_code!=GD_E_OK) {
+    *n_read = 0;
+    return(1);
+  }
+  
   ns = num_samp + num_frames*spf;
   tmpbuf = (unsigned *)malloc(ns*sizeof(unsigned));
 
@@ -1057,9 +1133,12 @@ static int DoIfBit(struct FormatType *F, const char *field_code,
     free(tmpbuf);
     return(1);
   }
-
+  
+  if (B->numbits==32) mask = 0xffffffff;
+  else mask = (unsigned)(pow(2,B->numbits)-0.9999);
+  
   for (i=0; i<*n_read; i++) {
-    tmpbuf[i] = (tmpbuf[i]>>B->bitnum) & 0x0001;
+    tmpbuf[i] = (tmpbuf[i]>>B->bitnum) & mask;
   }
 
   *error_code = ConvertType((unsigned char *)tmpbuf, 'U',
@@ -1224,7 +1303,10 @@ static int DoIfLinterp(struct FormatType *F, const char *field_code,
   /** if we got here, we found the field! **/
   if (I->n_interp<0) {
     *error_code = ReadLinterpFile(I);
-    if (*error_code != GD_E_OK) return(1);
+    if (*error_code != GD_E_OK) {
+      *n_read = 0;
+      return(1);
+    }
   }
   recurse_level++;
   *n_read = DoField(F, I->raw_field,
@@ -1244,7 +1326,7 @@ static int DoIfLinterp(struct FormatType *F, const char *field_code,
 /*  DoField: Doing one field once F has been identified                    */
 /*                                                                         */
 /***************************************************************************/
-int DoField(struct FormatType *F, const char *field_code,
+static int DoField(struct FormatType *F, const char *field_code,
 	    int first_frame, int first_samp,
 	    int num_frames, int num_samp, 
 	    char return_type, void *data_out,
@@ -1263,7 +1345,8 @@ int DoField(struct FormatType *F, const char *field_code,
       (strcmp(field_code,"INDEX")==0)) {
     n_read = num_frames + num_samp;
     if (data_out!=NULL) {
-      FillFileFrame(data_out, return_type, first_frame+first_samp, n_read);
+      FillFileFrame(data_out, return_type, first_frame+first_samp+
+                    F->frame_offset, n_read);
     }
     *error_code=GD_E_OK;
     return(n_read);
@@ -1342,7 +1425,8 @@ int GetData(const char *filename_in, const char *field_code,
   if (!F || *error_code != GD_E_OK) {
     return(0);
   }
-
+  first_frame -= F->frame_offset;
+  
   n_read = DoField(F, field_code,
 		   first_frame, first_samp,
 		   num_frames, num_samp,
@@ -1409,6 +1493,8 @@ int GetNFrames(const char *filename_in, int *error_code, const char *in_field) {
   nf = statbuf.st_size/
     (F->rawEntries[0].size*F->rawEntries[0].samples_per_frame);
 
+  nf += F->frame_offset;
+  
   nf -= 2;
   if (nf < 0)
     nf = 0;
@@ -1446,4 +1532,5 @@ int GetSamplesPerFrame(const char *filename_in, const char *field_name, int *err
 
   return (GetSPF(field_name, F, error_code));
 }
-/* vim: ts=2 sw=2 et */
+/* vim: ts=2 sw=2 et
+*/
