@@ -22,31 +22,31 @@
 
 
 class IndirectSource : public KstDataSource {
-public:
-  IndirectSource(const QString& filename, KstDataSourcePtr child);
+  public:
+    IndirectSource(KConfig *cfg, const QString& filename, KstDataSourcePtr child);
 
-  virtual ~IndirectSource();
+    virtual ~IndirectSource();
 
-  virtual KstObject::UpdateType update(int = -1);
+    virtual KstObject::UpdateType update(int = -1);
 
-  virtual int readField(double *v, const QString &field, int s, int n);
+    virtual int readField(double *v, const QString &field, int s, int n);
 
-  virtual bool isValidField(const QString &field) const;
+    virtual bool isValidField(const QString &field) const;
 
-  virtual int samplesPerFrame(const QString &field);
+    virtual int samplesPerFrame(const QString &field);
 
-  virtual int frameCount(const QString& field = QString::null) const;
+    virtual int frameCount(const QString& field = QString::null) const;
 
-  virtual QString fileType() const;
+    virtual QString fileType() const;
 
-  virtual void save(QTextStream &ts);
+    virtual void save(QTextStream &ts, const QString& indent = QString::null);
 
-  virtual bool isValid() const;
+    virtual bool isValid() const;
 
-  virtual bool isEmpty() const;
+    virtual bool isEmpty() const;
 
-private:
-  KstDataSourcePtr _child;
+  private:
+    KstDataSourcePtr _child;
 };
 
 
