@@ -20,7 +20,7 @@
 
 #include <config.h>
 
-#include <ksharedptr.h>
+#include <kstsharedptr.h>
 
 extern "C" {
 #include <HL2_PIOLIB/PIOLib.h>
@@ -34,7 +34,7 @@ namespace Planck {
     extern bool havePlanck();
     extern bool validDatabase(const QString&);
 
-class Source : public KShared {
+class Source : public KstShared {
   public:
     Source();
     virtual ~Source();
