@@ -176,7 +176,7 @@ int Object::readObject(const QString& object, double *buf, long start, long end)
       return 0;
     }
 
-    QString range;
+    QString range("");
 
     if (start < 0 || end < start) {
       // Leave blank
@@ -295,7 +295,7 @@ ObjectGroup::~ObjectGroup() {
 
 
 void ObjectGroup::close() {
-  kdDebug () << "Close group" << endl;
+  //kdDebug() << "Close group" << endl;
   if (_valid) {
     _PIOFREE(objNames);
     objNames = 0L;
@@ -311,8 +311,8 @@ void ObjectGroup::close() {
 
     PIOCloseVoidGrp(&_group);
     _group = 0L;
+    _valid = false;
   }
-  _valid = false;
 }
 
 
@@ -324,6 +324,8 @@ bool ObjectGroup::updateObjectList() {
     objTypes = 0L;
   }
 
+  _valid = false;
+
   PIOErr e = PIOGetObjectList(&objNames, &objTypes, _group);
   if (e > 0) {
     _valid = true;
@@ -338,52 +340,16 @@ bool ObjectGroup::updateObjectList() {
     /* there are two solution:
        - get directly only the group maximal size,
        - get the object size (it depend how KST manage index after. */
-#if 1     
-    PIOLONG LastIdx,FirstIdx;
+    PIOLONG LastIdx, FirstIdx;
     
-    PIOGetGrpSize(&FirstIdx,&LastIdx,_group);
+    PIOGetGrpSize(&FirstIdx, &LastIdx, _group);
 
     for (int i = 0; i < e; ++i) { 
       firstIndex[i] = FirstIdx;
       lastIndex[i] = LastIdx;
     }
-#else
-    for (int i = 0; i < e; ++i) { 
-      PIOSTRING ObjName;
-      
-      /* compute the complete name of an object */
-      /* Problem : if Objectname is bigger than 128 characters TODO */
-      sprintf(ObjName, "%s/%s", _group->HTMLName,objNames[i]);
-      firstIndex[i] = PIOGetBeginObjectIdx(ObjName, 0L);
-      lastIndex[i] = PIOGetEndObjectIdx(ObjName, 0L);
-    }
-      /* remove different vectorial size and set the bigger */
-    {
-      long max_index = 0;
-      for (int i = 0; i < e; ++i) {
-        if (max_index < lastIndex[i]) {
-          max_index = lastIndex[i];
-        }
-      /* add an index if no data were not written */
-      if (max_index < 1) {
-        max_index = 1;
-      }
-      for (int i = 0; i < e; ++i) {
-        firstIndex[i] = 0;
-        lastIndex[i] = max_index;
-      }
-    }
-#ifdef PIOLIBDEBUG
-      kdDebug() << "                  -> object: " << objNames[i] << endl;
-      kdDebug() << "                    -> first index: " << (long) firstIndex[i] << endl;
-      kdDebug() << "                    -> last index: " << (long) lastIndex[i] << endl;
-#endif
-
-#endif
-
-    return true;
   }
-  return false;
+  return _valid;
 }
 
 
