@@ -18,8 +18,8 @@
 #include "template.h"
 
  
-TemplateSource::TemplateSource(const QString& filename, const QString& type)
-: KstDataSource(filename, type) {
+TemplateSource::TemplateSource(KConfig *cfg, const QString& filename, const QString& type)
+: KstDataSource(cfg, filename, type) {
 }
 
 
@@ -65,8 +65,8 @@ QString TemplateSource::fileType() const {
 }
 
 
-void TemplateSource::save(QTextStream &ts) {
-  KstDataSource::save(ts);
+void TemplateSource::save(QTextStream &ts, const QString& indent) {
+  KstDataSource::save(ts, indent);
 }
 
 
@@ -76,24 +76,44 @@ bool TemplateSource::isEmpty() const {
 
 
 extern "C" {
-KstDataSource *create_template(const QString& filename, const QString& type) {
-  return new TemplateSource(filename, type);
+KstDataSource *create_template(KConfig *cfg, const QString& filename, const QString& type) {
+  return new TemplateSource(cfg, filename, type);
 }
 
+
 QStringList provides_template() {
   QStringList rc;
   // create the stringlist
   return rc;
 }
 
-int understands_template(const QString& filename) {
+
+int understands_template(KConfig *cfg, const QString& filename) {
   // determine if it is an X file
   // This should return a number 0 .. 100.  Normally you try to avoid 100
   // unless there is no way that this file is anything else except an X file.
+  Q_UNUSED(cfg)
   Q_UNUSED(filename)
   return 0;
 }
 
+
+QStringList fieldList_template(KConfig *cfg, const QString& filename, const QString& type, QString *typeSuggestion, bool *complete) {
+  Q_UNUSED(cfg)
+  Q_UNUSED(filename)
+  Q_UNUSED(type)
+  Q_UNUSED(typeSuggestion)
+  Q_UNUSED(complete)
+  return QStringList();
+}
+
+
+QWidget* widget_template(const QString& filename) {
+  Q_UNUSED(filename)
+  // Return 0L, or a widget that inherits from KstDataSourceConfigWidget
+  return 0L;
+}
+
 }
 
 // vim: ts=2 sw=2 et
