@@ -23,7 +23,7 @@
 
 class TemplateSource : public KstDataSource {
   public:
-    TemplateSource(const QString& filename, const QString& type);
+    TemplateSource(KConfig *cfg, const QString& filename, const QString& type);
 
     virtual ~TemplateSource();
 
@@ -39,7 +39,7 @@ class TemplateSource : public KstDataSource {
 
     virtual QString fileType() const;
 
-    virtual void save(QTextStream &ts);
+    virtual void save(QTextStream &ts, const QString& indent = QString::null);
 
     virtual bool isEmpty() const;
 };
