@@ -6,30 +6,28 @@
 ** init() function in place of a constructor, and a destroy() function in
 ** place of a destructor.
 *****************************************************************************/
+static const QString& defaultTag = KGlobal::staticQString("<Auto Name>");
 
-void DataWizard::init() {
+void DataWizard::init()
+{
+    _configWidget = 0L;
     KST::vectorDefaults.sync();
     QString default_source = KST::vectorDefaults.dataSource();
     _url->setMode(KFile::File | KFile::Directory | KFile::ExistingOnly | KFile::LocalOnly);
-    _url->setURL(default_source);
-
-    sourceChanged(default_source);
-
-    connect(_url, SIGNAL(textChanged(const QString&)), this, SLOT(sourceChanged(const QString&)));
-
     setAppropriate(_pageFilters, false);
     setIcon(BarIcon("kst_datawizard"));
 
     _kstDataRange->update();
     _kstFFTOptions->update();
 
-    _newWindowName->setText("W?");
+    _newWindowName->setText(defaultTag);
 
-    //setNextEnabled(_pageDataSource, false);
+    setNextEnabled(_pageDataSource, false);
     setNextEnabled(_pageVectors, false);
     setNextEnabled(_pageFilters, true);
     setFinishEnabled(_pagePlot, true);
 
+    disconnect(finishButton(), SIGNAL(clicked()), (QDialog*)this, SLOT(accept()));
     connect(finishButton(), SIGNAL(clicked()), this, SLOT(finished()));
 
     // No help button
@@ -39,22 +37,27 @@ void DataWizard::init() {
     setHelpEnabled(_pagePlot, false);
     _newFilter->setEnabled(false);
     _newFilter->hide(); // FIXME: implement this
+    _url->setURL(default_source);
+    _url->completionObject()->setDir(QDir::currentDirPath());
+    _url->setFocus();
 }
 
 
-bool DataWizard::xVectorOk() {
+bool DataWizard::xVectorOk()
+{
     QString txt = _xVector->currentText();
 
     for (int i = 0; i < _xVector->count(); ++i) {
-	if (_xVector->text(i) == txt) {
-	    return true;
-	}
+      if (_xVector->text(i) == txt) {
+       return true;
+      }
     }
     return false;
 }
 
 
-bool DataWizard::yVectorsOk() {
+bool DataWizard::yVectorsOk()
+{
     QListViewItemIterator it(_vectors);
 
     while (it.current()) {
@@ -84,58 +87,77 @@ void DataWizard::xChanged( const QString & txt )
 }
 
 
-void DataWizard::sourceChanged( const QString &txt ) {
-    bool ok = false;
-
+void DataWizard::sourceChanged( const QString &txt )
+{
+    delete _configWidget; // FIXME: very inefficient
+    _configWidget = 0L;
+    _configureSource->setEnabled(false);
+    _file = QString::null;
     if (!txt.isEmpty()) {
-	QString file = txt;
-	if (file.startsWith("file:")) {
-	    file.remove(0, 5);
-	}
-	KstDataSourcePtr ds = *KST::dataSourceList.findFileName(file);
-
-	if (!ds && file != "stdin" && file != "-") { // FIXME: deal with stdin properly
-	    ds = KstDataSource::loadSource(file);
-	    if (ds) {
-		KST::dataSourceList.lock().writeLock();
-		KST::dataSourceList.append(ds);
-		KST::dataSourceList.lock().writeUnlock();
-	    }
+	KURL url;
+	if (QFile::exists(txt) && QFileInfo(txt).isRelative()) {
+	    url.setPath(txt);
+	} else {
+	    url = KURL::fromPathOrURL(txt);
 	}
 
-	if (ds && _ds == ds) {
+	if (!url.isValid() || (!url.protocol().isEmpty() && url.protocol() != "file")) {
+	    setNextEnabled(_pageDataSource, false);
+	    _fileType->setText(QString::null);
 	    return;
 	}
 
-	_ds = ds;
+	QString file = url.path();
+	
+	KstDataSourcePtr ds = *KST::dataSourceList.findFileName(file);
+	QStringList fl;
+	bool complete = false;
+	QString fileType;
+
+	if (ds) {
+	    fl = ds->fieldList();
+	    fileType = ds->fileType();
+	    complete = ds->fieldListIsComplete();
+	    ds = 0L;
+	} else { // FIXME: deal with stdin?
+	    fl = KstDataSource::fieldListForSource(file, QString::null, &fileType, &complete);
+	}
+
+	if (!fl.isEmpty() && !fileType.isEmpty()) {
+	    _configWidget = KstDataSource::configWidgetForSource(file, fileType);
+	}
+
+	_configureSource->setEnabled(_configWidget);
+
+	_fileType->setText(fileType.isEmpty() ? QString::null : i18n("Data source of type: %1").arg(fileType));
+
+	if (fl.isEmpty()) {
+	    setNextEnabled(_pageDataSource, false);
+	    return;
+	}
 
 	_vectors->clear();
 	_xVector->clear();
 
-	if (_ds) {
-	    ok = true;
-	    QStringList fl = _ds->fieldList();
-	    _xVector->insertStringList(fl);
-	    _xVector->completionObject()->insertItems(fl);
-	    _xVector->setEditable(!_ds->fieldListIsComplete());
-	    for (QStringList::ConstIterator it = fl.begin(); it != fl.end(); ++it) {
-		new QCheckListItem(_vectors, *it, QCheckListItem::CheckBox);
-	    }
-	    KST::vectorDefaults.sync();
-	    QString defaultX = KST::vectorDefaults.wizardXVector();
-	    if (defaultX == "INDEX" || fl.contains(defaultX)) {
-		_xVector->setCurrentText(defaultX);
-	    }
+	_xVector->insertStringList(fl);
+	_xVector->completionObject()->insertItems(fl);
+	_xVector->setEditable(!complete);
+	for (QStringList::ConstIterator it = fl.begin(); it != fl.end(); ++it) {
+	    new QCheckListItem(_vectors, *it, QCheckListItem::CheckBox);
 	}
-    } else if (!_ds) {
+	KST::vectorDefaults.sync();
+	QString defaultX = KST::vectorDefaults.wizardXVector();
+	if (defaultX == "INDEX" || fl.contains(defaultX)) {
+	    _xVector->setCurrentText(defaultX);
+	}
+	_file = file;
+    } else {
+	_fileType->setText(QString::null);
 	setNextEnabled(_pageDataSource, false);
 	return;
-    } else {
-	setNextEnabled(_pageDataSource, false);
-	_ds = 0L; // deref
     }
     setNextEnabled(_pageVectors, xVectorOk() && yVectorsOk());
-    setNextEnabled(_pageDataSource, ok);
+    setNextEnabled(_pageDataSource, true);
 }
 
 
@@ -155,43 +177,6 @@ void DataWizard::showPage( QWidget *page )
 	QString save = _filterList->currentText();
 	_filterList->clear();
     } else if (page == _pagePlot) {
-	//
-	// check for sufficient memory...
-	//
-	unsigned long memoryRequested = 0, memoryAvailable = 1024*1024*1024; // 1GB
-#ifdef HAVE_LINUX
-	meminfo();
-	memoryAvailable = S(kb_main_free + kb_main_buffers + kb_main_cached);
-#endif
-	if (_kstDataRange->ReadToEnd->isChecked()) {
-	    int removeSamples = _kstDataRange->DoSkip->isChecked() ? _kstDataRange->Skip->value() : 0;
-	    memoryRequested += (_ds->frameCount(_xVector->currentText()) - removeSamples)*_ds->samplesPerFrame(_xVector->currentText())*sizeof(double);
-	} else {
-	    memoryRequested += _kstDataRange->N->value()*_ds->samplesPerFrame(_xVector->currentText())*sizeof(double);
-	}
-
-	{
-	    QListViewItemIterator it(_vectors);
-	    while (it.current()) {
-		QCheckListItem *i = static_cast<QCheckListItem*>(it.current());
-		if (i->isOn()) {
-		    QString field = it.current()->text(0);
-		    if (_kstDataRange->ReadToEnd->isChecked()) {
-			int removeSamples = _kstDataRange->DoSkip->isChecked() ? _kstDataRange->Skip->value() : 0;
-			memoryRequested += (_ds->frameCount(field) - removeSamples)*_ds->samplesPerFrame(field)*sizeof(double);
-		    } else {
-			memoryRequested += _kstDataRange->N->value()*_ds->samplesPerFrame(field)*sizeof(double);
-		    }
-		}
-		++it;
-	    }
-	}
-
-	if (memoryRequested > memoryAvailable) {
-	    KMessageBox::sorry(this, i18n("You requested to read in %1 MB of data but it seems that you only have approximately %2 MB of usable memory available.  Please load less data.").arg(memoryRequested/(1024*1024)).arg(memoryAvailable/(1024*1024)));
-	    return;
-	}
-
 	KST::vectorDefaults.setWizardXVector(_xVector->currentText());
 	KST::vectorDefaults.sync();
 
@@ -206,6 +191,8 @@ void DataWizard::showPage( QWidget *page )
 	    ++it;
 	}
 
+	_psdAxisGroup->setEnabled(_radioButtonPlotDataPSD->isChecked() || _radioButtonPlotPSD->isChecked());
+
 	// set window and plot defaults based on some guesses....
 	if (_radioButtonPlotDataPSD->isChecked()) { // plotting both psds and XY plots
 	    _newWindows->setEnabled(true);
@@ -376,16 +363,91 @@ void DataWizard::finished()
     KstApp *app = KstApp::inst();
     KstVectorList l;
     QString name = QString("V%1-%2").arg(KST::vectorList.count()).arg(_xVector->currentText());
+    QValueList<QColor> colors;
+    QColor color;
     uint n_curves = 0;
     uint n_steps = 0;
     int ptype = 0;
     int prg = 0;
 
+    KstDataSourcePtr ds = *KST::dataSourceList.findFileName(_file);
+    if (!ds) {
+	ds = KstDataSource::loadSource(_file);
+	if (!ds) {
+	    KMessageBox::sorry(this, i18n("Sorry, unable to load the data file."));
+	    return;
+	}
+	KST::dataSourceList.lock().writeLock();
+	KST::dataSourceList.append(ds);
+	KST::dataSourceList.lock().writeUnlock();
+    }
+
+    //
+    // check for sufficient memory...
+    //
+    unsigned long memoryRequested = 0, memoryAvailable = 1024*1024*1024; // 1GB
+    unsigned long frames;
+#ifdef HAVE_LINUX
+    meminfo();
+    memoryAvailable = S(kb_main_free + kb_main_buffers + kb_main_cached);
+#endif
+    
+    if (_kstDataRange->ReadToEnd->isChecked()) {
+	frames = ds->frameCount(_xVector->currentText()) - _kstDataRange->F0->value();
+    } else {
+	frames = _kstDataRange->N->value();
+	if (frames > ds->frameCount(_xVector->currentText())) {
+	    frames = ds->frameCount(_xVector->currentText());
+	}
+    }
+
+    if (_kstDataRange->DoSkip->isChecked()) {
+	memoryRequested += frames / _kstDataRange->Skip->value() * sizeof(double);;
+    } else {
+	memoryRequested += frames * ds->samplesPerFrame(_xVector->currentText())*sizeof(double);
+    }
+
     {
 	QListViewItemIterator it(_vectors);
 	while (it.current()) {
 	    QCheckListItem *i = static_cast<QCheckListItem*>(it.current());
 	    if (i->isOn()) {
+		QString field = it.current()->text(0);
+
+		if (_kstDataRange->ReadToEnd->isChecked()) {
+		    frames = ds->frameCount(field) - _kstDataRange->F0->value();
+		} else {
+		    frames = _kstDataRange->N->value();
+		    if (frames > ds->frameCount(field)) {
+			frames = ds->frameCount();
+		    }
+		}
+
+		if (_kstDataRange->DoSkip->isChecked()) {
+		    memoryRequested += frames / _kstDataRange->Skip->value()*sizeof(double);;
+		} else {
+		    memoryRequested += frames * ds->samplesPerFrame(field)*sizeof(double);
+		}                
+	    }
+	    ++it;
+	}
+    }
+
+    if (memoryRequested > memoryAvailable) {
+	KMessageBox::sorry(this, i18n("You requested to read in %1 MB of data but it seems that you only have approximately %2 MB of usable memory available.  Please load less data.").arg(memoryRequested/(1024*1024)).arg(memoryAvailable/(1024*1024)));
+	return;
+    }
+
+    accept();
+
+    {
+	QListViewItemIterator it(_vectors);
+	while (it.current()) {
+	    QCheckListItem *i = static_cast<QCheckListItem*>(it.current());
+	    if (i->isOn()) {
+		if (_radioButtonPlotPSD->isChecked() || _radioButtonPlotDataPSD->isChecked()) {
+		    n_steps++;
+		}
 		n_steps++;
 	    }
 	    ++it;
@@ -399,7 +461,7 @@ void DataWizard::finished()
     //
     // create the x-vector...
     //
-    KstVectorPtr xv = new KstRVector(_ds, _xVector->currentText(),
+    KstVectorPtr xv = new KstRVector(ds, _xVector->currentText(),
 	    name, _kstDataRange->CountFromEnd->isChecked() ? -1 : _kstDataRange->F0->value(),
 	    _kstDataRange->ReadToEnd->isChecked() ? -1 : _kstDataRange->N->value(),
 	    _kstDataRange->DoSkip->isChecked() ? _kstDataRange->Skip->value() : 0, _kstDataRange->DoSkip->isChecked(),
@@ -416,7 +478,7 @@ void DataWizard::finished()
 	    QCheckListItem *i = static_cast<QCheckListItem*>(it.current());
 	    if (i->isOn()) {
 		name = QString("V%1-%2").arg(KST::vectorList.count()).arg(it.current()->text(0));
-		KstVectorPtr v = new KstRVector(_ds, it.current()->text(0), name,
+		KstVectorPtr v = new KstRVector(ds, it.current()->text(0), name,
 			_kstDataRange->CountFromEnd->isChecked() ? -1 : _kstDataRange->F0->value(),
 			_kstDataRange->ReadToEnd->isChecked() ? -1 : _kstDataRange->N->value(),
 			_kstDataRange->DoSkip->isChecked() ? _kstDataRange->Skip->value() : 0, _kstDataRange->DoSkip->isChecked(),
@@ -437,9 +499,10 @@ void DataWizard::finished()
     KstViewWindow *w;
     if (_newWindow->isChecked() || _newWindows->isChecked()) {
 	newName = _newWindowName->text();
-	if (newName=="W?") newName = KST::suggestWinName();
-	QString n = app->newWindow(newName);
-	w = static_cast<KstViewWindow*>(app->findWindow(n));
+	if (newName == defaultTag) {
+	    newName = KST::suggestWinName();
+	}
+	w = static_cast<KstViewWindow*>(app->findWindow(app->newWindow(newName)));
     } else if (_currentWindow->isChecked()) {
 	w = static_cast<KstViewWindow*>(app->activeWindow());
     } else {
@@ -453,40 +516,40 @@ void DataWizard::finished()
     //
     // create the necessary plots...
     //
-    app->slotUpdateProgress(0, 0, i18n("Creating plots..."));
+    app->slotUpdateProgress(n_steps, prg, i18n("Creating plots..."));
     KstViewObjectList plots;
     bool relayout = true;
     if (_onePlot->isChecked()) {
-	Kst2DPlotPtr p = dynamic_cast<Kst2DPlot*>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())).data());
+	Kst2DPlotPtr p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
 	plots.append(p.data());
 	if (_radioButtonPlotDataPSD->isChecked()) {
 	    if (_newWindows->isChecked()) {
-		newName = newName+"-PSD";
+		newName += "-PSD";
 		QString n = app->newWindow(newName);
 		w = static_cast<KstViewWindow*>(app->findWindow(n));
 	    }
-	    Kst2DPlotPtr p = dynamic_cast<Kst2DPlot*>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())).data());
+	    Kst2DPlotPtr p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
 	    plots.append(p.data());
 	}
     } else if (_multiplePlots->isChecked()) {
 	Kst2DPlotPtr p;
 	if (_radioButtonPlotDataPSD->isChecked()) {
 	    for (uint i = 0; i < l.count(); ++i) {
-		p = dynamic_cast<Kst2DPlot*>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())).data());
+		p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
 		plots.append(p.data());
 	    }
 	    if (_newWindows->isChecked()) {
-		newName = newName+"-PSD";
+		newName += "-PSD";
 		QString n = app->newWindow(newName);
 		w = static_cast<KstViewWindow*>(app->findWindow(n));
 	    }
 	}
 	for (uint i = 0; i < l.count(); ++i) {
-	    p = dynamic_cast<Kst2DPlot*>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())).data());
+	    p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
 	    plots.append(p.data());
 	}
     } else if (_existingPlot->isChecked()) {
-	Kst2DPlotPtr p = dynamic_cast<Kst2DPlot*>(w->view()->findChild(_existingPlotName->currentText()).data());
+	Kst2DPlotPtr p = kst_cast<Kst2DPlot>(w->view()->findChild(_existingPlotName->currentText()));
 	plots.append(p.data());
 	relayout = false;
     } else if (_cycleExisting->isChecked()) {
@@ -499,16 +562,16 @@ void DataWizard::finished()
 	Kst2DPlotPtr p;
 	if (_radioButtonPlotDataPSD->isChecked()) {
 	    for (int i = 0; i < _plotNumber->value(); ++i) {
-		p = dynamic_cast<Kst2DPlot*>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())).data());
+		p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
 		plots.append(p.data());
 	    }
 	    if (_newWindows->isChecked()) {
-		QString n = app->newWindow(newName+"-PSD");
+		QString n = app->newWindow(newName + "-PSD");
 		w = static_cast<KstViewWindow*>(app->findWindow(n));
 	    }
 	}
 	for (int i = 0; i < _plotNumber->value(); ++i) {
-	    p = dynamic_cast<Kst2DPlot*>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())).data());
+	    p = kst_cast<Kst2DPlot>(w->view()->findChild(w->createPlot<Kst2DPlot>(KST::suggestPlotName())));
 	    plots.append(p.data());
 	}
     }
@@ -516,13 +579,19 @@ void DataWizard::finished()
     //
     // create the data curves...
     //
-    app->slotUpdateProgress(0, 0, i18n("Creating curves..."));
+    app->slotUpdateProgress(n_steps, prg, i18n("Creating curves..."));
     KstViewObjectList::Iterator pit = plots.begin();
     for (KstVectorList::Iterator it = l.begin(); it != l.end(); ++it) {
 	if (_radioButtonPlotData->isChecked() || _radioButtonPlotDataPSD->isChecked()) {
 	    name = KST::suggestCurveName((*it)->tagName());
-	    KstBaseCurvePtr c = new KstVCurve(name, xv, *it, 0L, 0L,
-		    KstColorSequence::next(static_cast<Kst2DPlot*>((*pit).data())->backgroundColor()));
+	    Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(*pit);
+	    if (plot) {
+		color = KstColorSequence::next(plot->Curves, plot->backgroundColor());
+	    } else {
+		color = KstColorSequence::next();
+	    }
+	    colors.append(color);
+	    KstBaseCurvePtr c = new KstVCurve(name, xv, *it, 0L, 0L, color);
 	    if (_drawBoth->isChecked()) {
 		c->setHasPoints(true);
 		c->Point.setType(ptype++ % KSTPOINT_MAXTYPE);
@@ -538,7 +607,9 @@ void DataWizard::finished()
 	    KST::dataObjectList.lock().writeLock();
 	    KST::dataObjectList.append(KstDataObjectPtr(c));
 	    KST::dataObjectList.lock().writeUnlock();
-	    static_cast<Kst2DPlot*>((*pit).data())->addCurve(c);
+	    if (plot) {
+		plot->addCurve(c);
+	    }
 	    if (!_onePlot->isChecked()) { // change plots if we are not onePlot...
 		if (_radioButtonPlotDataPSD->isChecked()) { // if xy and psd
 		    ++pit;
@@ -552,8 +623,10 @@ void DataWizard::finished()
 	}
     }
 
-    if (_onePlot->isChecked()) { // if we are one plot, now we can move to the psd plot
-	if (++pit == plots.end()) { // if _newWindows is not checked, there will not be another.
+    if (_onePlot->isChecked()) {
+        // if we are one plot, now we can move to the psd plot
+	if (++pit == plots.end()) {
+	    // if _newWindows is not checked, there will not be another.
 	    pit = plots.begin();
 	}
     } else if (_radioButtonPlotDataPSD->isChecked()) {
@@ -563,20 +636,44 @@ void DataWizard::finished()
     //
     // create the PSDs...
     //
-    for (KstVectorList::Iterator it = l.begin(); it != l.end(); ++it) {
-	if (_radioButtonPlotPSD->isChecked() || _radioButtonPlotDataPSD->isChecked()) {
+    if (_radioButtonPlotPSD->isChecked() || _radioButtonPlotDataPSD->isChecked()) {
+	KstVCurvePtr c;
+	int indexColor = 0;
+	ptype = 0;
+	app->slotUpdateProgress(n_steps, prg, i18n("Creating PSDs..."));
+	
+	for (KstVectorList::Iterator it = l.begin(); it != l.end(); ++it) {
 	    if ((*it)->length() > 0) {
+		Kst2DPlotPtr plot;
+		KstViewObjectList::Iterator startPlot = pit;
+		while (!plot) {
+		    plot = kst_cast<Kst2DPlot>(*pit);
+		    if (!plot) {
+		       	if (++pit == plots.end()) {
+			    pit = plots.begin();
+			}
+			// If this is ever false, we have no valid 2D plots
+			// which means that someone wrote some incomplete code
+			// or something is really broken
+			assert(pit != startPlot);
+		    }
+		}
 	        name = KST::suggestPSDName((*it)->tagName());
 
-		KstBaseCurvePtr c = new KstPSDCurve(name, *it,
+		KstPSDPtr p = new KstPSD(name, *it,
 			_kstFFTOptions->SampRate->text().toDouble(),
 			_kstFFTOptions->Interleaved->isChecked(),
 			_kstFFTOptions->FFTLen->text().toInt(),
 			_kstFFTOptions->Apodize->isChecked(),
 			_kstFFTOptions->RemoveMean->isChecked(),
 			_kstFFTOptions->VectorUnits->text(),
-			_kstFFTOptions->RateUnits->text(),
-			KstColorSequence::next(static_cast<Kst2DPlot*>((*pit).data())->backgroundColor()));
+			_kstFFTOptions->RateUnits->text());
+		if (_radioButtonPlotPSD->isChecked() || colors.count() <= indexColor) {
+		    c = new KstVCurve(name + "-curve", p->vX(), p->vY(), 0L, 0L, KstColorSequence::next(plot->Curves, plot->backgroundColor()));
+		} else {
+		    c = new KstVCurve(name + "-curve", p->vX(), p->vY(), 0L, 0L, colors[indexColor]);
+		    indexColor++;
+		}
 		if (_drawBoth->isChecked()) {
 		    c->setHasPoints(true);
 		    c->Point.setType(ptype++ % KSTPOINT_MAXTYPE);
@@ -590,16 +687,23 @@ void DataWizard::finished()
 		    c->setHasLines(false);
 		}
 		KST::dataObjectList.lock().writeLock();
+		KST::dataObjectList.append(KstDataObjectPtr(p));
 		KST::dataObjectList.append(KstDataObjectPtr(c));
 		KST::dataObjectList.lock().writeUnlock();
-		static_cast<Kst2DPlot*>((*pit).data())->addCurve(c);
+		plot->addCurve(c.data());
+	        plot->setLog(_psdLogX->isChecked(),_psdLogY->isChecked());
 		if (!_onePlot->isChecked()) { // change plots if we are not onePlot...
 		    if (++pit == plots.end()) {
+                      if (_radioButtonPlotDataPSD->isChecked()) { // if xy and psd
+                        pit = plots.at(plots.count()/2);
+                      } else {
 			pit = plots.begin();
+                      }
 		    }
 		}
 	    }
 	}
+	app->slotUpdateProgress(n_steps, ++prg, i18n("Creating PSDs..."));
     }
 
     //
@@ -611,7 +715,11 @@ void DataWizard::finished()
 
     pit = plots.begin();
     while (pit != plots.end()) {
-	Kst2DPlotPtr pp = static_cast<Kst2DPlot*>((*pit).data());
+	Kst2DPlotPtr pp = kst_cast<Kst2DPlot>(*pit);
+	if (!pp) {
+	    ++pit;
+	    continue;
+	}
 	pp->GenerateDefaultLabels();
 	if (!xl) {
 	    pp->setXLabel(QString::null);
@@ -703,4 +811,19 @@ void DataWizard::_enableWindowEntries()
     _windowGroup->setEnabled(true);
 }
 
+
+void DataWizard::configureSource()
+{
+    assert(_configWidget);
+    KDialogBase *dlg = new KDialogBase(this, "Data Config Dialog", true, i18n("Configure Data Source"));
+    connect(dlg, SIGNAL(okClicked()), _configWidget, SLOT(save()));
+    connect(dlg, SIGNAL(applyClicked()), _configWidget, SLOT(save()));
+    _configWidget->reparent(dlg, QPoint(0, 0));
+    dlg->setMainWidget(_configWidget);
+    dlg->exec();
+    _configWidget->reparent(0L, QPoint(0, 0));
+    dlg->setMainWidget(0L);
+    delete dlg;
+}
+
 // vim: ts=8 sw=4 noet
