@@ -52,6 +52,24 @@ function ScalarTrigger() {
 		this.dialog.show();
 	}
 
-	this.show();
+	this.hide = function() {
+		this.dialog.hide();
+	}
+
+	this.showhide_bool = function(b) {
+		if (b) {
+			this.show();
+		} else {
+			this.hide();
+		}
+	}
+
+	var gui = KstUIMerge.mergeUI('/home/staikos/cita/kdeextragear-2/kst/kst/extensions/js/examples/scalartrigger.rc');
+	this.action = new KToggleAction(gui, 'kst_scalar_trigger');
+	this.action.text = "Scalar Triggers";
+	this.action.connect(this.action, 'toggled(bool)', this, 'showhide_bool');
+	gui.merge();
 }
 
+KstScriptRegistry.addScript("Scalar Trigger", "ScalarTrigger");
+
