@@ -91,7 +91,7 @@ void KstJS::load(QDomElement& e) {
 }
 
 
-void KstJS::save(QTextStream& ts) {
+void KstJS::save(QTextStream& ts, const QString& indent) {
 }
 
 
