@@ -33,7 +33,7 @@ class KstJS : public KstExtension, public KXMLGUIClient {
 
     // To save state
     virtual void load(QDomElement& e);
-    virtual void save(QTextStream& ts);
+    virtual void save(QTextStream& ts, const QString& indent = QString::null);
 
     KJSEmbed::KJSEmbedPart *part() const { return _jsPart; }
 
