@@ -216,6 +216,10 @@ macro definitions
         .
 */
 
+void cdft(int n, int isgn, double *a);
+void rdft(int n, int isgn, double *a);
+void dfct(int n, double *a);
+void dfst(int n, double *a);
 
 static void cftfsub(int n, double *a);
 static void cftbsub(int n, double *a);
