@@ -15,35 +15,23 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <stdlib.h>
-#include <stdio.h>
+#include <assert.h>
 
 // include files for Qt
-#include <qevent.h>
 #include <qeventloop.h>
-#include <qdir.h>
-#include <qobjectlist.h>
-#include <qpainter.h>
 #include <qpaintdevicemetrics.h>
 #include <qprogressbar.h>
-#include <qpushbutton.h>
-#include <qtimer.h>
+#include <qvalidator.h>
 
 // include files for KDE
 #include <kaccel.h>
-#include <kconfig.h>
 #include <kdebug.h>
 #include <kfiledialog.h>
-#include <kiconloader.h>
 #include <kkeydialog.h>
-#include <klocale.h>
-#include <kmdimainfrm.h>
-#include <kmenubar.h>
 #include <kmessagebox.h>
 #include <kprinter.h>
 #include <kstandarddirs.h>
 #include <kstatusbar.h>
-#include <kstdaction.h>
 
 // application specific includes
 #include "datawizard.h"
@@ -54,13 +42,11 @@
 #include "kstchangenptsdialog_i.h"
 #include "kstcurvedialog_i.h"
 #include "kstdatamanager_i.h"
-#include "kstdebug.h"
 #include "kstdebugdialog_i.h"
 #include "kstdoc.h"
 #include "ksteqdialog_i.h"
 #include "kstevents.h"
 #include "ksteventmonitor_i.h"
-#include "kstextension.h"
 #include "kstfitdialog_i.h"
 #include "kstfilterdialog_i.h"
 #include "kstgraphfiledialog_i.h"
@@ -69,20 +55,18 @@
 #include "kstimagedialog_i.h"
 #include "kstlabeldialog_i.h"
 #include "kstmatrixdialog_i.h"
-#include "kstobject.h"
-#include "kstplotbase.h"
 #include "kstplotdialog_i.h"
 #include "kstplugindialog_i.h"
+#include "kstprintoptionspage.h"
 #include "kstpsddialog_i.h"
 #include "kstsettingsdlg.h"
-#include "ksttoplevelview.h"
 #include "kstvectordefaults.h"
 #include "kstvectordialog_i.h"
 #include "kstviewscalarsdialog_i.h"
 #include "kstviewvectorsdialog_i.h"
 #include "kstviewfitsdialog_i.h"
-#include "kstviewwidget.h"
 #include "kstviewwindow.h"
+#include "kstquickstartdialog_i.h"
 #include "pluginmanager.h"
 #include "psversion.h"
 #include "sysinfo.h"
@@ -93,19 +77,30 @@
 #define KST_STATUSBAR_DATA   1
 #define KST_STATUSBAR_STATUS 2
 
+#define KST_QUICKSTART_DLG
 
 static KstApp *inst = 0L;
 
+static const QString& defaultTag = KGlobal::staticQString("<Auto Name>");
+
+void KstApp::doubleCheckCleanup() {
+  delete ::inst;
+  ::inst = 0L;
+}
+
 KstApp* KstApp::inst() {
   return ::inst;
 }
 
+static KConfig *kConfigObject = 0L;
 
 KstApp::KstApp(QWidget *parent, const char *name)
 : KMdiMainFrm(parent, name) {
   assert(!::inst);
   ::inst = this;
 
+  _dataSourceConfig = kConfigObject;
+
   clearWFlags(WDestructiveClose);
 
   _stopping = false;
@@ -127,6 +122,9 @@ KstApp::KstApp(QWidget *parent, const char *name)
   graphFileDialog = new KstGraphFileDialogI(this);
   vectorSaveDialog = new VectorSaveDialog(this);
   _labelDialog = new KstLabelDialogI(this);
+#ifdef KST_QUICKSTART_DLG
+  _quickStartDialog = new KstQuickStartDialogI(this, 0 , true);
+#endif
 
   connect(_labelDialog, SIGNAL(applied()), doc, SLOT(wasModified()));
   connect(KstVectorDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
@@ -160,6 +158,9 @@ KstApp::KstApp(QWidget *parent, const char *name)
   /*** plot dialog ***/
   connect(_plotDialog, SIGNAL(docChanged()), this, SLOT(registerDocChange()));
 
+  //data manager signals
+  connect(dataManager, SIGNAL(docChanged()), this, SLOT(registerDocChange()));
+
   /*** ***/
   connect(doc, SIGNAL(updateDialogs()), this, SLOT(updateDialogs()));
   connect(doc, SIGNAL(dataChanged()), this, SLOT(updateDataDialogs()));
@@ -174,7 +175,9 @@ KstApp::KstApp(QWidget *parent, const char *name)
 
   QTimer::singleShot(0, this, SLOT(updateActions()));
 
-  // Load any extensions
+  //
+  // load any extensions
+  //
   ExtensionMgr *mgr = ExtensionMgr::self();
   mgr->setWindow(this);
   KService::List sl = KServiceType::offers("Kst Extension");
@@ -188,6 +191,11 @@ KstApp::KstApp(QWidget *parent, const char *name)
   }
 
   checkFontPresent("Symbol");
+  //checkFontPresent("Helvetica");
+
+  QFont defaultFont; // QT's current default font
+  QFontInfo info(defaultFont);
+  _defaultFont = info.family();
 
   if (!isFakingSDIApplication()) {
     bool addedWindowMenu = false;
@@ -207,9 +215,63 @@ KstApp::KstApp(QWidget *parent, const char *name)
     if (!addedWindowMenu) {
       menuBar()->insertItem(i18n("&Window"), windowMenu());
     }
+    QObject::connect( windowMenu(), SIGNAL(aboutToShow()), this, SLOT(addNewWindowMenu()) );
   }
 }
 
+
+KstApp::~KstApp() {
+  _updateThread->setFinished(true);
+#if 0
+  if (!_updateThread->wait(250)) { // 250ms
+    _updateThread->terminate();
+#endif
+    _updateThread->wait(3000); // QThread object destroyed while thread running on RH 9, but freezes on exit without it.  ->terminate() crashes.  Can we win?
+#if 0
+  }
+#endif
+
+  KstDataSource::cleanupForExit(); // must be before deletions
+  delete _updateThread;
+  _updateThread = 0L;
+  delete _dcopIface;
+  _dcopIface = 0L;
+  ::inst = 0L;
+  if (_dataSourceConfig) {
+    _dataSourceConfig->sync();
+    _dataSourceConfig = 0L;
+  }
+  delete kConfigObject; // must be after cleanupForExit
+  kConfigObject = 0L;
+}
+
+
+void KstApp::initialize() {
+  kConfigObject = new KConfig("kstdatarc", false, false);
+  KstDataSource::setupOnStartup(kConfigObject);
+  // NOTE: This is leaked in commandline mode if we never create a KstApp.
+  //       Not too much of a problem right now, and less messy than hooking in
+  //       cleanups in main.
+}
+
+
+void KstApp::addNewWindowMenu() {
+  int id = windowMenu()->insertItem(i18n("&New..."), this, SLOT(slotFileNewWindow()), 0, -1, 0);
+  windowMenu()->setWhatsThis(id, i18n("Create a new plot window."));
+
+  id = windowMenu()->insertItem(i18n("&Rename..."), this, SLOT(slotFileRenameWindow()), 0, -1, 0);
+  windowMenu()->setWhatsThis(id, i18n("Rename an existing plot window."));
+  if (!activeView()) {
+    windowMenu()->setItemEnabled(id, false);
+  }
+}
+
+
+const QString& KstApp::defaultFont() const {
+  return _defaultFont;
+}
+
+
 void KstApp::checkFontPresent(const QString& font) {
   QFont f(font);
   QFontInfo info(f);
@@ -231,12 +293,18 @@ void KstApp::EventELOGConfigure() {
 
 
 void KstApp::customEvent(QCustomEvent *pEvent) {
-  if(pEvent->type() == KstELOGDebugInfoEvent) {
+  if (pEvent->type() == KstELOGAliveEvent) {
+    KstEventMonitorI::globalInstance()->enableELOG();
+  } else if (pEvent->type() == KstELOGDeathEvent) {
+    KstEventMonitorI::globalInstance()->disableELOG();
+  } else if (pEvent->type() == KstELOGDebugInfoEvent) {
     QTextStream *pTextStream = (QTextStream*)pEvent->data();
     (*pTextStream) << KstDebug::self()->text();
   } else if (pEvent->type() == KstELOGConfigureEvent) {
     QTextStream *pTextStream = (QTextStream*)pEvent->data();
-    document()->saveDocument(*pTextStream);
+    if (document()) {
+      document()->saveDocument(*pTextStream, true);
+    }
   } else if (pEvent->type() == KstELOGCaptureEvent) {
     KstELOGCaptureStruct* pCapture = (KstELOGCaptureStruct*)pEvent->data();
 
@@ -252,32 +320,20 @@ void KstApp::customEvent(QCustomEvent *pEvent) {
   }
 }
 
+
 void KstApp::updateActions() {
   // Hack
   ToolBarAction->setChecked(toolBar("mainToolBar")->isShown());
   StatusBarAction->setChecked(statusBar()->isShown());
 }
 
-KstApp::~KstApp() {
-  _updateThread->setFinished(true);
-  if (!_updateThread->wait(250)) { // 250ms
-    _updateThread->terminate();
-    _updateThread->wait();
-  }
-
-  delete _updateThread;
-  delete _dcopIface;
-  _dcopIface = 0L;
-  ::inst = 0L;
-}
 
 void KstApp::initActions() {
-
+#if 0
   fileNewWindow = new KAction(i18n("New &Window..."), 0, 0,
                               this, SLOT(slotFileNewWindow()),
                               actionCollection(),"file_new_window");
 
-#if 0
   fileOpenNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
   fileOpenNew->setWhatsThis(i18n("Open Kst plot file."));
 #endif
@@ -294,7 +350,12 @@ void KstApp::initActions() {
   fileQuit = KStdAction::quit(this, SLOT(slotFileClose()), actionCollection());
   fileQuit->setWhatsThis(i18n("Quit Kst."));
 
+#if 0
+  // KDE 3.3 only
+  fileKeyBindings = KStdAction::keyBindings(guiFactory(), SLOT(slotConfigureKeys()), actionCollection());
+#else
   fileKeyBindings = KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), actionCollection());
+#endif
   fileKeyBindings->setWhatsThis(i18n("Bring up a dialog box\n"
                                      "to configure shortcuts."));
 
@@ -390,7 +451,7 @@ void KstApp::initActions() {
                                       "to edit plot settings."));
 
   /************/
-  DataManagerAction = new KAction(i18n("&Data Manager..."), "kst_datamanager", 0,
+  DataManagerAction = new KAction(i18n("&Data Manager"), "kst_datamanager", 0,
                                   dataManager, SLOT(show_I()),
                                   actionCollection(), "datamanager_action");
   DataManagerAction->setWhatsThis(i18n("Bring up a dialog box\n"
@@ -470,7 +531,7 @@ void KstApp::initActions() {
                                             "to change input files."));
 
   /************/
-  ViewScalarsDialogAction = new KAction(i18n("View &Scalar Values..."),
+  ViewScalarsDialogAction = new KAction(i18n("View &Scalar Values"),
                                        0, 0, this,
                                        SLOT(showViewScalarsDialog()),
                                        actionCollection(),
@@ -479,7 +540,7 @@ void KstApp::initActions() {
                                             "to view scalar values."));
 
   /************/
-  ViewVectorsDialogAction = new KAction(i18n("View Vec&tor Values..."),
+  ViewVectorsDialogAction = new KAction(i18n("View Vec&tor Values"),
                                        0, 0, this,
                                        SLOT(showViewVectorsDialog()),
                                        actionCollection(),
@@ -488,7 +549,7 @@ void KstApp::initActions() {
                                             "to view vector values."));
 
   /************/
-  ViewFitsDialogAction = new KAction(i18n("View &Fit Results..."),
+  ViewFitsDialogAction = new KAction(i18n("View &Fit Results"),
                                        0, 0, this,
                                        SLOT(showViewFitsDialog()),
                                        actionCollection(),
@@ -584,7 +645,7 @@ void KstApp::initActions() {
 
 
   /************/
-  DataWizardAction = new KAction(i18n("Data &Wizard..."), "kst_datawizard", 0,
+  DataWizardAction = new KAction(i18n("Data &Wizard"), "kst_datawizard", 0,
                                  this, SLOT(showDataWizard()),
                                  actionCollection(), "datawizard_action");
   DataWizardAction->setWhatsThis(i18n("Bring up a wizard\n"
@@ -622,9 +683,28 @@ void KstApp::initActions() {
 }
 
 
-void KstApp::slotConfigureKeys()
-{
-  KKeyDialog::configure( actionCollection(), this );
+void KstApp::slotConfigureKeys() {
+#if 0
+  // KDE 3.3 only:
+  KKeyDialog::configure(actionCollection(), this);
+#else
+  KKeyDialog dlg(true, this);
+  dlg.insert(actionCollection());
+  ExtensionMgr *mgr = ExtensionMgr::self();
+  const QMap<QString, bool>& extensions = mgr->extensions();
+  for (QMap<QString, bool>::ConstIterator it = extensions.begin(); it != extensions.end(); ++it) {
+    if (it.data()) {
+      KXMLGUIClient *gc = dynamic_cast<KXMLGUIClient*>(mgr->extension(it.key()));
+      if (gc) {
+        KActionCollection *ac = gc->actionCollection();
+        if (ac) {
+          dlg.insert(ac);
+        }
+      }
+    }
+  }
+  dlg.configure(true);
+#endif
 }
 
 
@@ -651,13 +731,19 @@ KstApp::KstZoomType KstApp::getZoomRadio() {
   }
 }
 
-bool KstApp::getTieZoom() {
-  return 0; // (TiedZoomAction->isChecked());
-}
 
 void KstApp::initStatusBar() {
   _dataBar = new QLabel(QString::null, statusBar());
-  statusBar()->addWidget(_dataBar, 0, true);
+  statusBar()->addWidget(_dataBar, 5, true);
+
+  _readyBar = new KstQuickLabel(i18n("Almost Ready"), statusBar());
+  _readyBar->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
+  statusBar()->addWidget(_readyBar, 5, true);
+
+  _progressBar = new KstQuickProgressBar(statusBar());
+  _progressBar->setPercentageVisible(false);
+  _progressBar->setCenterIndicator(true);
+  statusBar()->addWidget(_progressBar, 2, true);
 
 #ifdef HAVE_LINUX
   _memoryBar = new QLabel(i18n("0 MB available"), statusBar());
@@ -666,18 +752,6 @@ void KstApp::initStatusBar() {
   _memTimer.start(5000);
 #endif
 
-  _progressMsg = new QLabel(QString::null, statusBar());
-  statusBar()->addWidget(_progressMsg, 0, true);
-
-  _progressBar = new QProgressBar(statusBar());
-  _progressBar->setPercentageVisible(false);
-  _progressBar->setCenterIndicator(true);
-  statusBar()->addWidget(_progressBar, 0, true);
-  _progressBar->hide();
-  _progressMsg->hide();
-
-  statusBar()->insertItem(i18n("Almost Ready"), KST_STATUSBAR_STATUS);
-
   statusBar()->show();
   slotUpdateStatusMsg(i18n("Ready"));
 }
@@ -801,8 +875,7 @@ bool KstApp::queryClose() {
   if (doc->saveModified()) {
      doc->cancelUpdate();
     _stopping = true;
-    kapp->processEvents(); // danger
-    doc->deleteContents();
+    QTimer::singleShot(0, doc, SLOT(deleteContents()));
     return true;
   }
   return false;
@@ -833,24 +906,29 @@ void KstApp::slotFileOpen() {
     if (!fileToOpen.isEmpty()) {
       doc->deleteContents();
       doc->setModified(false);
-      doc->openDocument(fileToOpen);
-      setCaption(doc->title());
-      addRecentFile(fileToOpen);
+      if (doc->openDocument(fileToOpen)) {
+        setCaption(doc->title());
+        addRecentFile(fileToOpen);
+      }
     }
   }
 
   slotUpdateStatusMsg(i18n("Ready"));
 }
 
-void KstApp::slotFileOpenRecent(const KURL& newfile) {
+bool KstApp::slotFileOpenRecent(const KURL& newfile) {
+  bool returnVal = false;
   slotUpdateStatusMsg(i18n("Opening file..."));
 
   if(doc->saveModified()) {
-    doc->openDocument(newfile);
+    if (doc->openDocument(newfile)) {
+      returnVal = true;
+    }
     setCaption(kapp->caption() + ": " + doc->title());
   }
 
   slotUpdateStatusMsg(i18n("Ready"));
+  return returnVal;
 }
 
 void KstApp::slotFileSave() {
@@ -868,100 +946,107 @@ void KstApp::slotFileSave() {
 bool KstApp::slotFileSaveAs() {
   slotUpdateStatusMsg(i18n("Saving file with a new filename..."));
 
-  QString newName=
-    KFileDialog::getSaveFileName(QDir::currentDirPath(),
-                 i18n("*.kst|Kst Plot File (*.kst)\n*|All Files"),
-                                 this, i18n("Save As"));
-  if(!newName.isEmpty()) {
-    if (doc->saveDocument(newName)) {
-      addRecentFile(newName);
+  while (true) {
+    QString newName=
+      KFileDialog::getSaveFileName(QDir::currentDirPath(),
+                  i18n("*.kst|Kst Plot File (*.kst)\n*|All Files"),
+                                  this, i18n("Save As"));
+    if(!newName.isEmpty()) {
+      QRegExp extension("*.kst", false, true);
+      QString longName = newName;
+      if (!extension.exactMatch(newName)) {
+        longName = newName + QString(".kst");
+      }
+      if (doc->saveDocument(longName)) {
+        addRecentFile(longName);
+        QFileInfo saveAsInfo(longName);
+        doc->setTitle(saveAsInfo.fileName());
+        doc->setAbsFilePath(saveAsInfo.absFilePath());
 
-      QFileInfo saveAsInfo(newName);
-      doc->setTitle(saveAsInfo.fileName());
-      doc->setAbsFilePath(saveAsInfo.absFilePath());
+        setCaption(kapp->caption() + ": " + doc->title());
 
-      setCaption(kapp->caption() + ": " + doc->title());
+        slotUpdateStatusMsg(i18n("Ready"));
+        return true;
+      }
+    } else {
+      slotUpdateStatusMsg(i18n("Ready"));
+      return false;
     }
-  } else {
-    slotUpdateStatusMsg(i18n("Ready"));
-    return false;
   }
-  slotUpdateStatusMsg(i18n("Ready"));
-  return true;
 }
 
 void KstApp::slotFileClose() {
   if (doc->saveModified()) {
      doc->cancelUpdate();
     _stopping = true;
-    kapp->processEvents(); // danger
-    doc->deleteContents();
+    QTimer::singleShot(0, doc, SLOT(deleteContents()));
     close();
   }
 }
 
 void KstApp::slotFilePrintActiveWindow() {
-  KstViewWindow *pView = dynamic_cast<KstViewWindow*>(activeWindow());
-  if (pView) {
-    pView->slotFilePrint();
+  KstViewWindow *view = dynamic_cast<KstViewWindow*>(activeWindow());
+  if (view) {
+    view->slotFilePrint();
   }
 }
 
-void KstApp::immediatePrintWindowToFile(KMdiChildView *pWindow, const QString& filename) {
-  KstViewWindow *pView = dynamic_cast<KstViewWindow*>(pWindow);
-  if (pView && !pView->view()->children().isEmpty()) {
-    pView->immediatePrintToFile(filename);
+void KstApp::immediatePrintWindowToFile(KMdiChildView *win, const QString& filename) {
+  KstViewWindow *view = dynamic_cast<KstViewWindow*>(win);
+  if (view && !view->view()->children().isEmpty()) {
+    view->immediatePrintToFile(filename);
   }
 }
 
 void KstApp::immediatePrintActiveWindowToFile(const QString& filename) {
-  KstViewWindow *pView = dynamic_cast<KstViewWindow*>(activeWindow());
-  if (pView) {
-    pView->immediatePrintToFile(filename);
+  KstViewWindow *view = dynamic_cast<KstViewWindow*>(activeWindow());
+  if (view) {
+    view->immediatePrintToFile(filename);
   }
 }
 
-void KstApp::immediatePrintWindowToPng(KMdiChildView *pWindow, const QString& filename, const QString& format, int iWidth, int iHeight) {
-  QSize size(iWidth, iHeight);
+void KstApp::immediatePrintWindowToPng(KMdiChildView *win, const QString& filename, const QString& format, int width, int height) {
+  QSize size(width, height);
 
-  KstViewWindow *pView = dynamic_cast<KstViewWindow*>(pWindow);
-  if (pView && !pView->view()->children().isEmpty()) {
-    pView->immediatePrintToPng(filename,size,format);
+  KstViewWindow *view = dynamic_cast<KstViewWindow*>(win);
+  if (view && !view->view()->children().isEmpty()) {
+    view->immediatePrintToPng(filename,size,format);
   }
 }
 
-void KstApp::immediatePrintActiveWindowToPng(const QString& filename, const QString& format, int iWidth, int iHeight) {
-  KMdiChildView *pWindow = activeWindow();
+void KstApp::immediatePrintActiveWindowToPng(const QString& filename, const QString& format, int width, int height) {
+  KMdiChildView *win = activeWindow();
 
-  if (pWindow) {
-    immediatePrintWindowToPng(pWindow, filename, format, iWidth, iHeight );
+  if (win) {
+    immediatePrintWindowToPng(win, filename, format, width, height);
   }
 }
 
 void KstApp::slotFilePrint() {
-  KstViewWindow *pView;
-  int iNumPages = 0;
+  KstViewWindow *win;
+  int pages = 0;
 
-  KMdiIterator<KMdiChildView*> *pIterator = createIterator();
-  if (pIterator) {
-    while (pIterator->currentItem()) {
-      pView = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
-      if (pView && !pView->view()->children().isEmpty()) {
-        iNumPages++;
+  KMdiIterator<KMdiChildView*> *it = createIterator();
+  if (it) {
+    while (it->currentItem()) {
+      win = dynamic_cast<KstViewWindow*>(it->currentItem());
+      if (win && !win->view()->children().isEmpty()) {
+        pages++;
       }
-      pIterator->next();
+      it->next();
     }
-    deleteIterator(pIterator);
+    deleteIterator(it);
   }
 
-  if (iNumPages > 0) {
+  if (pages > 0) {
     KPrinter printer;
     printer.setPageSize(KPrinter::Letter);
     printer.setOrientation(KPrinter::Landscape);
     printer.setFromTo(0, 0);
-    printer.setMinMax(1, iNumPages);
+    printer.setMinMax(1, pages);
 
-    iNumPages = 0;
+    pages = 0;
+    printer.addDialogPage(new KstPrintOptionsPage);
     if (printer.setup(this, i18n("Print"))) {
       QDateTime dateTime = QDateTime::currentDateTime();
       QPainter paint(&printer);
@@ -970,41 +1055,69 @@ void KstApp::slotFilePrint() {
       QSize size(metrics.width(), metrics.height());
       int iFromPage = printer.fromPage();
       int iToPage = printer.toPage();
+      bool datetimeFooter = printer.option("kst-plot-datetime-footer") == "1";
+      int lineAdjust = printer.option("kst-plot-curve-width-adjust").toInt();
+      bool monochrome = printer.option("kst-plot-monochrome") == "1";
 
       slotUpdateStatusMsg(i18n("Printing..."));
 
       rect.setLeft(0);
       rect.setRight(size.height());
       rect.setBottom(size.height());
-      size.setHeight(9 * size.height() / 10);
+      if (datetimeFooter) {
+        size.setHeight(9 * size.height() / 10);
+      }
       rect.setTop(size.height());
 
-      pIterator = createIterator();
-      if (pIterator) {
+      it = createIterator();
+      if (it) {
         bool firstPage = true;
-        while (pIterator->currentItem()) {
-          pView = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
-          if (pView && !pView->view()->children().isEmpty()) {
-            iNumPages++;
+        while (it->currentItem()) {
+          win = dynamic_cast<KstViewWindow*>(it->currentItem());
+          if (win && !win->view()->children().isEmpty()) {
+            pages++;
             if ((iFromPage == 0 && iToPage == 0) ||
-                (iFromPage <= iNumPages && iToPage >= iNumPages)) {
+                (iFromPage <= pages && iToPage >= pages)) {
               if (!firstPage && !printer.newPage()) {
                 break;
               }
 
-              QString title = i18n("Page: %1  Name: %2  Date: %3").arg(iNumPages).arg(pView->caption()).arg(dateTime.toString(Qt::ISODate));
+              KstTopLevelViewPtr tlv = kst_cast<KstTopLevelView>(win->view());
+              if (tlv) {
+                if (lineAdjust != 0) {
+                  tlv->forEachChild(&Kst2DPlot::pushAdjustLineWidth, lineAdjust, false);
+                }
+                if (monochrome) {
+                  tlv->forEachChild2(&Kst2DPlot::pushPlotColors);
+                  tlv->forEachChild<const QColor&>(&Kst2DPlot::pushCurveColor, Qt::black, false);
+                }
+              }
 
-              pView->view()->resizeForPrint(size);
-              paint.drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, title);
-              pView->view()->paint(P_PRINT, paint);
-              pView->view()->revertForPrint();
+              win->view()->resizeForPrint(size);
+              if (datetimeFooter) {
+                QString title = i18n("Page: %1  Name: %2  Date: %3").arg(pages).arg(win->caption()).arg(dateTime.toString(Qt::ISODate));
+                paint.drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, title);
+              }
+              win->view()->paint(P_PRINT, paint);
+
+              if (tlv) {
+                if (lineAdjust != 0) {
+                  tlv->forEachChild2(&Kst2DPlot::popLineWidth);
+                }
+                if (monochrome) {
+                  tlv->forEachChild2(&Kst2DPlot::popPlotColors);
+                  tlv->forEachChild2(&Kst2DPlot::popCurveColor);
+                }
+              }
+
+              win->view()->revertForPrint();
 
               firstPage = false;
             }
           }
-          pIterator->next();
+          it->next();
         }
-        deleteIterator(pIterator);
+        deleteIterator(it);
       }
 
       slotUpdateStatusMsg(i18n("Ready"));
@@ -1023,10 +1136,9 @@ void KstApp::immediatePrintToFile(const QString& filename) {
   printer.setOutputFileName(filename);
   printer.setFromTo(0, 0);
 
-  KMdiIterator<KMdiChildView*> *pIterator = createIterator();
-  if (pIterator) {
+  KMdiIterator<KMdiChildView*> *it = createIterator();
+  if (it) {
     bool firstPage = true;
-    QDateTime dateTime = QDateTime::currentDateTime();
     QPainter paint(&printer);
     QPaintDeviceMetrics metrics(&printer);
     QRect rect;
@@ -1035,60 +1147,63 @@ void KstApp::immediatePrintToFile(const QString& filename) {
     rect.setLeft(0);
     rect.setRight(size.height());
     rect.setBottom(size.height());
+#if 0
+    QDateTime dateTime = QDateTime::currentDateTime();
     size.setHeight(9 * size.height() / 10);
+#endif
     rect.setTop(size.height());
 
-    while (pIterator->currentItem()) {
-      KstViewWindow *view = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
+    while (it->currentItem()) {
+      KstViewWindow *view = dynamic_cast<KstViewWindow*>(it->currentItem());
       if (view && !view->view()->children().isEmpty()) {
         if (!firstPage && !printer.newPage()) {
           break;
         }
 
+#if 0
         QString title = i18n("Name: %1  Date: %2").arg(view->caption()).arg(dateTime.toString(Qt::ISODate));
+#endif
 
         view->view()->resizeForPrint(size);
+#if 0
         paint.drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, title);
+#endif
         view->view()->paint(P_PRINT, paint);
         view->view()->revertForPrint();
 
         firstPage = false;
       }
-      pIterator->next();
+      it->next();
     }
-    deleteIterator(pIterator);
+    deleteIterator(it);
   }
 }
 
-void KstApp::immediatePrintToPng(const QString& filename, const QString& format, int iWidth, int iHeight, bool bAll) {
-  if (bAll) {
-    KMdiIterator<KMdiChildView*>* pIterator;
+void KstApp::immediatePrintToPng(const QString& filename, const QString& format, int width, int height, bool all) {
+  if (all) {
     QString filenameSub;
-    QString filenameNew;
-    QString dotFormat;
-    int iNumPages = 0;
-    int iPos;
+    int pages = 0;
 
-    dotFormat = i18n(".%1").arg(format);
-    iPos = filename.findRev(dotFormat, -1, FALSE);
+    QString dotFormat = i18n(".%1").arg(format);
+    int iPos = filename.findRev(dotFormat, -1, FALSE);
     if (iPos != -1 && iPos == (int)(filename.length() - dotFormat.length())) {
       filenameSub = filename.left(filename.length() - dotFormat.length());
     } else {
       filenameSub = filename;
     }
 
-    pIterator = createIterator();
-    if (pIterator) {
-      while (pIterator->currentItem()) {
-        iNumPages++;
-        filenameNew = i18n("%1_%2").arg(filenameSub).arg(iNumPages);
-        immediatePrintWindowToPng(pIterator->currentItem(), filenameNew, format, iWidth, iHeight);
-        pIterator->next();
+    KMdiIterator<KMdiChildView*>* it = createIterator();
+    if (it) {
+      while (it->currentItem()) {
+        pages++;
+        QString filenameNew = i18n("%1_%2").arg(filenameSub).arg(pages);
+        immediatePrintWindowToPng(it->currentItem(), filenameNew, format, width, height);
+        it->next();
       }
-      deleteIterator(pIterator);
+      deleteIterator(it);
     }
   } else {
-    immediatePrintActiveWindowToPng(filename, format, iWidth, iHeight);
+    immediatePrintActiveWindowToPng(filename, format, width, height);
   }
 }
 
@@ -1118,7 +1233,7 @@ void KstApp::slotViewStatusBar() {
 
 
 void KstApp::slotUpdateStatusMsg(const QString& msg) {
-  statusBar()->changeItem(msg, KST_STATUSBAR_STATUS);
+  _readyBar->setText(msg);
 }
 
 
@@ -1129,29 +1244,25 @@ void KstApp::slotUpdateDataMsg(const QString& msg) {
 
 void KstApp::slotUpdateProgress(int total, int step, const QString &msg) {
   if (step > 0) {
-    _progressBar->show();
     if (!_progressBar->percentageVisible()) {
       _progressBar->setPercentageVisible(true);
     }
     if (total != _progressBar->totalSteps()) {
       _progressBar->setTotalSteps(total);
     }
-    _progressBar->setProgress(step);
+    if (_progressBar->progress() != step) {
+      _progressBar->setProgress(step);
+    }
   } else {
     _progressBar->setPercentageVisible(false);
     _progressBar->reset();
-    _progressBar->hide();
   }
 
   if (msg.isEmpty()) {
-    _progressMsg->setText(QString::null);
-    _progressMsg->hide();
+    _readyBar->setText(i18n("Ready"));
   } else {
-    _progressMsg->setText(msg);
-    _progressMsg->show();
+    _readyBar->setText(msg);
   }
-
-  while (kapp->eventLoop()->processEvents(QEventLoop::ExcludeUserInput));
 }
 
 
@@ -1171,9 +1282,16 @@ void KstApp::toggleDataMode() {
 
 
 void KstApp::toggleMouseMode() {
-  KstTopLevelViewPtr pView = activeView();
-  if (pView) {
-    pView->setViewMode(LayoutAction->isChecked() ? KstTopLevelView::LayoutMode : KstTopLevelView::DisplayMode);
+  KMdiIterator<KMdiChildView*> *it = createIterator();
+  if (it) {
+    while (it->currentItem()) {
+      KstViewWindow *pView = dynamic_cast<KstViewWindow*>(it->currentItem());
+      if (pView) {
+        pView->view()->setViewMode(LayoutAction->isChecked() ? KstTopLevelView::LayoutMode : KstTopLevelView::DisplayMode);
+      }
+      it->next();
+    }
+    deleteIterator(it);
   }
 }
 
@@ -1181,7 +1299,7 @@ void KstApp::toggleMouseMode() {
 void KstApp::tieAll() {
   int tied = 0;
   Kst2DPlotList pl = Kst2DPlot::globalPlotList();
-  for (Kst2DPlotList::Iterator i = pl.begin(); i != pl.end(); ++i) {
+  for (Kst2DPlotList::ConstIterator i = pl.begin(); i != pl.end(); ++i) {
     if ((*i)->isTied()) {
       ++tied;
     } else {
@@ -1198,16 +1316,16 @@ void KstApp::tieAll() {
 
 
 void KstApp::paintAll(KstPaintType pt) {
-  KMdiIterator<KMdiChildView*> *pIterator = createIterator();
-  if (pIterator) {
-    while (pIterator->currentItem()) {
-      KstViewWindow *pView = dynamic_cast<KstViewWindow*>(pIterator->currentItem());
+  KMdiIterator<KMdiChildView*> *it = createIterator();
+  if (it) {
+    while (it->currentItem()) {
+      KstViewWindow *pView = dynamic_cast<KstViewWindow*>(it->currentItem());
       if (pView) {
         pView->view()->paint(pt);
       }
-      pIterator->next();
+      it->next();
     }
-    deleteIterator(pIterator);
+    deleteIterator(it);
   }
 }
 
@@ -1361,8 +1479,8 @@ void KstApp::registerDocChange() {
 
 void KstApp::reload() {
   KstReadLocker ml(&KST::vectorList.lock());
-  for (KstVectorList::Iterator i = KST::vectorList.begin(); i != KST::vectorList.end(); ++i) {
-    KstRVectorPtr r = dynamic_cast<KstRVector*>((*i).data());
+  for (KstVectorList::ConstIterator i = KST::vectorList.begin(); i != KST::vectorList.end(); ++i) {
+    KstRVectorPtr r = kst_cast<KstRVector>(*i);
     if (r) {
       r->reload();
     }
@@ -1388,7 +1506,7 @@ void KstApp::slotCopy() {
   if (tlv) {
     KstViewWidget *w = tlv->widget();
     KstViewObjectPtr o = w->findChildFor(w->mapFromGlobal(QCursor::pos()));
-    Kst2DPlotPtr p = dynamic_cast<Kst2DPlot*>(o.data());
+    Kst2DPlotPtr p = kst_cast<Kst2DPlot>(o);
     if (p) {
       p->copy();
     }
@@ -1402,6 +1520,22 @@ void KstApp::slotFileNewWindow() {
 }
 
 
+void KstApp::slotFileRenameWindow() {
+  KstViewWindow *vw = dynamic_cast<KstViewWindow*>(activeWindow());
+
+  if (vw) {
+    QString name = windowName(true, vw->caption(), true);
+
+    if (!name.isEmpty() && vw->caption() != name) {
+      vw->setCaption(name);
+      vw->setTabCaption(name);
+      updateDialogsForWindow();
+      doc->setModified();
+    }
+  }
+}
+
+
 QString KstApp::newWindow(const QString& name_in) {
   KstViewWindow *w = new KstViewWindow;
   QString nameToUse = QString::null;
@@ -1421,14 +1555,22 @@ QString KstApp::newWindow(const QString& name_in) {
 }
 
 
-QString KstApp::newWindow(bool prompt) {
+QString KstApp::windowName(bool prompt, const QString& nameOriginal, bool rename) {
   bool ok = false;
-  QString name;
-  QString nameUsed;
+  QString name = nameOriginal;
 
   do {
     if (prompt) {
-      name = KLineEditDlg::getText(i18n("Enter a name for the new window:"), name, &ok, 0L);
+      QRegExp exp("\\S+.*");
+      QRegExpValidator val(exp, 0L);
+      if (rename) {
+        name = KLineEditDlg::getText(i18n("Enter a new name for the window:"), name, &ok, 0L, &val).stripWhiteSpace();
+      } else {
+        name = KLineEditDlg::getText(i18n("Enter a name for the new window:"), name, &ok, 0L, &val).stripWhiteSpace();
+      }
+      if (ok && name==defaultTag) {
+        name = KST::suggestWinName();
+      }
     } else {
       name = KST::suggestWinName();
       ok = true;
@@ -1440,9 +1582,21 @@ QString KstApp::newWindow(bool prompt) {
     } else {
       return QString::null;
     }
+    if (prompt) {
+      KMessageBox::sorry(this, i18n("A window with the same name already exists.  Please enter a unique window name."));
+    }
   } while(true);
 
-  nameUsed = newWindow(name);
+  return name;
+}
+
+
+QString KstApp::newWindow(bool prompt) {
+  QString nameUsed;
+  QString name = windowName(prompt, defaultTag);
+  if (!name.isEmpty()) {
+    nameUsed = newWindow(name);
+  }
 
   return nameUsed;
 }
@@ -1488,6 +1642,17 @@ void KstApp::updateMemoryStatus() {
 #endif
 }
 
+const QStringList KstApp::recentFiles() const {
+  return recent->items();
+}
+
+void KstApp::showQuickStartDialog() {
+#ifdef KST_QUICKSTART_DLG
+  if (KstSettings::globalSettings()->showQuickStart) {
+    _quickStartDialog->show_I();
+  }
+#endif
+}
 
 #include "kst.moc"
 // vim: ts=2 sw=2 et
