@@ -18,18 +18,23 @@
 #ifndef KST2DPLOT_H
 #define KST2DPLOT_H
 
+#include <time.h>
+
 #include "kstbackbuffer.h"
 #include "kstbasecurve.h"
+#include "kstcolorsequence.h"
+#include "kstplotdefines.h"
 #include "kstimage.h"
 #include "kstlabel.h"
 #include "kstviewwidget.h"
 
+#include <qvaluestack.h>
 
 class KstLegend;
 class Kst2DPlot;
 typedef KstObjectList<KstSharedPtr<Kst2DPlot> > Kst2DPlotList;
 
-typedef enum { AUTO, AC, FIXED, AUTOUP, NOSPIKE } KstScaleModeType;
+enum KstScaleModeType { AUTO, AC, FIXED, AUTOUP, NOSPIKE };
 
 struct KstPlotScale {
   double xmin;
@@ -41,7 +46,16 @@ struct KstPlotScale {
   bool xlog, ylog;
 };
 
-typedef enum { INACTIVE, XY_ZOOMBOX, Y_ZOOMBOX, X_ZOOMBOX, LABEL_TOOL, LAYOUT_TOOL } KstMouseModeType;
+struct KstMarker {
+  double value;
+  bool isRising;
+  bool isFalling;
+};
+
+typedef QValueList<KstMarker> KstMarkerList;
+
+enum KstMouseModeType { INACTIVE, XY_ZOOMBOX, Y_ZOOMBOX, X_ZOOMBOX, LABEL_TOOL, LAYOUT_TOOL };
+
 struct KstMouse {
   KstMouse() { mode = INACTIVE; label = -1; minMove = 2; }
   KstMouseModeType mode;
@@ -68,41 +82,41 @@ public:
   virtual ~Kst2DPlot();
 
   static Kst2DPlotList globalPlotList();
-  static bool checkRange(double &min_in, double &max_in, bool bIsLog);
-  static void genAxisTickLabelFullPrecision(QString& label, double z, bool isLog);
-  static void genAxisTickLabelDifference(QString& label, double zbase, double zvalue, bool isLog);
+  static bool checkRange(double& min_in, double& max_in);
+  static bool checkLRange(double& min_in, double& max_in, bool isLog);
   static void genAxisTickLabel(QString& label, double z, bool isLog);
-  static void genAxisTickLabels(QPainter& p, double Min, double Max, double Org, double Tick,
-                              bool bLog, KstLabel* Label, bool &bDelta,
-                              double &dMaxWidth, double &dMaxHeight, QStringList &labelList,
-                              int& iIndexLo, int& iIndexHi);
 
   virtual UpdateType update();
-  virtual void save(QTextStream& ts);
-  virtual void saveTag(QTextStream& ts);
+  virtual void save(QTextStream& ts, const QString& indent = QString::null);
+  virtual void saveTag(QTextStream& ts, const QString& indent = QString::null);
 
   virtual bool popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topLevelParent);
   virtual bool layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topLevelParent);
 
-  void drawDotAt(QPainter& p, double x, double y);
-  double xInternalAlignment();
+  void drawDotAt(QPainter& p, double x, double y, KstBaseCurvePtr curve);
+  void internalAlignment(QRect& plotRegion);
   void addCurve(KstBaseCurvePtr curve);
   void addLabel(KstLabel* label);
   void removeCurve(KstBaseCurvePtr curve);
   void setXScaleMode(KstScaleModeType scalemode);
   void setYScaleMode(KstScaleModeType scalemode);
   
+  void pushAdjustLineWidth(int adjustment);
+  void popLineWidth();
+  void pushCurveColor(const QColor& c);
+  void popCurveColor();
+  void pushPlotColors();
+  void popPlotColors();
+
   /** Set the scale */
-  void setScale(double xmin, double ymin,
-                double xmax, double ymax);
+  void setScale(double xmin, double ymin, double xmax, double ymax);
   /** Set the X scale */
   bool setXScale(double xmin, double xmax);
   /** Set the y scale */
   bool setYScale(double ymin, double ymax);
 
   /** Set the log_scale */
-  bool setLScale(double xmin, double ymin,
-                double xmax, double ymax);
+  bool setLScale(double xmin, double ymin, double xmax, double ymax);
   /** Set the X log_scale */
   bool setLXScale(double xmin, double xmax);
   /** Set the y log_scale */
@@ -115,12 +129,13 @@ public:
   void setLog(bool x_log, bool y_log);
   bool isXLog() const;
   bool isYLog() const;
-
+  void setXAxisInterpretation( bool isXAxisInterpreted, KstXAxisInterpretation xAxisInterpretation, KstXAxisDisplay xAxisDisplay);
+  void getXAxisInterpretation( bool& isXAxisInterpreted, KstXAxisInterpretation& xAxisInterpretation, KstXAxisDisplay& xAxisDisplay) const;
   void getScale(double& xmin, double& ymin, double& xmax, double& ymax) const;
   void getLScale(double& xmin, double& ymin, double& xmax, double& ymax) const;
-  
-  KstScaleModeType getXScaleMode() const;
-  KstScaleModeType getYScaleMode() const;
+
+  KstScaleModeType xScaleMode() const;
+  KstScaleModeType yScaleMode() const;
 
   void setXLabel(const QString& xlabel);
   void setYLabel(const QString& ylabel);
@@ -161,8 +176,10 @@ public:
   /* kstview tells kstplot where to offset the plot to */
   void setPixRect(const QRect& RelPlotRegion, const QRect& RelWinRegion, const QRect& RelPlotAndAxisRegion);
 
+  virtual void move(const QPoint&);
   virtual void resize(const QSize&);
   virtual void parentResized();
+  virtual void parentMoved(const QPoint&);
 
   virtual bool mouseHandler() const;
   virtual void mouseMoveEvent(QWidget *view, QMouseEvent *e);
@@ -174,6 +191,8 @@ public:
   virtual void dropEvent(QWidget *view, QDropEvent *e);
   virtual void dragMoveEvent(QWidget *view, QDragMoveEvent *e);
   virtual void wheelEvent(QWidget *view, QWheelEvent *e);
+  virtual void setHasFocus(bool hasFocus);
+  virtual void removeFocus(QPainter& p);
 
   void cancelZoom(QWidget *view);
   bool moveSelfHorizontal(bool left);
@@ -181,8 +200,6 @@ public:
   bool zoomSelfHorizontal(bool in);
   bool zoomSelfVertical(bool in);
   double xLeft() const;
-  void setHasFocus(bool hasFocus);
-  void removeFocus(QPainter& p);
   void moveUp(KstViewWidget *);
   void moveDown(KstViewWidget *);
   void moveLeft(KstViewWidget *);
@@ -198,20 +215,27 @@ public:
   void yLogSlot(KstViewWidget *);
   void zoomPrev(KstViewWidget *);
   void yZoomAc(KstViewWidget *);
+  void xZoomNormal(KstViewWidget *);
+  void yZoomNormal(KstViewWidget *);
   void zoomSpikeInsensitiveMax(KstViewWidget *);
 
-  //Plot Markers
-  bool setPlotMarker(const double xValue);
+  //
+  // plot Markers
+  //
+  bool setPlotMarker(const double xValue, bool isRisng = false, bool isFalling = false);
   bool removePlotMarker(const double xValue);
-  void setPlotMarkerList(const QValueList<double>& newMarkers);
-  const QValueList<double> plotMarkers(const double minX, const double maxX);
-  const QValueList<double>& plotMarkers();
+  void setPlotMarkerList(const KstMarkerList& newMarkers);
+  const KstMarkerList plotMarkers(const double minX, const double maxX);
+  const KstMarkerList& plotMarkers();
   bool nextMarker(const double currentPosition, double& marker);
   bool prevMarker(const double currentPosition, double& marker);
   void moveToNextMarker(KstViewWidget *);
   void moveToPrevMarker(KstViewWidget *);
-  void moveSelfToMarker(bool next);
-  //curve related Plot Marker functions
+  void moveSelfToCenter(double center);
+
+  //
+  // curve related plot marker functions
+  //
   void setCurveToMarkers(KstBaseCurvePtr curve, bool risingDetect, bool fallingDetect);
   bool hasCurveToMarkers() const;
   void removeCurveToMarkers();
@@ -219,12 +243,45 @@ public:
   bool curveToMarkersRisingDetect() const;
   bool curveToMarkersFallingDetect() const;
 
-  //Images
+  void setXOffsetMode( bool xOffsetMode ) { _xOffsetMode = xOffsetMode; }
+  void setYOffsetMode( bool yOffsetMode ) { _yOffsetMode = yOffsetMode; }
+  bool xOffsetMode( ) const { return _xOffsetMode; }
+  bool yOffsetMode( ) const { return _yOffsetMode; }
+  
+  //
+  // images
+  //
   void addImage(KstImagePtr inImage, bool set_dirty = true);
   void removeImage(KstImagePtr inImage, bool set_dirty = true);
   void removeAllImages(bool set_dirty = true);
   bool hasImage(KstImagePtr inImage) const;
 
+  //
+  // plot grid lines
+  //
+  void setGridLines(bool xMajor, bool yMajor, bool xMinor, bool yMinor,
+                    const QColor& majorColor, const QColor& minorColor,
+                    bool majorGridColorDefault, bool minorGridColorDefault);
+  bool hasXMajorGrid() const { return _xMajorGrid; }
+  bool hasYMajorGrid() const { return _yMajorGrid; }
+  bool hasXMinorGrid() const { return _xMinorGrid; }
+  bool hasYMinorGrid() const { return _yMinorGrid; }
+  const QColor& majorGridColor() const { return _majorGridColor; }
+  const QColor& minorGridColor() const { return _minorGridColor; }
+  bool defaultMajorGridColor() const { return _majorGridColorDefault; }
+  bool defaultMinorGridColor() const { return _minorGridColorDefault; }
+
+  void setXMinorTicks(int minorTicks); //set to -1 for auto
+  void setYMinorTicks(int minorTicks); //set to -1 for auto
+  int xMinorTicks() const { return _xMinorTicks - 1; }
+  int yMinorTicks() const { return _yMinorTicks - 1; }
+  bool xMinorTicksAuto() const { return _reqXMinorTicks < 0; }
+  bool yMinorTicksAuto() const { return _reqYMinorTicks < 0; }
+  void setXMajorTicks(int majorTicks);
+  void setYMajorTicks(int majorTicks);
+  int xMajorTicks() const { return _xMajorTicks; }
+  int yMajorTicks() const { return _yMajorTicks; }
+
 signals:
   void modified();
 
@@ -260,6 +317,8 @@ protected slots:
   void menuYLogSlot();
   void menuZoomPrev();
   void menuYZoomAc();
+  void menuXNormalize();
+  void menuYNormalize();
   void menuZoomSpikeInsensitiveMax();
   void menuNextMarker();
   void menuPrevMarker();
@@ -272,7 +331,7 @@ private:
   template<class T> void updateTiedPlots(bool (Kst2DPlot::*method)(T), T arg);
 
   void updateMousePos(QPoint pos);
-  void highlightNearestDataPoint(bool bRepaint, QWidget *view, const QPoint& pos);
+  void highlightNearestDataPoint(bool repaint, QWidget *view, const QPoint& pos);
   void updateTieBox(QPainter&);
   int labelNumber(QMouseEvent *e);
   bool legendUnder(QMouseEvent *e);
@@ -290,28 +349,55 @@ private:
                                 bool x_log = false,
                                 bool y_log = false);
   void setBorders(double& xleft_bdr_px, double& xright_bdr_px,
-                  double& ytop_bdr_px, double& ybot_bdr_px,
-                  double XTick, double Xorg, double YTick, double Yorg,
-                  QPainter &p);
+                         double& ytop_bdr_px, double& ybot_bdr_px,
+                         TickParameters &tpx,  TickParameters &tpy,
+                         QPainter& p, bool& bOffsetX, bool& bOffsetY);
   void setTicks(double& tick, double& org,
-                double max, double min, bool is_log);
+                double max, double min, bool is_log, bool isX);
+  double convertTimeValueToJD(double valueIn);
+  double convertTimeDiffValueToDays(double diffIn);
+  void convertJDToDateString(QString& label, uint& length, KstXAxisDisplay display, double dJD);
+  void convertTimeValueToString(QString& label, uint& length, double z, bool isLog);
+  void convertDiffTimevalueToString(QString& label, double zbase, double zvalue, bool isLog, double scale);
+  void genAxisTickLabelFullPrecision(QString& label, uint& length, double z, bool isLog, bool isX);
+  void genAxisTickLabelDifference(QString& label, double zbase, double zvalue, bool isLog, bool isX, double scale);
+  void genAxisTickLabels(QPainter& p, TickParameters &tp,
+			 double Min, double Max,
+			 bool bLog, KstLabel* Label, bool isX);
 
-  /** range and domain of plot: not plot dimentions */
+  // range and domain of plot: not plot dimentions
   double XMin, XMax, YMin, YMax;
   double _copy_x, _copy_y;
 
+  // grid lines
+  QColor _majorGridColor;
+  QColor _minorGridColor;
+  bool _xMajorGrid : 1;
+  bool _xMinorGrid : 1;
+  bool _yMajorGrid : 1;
+  bool _yMinorGrid : 1;
+  bool _majorGridColorDefault : 1;
+  bool _minorGridColorDefault : 1;
+  
   bool _xLog : 1, _yLog : 1;
   bool _isTied : 1;
-  bool _hasFocus : 1;
   bool _dirty : 1;
   bool _zoomPaused : 1;
   bool _highlighting : 1; // lock to prevent reentrancy on data mode
                           // (shouldn't this be avoidable?)
   bool _curveToMarkersRisingDetect : 1;
   bool _curveToMarkersFallingDetect : 1;
+  bool _isLogLast : 1;
+
+  bool _xOffsetMode : 1;
+  bool _yOffsetMode : 1;
 
   int _draggableLabel : 15; // I think this should be enough
 
+  bool _isXAxisInterpreted;
+  KstXAxisInterpretation _xAxisInterpretation;
+  KstXAxisDisplay _xAxisDisplay;
+  
   KstScaleModeType _xScaleMode, _yScaleMode;
 
   QPtrList<KstPlotScale> _plotScaleList;
@@ -333,33 +419,58 @@ private:
   KstBackBuffer _buffer;
   QGuardedPtr<KstViewWidget> _menuView;
   QSize _oldSize;
-  double _oldXAlignment;
+  QRect _oldAlignment;
   double _m_X, _b_X, _m_Y, _b_Y;
-
+  double _tickYLast, _stLast;
+  int _autoTickYLast;
+  
   //the list of more than one image
   KstImageList _images;
 
-  /** Plot Markers.  This needs to remain sorted **/
-  QValueList<double> _plotMarkers;
-  //the curve used for auto marker generation
+  // plot markers. This needs to remain sorted
+  KstMarkerList _plotMarkers;
+  // the curve used for auto marker generation
   KstBaseCurvePtr _curveToMarkers;
   void updateMarkersFromCurve();  //auto-generates the markers from _curveToMarkers
 
-  /** helper functions for draw(...) **/
+  // minor tick marks.
+  int _reqXMinorTicks; // -1 means use auto
+  int _reqYMinorTicks; // -1 means use auto
+  int _xMinorTicks;
+  int _yMinorTicks;
+
+  int _xMajorTicks;
+  int _yMajorTicks;
+
+  QValueStack<QColor> _colorStack;
+
+  // helper functions for draw(...)
   void plotLabels(QPainter& p, int x_px, int y_px, double xleft_bdr_px, double ytop_bdr_px);
-  void plotAxes(QPainter& p, QRect& plotRegion, double x_max, double y_max, double x_min, double y_min,
-                        double XTick, double Xorg, double xleft_bdr_px, double xright_bdr_px, double x_orig_px, double xtick_px, double xtick_len_px, int x_px,
-                        double YTick, double Yorg, double ytop_bdr_px, double ybot_bdr_px, double y_orig_px, double ytick_px, double ytick_len_px, int y_px);
+  void plotAxes(QPainter& p, QRect& plotRegion,
+		TickParameters tpx, 
+		double xleft_bdr_px, double xright_bdr_px,
+		double x_orig_px, double xtick_px,
+		double xtick_len_px, int x_px,
+		TickParameters tpy, 
+		double ytop_bdr_px, double ybot_bdr_px,
+		double y_orig_px, double ytick_px,
+		double ytick_len_px, int y_px,
+		bool offsetX, bool offsetY);
+  void plotGridLines(QPainter& p,
+      double XTick, double xleft_bdr_px, double xright_bdr_px,
+      double x_orig_px, double xtick_px, double xtick_len_px, int x_px,
+      double YTick, double ytop_bdr_px, double ybot_bdr_px,
+      double y_orig_px, double ytick_px, double ytick_len_px, int y_px);
   void plotCurves(QPainter& p,
-                           double Lx, double Hx, double Ly, double Hy,
-                           double m_X, double m_Y, double b_X, double b_Y, int penWidth);
+      double Lx, double Hx, double Ly, double Hy,
+      double m_X, double m_Y, double b_X, double b_Y, int penWidth);
   void plotImages(QPainter& p,
-                           double Lx, double Hx, double Ly, double Hy,
-                           double m_X, double m_Y, double b_X, double b_Y,
-                           double x_max, double y_max, double x_min, double y_min); 
-  void plotMarkers(QPainter& p,
-                           double b_X, double b_Y, double x_max, double x_min,
-                           double y_px, double ytop_bdr_px, double ybot_bdr_px);
+      double Lx, double Hx, double Ly, double Hy,
+      double m_X, double m_Y, double b_X, double b_Y,
+      double x_max, double y_max, double x_min, double y_min);
+  void plotPlotMarkers(QPainter& p,
+      double b_X, double b_Y, double x_max, double x_min,
+      double y_px, double ytop_bdr_px, double ybot_bdr_px);
 };
 
 typedef KstSharedPtr<Kst2DPlot> Kst2DPlotPtr;
