@@ -3,6 +3,7 @@
                              -------------
     begin                : June 24, 2004
     copyright            : (C) 2004 The University of Toronto
+                           (C) 2004 The University of British Columbia
     email                :
  ***************************************************************************/
 
@@ -18,19 +19,26 @@
 #ifndef KSTALIGNMENT_H
 #define KSTALIGNMENT_H
 
-#include <qsize.h>
+#include <qmap.h>
+#include <qpoint.h>
+#include <qrect.h>
+
+Q_EXPORT bool operator<(const QPoint &p1, const QPoint &p2);
+
+typedef QMap<QPoint, QPoint> alignmentMap;
 
 // this class keeps track of the plot line alignment (eg, xaxis lineup, etc)
-// currently, it only returns 1 value for all 'plots'.  Eventually, it will
-// set alignment based on the size of the 'plot'.
 class KstAlignment {
  public:
   KstAlignment();
-  void setX(double x);
+  void reset();
+  void setPosition(const QRect& geometry, const QRect& plotRegion);
+  void limits(const QRect& geometry, double& xleft, double& xright, double& ytop, double& ybottom, double dFactor);
+  QRect limits(const QRect& geometry);
 
-  double x(QSize size);
- private:
-  double _xalignment;
+private:
+  alignmentMap  _xAlignments;
+  alignmentMap  _yAlignments;
 };
 
 namespace KST {
