@@ -15,8 +15,9 @@
  *   Permission is granted to link with any opensource library             *
  *                                                                         *
  ***************************************************************************/
-#include "kstbasecurve.h"
 
+#include "kst2dplot.h"
+#include "kstbasecurve.h"
 
 KstBaseCurve::KstBaseCurve(QDomElement& e) : KstDataObject(e) {
   commonConstructor();
@@ -32,13 +33,12 @@ void KstBaseCurve::commonConstructor() {
   QColor in_color("red");
   setHasPoints(false);
   setHasLines(true);
+  setHasBars(false);
   setLineWidth(0);
   setLineStyle(0);
+  setBarStyle(0);
   MaxX = MinX = MeanX = MaxY = MinY = MeanY = NS = 0;
   MinPosX = MinPosY = 0;
-  NumUsed = 0;
-  HasPoints = false;
-  HasLines = true;
   Color = in_color;
   NS = 0;
 }
@@ -47,6 +47,7 @@ void KstBaseCurve::commonConstructor() {
 KstBaseCurve::~KstBaseCurve() {
 }
 
+
 int KstBaseCurve::getIndexNearX(double x) {
   // monotonically rising: we can do a binary search
   // should be reasonably fast
@@ -71,9 +72,9 @@ int KstBaseCurve::getIndexNearX(double x) {
     point(i_top, xt, rY);
     point(i_bot, xb, rY);
     if (xt - x < x - xb) {
-      return(i_top);
+      return i_top;
     } else {
-      return(i_bot);
+      return i_bot;
     }
   } else {
     // Oh Oh... not monotonically rising - we have to search the entire curve!
@@ -94,7 +95,7 @@ int KstBaseCurve::getIndexNearX(double x) {
         i0 = i;
       }
     }
-    return(i0);
+    return i0;
   }
 }
 
@@ -102,21 +103,21 @@ int KstBaseCurve::getIndexNearX(double x) {
     x +- dx which is closest to y **/
 int KstBaseCurve::getIndexNearXY(double x, double dx_per_pix, double y) {
   double xi, yi, dx, dxi, dy, dyi;
-  bool bFirst = true;
+  bool first = true;
   int i,i0, iN, index;
   int sc = sampleCount();
 
   if (xIsRising()) {
     iN = i0 = getIndexNearX(x);
-    point(i0, xi, yi);
 
-    while ((i0>0) && (x-dx<xi)) {
+    point(i0, xi, yi);
+    while (i0 > 0 && x-dx_per_pix < xi) {
       i0--;
       point(i0, xi, yi);
     }
+    
     point(iN, xi, yi);
-
-    while ((iN<sc-1) && (x+dx>xi)) {
+    while (iN < sc-1 && x+dx_per_pix > xi) {
       iN++;
       point(iN, xi, yi);
     }
@@ -136,18 +137,45 @@ int KstBaseCurve::getIndexNearXY(double x, double dx_per_pix, double y) {
     if (dxi < dx_per_pix) {
       dx = dxi;
       dyi = fabs(y-yi);
-      if (bFirst || dyi < dy) {
-        bFirst = false;
+      if (first || dyi < dy) {
+        first = false;
         index = i;
         dy = dyi;
       }
-    } else if(dxi < dx) {
+    } else if (dxi < dx) {
       dx = dxi;
       index = i;
     }
   }
-  return(index);
+  return index;
 }
 
 
+double KstBaseCurve::maxX() const {
+  if (hasBars() && sampleCount() > 0) {
+    return MaxX + (MaxX - MinX)/(2*(sampleCount()-1));
+  } else {
+    return MaxX;
+  }
+}
+
+
+double KstBaseCurve::minX() const {
+  if (hasBars() && sampleCount() > 0) {
+    return MinX - (MaxX - MinX)/(2*(sampleCount()-1));
+  } else {
+    return MinX;
+  }
+}
+
+
+bool KstBaseCurve::deleteDependents() {
+  bool rc = KstDataObject::deleteDependents();
+  Kst2DPlotList pl = Kst2DPlot::globalPlotList();
+  for (Kst2DPlotList::Iterator i = pl.begin(); i != pl.end(); ++i) {
+    (*i)->Curves.remove(this);
+  }
+  return rc;
+}
+
 // vim: ts=2 sw=2 et
