@@ -18,8 +18,7 @@
 #include "kstborderedviewobject.h"
 
 #include <qpainter.h>
-#include <qpen.h>
-#include <qmessagebox.h>
+#include <qstylesheet.h>
 
 KstBorderedViewObject::KstBorderedViewObject(const QString& type)
 : KstViewObject(type), _borderColor(QColor(0, 0, 0)), _borderWidth(0), _padding(0), _margin(0) {
@@ -53,21 +52,23 @@ KstObject::UpdateType KstBorderedViewObject::update() {
 }
 
 
-void KstBorderedViewObject::save(QTextStream& ts) {
-  ts << "  <border color=\"" << _borderColor.name() <<
-               "\" width=\"" << _borderWidth <<
-               "\" padding=\"" << _padding <<
-               "\" margin=\"" << _margin << "\" />" << endl;
-  KstViewObject::save(ts);
+void KstBorderedViewObject::save(QTextStream& ts, const QString& indent) {
+  ts << indent << "<border color=\""
+    << QStyleSheet::escape(_borderColor.name())
+    << "\" width=\"" << _borderWidth
+    << "\" padding=\"" << _padding
+    << "\" margin=\"" << _margin << "\" />" << endl;
+  KstViewObject::save(ts, indent);
 }
 
 
-void KstBorderedViewObject::saveTag(QTextStream& ts) {
-  ts << "  <border color=\"" << _borderColor.name() <<
-               "\" width=\"" << _borderWidth <<
-               "\" padding=\"" << _padding <<
-               "\" margin=\"" << _margin << "\" />" << endl;
-  KstViewObject::saveTag(ts);
+void KstBorderedViewObject::saveTag(QTextStream& ts, const QString& indent) {
+  ts << indent << "<border color=\""
+    << QStyleSheet::escape(_borderColor.name())
+    << "\" width=\"" << _borderWidth
+    << "\" padding=\"" << _padding
+    << "\" margin=\"" << _margin << "\" />" << endl;
+  KstViewObject::saveTag(ts, indent);
 }
 
 
