@@ -20,22 +20,25 @@
 
 #include <qcolor.h>
 #include <kstaticdeleter.h>
+#include "kstbasecurve.h"
 
 class KPalette;
 
 class KstColorSequence {
-friend class KStaticDeleter<KstColorSequence>;
+  friend class KStaticDeleter<KstColorSequence>;
   public:
     static QColor next();
     static QColor next(const QColor& badColor);
+    static QColor next(const KstBaseCurveList& Curves, const QColor& badColor);
+    static bool colorsTooClose(const QColor& color, const QColor& badColor);
 
   private:
     KstColorSequence();
     ~KstColorSequence();
     static KstColorSequence* _self;
-    int _ptr;  // pointer to the next color
-    KPalette *_pal;
+    KPalette* _pal;
     int _count;
+    int _ptr;  // pointer to the next color
 };
 
 #endif
