@@ -17,12 +17,12 @@
 
 #include "kstdatacollection.h"
 
-bool KST::tagNameNotUnique(const QString &tag, bool warn, void *p) {
+bool KST::tagNameNotUnique(const QString& tag, bool warn, void *p) {
   Q_UNUSED(p)
   return dataTagNameNotUnique(tag, warn) || vectorTagNameNotUnique(tag, warn);
 }
 
-bool KST::dataTagNameNotUnique(const QString &tag, bool warn, void *p) {
+bool KST::dataTagNameNotUnique(const QString& tag, bool warn, void *p) {
   Q_UNUSED(p)
   Q_UNUSED(warn)
   /* verify that the tag name is not empty */
@@ -31,6 +31,7 @@ bool KST::dataTagNameNotUnique(const QString &tag, bool warn, void *p) {
   }
 
   /* verify that the tag name is not used by a data object */
+  KstReadLocker ml(&KST::dataObjectList.lock());
   if (KST::dataObjectList.findTag(tag) != KST::dataObjectList.end()) {
       return true;
   }
@@ -38,7 +39,7 @@ bool KST::dataTagNameNotUnique(const QString &tag, bool warn, void *p) {
   return false;
 }
 
-bool KST::vectorTagNameNotUnique(const QString &tag, bool warn, void *p) {
+bool KST::vectorTagNameNotUnique(const QString& tag, bool warn, void *p) {
   Q_UNUSED(p)
   Q_UNUSED(warn)
   /* verify that the tag name is not empty */
@@ -47,6 +48,8 @@ bool KST::vectorTagNameNotUnique(const QString &tag, bool warn, void *p) {
   }
 
   /* verify that the tag name is not used by a data object */
+  KstReadLocker ml(&KST::vectorList.lock());
+  KstReadLocker ml2(&KST::scalarList.lock());
   if (KST::vectorList.findTag(tag) != KST::vectorList.end() ||
       KST::scalarList.findTag(tag) != KST::scalarList.end()) {
       return true;
