@@ -20,7 +20,6 @@
 #include <stdlib.h>
 
 #include "kstdatacollection.h"
-#include "kstrvector.h"
 
 #include "sysinfo.h"
 #include "psversion.h"
@@ -34,6 +33,9 @@ KstVectorList KST::vectorList;
 /** The list of Scalars which have been generated */
 KstScalarList KST::scalarList;
 
+/** The list of Strings */
+KstStringList KST::stringList;
+
 /** The list of data objects which are in use */
 KstDataObjectList KST::dataObjectList;
 
