@@ -18,25 +18,14 @@
 #include <assert.h>
 
 // include files for Qt
-#include <qlistview.h>
-#include <qmap.h>
-#include <qptrlist.h>
 #include <qptrstack.h>
-#include <qpushbutton.h>
-#include <qstring.h>
-#include <qwidget.h>
 
 // include files for KDE
 #include <kdebug.h>
 #include <klistview.h>
-#include <klocale.h>
-#include <kmdimainfrm.h>
 #include <kmessagebox.h>
-#include <kpopupmenu.h>
 
 // application specific includes
-#include "imagedialog.h"
-#include "kst.h"
 #include "kst2dplot.h"
 #include "kstcurvedialog_i.h"
 #include "kstdatacollection.h"
@@ -45,15 +34,10 @@
 #include "ksteqdialog_i.h"
 #include "ksteventmonitor_i.h"
 #include "ksthsdialog_i.h"
-#include "kstimage.h"
 #include "kstimagedialog_i.h"
-#include "kstmatrix.h"
 #include "kstmatrixdialog_i.h"
-#include "kstplotdialog_i.h"
-#include "kstplugin.h"
 #include "kstplugindialog_i.h"
 #include "kstpsddialog_i.h"
-#include "kstrvector.h"
 #include "kstvectordialog_i.h"
 #include "kstviewwindow.h"
 #include "matrixselector.h"
@@ -68,25 +52,25 @@ static QMap<int,QString> FilterMap;
 #define RTTI_OBJ_DATA_VECTOR     4203
 
 
-KstObjectItem::KstObjectItem(QListView *parent, KstRVectorPtr x, KstDataManagerI *dm)
+KstObjectItem::KstObjectItem(QListView *parent, KstRVectorPtr x, KstDataManagerI *dm, int localUseCount)
 : QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_DATA_VECTOR), _name(x->tagName()), _dm(dm) {
   assert(x);
   setText(0, x->tagName());
   setText(1, i18n("Data Vector"));
   x = 0L; // keep the counts in sync
-  update();
+  update(true, localUseCount);
 }
 
-KstObjectItem::KstObjectItem(QListViewItem *parent, KstVectorPtr x, KstDataManagerI *dm)
+KstObjectItem::KstObjectItem(QListViewItem *parent, KstVectorPtr x, KstDataManagerI *dm, int localUseCount)
 : QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_VECTOR), _name(x->tagName()), _dm(dm) {
   assert(x);
   setText(0, x->tagName());
   setText(1, i18n("Slave Vector"));
   x = 0L; // keep the counts in sync
-  update();
+  update(true, localUseCount);
 }
 
-KstObjectItem::KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManagerI *dm)
+KstObjectItem::KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManagerI *dm, int localUseCount)
 : QObject(), QListViewItem(parent), _rtti(RTTI_OBJ_OBJECT), _name(x->tagName()), _dm(dm) {
   assert(x);
   setText(0, x->tagName());
@@ -97,84 +81,94 @@ KstObjectItem::KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManag
     connect(item, SIGNAL(updated()), this, SIGNAL(updated()));
   }
   x = 0L; // keep the counts in sync
-  update(false);
+  update(false, localUseCount);
 }
 
 KstObjectItem::~KstObjectItem() {
 }
 
-void KstObjectItem::update(bool recursive) {
+void KstObjectItem::update(bool recursive, int localUseCount) {
   switch (_rtti) {
     case RTTI_OBJ_DATA_VECTOR:
       {
-        KstVectorPtr px = *KST::vectorList.findTag(_name);
-        assert(px.data());
-        assert(dynamic_cast<KstRVector*>(px.data()));
-        KstRVectorPtr x = static_cast<KstRVector*>(px.data());
-        x->readLock();
-        setText(2, QString::number(x->getUsage() - 2)); // caller has a ptr
-        setText(3, QString::number(x->sampleCount()));
-        setText(4, i18n("%3: %4 [%1..%2]").arg(x->reqStartFrame())
-            .arg(x->reqStartFrame() + x->reqNumFrames())
-            .arg(x->filename())
-            .arg(x->field()));
-        _removable = x->getUsage() == 2;
-        x->readUnlock();
+        KstRVectorPtr x = kst_cast<KstRVector>(*KST::vectorList.findTag(_name));
+        if (x) {
+          x->readLock();
+          // getUsage: subtract 1 for KstRVectorPtr x
+          setText(2, QString::number(x->getUsage() - 1 - localUseCount));
+          setText(3, QString::number(x->sampleCount()));
+          setText(4, i18n("%3: %4 [%1..%2]").arg(x->reqStartFrame())
+              .arg(x->reqStartFrame() + x->reqNumFrames())
+              .arg(x->filename())
+              .arg(x->field()));
+          _removable = x->getUsage() == 2;
+          x->readUnlock();
+        }
+        // Hmmm what happens if this if() fails??  We become inconsistent?
         break;
       }
     case RTTI_OBJ_VECTOR:
       {
         KstVectorPtr x = *KST::vectorList.findTag(_name);
-        assert(x.data());
-        x->readLock();
-        setText(2, QString::number(x->getUsage() - 2)); //caller also points
-        setText(3, QString::number(x->sampleCount()));
-        setText(4, i18n("[%1..%2]").arg(x->min()).arg(x->max()));
-        x->readUnlock();
-        _removable = false;
+        if (x) {
+          x->readLock();
+          //
+          // getUsage:
+          //  subtract 1 for KstVectorPtr x
+          //
+          setText(2, QString::number(x->getUsage() - 1 - localUseCount));
+          setText(3, QString::number(x->sampleCount()));
+          setText(4, i18n("[%1..%2]").arg(x->min()).arg(x->max()));
+          x->readUnlock();
+          _removable = false;
+        }
         break;
       }
     case RTTI_OBJ_OBJECT:
       {
         KstDataObjectPtr x = *KST::dataObjectList.findTag(_name);
-        assert(x.data());
-        x->readLock();
-        setText(1, x->typeString());
-        setText(2, QString::number(x->getUsage() - 1)); // our pointer
-        setText(3, QString::number(x->sampleCount()));
-        setText(4, x->propertyString());
-        if (recursive) {
-          QPtrStack<QListViewItem> trash;
+        if (x) {
+          x->readLock();
+          //
+          // getUsage:
+          //  subtract 1 for KstDataObjectPtr x
+          //
+          setText(1, x->typeString());
+          setText(2, QString::number(x->getUsage() - 1 - localUseCount));
+          setText(3, QString::number(x->sampleCount()));
+          setText(4, x->propertyString());
+          if (recursive) {
+            QPtrStack<QListViewItem> trash;
 
-          for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
-            KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-            if (x->outputVectors().findTag(oi->tagName()) == x->outputVectors().end()) {
-              trash.push(i);
-            }
-          }
-          trash.setAutoDelete(true);
-          trash.clear();
-
-          for (KstVectorMap::Iterator p = x->outputVectors().begin();
-              p != x->outputVectors().end();
-              ++p) {
-            bool found = false;
             for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
               KstObjectItem *oi = static_cast<KstObjectItem*>(i);
-              if (oi->tagName() == p.data()->tagName()) {
-                oi->update();
-                found = true;
-                break;
+              if (x->outputVectors().findTag(oi->tagName()) == x->outputVectors().end()) {
+                trash.push(i);
               }
             }
-            if (!found) {
-              KstObjectItem *item = new KstObjectItem(this, p.data(), _dm);
-              connect(item, SIGNAL(updated()), this, SIGNAL(updated()));
+            trash.setAutoDelete(true);
+            trash.clear();
+
+            for (KstVectorMap::Iterator p = x->outputVectors().begin();
+                p != x->outputVectors().end(); ++p) {
+              bool found = false;
+              for (QListViewItem *i = firstChild(); i; i = i->nextSibling()) {
+                KstObjectItem *oi = static_cast<KstObjectItem*>(i);
+                if (oi->tagName() == p.data()->tagName()) {
+                  oi->update();
+                  found = true;
+                  break;
+                }
+              }
+              if (!found) {
+                KstObjectItem *item = new KstObjectItem(this, p.data(), _dm);
+                connect(item, SIGNAL(updated()), this, SIGNAL(updated()));
+              }
             }
           }
+          _removable = x->getUsage() == 1;
+          x->readUnlock();
         }
-        _removable = x->getUsage() == 1;
-        x->readUnlock();
         break;
       }
     default:
@@ -184,14 +178,14 @@ void KstObjectItem::update(bool recursive) {
 
 void KstObjectItem::updateButtons() {
   _dm->Edit->setEnabled(RTTI_OBJ_VECTOR != _rtti);
-  _dm->Delete->setEnabled(_removable);
+  _dm->Delete->setEnabled(RTTI_OBJ_VECTOR != _rtti);
 }
 
 void KstObjectItem::reload() {
   KstReadLocker ml(&KST::vectorList.lock());
   KstVectorList::Iterator v = KST::vectorList.findTag(_name);
   if (v != KST::vectorList.end()) {
-    KstRVector *r = dynamic_cast<KstRVector*>((*v).data());
+    KstRVectorPtr r = kst_cast<KstRVector>(*v);
     if (r) {
       r->writeLock();
       r->reload();
@@ -230,42 +224,84 @@ void KstObjectItem::applyFilter(int id) {
 */
 }
 
+void KstObjectItem::activateHint(int id) {
+  KstDataObjectPtr d = dataObject();
+  const KstCurveHintList* hints = d->curveHints();
+  int cnt = 0;
+  for (KstCurveHintList::ConstIterator i = hints->begin(); i != hints->end(); ++i) {
+    if (cnt == id) {
+      KstBaseCurvePtr c = (*i)->makeCurve(d->tagName() + "-curve" /* FIXME */, KstColorSequence::next());
+      if (c) {
+        KST::dataObjectList.lock().writeLock();
+        KST::dataObjectList.append(c.data());
+        KST::dataObjectList.lock().writeUnlock();
+        emit updated();
+      } else {
+        KMessageBox::sorry(0L, i18n("Unable to create quick curve."));
+      }
+      break;
+    }
+    cnt++;
+  }
+}
+
 void KstObjectItem::addToPlot(int id) {
   Kst2DPlotPtr p = PlotMap[id];
-  KstBaseCurvePtr c = dynamic_cast<KstBaseCurve*>(dataObject().data());
+  KstBaseCurvePtr c = kst_cast<KstBaseCurve>(dataObject());
   if (p && c) {
     p->addCurve(c);
+    p->setDirty();
+    paintPlot(p);
     emit updated();
   }
 }
 
 void KstObjectItem::removeFromPlot(int id) {
   Kst2DPlotPtr p = PlotMap[id];
-  KstBaseCurvePtr c = dynamic_cast<KstBaseCurve*>(dataObject().data());
+  KstBaseCurvePtr c = kst_cast<KstBaseCurve>(dataObject());
   if (p && c) {
     p->removeCurve(c);
+    p->setDirty();
+    paintPlot(p);
     emit updated();
   }
 }
 
 void KstObjectItem::addImageToPlot(int id) {
   Kst2DPlotPtr p = PlotMap[id];
-  KstImagePtr img = dynamic_cast<KstImage*>(dataObject().data());
+  KstImagePtr img = kst_cast<KstImage>(dataObject());
   if (p && img) {
     p->addImage(img);
+    p->setDirty();
+    paintPlot(p);
     emit updated();
   }
 }
 
 void KstObjectItem::removeImageFromPlot(int id) {
   Kst2DPlotPtr p = PlotMap[id];
-  KstImagePtr img = dynamic_cast<KstImage*>(dataObject().data());
+  KstImagePtr img = kst_cast<KstImage>(dataObject());
   if (p && img) {
     p->removeImage(img);
+    p->setDirty();
+    paintPlot(p);
     emit updated();
   }
 }
 
+void KstObjectItem::paintPlot(Kst2DPlotPtr p) {
+  KstApp *app = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *it = app->createIterator();
+  while (it->currentItem()) {
+    KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
+    if (v && v->view()->contains(static_cast<KstViewObject*>(p))) {
+      v->view()->paint(P_PLOT);
+      break;
+    }
+    it->next();
+  }
+  app->deleteIterator(it);
+}
 
 KstDataManagerI::KstDataManagerI(KstDoc *in_doc, QWidget* parent, const char* name, bool modal, WFlags fl)
 : KstDataManager(parent, name, modal, fl) {
@@ -344,7 +380,10 @@ void KstDataManagerI::update() {
 
   KST::dataObjectList.lock().writeLock();
   KST::vectorList.lock().writeLock();
-  // Garbage collect first
+
+  //
+  // garbage collect first
+  //
   for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
     KstObjectItem *oi = static_cast<KstObjectItem*>(i);
     if (i->rtti() == RTTI_OBJ_OBJECT) {
@@ -363,6 +402,9 @@ void KstDataManagerI::update() {
   trash.clear();
   DataView->blockSignals(false);
 
+  //
+  // update the data objects
+  //
   for (KstDataObjectList::iterator it = KST::dataObjectList.begin();
                                     it != KST::dataObjectList.end();
                                                                ++it) {
@@ -376,31 +418,35 @@ void KstDataManagerI::update() {
       }
     }
     if (!found) {
-        KstObjectItem *i = new KstObjectItem(DataView, *it, this);
-        connect(i, SIGNAL(updated()), this, SLOT(doUpdates()));
+      KstObjectItem *i = new KstObjectItem(DataView, *it, this);
+      connect(i, SIGNAL(updated()), this, SLOT(doUpdates()));
     }
   }
+
   KST::vectorList.lock().writeUnlock();
   KST::dataObjectList.lock().writeUnlock();
 
+  //
+  // update the vectors
+  //
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (KstRVectorList::iterator it = rvl.begin(); it != rvl.end(); ++it) {
     bool found = false;
     for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
       KstObjectItem *oi = static_cast<KstObjectItem*>(i);
       if (oi->rtti() == RTTI_OBJ_DATA_VECTOR && oi->tagName() == (*it)->tagName()) {
-        oi->update();
+        oi->update(true, 1);
         found = true;
         break;
       }
     }
     if (!found) {
-        KstObjectItem *i = new KstObjectItem(DataView, *it, this);
-        connect(i, SIGNAL(updated()), this, SLOT(doUpdates()));
+      KstObjectItem *i = new KstObjectItem(DataView, *it, this, 1);
+      connect(i, SIGNAL(updated()), this, SLOT(doUpdates()));
     }
   }
 
-  // is this really necessary?  I would think not.....
+  // is this really necessary?  I would think not...
   for (QListViewItem *i = DataView->firstChild(); i; i = i->nextSibling()) {
     if (i == currentItem) {
       DataView->setCurrentItem(i);
@@ -447,13 +493,39 @@ void KstDataManagerI::delete_I() {
   QListViewItem *qi = DataView->selectedItems().at(0);
   KstObjectItem *koi = static_cast<KstObjectItem*>(qi);
 
-  KstWriteLocker ml(&KST::vectorList.lock());
+  if (koi->removable()) {
+    KstWriteLocker ml(&KST::vectorList.lock());
 
-  if (qi->rtti() == RTTI_OBJ_OBJECT) {
-    doc->removeDataObject(koi->tagName());
-  } else if (qi->rtti() == RTTI_OBJ_DATA_VECTOR) {
-    KST::vectorList.removeTag(koi->tagName());
-    doUpdates();
+    if (qi->rtti() == RTTI_OBJ_OBJECT) {
+      doc->removeDataObject(koi->tagName());
+    } else if (qi->rtti() == RTTI_OBJ_DATA_VECTOR) {
+      KST::vectorList.removeTag(koi->tagName());
+      doUpdates();
+    }
+  } else {
+    // Don't prompt for base curves
+    KstBaseCurvePtr bc = kst_cast<KstBaseCurve>(koi->dataObject());
+    if (bc || KMessageBox::warningYesNo(this, i18n("There are other objects in memory that depend on %1.  Do you wish to delete them too?").arg(koi->tagName())) == KMessageBox::Yes) {
+
+      if (qi->rtti() == RTTI_OBJ_OBJECT) {
+        koi->dataObject()->deleteDependents();
+        doc->removeDataObject(koi->tagName());
+      } else if (qi->rtti() == RTTI_OBJ_DATA_VECTOR) {
+        KstRVectorPtr x = kst_cast<KstRVector>(*KST::vectorList.findTag(koi->tagName()));
+        if (x) {
+          x->deleteDependents();
+          x = 0L;
+          KstWriteLocker ml(&KST::vectorList.lock());
+          KST::vectorList.removeTag(koi->tagName());
+          doUpdates();
+        } else {
+          KMessageBox::sorry(this, i18n("Unknown error deleting data vector."));
+        }
+      }
+      KstApp::inst()->paintAll(P_PLOT);
+    } else {
+      KMessageBox::sorry(this, i18n("Cannot delete objects with dependencies."));
+    }
   }
 }
 
@@ -500,6 +572,19 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
     m->setItemEnabled(id, !FilterMap.isEmpty());
   }
 
+  if (koi->dataObject()) {
+    const KstCurveHintList* hints = koi->dataObject()->curveHints();
+    if (!hints->isEmpty()) {
+      KPopupMenu *hintMenu = new KPopupMenu(this);
+      int cnt = 0;
+      for (KstCurveHintList::ConstIterator i = hints->begin(); i != hints->end(); ++i) {
+        hintMenu->insertItem((*i)->curveName(), koi, SLOT(activateHint(int)), 0, cnt);
+        cnt++;
+      }
+      id = m->insertItem(i18n("&Quick Curve"), hintMenu);
+    }
+  }
+
   if (koi->rtti() == RTTI_OBJ_DATA_VECTOR) {
     id = m->insertItem(i18n("&Make Curve..."), koi, SLOT(makeCurve()));
     id = m->insertItem(i18n("Make M&atrix..."), koi, SLOT(makeMatrix()));
@@ -507,13 +592,13 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
   } else if (koi->rtti() == RTTI_OBJ_VECTOR) {
     id = m->insertItem(i18n("&Make Curve..."), koi, SLOT(makeCurve()));
     id = m->insertItem(i18n("Make M&atrix..."), koi, SLOT(makeMatrix()));
-  } else if (dynamic_cast<KstMatrix*>(koi->dataObject().data())) {
+  } else if (kst_cast<KstMatrix>(koi->dataObject())) {
     id = m->insertItem(i18n("Make &Image..."), koi, SLOT(makeImage()));
-  } else if ((c = dynamic_cast<KstBaseCurve*>(koi->dataObject().data()))) {
+  } else if ((c = kst_cast<KstBaseCurve>(koi->dataObject()))) {
     KPopupMenu *addMenu = new KPopupMenu(this);
     KPopupMenu *removeMenu = new KPopupMenu(this);
     PlotMap.clear();
-    id = 100;
+    id = 300;
     bool haveAdd = false, haveRemove = false;
 
     KstApp *app = KstApp::inst();
@@ -539,11 +624,11 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
 
     app->deleteIterator(it);
 
-    id = m->insertItem(i18n("&Add To Plot"), addMenu);
+    id = m->insertItem(i18n("&Add to Plot"), addMenu);
     m->setItemEnabled(id, haveAdd);
     id = m->insertItem(i18n("&Remove From Plot"), removeMenu);
     m->setItemEnabled(id, haveRemove);
-  } else if ((img = dynamic_cast<KstImage*>(koi->dataObject().data()))) {
+  } else if ((img = kst_cast<KstImage>(koi->dataObject()))) {
     KPopupMenu *addMenu = new KPopupMenu(this);
     KPopupMenu *removeMenu = new KPopupMenu(this);
     PlotMap.clear();
@@ -573,7 +658,7 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
 
     app->deleteIterator(it);
 
-    id = m->insertItem(i18n("&Add To Plot"), addMenu);
+    id = m->insertItem(i18n("&Add to Plot"), addMenu);
     m->setItemEnabled(id, haveAdd);
     id = m->insertItem(i18n("&Remove From Plot"), removeMenu);
     m->setItemEnabled(id, haveRemove);
@@ -590,8 +675,9 @@ void KstDataManagerI::contextMenu(QListViewItem *i, const QPoint& p, int col) {
 
 
 void KstDataManagerI::doUpdates() {
-  doc->forceUpdate();
-  doc->setModified();
+//  doc->forceUpdate();
+//  doc->setModified();
+  emit docChanged();
 }
 
 
