@@ -21,44 +21,46 @@
 class KstDoc;
 
 #include "datamanager.h"
-#include <qlistview.h>
+#include "kstrvector.h"
 
 class KstDataManagerI: public KstDataManager {
   Q_OBJECT
-public:
-  KstDataManagerI(KstDoc *doc, QWidget* parent=0,
-                  const char* name = 0,
-                  bool modal = FALSE, WFlags fl = 0 );
-  virtual ~KstDataManagerI();
+  public:
+    KstDataManagerI(KstDoc *doc, QWidget* parent = 0,
+        const char *name = 0,
+        bool modal = false, WFlags fl = 0);
+    virtual ~KstDataManagerI();
 
-public slots:
-  void update();
-  void updateContents();
-  void show_I();
-  void edit_I();
-  void filter_I();
-  void delete_I();
+  public slots:
+    void update();
+    void updateContents();
+    void show_I();
+    void edit_I();
+    void filter_I();
+    void delete_I();
 
-private slots:
-  void doUpdates();
-  void contextMenu(QListViewItem *i, const QPoint& p, int c);
-  void currentChanged(QListViewItem *);
+  private slots:
+    void doUpdates();
+    void contextMenu(QListViewItem *i, const QPoint& p, int c);
+    void currentChanged(QListViewItem *);
 
-private:
-  KstDoc *doc;
-signals:
-  void editDataVector(const QString &);
+  private:
+    KstDoc *doc;
+
+  signals:
+    void editDataVector(const QString &);
+    void docChanged();
 };
 
 class KstObjectItem : public QObject, public QListViewItem {
   Q_OBJECT
   public:
-    KstObjectItem(QListView *parent, KstRVectorPtr x, KstDataManagerI *dm);
-    KstObjectItem(QListViewItem *parent, KstVectorPtr x, KstDataManagerI *dm);
-    KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManagerI *dm);
+    KstObjectItem(QListView *parent, KstRVectorPtr x, KstDataManagerI *dm, int localUseCount = 0);
+    KstObjectItem(QListViewItem *parent, KstVectorPtr x, KstDataManagerI *dm, int localUseCount = 0);
+    KstObjectItem(QListView *parent, KstDataObjectPtr x, KstDataManagerI *dm, int localUseCount = 0);
     virtual ~KstObjectItem();
 
-    void update(bool recursive = true);
+    void update(bool recursive = true, int localUseCount = 0);
     virtual int rtti() const { return _rtti; }
     
     const QString& tagName() const { return _name; }
@@ -69,6 +71,7 @@ class KstObjectItem : public QObject, public QListViewItem {
   public slots:
     void applyFilter(int);
     void addToPlot(int);
+    void activateHint(int);
     void addImageToPlot(int);
     void removeFromPlot(int);
     void removeImageFromPlot(int);
@@ -85,6 +88,7 @@ class KstObjectItem : public QObject, public QListViewItem {
     QString          _name;
     KstDataManagerI *_dm;
     bool             _removable;
+    void paintPlot(Kst2DPlotPtr p); //used by add to/remove from plot slots
 };
 
 #endif
