@@ -20,7 +20,7 @@
 #include "kstdataobject.h"
 #include "kstdebug.h"
 
-#include <qthread.h>
+#include <qdeepcopy.h>
 #include <qtimer.h>
 
 #include <kdebug.h>
@@ -30,15 +30,25 @@
 
 KstDataObject::KstDataObject() : KstObject() {
   //kdDebug() << "+++ CREATING DATA OBJECT: " << (void*)this << endl;
+  _curveHints = new KstCurveHintList;
 }
 
 KstDataObject::KstDataObject(QDomElement& e) : KstObject() {
   Q_UNUSED(e)
   //kdDebug() << "+++ CREATING DATA OBJECT: " << (void*)this << endl;
+  _curveHints = new KstCurveHintList;
 }
 
 KstDataObject::~KstDataObject() {
-  // Remove our slave vectors and scalars
+  // Remove our slave vectors, scalars, and strings
+  KST::stringList.lock().writeLock();
+  for (KstStringMap::Iterator it = _outputStrings.begin();
+                               it != _outputStrings.end();
+                                                      ++it) {
+    KST::stringList.remove(it.data());
+  }
+  KST::stringList.lock().writeUnlock();
+
   KST::scalarList.lock().writeLock();
   for (KstScalarMap::Iterator it = _outputScalars.begin();
                                it != _outputScalars.end();
@@ -55,9 +65,10 @@ KstDataObject::~KstDataObject() {
   }
   KST::vectorList.lock().writeUnlock();
   //kdDebug() << "+++ DESTROYING DATA OBJECT: " << (void*)this << endl;
+  delete _curveHints;
 }
 
-double *KstDataObject::vectorRealloced(KstVectorPtr v, double *memptr, int newSize) const {
+double *KstDataObject::vectorRealloced(KstVectorPtr v, double *memptr, int newSize, bool bNew) const {
   if (v.data() == 0L) {
     return 0L;
   }
@@ -66,13 +77,14 @@ double *KstDataObject::vectorRealloced(KstVectorPtr v, double *memptr, int newSi
   // in complicated object hierarchies such as filtered vectors.  Therefore if
   // you call vectorRealloced() and v is not locked by you already, you'd
   // better lock it!
-  double *rc = v->realloced(memptr, newSize);
+  double *rc = v->realloced(memptr, newSize, bNew);
 
   return rc;
 }
 
-void KstDataObject::save(QTextStream& ts) {
+void KstDataObject::save(QTextStream& ts, const QString& indent) {
   Q_UNUSED(ts)
+  Q_UNUSED(indent)
 }
 
 bool KstDataObject::loadInputs() {
@@ -102,8 +114,21 @@ bool KstDataObject::loadInputs() {
   }
   KST::scalarList.lock().readUnlock();
 
+  KST::stringList.lock().readLock();
+  for (i = _inputStringLoadQueue.begin(); i != _inputStringLoadQueue.end(); ++i) {
+    KstStringList::Iterator it = KST::stringList.findTag((*i).second);
+    if (it != KST::stringList.end()) {
+      _inputStrings.insert((*i).first, *it);
+    } else {
+      KstDebug::self()->log(i18n("Unable to find required string [%1] for data object %2.").arg((*i).second).arg(tagName()), KstDebug::Error);
+      rc = false;
+    }
+  }
+  KST::stringList.lock().readUnlock();
+
   _inputVectorLoadQueue.clear();
   _inputScalarLoadQueue.clear();
+  _inputStringLoadQueue.clear();
   update();
   return rc;
 }
@@ -123,6 +148,12 @@ int KstDataObject::getUsage() const {
     }
   }
 
+  for (KstScalarMap::ConstIterator i = _outputScalars.begin(); i != _outputScalars.end(); ++i) {
+    if (i.data().data()) {
+      rc += i.data()->getUsage() - 1;
+    }
+  }
+
   return KstObject::getUsage() + rc;
 }
 
@@ -137,6 +168,12 @@ void KstDataObject::readLock() const {
   #ifdef LOCKTRACE
   kdDebug() << "Read lock by tid=" << (int)QThread::currentThread() << endl;
   #endif
+  for (KstStringMap::ConstIterator i = _inputStrings.begin(); i != _inputStrings.end(); ++i) {
+    (*i)->readLock();
+  }
+  for (KstStringMap::ConstIterator i = _outputStrings.begin(); i != _outputStrings.end(); ++i) {
+    (*i)->readLock();
+  }
   for (KstScalarMap::ConstIterator i = _inputScalars.begin(); i != _inputScalars.end(); ++i) {
     (*i)->readLock();
   }
@@ -168,6 +205,12 @@ void KstDataObject::readUnlock() const {
   for (KstScalarMap::ConstIterator i = _inputScalars.begin(); i != _inputScalars.end(); ++i) {
     (*i)->readUnlock();
   }
+  for (KstStringMap::ConstIterator i = _outputStrings.begin(); i != _outputStrings.end(); ++i) {
+    (*i)->readUnlock();
+  }
+  for (KstStringMap::ConstIterator i = _inputStrings.begin(); i != _inputStrings.end(); ++i) {
+    (*i)->readUnlock();
+  }
   KstObject::readUnlock();
 }
 
@@ -180,6 +223,12 @@ void KstDataObject::writeLock() const {
   kdDebug() << (void*) this << " Write lock by tid=" << (int)QThread::currentThread() << endl;
   #endif
   KstRWLock::writeLock();
+  for (KstStringMap::ConstIterator i = _inputStrings.begin(); i != _inputStrings.end(); ++i) {
+    (*i)->writeLock();
+  }
+  for (KstStringMap::ConstIterator i = _outputStrings.begin(); i != _outputStrings.end(); ++i) {
+    (*i)->writeLock();
+  }
   for (KstScalarMap::ConstIterator i = _inputScalars.begin(); i != _inputScalars.end(); ++i) {
     (*i)->writeLock();
   }
@@ -211,6 +260,12 @@ void KstDataObject::writeUnlock() const {
   for (KstScalarMap::ConstIterator i = _inputScalars.begin(); i != _inputScalars.end(); ++i) {
     (*i)->writeUnlock();
   }
+  for (KstStringMap::ConstIterator i = _outputStrings.begin(); i != _outputStrings.end(); ++i) {
+    (*i)->writeUnlock();
+  }
+  for (KstStringMap::ConstIterator i = _inputStrings.begin(); i != _inputStrings.end(); ++i) {
+    (*i)->writeUnlock();
+  }
   KstRWLock::writeUnlock();
 }
 
@@ -219,5 +274,49 @@ bool KstDataObject::isValid() const {
   return true;
 }
 
+
+const KstCurveHintList* KstDataObject::curveHints() const {
+  return _curveHints;
+}
+
+
+bool KstDataObject::deleteDependents() {
+  KST::dataObjectList.lock().readLock();
+  KstDataObjectList dol = QDeepCopy<KstDataObjectList>(KST::dataObjectList);
+  KST::dataObjectList.lock().readUnlock();
+  for (KstDataObjectList::Iterator i = dol.begin(); i != dol.end(); ++i) {
+    bool user = (*i)->uses(this);
+    if (!user) {
+      // FIXME: should we care about scalars and strings? Needs policy decision
+      for (KstVectorMap::Iterator j = _outputVectors.begin(); !user && j != _outputVectors.end(); ++j) {
+        user = (*i)->uses(j.data().data());
+      }
+    }
+    if (user) {
+      KstDataObjectPtr dop = *i;
+      KST::dataObjectList.lock().writeLock();
+      KST::dataObjectList.remove(dop);
+      KST::dataObjectList.lock().writeUnlock();
+      dop->deleteDependents();
+    }
+  } 
+
+  return true;
+}
+
+
+bool KstDataObject::uses(KstObjectPtr p) const {
+  KstVectorPtr v = kst_cast<KstVector>(p);
+  if (v) {
+    for (KstVectorMap::ConstIterator j = _inputVectors.begin(); j != _inputVectors.end(); ++j) {
+      if (j.data() == v) {
+        return true;
+      }
+    }
+  }
+  return false;
+}
+
+
 #include "kstdataobject.moc"
 // vim: ts=2 sw=2 et
