@@ -22,13 +22,13 @@
 #include <kdebug.h>
 #include <klibloader.h>
 #include <klocale.h>
-#include <kservice.h>
 #include <kservicetype.h>
 
 #include <qdeepcopy.h>
 #include <qfile.h>
 #include <qstylesheet.h>
 
+#include "kst.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "stdinsource.h"
@@ -52,11 +52,11 @@ namespace KST {
         }
       }
 
-      KstDataSource *create(const QString& filename, const QString& type = QString::null) const {
-        KstDataSource *(*sym)(const QString&, const QString&) = (KstDataSource*(*)(const QString&, const QString&))symbol("create");
+      KstDataSource *create(KConfig *cfg, const QString& filename, const QString& type = QString::null) const {
+        KstDataSource *(*sym)(KConfig*, const QString&, const QString&) = (KstDataSource*(*)(KConfig*, const QString&, const QString&))symbol("create");
         if (sym) {
           //kdDebug() << "Trying to create " << filename << " type=" << type << " with " << service->property("Name").toString() << endl;
-          KstDataSource *ds = (sym)(filename, type);
+          KstDataSource *ds = (sym)(cfg, filename, type);
           if (ds) {
             ds->_source = service->property("Name").toString();
           }
@@ -67,27 +67,34 @@ namespace KST {
         return 0L;
       }
 
-      QStringList fieldList(const QString& filename, const QString& type = QString::null) const {
-        QStringList (*sym)(const QString&, const QString&) = (QStringList(*)(const QString&, const QString&))symbol("fieldList");
+      QStringList fieldList(KConfig *cfg, const QString& filename, const QString& type = QString::null, QString *typeSuggestion = 0L, bool *complete = 0L) const {
+        QStringList (*sym)(KConfig*, const QString&, const QString&, QString*, bool*) = (QStringList(*)(KConfig*, const QString&, const QString&, QString*, bool*))symbol("fieldList");
         if (sym) {
-          return (sym)(filename, type);
+          return (sym)(cfg, filename, type, typeSuggestion, complete);
         }
 
         // fallback incase the helper isn't implemented
         //  (note: less efficient now)
-        KstDataSourcePtr ds = create(filename, type);
+        KstDataSourcePtr ds = create(cfg, filename, type);
         if (ds) {
-          return ds->fieldList();
+          QStringList rc = ds->fieldList();
+          if (typeSuggestion) {
+            *typeSuggestion = ds->fileType();
+          }
+          if (complete) {
+            *complete = ds->fieldListIsComplete();
+          }
+          return rc;
         }
 
         return QStringList();
       }
 
-      int understands(const QString& filename) const {
-        int (*sym)(const QString&) = (int(*)(const QString&))symbol("understands");
+      int understands(KConfig *cfg, const QString& filename) const {
+        int (*sym)(KConfig*, const QString&) = (int(*)(KConfig*, const QString&))symbol("understands");
         if (sym) {
           //kdDebug() << "Checking if " << service->property("Name").toString() << " understands " << filename << endl;
-          int rc = (sym)(filename);
+          int rc = (sym)(cfg, filename);
           //kdDebug() << "result: " << rc << endl;
           return rc;
         }
@@ -96,15 +103,33 @@ namespace KST {
       }
 
       bool provides(const QString& type) const {
+        return provides().contains(type);
+      }
+
+      QStringList provides() const {
         QStringList (*sym)() = (QStringList(*)())symbol("provides");
         if (sym) {
           //kdDebug() << "Checking if " << service->property("Name").toString() << " provides " << type << endl;
-          bool rc = ((sym)()).contains(type);
-          //kdDebug() << "result: " << rc << endl;
-          return rc;
+          return (sym)();
         }
 
-        return false;
+        return QStringList();
+      }
+
+      KstDataSourceConfigWidget *configWidget(KConfig *cfg, const QString& filename) const {
+        QWidget *(*sym)(const QString&) = (QWidget *(*)(const QString&))symbol("widget");
+        if (sym) {
+          QWidget *rc = (sym)(filename);
+          KstDataSourceConfigWidget *cw = dynamic_cast<KstDataSourceConfigWidget*>(rc);
+          if (cw) {
+            cw->setConfig(cfg);
+            return cw;
+          }
+          KstDebug::self()->log(i18n("Error in plugin %1: Configuration widget is of the wrong type.").arg(service->property("Name").toString()), KstDebug::Error);
+          delete rc;
+        }
+
+        return 0L;
       }
 
       KService::Ptr service;
@@ -117,7 +142,11 @@ namespace KST {
 
         // FIXME: might be a good idea to cache this per-symbol
 
-        return _lib->symbol(QFile::encodeName(sym + "_" + _plugLib));
+        QCString s = QFile::encodeName(sym + "_" + _plugLib);
+        if (_lib->hasSymbol(s)) {
+          return _lib->symbol(s);
+        }
+        return 0L;
       }
 
       bool loadLibrary() const {
@@ -144,7 +173,17 @@ namespace KST {
 }
 
 
+static KConfig *kConfigObject = 0L;
+void KstDataSource::setupOnStartup(KConfig *cfg) {
+  kConfigObject = cfg;
+}
+
+
 static KST::PluginInfoList pluginInfo;
+void KstDataSource::cleanupForExit() {
+  pluginInfo.clear();
+  kConfigObject = 0L;
+}
 
 
 // Scans for plugins and stores the information for them in "pluginInfo"
@@ -202,9 +241,9 @@ class PluginSortContainer {
 };
 }
 
-static KstDataSourcePtr findPluginFor(const QString& filename, const QString& type) {
-  KstDataSourcePtr plugin;
 
+static QValueList<PluginSortContainer> bestPluginsForSource(const QString& filename, const QString& type) {
+  QValueList<PluginSortContainer> bestPlugins;
   scanPlugins();
 
   KST::PluginInfoList info = QDeepCopy<KST::PluginInfoList>(pluginInfo);
@@ -212,40 +251,46 @@ static KstDataSourcePtr findPluginFor(const QString& filename, const QString& ty
   if (!type.isEmpty()) {
     for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
       if ((*it)->provides(type)) {
-        plugin = (*it)->create(filename, type);
-        break;
+        PluginSortContainer psc;
+        psc.match = 100;
+        psc.plugin = *it;
+        bestPlugins.append(psc);
+        return bestPlugins;
       }
     }
-
-    return plugin;
   }
 
-  QValueList<PluginSortContainer> bestPlugins;
-
   for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
     PluginSortContainer psc;
-    if ((psc.match = (*it)->understands(filename)) > 0) {
+    if ((psc.match = (*it)->understands(kConfigObject, filename)) > 0) {
       psc.plugin = *it;
       bestPlugins.append(psc);
     }
   }
 
   qHeapSort(bestPlugins);
+  return bestPlugins;
+}
+
+
+static KstDataSourcePtr findPluginFor(const QString& filename, const QString& type) {
+
+  QValueList<PluginSortContainer> bestPlugins = bestPluginsForSource(filename, type);
 
   for (QValueList<PluginSortContainer>::Iterator i = bestPlugins.begin(); i != bestPlugins.end(); ++i) {
-      plugin = (*i).plugin->create(filename);
-      if (plugin) {
-        break;
-      }
+    KstDataSourcePtr plugin = (*i).plugin->create(kConfigObject, filename);
+    if (plugin) {
+      return plugin;
+    }
   }
 
-  return plugin;
+  return 0L;
 }
 
 
 KstDataSourcePtr KstDataSource::loadSource(const QString& filename, const QString& type) {
   if (filename == "stdin" || filename == "-") {
-    return new KstStdinSource;
+    return new KstStdinSource(kConfigObject);
   }
 
   if (filename.isEmpty()) {
@@ -256,41 +301,63 @@ KstDataSourcePtr KstDataSource::loadSource(const QString& filename, const QStrin
 }
 
 
-QStringList KstDataSource::fieldListForSource(const QString& filename, const QString& type) {
-  if (filename.isEmpty() || filename == "stdin" || filename == "-") {
-    return QStringList();
-  }
+KstDataSourceConfigWidget* KstDataSource::configWidget() const {
+  return configWidgetForSource(_filename, fileType());
+}
 
+
+KstDataSourceConfigWidget* KstDataSource::configWidgetForPlugin(const QString& plugin) {
   scanPlugins();
 
   KST::PluginInfoList info = QDeepCopy<KST::PluginInfoList>(pluginInfo);
 
-  if (!type.isEmpty()) {
-    for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
-      if ((*it)->provides(type)) {
-        return (*it)->fieldList(filename, type);
-      }
-    }
-  }
-
-  QValueList<PluginSortContainer> bestPlugins;
-
   for (KST::PluginInfoList::ConstIterator it = info.begin(); it != info.end(); ++it) {
-    PluginSortContainer psc;
-    if ((psc.match = (*it)->understands(filename)) > 0) {
-      psc.plugin = *it;
-      bestPlugins.append(psc);
+    if ((*it)->service->property("Name").toString() == plugin) {
+      return (*it)->configWidget(kConfigObject, QString::null);
     }
   }
 
-  qHeapSort(bestPlugins);
+  return 0L;
+}
 
+
+KstDataSourceConfigWidget* KstDataSource::configWidgetForSource(const QString& filename, const QString& type) {
+  if (filename.isEmpty() || filename == "stdin" || filename == "-") {
+    return 0L;
+  }
+
+  QValueList<PluginSortContainer> bestPlugins = bestPluginsForSource(filename, type);
+  for (QValueList<PluginSortContainer>::Iterator i = bestPlugins.begin(); i != bestPlugins.end(); ++i) {
+    KstDataSourceConfigWidget *w = (*i).plugin->configWidget(kConfigObject, filename);
+    // Don't iterate.
+    return w;
+  }
+
+  KstDebug::self()->log(i18n("Could not find a datasource for '%1'(%2), but we found one just prior.  Something is wrong with Kst.").arg(filename).arg(type), KstDebug::Error);
+  return 0L;
+}
+
+
+QStringList KstDataSource::fieldListForSource(const QString& filename, const QString& type, QString *outType, bool *complete) {
+  if (filename.isEmpty() || filename == "stdin" || filename == "-") {
+    return QStringList();
+  }
+
+  QValueList<PluginSortContainer> bestPlugins = bestPluginsForSource(filename, type);
   QStringList rc;
   for (QValueList<PluginSortContainer>::Iterator i = bestPlugins.begin(); i != bestPlugins.end(); ++i) {
-      rc = (*i).plugin->fieldList(filename);
-      if (!rc.isEmpty()) {
-        break;
+    QString typeSuggestion;
+    rc = (*i).plugin->fieldList(kConfigObject, filename, QString::null, &typeSuggestion, complete);
+    if (!rc.isEmpty()) {
+      if (outType) {
+        if (typeSuggestion.isEmpty()) {
+          *outType = (*i).plugin->provides()[0];
+        } else {
+          *outType = typeSuggestion;
+        }
       }
+      break;
+    }
   }
 
   return rc;
@@ -318,27 +385,35 @@ KstDataSourcePtr KstDataSource::loadSource(QDomElement& e) {
   }
 
   if (filename == "stdin" || filename == "-") {
-    return new KstStdinSource;
+    return new KstStdinSource(kConfigObject);
   }
 
   return findPluginFor(filename, type);
 }
 
 
-KstDataSource::KstDataSource(const QString& filename, const QString& type)
-: KstObject(), _filename(filename) {
+class KstDataSourcePrivate {
+  public:
+    KstDataSourcePrivate() {}
+    ~KstDataSourcePrivate() {}
+};
+
+
+KstDataSource::KstDataSource(KConfig *cfg, const QString& filename, const QString& type)
+: KstObject(), _filename(filename), _cfg(cfg) {
   Q_UNUSED(type)
   _valid = false;
-  d = new KstDataSourcePrivate;
-  d->numFrames = new KstScalar(filename + i18n("-frames"));
+  //d = new KstDataSourcePrivate;
+  _numFramesScalar = new KstScalar(filename + i18n("-frames"));
 }
 
+
 KstDataSource::~KstDataSource() {
   KST::scalarList.lock().writeLock();
-  KST::scalarList.remove(d->numFrames);
+  KST::scalarList.remove(_numFramesScalar);
   KST::scalarList.lock().writeUnlock();
-  d->numFrames = 0;
-  delete d;
+  _numFramesScalar = 0L;
+  //delete d;
 }
 
 
@@ -347,10 +422,23 @@ KstObject::UpdateType KstDataSource::update(int u) {
   return KstObject::NO_CHANGE;
 }
 
+
 void KstDataSource::updateNumFramesScalar() {
-  d->numFrames->setValue(frameCount());
+  _numFramesScalar->setValue(frameCount());
 }
 
+
+int KstDataSource::readField(double *v, const QString& field, int s, int n, int skip, int *lastFrameRead) {
+  Q_UNUSED(v)
+  Q_UNUSED(field)
+  Q_UNUSED(s)
+  Q_UNUSED(n)
+  Q_UNUSED(skip)
+  Q_UNUSED(lastFrameRead)
+  return -9999; // unsupported
+}
+
+
 int KstDataSource::readField(double *v, const QString& field, int s, int n) {
   Q_UNUSED(v)
   Q_UNUSED(field)
@@ -398,9 +486,9 @@ QString KstDataSource::fileType() const {
 }
 
 
-void KstDataSource::save(QTextStream &ts) {
-  ts << " <filename>" << QStyleSheet::escape(_filename) << "</filename>" << endl;
-  ts << " <type>" << QStyleSheet::escape(fileType()) << "</type>" << endl;
+void KstDataSource::save(QTextStream &ts, const QString& indent) {
+  ts << indent << "<filename>" << QStyleSheet::escape(_filename) << "</filename>" << endl;
+  ts << indent << "<type>" << QStyleSheet::escape(fileType()) << "</type>" << endl;
 }
 
 
@@ -426,11 +514,6 @@ void *KstDataSource::bufferRealloc(void *ptr, size_t size) {
 }
 
 
-QGuardedPtr<QWidget> KstDataSource::configWidget() const {
-  return 0L;
-}
-
-
 bool KstDataSource::fieldListIsComplete() const {
   return true;
 }
@@ -445,4 +528,92 @@ bool KstDataSource::reset() {
   return false;
 }
 
+
+const QMap<QString, QString>& KstDataSource::metaData() const {
+  return _metaData;
+}
+
+
+const QString& KstDataSource::metaData(const QString& key) const {
+  if (_metaData.contains(key)) {
+    return _metaData[key];
+  } else {
+    return QString::null;
+  }
+}
+
+
+bool KstDataSource::hasMetaData() const {
+  return !_metaData.isEmpty();
+}
+
+
+bool KstDataSource::hasMetaData(const QString& key) const {
+  return _metaData.contains(key);
+}
+
+
+bool KstDataSource::supportsTimeConversions() const {
+  return false;
+}
+
+
+int KstDataSource::sampleForTime(const QDateTime& time, bool *ok) {
+  Q_UNUSED(time)
+  if (ok) {
+    *ok = false;
+  }
+  return 0;
+}
+
+
+
+int KstDataSource::sampleForTime(int milliseconds, bool *ok) {
+  Q_UNUSED(milliseconds)
+  if (ok) {
+    *ok = false;
+  }
+  return 0;
+}
+
+
+
+QDateTime KstDataSource::timeForSample(int sample, bool *ok) {
+  Q_UNUSED(sample)
+  if (ok) {
+    *ok = false;
+  }
+  return QDateTime::currentDateTime();
+}
+
+
+
+int KstDataSource::relativeTimeForSample(int sample, bool *ok) {
+  Q_UNUSED(sample)
+  if (ok) {
+    *ok = false;
+  }
+  return 0;
+}
+
+
+/////////////////////////////////////////////////////////////////////////////
+KstDataSourceConfigWidget::KstDataSourceConfigWidget()
+: QWidget(0L), _cfg(0L) {
+}
+
+
+KstDataSourceConfigWidget::~KstDataSourceConfigWidget() {
+}
+
+
+void KstDataSourceConfigWidget::save() {
+}
+
+
+void KstDataSourceConfigWidget::setConfig(KConfig *cfg) {
+  _cfg = cfg;
+}
+
+#include "kstdatasource.moc"
 // vim: ts=2 sw=2 et
