@@ -17,38 +17,18 @@
 
 // include files for Qt
 #include <qcheckbox.h>
-#include <qdir.h>
-#include <qfileinfo.h>
-#include <qfontdatabase.h>
+#include <qlabel.h>
 #include <qlayout.h>
-#include <qlineedit.h>
 #include <qlistview.h>
-#include <qmessagebox.h>
-#include <qpushbutton.h>
-#include <qradiobutton.h>
-#include <qspinbox.h>
-#include <qstring.h>
-#include <qtable.h>
-#include <qtabwidget.h>
-#include <qtextbrowser.h>
-#include <qtextstream.h>
-#include <qwidget.h>
 
 // include files for KDE
 #include <kcolorbutton.h>
 #include <kdebug.h>
-#include <kdualcolorbutton.h>
-#include <kfontcombo.h>
-#include <klistbox.h>
 #include <klocale.h>
-#include <kmessagebox.h>
 
 #include "kstdatacollection.h"
-#include "kstdebug.h"
 #include "kstdebugdialog_i.h"
-#include "kstlegend.h"
 #include "kstlogtable.h"
-#include "kstsettings.h"
 #include "kstversion.h"
 
 /*
