@@ -17,11 +17,9 @@
 #ifndef KSTDEBUGI_H
 #define KSTDEBUGI_H
 
-
-#include <qlabel.h>
-
 #include "debugdialog.h"
-#include "kstlogtable.h"
+
+class KstLogTable;
 
 class KstDebugDialogI : public DebugDialog {
   Q_OBJECT
