@@ -26,45 +26,24 @@
 #include <unistd.h>
 
 // include files for Qt
-#include <qcheckbox.h>
 #include <qdeepcopy.h>
-#include <qdir.h>
-#include <qfileinfo.h>
-#include <qfontdatabase.h>
-#include <qlineedit.h>
-#include <qlistbox.h>
-#include <qpushbutton.h>
-#include <qradiobutton.h>
-#include <qspinbox.h>
-#include <qstring.h>
-#include <qtextstream.h>
-#include <qwidget.h>
+#include <qstylesheet.h>
 
 // include files for KDE
-#include <kapplication.h>
-#include <kcolordialog.h>
 #include <kdebug.h>
 #include <kfiledialog.h>
-#include <klocale.h>
-#include <kmdimainfrm.h>
 #include <kmessagebox.h>
 #include <kprogress.h>
 #include <ksavefile.h>
-#include <kurl.h>
 
 // application specific includes
-#include "kst.h"
-#include "kst2dplot.h"
-#include "kstdatacollection.h"
-#include "kstdebug.h"
 #include "kstdoc.h"
-#include "kstequationcurve.h"
+#include "kstgraphfiledialog_i.h"
+#include "kstequation.h"
 #include "ksthistogram.h"
-#include "kstimage.h"
-#include "kstmatrix.h"
 #include "kstplugin.h"
-#include "kstpoint.h"
-#include "kstpsdcurve.h"
+#include "kstpsd.h"
+#include "kstrvector.h"
 #include "kstvcurve.h"
 #include "kstvectordefaults.h"
 #include "kstviewwindow.h"
@@ -77,6 +56,7 @@ KstDoc::KstDoc(QWidget *parent, const char *name)
   _lock = 0;
   _updating = false;
   _stopping = false;
+  _updateDelay = 5;
 }
 
 KstDoc::~KstDoc() {
@@ -166,32 +146,38 @@ bool KstDoc::newDocument() {
 
 bool KstDoc::openDocument(const KURL& url, const QString& o_file,
                           int o_n, int o_f, int o_s, bool o_ave) {
-  // Temp variables
-  KstDataSourcePtr file;
   KstRVectorPtr vector;
+  KstApp *app = KstApp::inst();
+  QString readingDocument = i18n("Reading document");
+  QString openingDocument = i18n("Opening document");
 
   deleteContents();
+  _updating = true; // block update thread from sending events 'till we're done
+
   QFile f(url.path());
   if (!f.exists()) {
     KMessageBox::sorry(0L, i18n("%1: There is no file with that name to open.").arg(url.path()));
+    _updating = false;
     return false;
   }
 
   _title = url.fileName(false);
   _absFilePath = url.path();
-  if (_title.isNull()) { // isEmpty()???
+  if (_title.isEmpty()) {
     _title = _absFilePath;
   }
   QDomDocument doc(_title);
 
   if (!f.open(IO_ReadOnly)) {
     KMessageBox::sorry(0L, i18n("%1: File exists, but kst could not open it.").arg(url.path()));
+    _updating = false;
     return false;
   }
 
   if (!doc.setContent(&f)) {
     KMessageBox::sorry(0L, i18n("%1: Not a valid kst plot specification file.").arg(url.path()));
     f.close();
+    _updating = false;
     return false;
   }
 
@@ -199,8 +185,14 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
 
   QDomElement docElem = doc.documentElement();
   int columns = -1;
+  int count;
+  int handled = 0;
+  bool warnOldKstFile = false;
 
   QDomNode n = docElem.firstChild();
+  count = docElem.childNodes().length();
+  app->slotUpdateProgress(count, handled, readingDocument);
+  
   while (!n.isNull()) {
     QDomElement e = n.toElement(); // try to convert the node to an element.
     if (!e.isNull()) { // the node was really an element.
@@ -217,6 +209,30 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
           nn = nn.nextSibling();
         }
         static_cast<KstApp*>(parent())->resize(size);
+      } else if (e.tagName() == "graphicsautosave") {
+        KstGraphFileDialogI *gdlg = KstApp::inst()->graphFileDlg();
+        int time = e.attribute("time").toInt();
+        bool en = e.attribute("enabled") == "true";
+        QString fmt = e.attribute("format");
+        QString url = e.attribute("url");
+        bool sq = e.attribute("square") == "true";
+        int sizeX = e.attribute("xsize").toInt();
+        int sizeY = e.attribute("ysize").toInt();
+
+        if (sizeX > 0 && sizeY > 0) {
+          gdlg->setImageSize(sizeX, sizeY);
+          gdlg->setSquare(sq);
+        }
+        if (!url.isEmpty()) {
+          gdlg->setURL(url);
+        }
+        if (!fmt.isEmpty()) {
+          gdlg->setFormat(fmt);
+        }
+        if (time >= 0) {
+          gdlg->setAutoSaveTimer(time);
+        }
+        gdlg->setAutoSave(en);
       } else if (e.tagName() == "plotcols") {
         columns = e.text().toInt();
       } else if (e.tagName() == "updateDelay") {
@@ -224,6 +240,7 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         setDelay(e.text().toInt());
       } else if (e.tagName() == "kstfile") {
         KstWriteLocker dswl(&KST::dataSourceList.lock());
+        KstDataSourcePtr file;
         if (o_file == "|") {
           file = KstDataSource::loadSource(e);
         } else {
@@ -237,6 +254,8 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         }
       } else if (e.tagName() == "scalar") {
         new KstScalar(e);
+      } else if (e.tagName() == "string") {
+        new KstString(e);
       } else if (e.tagName() == "vector") {
         vector = new KstRVector(e, o_file, o_n, o_f, o_s, o_ave);
         KST::addVectorToList(KstVectorPtr(vector));
@@ -250,11 +269,21 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         KstWriteLocker dowl(&KST::dataObjectList.lock());
         KST::dataObjectList.append(p);
       } else if (e.tagName() == "equation") {
-        KstDataObjectPtr p = new KstEquationCurve(e);
+        KstEquationPtr p = new KstEquation(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p.data());
+        warnOldKstFile = true;
+      } else if (e.tagName() == "psd") {
+        KstPSDPtr p = new KstPSD(e);
+        KstWriteLocker dowl(&KST::dataObjectList.lock());
+        KST::dataObjectList.append(p.data());
+        warnOldKstFile = true;
+      } else if (e.tagName() == "equationobject") {
+        KstDataObjectPtr p = new KstEquation(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
         KST::dataObjectList.append(p);
-      } else if (e.tagName() == "psd") {
-        KstDataObjectPtr p = new KstPSDCurve(e);
+      } else if (e.tagName() == "psdobject") {
+        KstDataObjectPtr p = new KstPSD(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
         KST::dataObjectList.append(p);
       } else if (e.tagName() == "histogram") {
@@ -267,8 +296,8 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
         KST::dataObjectList.append(p);
       } else if (e.tagName() == "plot") {
         KstBaseCurveList l = kstObjectSubList<KstDataObject,KstBaseCurve>(KST::dataObjectList);
-        Kst2DPlotPtr plotPtr = new Kst2DPlot(e);
-        KstApp::inst()->plotHolderWhileOpeningDocument().insert(plotPtr->tagName(), plotPtr);
+        Kst2DPlotPtr plot = new Kst2DPlot(e);
+        KstApp::inst()->plotHolderWhileOpeningDocument().insert(plot->tagName(), plot);
       } else if (e.tagName() == "matrix") {
         KstDataObjectPtr p = new KstMatrix(e);
         KstWriteLocker dowl(&KST::dataObjectList.lock());
@@ -298,18 +327,24 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
           KstApp::inst()->closeWindow(toDelete);
         }
       } else {
-        KstDebug::self()->log(i18n("Unsupported element '%1' in file %2.").arg(e.tagName()).arg(file->fileName()), KstDebug::Warning);
+        KstDebug::self()->log(i18n("Unsupported element '%1' in file %2.").arg(e.tagName()).arg(o_file), KstDebug::Warning);
       }
     }
+    handled++;
+    app->slotUpdateProgress(count, handled, readingDocument);
     n = n.nextSibling();
   }
+  app->slotUpdateProgress(0, 0, openingDocument);
+
+  if (warnOldKstFile) {
+    KMessageBox::sorry(0L, i18n("You tried to load an old Kst file.  Curves created by equations or PSDs will not be loaded."));
+  }
 
   //
   // if we have anything left in plotHolderWhileOpeningDocument then
   //  we are most likely reading an old style document, so we create
   //  a default view and fill it with whatever is left...
   //
-  KstApp *app = KstApp::inst();
   if (!app->plotHolderWhileOpeningDocument().isEmpty() && !app->activeWindow()) {
 
     QString winName = app->newWindow(QString::null);
@@ -343,7 +378,9 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
 
   KstApp::inst()->plotHolderWhileOpeningDocument().clear();
 
-  // Lazy load data
+  //
+  // lazy load data
+  //
   KstDataObjectList bitBucket;
   {
     KST::dataObjectList.lock().readLock();
@@ -356,7 +393,9 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
       bool rc = (*i)->loadInputs();
       (*i)->KstRWLock::writeUnlock();
       if (!rc) {
+        //
         // schedule for removal
+        //
         bitBucket.append(*i);
       }
     }
@@ -374,7 +413,9 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
 
   createScalars();
 
-  // Update plots now that lazy loading is done
+  //
+  // update plots now that lazy loading is done
+  //
   KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
   while (it->currentItem()) {
     KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
@@ -390,54 +431,93 @@ bool KstDoc::openDocument(const KURL& url, const QString& o_file,
 
   emit updateDialogs();
 
+  app->slotUpdateProgress(0, 0, QString::null);
+
   _modified = false;
+  
+  QTimer::singleShot(0, this, SLOT(enableUpdates()));
+  
   return true;
 }
 
-void KstDoc::saveDocument(QTextStream& ts) {
+void KstDoc::saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions) {
   KstApp *app = KstApp::inst();
   ts << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
   ts << "<kstdoc>" << endl;
   // save window geometry for this kst file
-  ts << " <windowsize>" << endl;
-  ts << "  <width>" << ((KstApp *)parent())->width() << "</width>" << endl;
-  ts << "  <height>"<< ((KstApp *)parent())->height()<< "</height>" << endl;
-  ts << " </windowsize>" << endl;
+  ts << "  <windowsize>" << endl;
+  ts << "    <width>" << ((KstApp *)parent())->width() << "</width>" << endl;
+  ts << "    <height>"<< ((KstApp *)parent())->height()<< "</height>" << endl;
+  ts << "  </windowsize>" << endl;
+
+  // save graphics autosave settings
+  ts << "  <graphicsautosave time=\""
+    << KstApp::inst()->graphFileDlg()->autoSaveTimer()
+    << "\" enabled=\""
+    << (KstApp::inst()->graphFileDlg()->autoSaving() ? "true" : "false")
+    << "\" format=\""
+    << QStyleSheet::escape(KstApp::inst()->graphFileDlg()->format())
+    << "\" xsize=\""
+    << KstApp::inst()->graphFileDlg()->imageXSize()
+    << "\" ysize=\""
+    << KstApp::inst()->graphFileDlg()->imageYSize()
+    << "\" square=\""
+    << (KstApp::inst()->graphFileDlg()->isSquare() ? "true" : "false");
+  if (KstApp::inst()->graphFileDlg()->autoSaving()) {
+    ts << "\" url=\"" << QStyleSheet::escape(KstApp::inst()->graphFileDlg()->url().url());
+  }
+  ts << "\" />" << endl;
 
   // save update delay
 //FIXME XXXX
   ts << " <updateDelay>" << delay() << "</updateDelay>" << endl;
 
+  KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+
   // save files
   KST::dataSourceList.lock().readLock();
   for (uint i = 0; i < KST::dataSourceList.count(); i++) {
-    ts << " <kstfile>" << endl;
-    KST::dataSourceList[i]->save(ts);
-    ts << " </kstfile>" << endl;
+    KstDataSourcePtr dsp = KST::dataSourceList[i];
+    for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
+      if ((*it)->dataSource() == dsp) {
+        ts << "  <kstfile>" << endl;
+        dsp->save(ts, "    ");
+        ts << "  </kstfile>" << endl;
+        break;
+      }
+    }
   }
   KST::dataSourceList.lock().readUnlock();
 
   // save orphan scalars
   for (KstScalarList::Iterator it = KST::scalarList.begin(); it != KST::scalarList.end(); ++it) {
     if ((*it)->orphan()) {
-      ts << " <scalar>" << endl;
-      (*it)->save(ts);
-      ts << " </scalar>" << endl;
+      ts << "  <scalar>" << endl;
+      (*it)->save(ts, "    ");
+      ts << "  </scalar>" << endl;
+    }
+  }
+
+  // save orphan strings
+  for (KstStringList::Iterator it = KST::stringList.begin(); it != KST::stringList.end(); ++it) {
+    if ((*it)->orphan()) {
+      ts << "  <string>" << endl;
+      (*it)->save(ts, "    ");
+      ts << "  </string>" << endl;
     }
   }
 
   // save vectors
-  KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
-    ts << " <vector>" << endl;
-    (*it)->save(ts);
-    ts << " </vector>" << endl;
+    ts << "  <vector>" << endl;
+    (*it)->save(ts, "    ", saveAbsoluteVectorPositions);
+    ts << "  </vector>" << endl;
   }
 
   // save data objects
   KST::dataObjectList.lock().readLock();
   for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
-    (*it)->save(ts);
+    (*it)->save(ts, "  ");
   }
   KST::dataObjectList.lock().readUnlock();
   // save plots
@@ -447,11 +527,11 @@ void KstDoc::saveDocument(QTextStream& ts) {
     if (c) {
       KstPlotBaseList plots = c->view()->findChildrenType<KstPlotBase>(true);
       for (KstPlotBaseList::Iterator i = plots.begin(); i != plots.end(); ++i) {
-        Kst2DPlotPtr plot = dynamic_cast<Kst2DPlot*>((*i).data());
+        Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(*i);
         if (plot) {
-          ts << " <plot>" << endl;
-          plot->save(ts);
-          ts << " </plot>" << endl;
+          ts << "  <plot>" << endl;
+          plot->save(ts, "    ");
+          ts << "  </plot>" << endl;
         } else {
         }
       }
@@ -466,9 +546,9 @@ void KstDoc::saveDocument(QTextStream& ts) {
     while (it->currentItem()) {
       KstViewWindow *v = dynamic_cast<KstViewWindow*>(it->currentItem());
       if (v) {
-        ts << " <window>" << endl;
-        v->save(ts);
-        ts << " </window>" << endl;
+        ts << "  <window>" << endl;
+        v->save(ts, "    ");
+        ts << "  </window>" << endl;
       }
       it->next();
     }
@@ -478,11 +558,11 @@ void KstDoc::saveDocument(QTextStream& ts) {
   ts << "</kstdoc>" << endl;
 }
 
-bool KstDoc::saveDocument(const QString &filename) {
+bool KstDoc::saveDocument(const QString &filename, bool saveAbsoluteVectorPositions) {
   backupFile(filename);
   QFile f(filename);
 
-  if (f.exists() && (filename != absFilePath())) {
+  if (f.exists() && filename != absFilePath()) {
     if (KMessageBox::warningYesNo(0L, i18n("%1: A file of this name already exists.").arg(filename),
                                   i18n("Warning"),i18n("&Overwrite"),
                                   i18n("&Cancel")) == KMessageBox::No) {
@@ -497,7 +577,7 @@ bool KstDoc::saveDocument(const QString &filename) {
 
   QTextStream ts(&f);
 
-  saveDocument(ts);
+  saveDocument(ts, saveAbsoluteVectorPositions);
 
   f.close();
 
@@ -530,6 +610,10 @@ void KstDoc::deleteContents() {
   KST::scalarList.clear();
   KST::scalarList.lock().writeUnlock();
 
+  KST::stringList.lock().writeLock();
+  KST::stringList.clear();
+  KST::stringList.lock().writeUnlock();
+
   KST::dataSourceList.lock().writeLock();
   KST::dataSourceList.clear();
   KST::dataSourceList.lock().writeUnlock();
@@ -646,22 +730,20 @@ RemoveStatus KstDoc::removeDataObject(const QString& tag) {
 
 
 void KstDoc::purge() {
+  QString purging = i18n("Purging unused objects");
   bool modified = false;
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-
+  KstApp *app = KstApp::inst();
+  
   KST::dataObjectList.lock().writeLock();
   int cnt = rvl.count() + KST::dataObjectList.count();
   int prg = 0;
 
-  KProgressDialog *dlg = new KProgressDialog((QWidget*)static_cast<KstApp*>(parent()), "Progress Dialog");
-
-  dlg->setAllowCancel(false);
-  dlg->progressBar()->setTotalSteps(cnt);
-  dlg->progressBar()->setProgress(0);
-  dlg->setLabel(i18n("Purging unused objects..."));
-  dlg->show();
+  app->slotUpdateProgress(cnt, prg, purging);
 
+  //
   // ASSUMPTION: this only gets called from the data manager!
+  //
   for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
     //kdDebug() << "OBJECT: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
     if ((*it)->getUsage() == 0) {
@@ -671,8 +753,8 @@ void KstDoc::purge() {
       KST::dataObjectList.remove(byebye);
       modified = true;
     }
-    dlg->progressBar()->setProgress(++prg);
-    kapp->processEvents();
+    prg++;
+    app->slotUpdateProgress(cnt, prg, purging);
   }
   KST::dataObjectList.lock().writeUnlock();
 
@@ -685,15 +767,15 @@ void KstDoc::purge() {
       KST::vectorList.lock().writeUnlock();
       modified = true;
     }
-    dlg->progressBar()->setProgress(++prg);
-    kapp->processEvents();
+    prg++;
+    app->slotUpdateProgress(cnt, prg, purging);
   }
 
+  app->slotUpdateProgress(0, 0, QString::null);
+
   rvl.clear();
   setModified(modified);
   emit updateDialogs();
-
-  delete dlg;
 }
 
 
@@ -702,6 +784,11 @@ void KstDoc::forceUpdate() {
 }
 
 
+void KstDoc::enableUpdates() {
+  _updating = false;
+}
+
+
 bool KstDoc::event(QEvent *e) {
   if (e->type() == KstEventTypeThread) {
     _updating = true; // block update thread from sending events 'till we're done
@@ -733,8 +820,8 @@ bool KstDoc::event(QEvent *e) {
       KstApp::inst()->paintAll();
     } else if (te->_eventType == ThreadEvent::Done) {
     }
-    kapp->processEvents(); // actually do the paints and check for inputs
-    _updating = false; // ok, we're done... update thread can send us a new event.
+
+    QTimer::singleShot(0, this, SLOT(enableUpdates()));
 
     return true;
   }
