@@ -21,14 +21,9 @@
 #include <config.h>
 
 // include files for QT
-#include <qobject.h>
-#include <qptrlist.h>
-#include <qstring.h>
 
 #include <kapplication.h>
-#include <kurl.h>
 
-#include "kstobject.h"
 #include "kstlistenums.h"
 
 class KstDoc : public QObject {
@@ -45,8 +40,6 @@ public:
   bool isModified() const { return _modified; }
   /** asks the user for saving if the document is modified */
   bool saveModified(bool doDelete = true);
-  /** deletes the document's contents */
-  void deleteContents();
   /** initializes the document generally */
   bool newDocument();
   /** closes the acutal document */
@@ -56,9 +49,9 @@ public:
 		    int o_n = -2, int o_f = -2,
 		    int o_s = -1, bool o_ave = false);
   /** saves the document to a QTextStream */
-  void saveDocument(QTextStream& ts);
+  void saveDocument(QTextStream& ts, bool saveAbsoluteVectorPositions = false);
   /** saves the document under filename */
-  bool saveDocument(const QString& filename);
+  bool saveDocument(const QString& filename, bool saveAbsoluteVectorPositions = false);
   /** returns the pathname of the current document file*/
   const QString& absFilePath() const;
   /** returns the title of the document */
@@ -75,6 +68,8 @@ public:
   bool updating() const { return _updating; }
 
 public slots:
+  /** deletes the document's contents */
+  void deleteContents();
   void purge();
   void wasModified();
   void forceUpdate();
@@ -96,8 +91,11 @@ public slots:
 
   void setDelay(int in_delay) { _updateDelay = in_delay; }
 
+private slots:
+  void enableUpdates();
+
 private:
-   void createScalars() const;
+  void createScalars() const;
 
   /** the modified flag of the current document */
   bool _modified;
@@ -109,6 +107,7 @@ private:
   QString _absFilePath;
   int _lock;
   bool _updating;
+
 signals:
   /** if something has changed the vectors */
   void dataChanged();
@@ -116,3 +115,4 @@ signals:
 };
 
 #endif // KSTDOC_H
+// vim: ts=2 sw=2 et
