@@ -20,44 +20,46 @@
 #define KSTEQDIALOGI_H
 
 #include "eqdialog.h"
-#include "kstequationcurve.h"
+#include "kstequation.h"
 
 class KstEqDialogI : public KstEqDialog {
   Q_OBJECT
-public:
-  KstEqDialogI(QWidget* parent = 0, const char* name = 0,
-               bool modal = FALSE, WFlags fl = 0 );
-  virtual ~KstEqDialogI();
-public slots:
+  public:
+    KstEqDialogI(QWidget* parent = 0, const char* name = 0,
+        bool modal = FALSE, WFlags fl = 0 );
+    virtual ~KstEqDialogI();
 
-  void update();
-  void updateWindow();
+  public slots:
+    void update();
+    void updateWindow();
 
-  bool new_I();
-  bool edit_I();
+    bool new_I();
+    bool edit_I();
 
-  static KstEqDialogI *globalInstance();
+    static KstEqDialogI *globalInstance();
 
-private:
-  static KstEqDialogI *_inst;
-  KstEquationCurvePtr _getPtr(const QString &tagin);
-  bool _checkEntries();
-  bool _newDialog;
-  KstEquationCurvePtr DP;
+  private:
+    static KstEqDialogI *_inst;
+    KstEquationPtr _getPtr(const QString &tagin);
+    bool _checkEntries();
+    bool _newDialog;
+    KstEquationPtr DP;
 
-/***********************************/
-/** defined in dataobjectdialog.h **/
-public slots:
-  void show_Edit(const QString &field);
-  void show_New();
-  void OK();
-  void Init();
-  void close();
-  void reject();
-private:
-  void _fillFieldsForEdit();
-  void _fillFieldsForNew();
+    /***********************************/
+    /** defined in dataobjectdialog.h **/
+  public slots:
+    void show_Edit(const QString &field);
+    void show_New();
+    void OK();
+    void Init();
+    void close();
+    void reject();
+    void populateFunctionList();
 
+  private:
+    void _fillFieldsForEdit();
+    void _fillFieldsForNew();
 };
 
 #endif
+// vim: ts=2 sw=2 et
