@@ -16,33 +16,18 @@
  ***************************************************************************/
 
 // include files for Qt
-#include <qbuttongroup.h>
 #include <qcheckbox.h>
-#include <qcombobox.h>
-#include <qlabel.h>
-#include <qlayout.h>
 #include <qlineedit.h>
-#include <qpushbutton.h>
 #include <qradiobutton.h>
-#include <qstring.h>
-#include <qtable.h>
-#include <qtimer.h>
-#include <qwidget.h>
 
 // include files for KDE
 #include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
-#include <kstaticdeleter.h>
 
 // application specific includes
 #include "kst.h"
-#include "kstdatacollection.h"
-#include "kstdebug.h"
-#include "kstdoc.h"
 #include "ksteventmonitor_i.h"
-#include "ksteventmonitorentry.h"
-#include "kstrvector.h"
 #include "scalarselector.h"
 #include "vectorselector.h"
 
@@ -78,6 +63,8 @@ KstEventMonitorI::KstEventMonitorI(QWidget* parent,
           lineEditEquation, SLOT(insert(const QString&)));
   connect(_pushButtonELOGConfigure, SIGNAL(clicked()),
           KstApp::inst(), SLOT(EventELOGConfigure()));
+
+  setFixedHeight(height());
 }
 
 KstEventMonitorI::~KstEventMonitorI() {
@@ -164,11 +151,26 @@ void KstEventMonitorI::fillEvent(EventMonitorEntryPtr& event) {
   event->update();
 }
 
+
+void KstEventMonitorI::enableELOG() {
+  checkBoxELOGNotify->setEnabled(true);
+  _pushButtonELOGConfigure->setEnabled(true);
+}
+
+
+void KstEventMonitorI::disableELOG() {
+  checkBoxELOGNotify->setEnabled(false);
+  _pushButtonELOGConfigure->setEnabled(false);
+}
+
+
 bool KstEventMonitorI::new_I() {
   QString tag_name = _tagName->text();
   tag_name.replace("<New_Event>", lineEditEquation->text());
 
-  /* verify that the event name is unique */
+  //
+  // verify that the event name is unique
+  //
   if (KST::dataTagNameNotUnique(tag_name)) {
     _tagName->setFocus();
     return false;
@@ -180,7 +182,7 @@ bool KstEventMonitorI::new_I() {
   if (!event->isValid()) {
     event = 0L;
 
-    KMessageBox::sorry(0L, i18n("There is a syntax error in the equation "
+    KMessageBox::sorry(this, i18n("There is a syntax error in the equation "
           "you entered.  Please fix it."));
     return false;
   }
@@ -195,7 +197,9 @@ bool KstEventMonitorI::new_I() {
 }
 
 bool KstEventMonitorI::edit_I() {
-  /* verify that the tag name is unique */
+  //
+  // verify that the tag name is unique
+  //
   QString tag_name = _tagName->text();
   DP->writeLock();
   if (tag_name != DP->tagName() && KST::dataTagNameNotUnique(tag_name)) {
