@@ -32,7 +32,9 @@ class KstEventMonitorI : public EventMonitor {
     void update();
     bool new_I();
     bool edit_I();
-
+    void enableELOG();
+    void disableELOG();
+    
     static KstEventMonitorI *globalInstance();
 
   signals:
