@@ -16,31 +16,18 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <assert.h>
-#include <ctype.h>
-#include <math.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
-
 // include files for Qt
 #include <qstylesheet.h>
-#include <qmessagebox.h>
-#include <qprocess.h>
-#include <qtimer.h>
 
 // include files for KDE
 #include <kdebug.h>
 #include <klocale.h>
-#include <kstaticdeleter.h>
 
 // application specific includes
 #include "enodes.h"
 #include "emailthread.h"
 #include "kst.h"
-#include "kstdoc.h"
 #include "kstdatacollection.h"
-#include "kstdebug.h"
 #include "ksteventmonitorentry.h"
 #include "ksteventmonitor_i.h"
 
@@ -52,12 +39,22 @@ const QString EventMonitorEntry::OUTXVECTOR = "X";
 const QString EventMonitorEntry::OUTYVECTOR = "Y";
 
 EventMonitorEntry::EventMonitorEntry(const QString &in_tag) : KstDataObject() {
+  _level = KstDebug::Warning;
+  _bLogKstDebug = true;
+  _bLogEMail = false;
+  _bLogELOG = false;
+
   commonConstructor( in_tag );
 }
 
 EventMonitorEntry::EventMonitorEntry(QDomElement &e) {
   QString strTag;
 
+  _level = KstDebug::Warning;
+  _bLogKstDebug = true;
+  _bLogEMail = false;
+  _bLogELOG = false;
+
   QDomNode n = e.firstChild();
   while( !n.isNull() ) {
     QDomElement e = n.toElement(); // try to convert the node to an element.
@@ -89,20 +86,20 @@ EventMonitorEntry::EventMonitorEntry(QDomElement &e) {
 
   commonConstructor( strTag );
 
-  update();
+  //
+  // wait for the initial upadte, as we don't want to trigger elog entries
+  //  until we are sure the document is open...
+  //
+  QTimer::singleShot( 500, this, SLOT(slotUpdate()) );
 }
 
 void EventMonitorEntry::commonConstructor(const QString &in_tag) {
   int NS = 1;
 
-  _level = KstDebug::Warning;
-  _pExpression = 0L;
-  _bLogKstDebug = true;
-  _bLogEMail = false;
-  _bLogELOG = false;
-  _bIsValid = false;
   _iNumDone = 0;
-
+  _bIsValid = false;
+  _pExpression = 0L;
+  
   _typeString = i18n("Event");
   KstObject::setTagName(in_tag);
 
@@ -117,19 +114,20 @@ void EventMonitorEntry::commonConstructor(const QString &in_tag) {
   _yVector = _outputVectors.insert(OUTYVECTOR, yv);
 }
 
-void EventMonitorEntry::save(QTextStream &ts) {
-  ts << "<event>" << endl;
-  ts << "  <tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
-  ts << "  <equation>" << QStyleSheet::escape(_strEvent) << "</equation>" << endl;
-  ts << "  <description>" << QStyleSheet::escape(_strDescription) << "</description>" << endl;
-  ts << "  <logdebug>" << QString::number(_bLogKstDebug) << "</logdebug>" << endl;
-  ts << "  <loglevel>" << QString::number(_level) << "</loglevel>" << endl;
-  ts << "  <logemail>" << QString::number(_bLogEMail) << "</logemail>" << endl;
-  ts << "  <logelog>" << QString::number(_bLogELOG) << "</logelog>" << endl;
-  ts << "  <emailRecipients>" << QStyleSheet::escape(_strEMailRecipients) << "</emailRecipients>" << endl;
-  ts << "  <emailSender>" << QStyleSheet::escape(_strEMailSender) << "</emailSender>" << endl;
-  ts << "  <smtpServer>" << QStyleSheet::escape(_strSMTPServer) << "</smtpServer>" << endl;
-  ts << "</event>" << endl;
+void EventMonitorEntry::save(QTextStream &ts, const QString& indent) {
+  QString l2 = indent + "  ";
+  ts << indent << "<event>" << endl;
+  ts << l2 << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+  ts << l2 << "<equation>" << QStyleSheet::escape(_strEvent) << "</equation>" << endl;
+  ts << l2 << "<description>" << QStyleSheet::escape(_strDescription) << "</description>" << endl;
+  ts << l2 << "<logdebug>" << QString::number(_bLogKstDebug) << "</logdebug>" << endl;
+  ts << l2 << "<loglevel>" << QString::number(_level) << "</loglevel>" << endl;
+  ts << l2 << "<logemail>" << QString::number(_bLogEMail) << "</logemail>" << endl;
+  ts << l2 << "<logelog>" << QString::number(_bLogELOG) << "</logelog>" << endl;
+  ts << l2 << "<emailRecipients>" << QStyleSheet::escape(_strEMailRecipients) << "</emailRecipients>" << endl;
+  ts << l2 << "<emailSender>" << QStyleSheet::escape(_strEMailSender) << "</emailSender>" << endl;
+  ts << l2 << "<smtpServer>" << QStyleSheet::escape(_strSMTPServer) << "</smtpServer>" << endl;
+  ts << indent << "</event>" << endl;
 }
 
 EventMonitorEntry::~EventMonitorEntry() {
@@ -139,6 +137,10 @@ EventMonitorEntry::~EventMonitorEntry() {
   _pExpression = 0L;
 }
 
+void EventMonitorEntry::slotUpdate() {
+  update();
+}
+
 KstObject::UpdateType EventMonitorEntry::update(int updateCounter) {
   KstObject::UpdateType retVal = NO_CHANGE;
   KstVectorPtr xv = *_xVector;
@@ -180,14 +182,17 @@ KstObject::UpdateType EventMonitorEntry::update(int updateCounter) {
     }
 
     if (xv && yv) {
-      bool rc = xv->resize(ns);
-      rc = yv->resize(ns) && rc;
-      if (!rc) {
-        // FIXME: handle error
-        return retVal;
+      bool rc;
+
+      rc = xv->resize(ns);
+      if (rc) {
+        pdRawValuesX = xv->value();
+      }
+
+      rc = yv->resize(ns);
+      if (rc) {
+        pdRawValuesY = yv->value();
       }
-      pdRawValuesX = xv->value();
-      pdRawValuesY = yv->value();
     }
 
     ctx.sampleCount = ns;
@@ -252,32 +257,32 @@ void EventMonitorEntry::logImmediately( ) {
   int i;
 
   if( iSize > 0 ) {
-	  QString str;
-    if( _strDescription.isEmpty( ) ) {
-      str = _strEvent;
-    } else {
-      str = _strDescription;
-    }
-
-    // FIXME: i18n completely broken here for RTL.
-    str = i18n("Event Monitor: %1: ").arg( str );
+    QString str;
+    QString strRange;
+    
     for( i=0; i<iSize; i++ ) {
       iIndex = *(_indexArray.at(i));
       if( i == 0 ) {
-        str += i18n("%1").arg( iIndex );
+        strRange.setNum( iIndex );
       } else if( !bRange && iIndex == iIndexOld+1 ) {
         bRange = true;
       } else if( bRange && iIndex != iIndexOld+1 ) {
-        str += i18n("-%1,%2").arg( iIndexOld ).arg( iIndex );
+        strRange = i18n("%1-%2,%3").arg( strRange ).arg( iIndexOld ).arg( iIndex );
         bRange = false;
       } else if( iIndex != iIndexOld+1 ) {
-        str += i18n(",%1").arg( iIndex );
+        strRange = i18n("%1,%2").arg( strRange ).arg( iIndex );
       }
       iIndexOld = iIndex;
     }
 
     if( bRange ) {
-      str += i18n("-%1").arg( iIndex );
+      strRange = i18n("%1-%2").arg( strRange ).arg( iIndex );
+    }
+
+    if( _strDescription.isEmpty() ) {
+      str = i18n("Event Monitor: %1: %2").arg( _strEvent ).arg( strRange );
+    } else {
+      str = i18n("Event Monitor: %1: %2").arg( _strDescription ).arg( strRange );
     }
 
     if( _bLogKstDebug ) {
