@@ -20,8 +20,6 @@
 
 #include <qdom.h>
 #include <qobject.h>
-#include <qstringlist.h>
-#include <qtextstream.h>
 
 class KstApp;
 
@@ -33,7 +31,7 @@ class KstExtension : public QObject {
 
     // To save state
     virtual void load(QDomElement& e);
-    virtual void save(QTextStream& ts);
+    virtual void save(QTextStream& ts, const QString& indent = QString::null);
 
     // Clear internal state
     virtual void clear();
