@@ -17,48 +17,25 @@
  ***************************************************************************/
 
 #include <assert.h>
-#include <stdlib.h>
 
 // include files for Qt
-#include <qcheckbox.h>
-#include <qcombobox.h>
-#include <qlabel.h>
-#include <qlayout.h>
 #include <qlineedit.h>
-#include <qlistview.h>
-#include <qmessagebox.h>
-#include <qobjectlist.h>
-#include <qpushbutton.h>
-#include <qspinbox.h>
-#include <qstring.h>
-#include <qstringlist.h>
-#include <qtimer.h>
 #include <qtooltip.h>
 #include <qwhatsthis.h>
-#include <qwidget.h>
 
 // include files for KDE
-#include <kcolorbutton.h>
-#include <kcolordialog.h>
 #include <kdebug.h>
-#include <klocale.h>
-#include <kmdimainfrm.h>
 #include <kmessagebox.h>
-#include <kstaticdeleter.h>
 
 // application specific includes
 #include "curveappearancewidget.h"
-#include "kst.h"
-#include "kst2dplot.h"
-#include "kstdatacollection.h"
-#include "kstdoc.h"
 #include "kstfilterdialog_i.h"
 #include "kstvcurve.h"
 #include "kstviewwindow.h"
 #include "plugincollection.h"
-#include "plugin.h"
 #include "pluginmanager.h"
 #include "scalarselector.h"
+#include "stringselector.h"
 #include "vectorselector.h"
 
 KstFilterDialogI *KstFilterDialogI::_inst = 0L;
@@ -104,16 +81,6 @@ void KstFilterDialogI::OK() {
 }
 
 
-void KstFilterDialogI::close() {
-  QDialog::close();
-}
-
-
-void KstFilterDialogI::reject() {
-  QDialog::reject();
-}
-
-
 void KstFilterDialogI::show_setCurve(const QString& curveName,
                                   const QString& plotName,
                                   const QString& window) {
@@ -125,8 +92,8 @@ void KstFilterDialogI::show_setCurve(const QString& curveName,
   if (curves.findTag(curveName) != curves.end()) {
     KstBaseCurvePtr curve = *curves.findTag(curveName);
     curve->readLock();
-    _xvector = curve->getXVTag();
-    _yvector = curve->getYVTag();
+    _xvector = curve->xVTag();
+    _yvector = curve->yVTag();
     curve->readUnlock();
   }
 
@@ -172,9 +139,7 @@ void KstFilterDialogI::updatePluginList() {
 
 
 void KstFilterDialogI::update() {
-  KstSharedPtr<Plugin> plugin;
-
-  plugin = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
+  KstSharedPtr<Plugin> plugin = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
   if (plugin) {
     const QValueList<Plugin::Data::IOValue>& itable = plugin->data()._inputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
@@ -184,6 +149,14 @@ void KstFilterDialogI::update() {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
           vs->update();
         }
+      } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+        QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+        if (field) {
+          StringSelector *ss = static_cast<StringSelector*>(field);
+          ss->update();
+        }
+      } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+        // Nothing
       } else {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
         if (field) {
@@ -200,6 +173,7 @@ void KstFilterDialogI::fixupLayout() {
   _frameWidget->resize(_frameWidget->sizeHint());
   _pluginFrame->resize(_pluginFrame->sizeHint());
   resize(sizeHint());
+  setMinimumSize(size());
   updateGeometry();
 }
 
@@ -207,6 +181,7 @@ void KstFilterDialogI::fixupLayout() {
 bool KstFilterDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
   KST::vectorList.lock().readLock();
   KST::scalarList.lock().readLock();
+  KST::stringList.lock().readLock();
   const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
@@ -217,7 +192,7 @@ bool KstFilterDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
         }
         if (plugin->inputVectors().contains((*it)._name) && plugin->inputVectors()[(*it)._name] != v) {
           plugin->inputVectors()[(*it)._name]->writeUnlock();
-        } 
+        }
         plugin->inputVectors().insert((*it)._name, v);
       } else {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "VectorSelector");
@@ -229,10 +204,33 @@ bool KstFilterDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
           }
           if (plugin->inputVectors().contains((*it)._name) && plugin->inputVectors()[(*it)._name] != v) {
             plugin->inputVectors()[(*it)._name]->writeUnlock();
-          } 
+          }
           plugin->inputVectors().insert((*it)._name, v);
         }
       }
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+      if (field) {
+        StringSelector *ss = static_cast<StringSelector*>(field);
+        KstStringPtr s = *KST::stringList.findTag(ss->selectedString());
+        if (s == *KST::stringList.end()) {
+          QString val = ss->_string->currentText();
+          KstStringPtr newString = new KstString(ss->_string->currentText(), val, true, false);
+          newString->writeLock(); // to match with plugin->writeLock()
+          if (plugin->inputStrings().contains((*it)._name) && plugin->inputStrings()[(*it)._name] != newString) {
+            plugin->inputStrings()[(*it)._name]->writeUnlock();
+          }
+          plugin->inputStrings().insert((*it)._name, newString);
+        } else {
+          s->writeLock(); // to match with plugin->writeLock()
+          if (plugin->inputStrings().contains((*it)._name) && plugin->inputStrings()[(*it)._name] != s) {
+            plugin->inputStrings()[(*it)._name]->writeUnlock();
+          }
+          plugin->inputStrings().insert((*it)._name, s);
+        }
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
     } else {
       QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
       if (field) {
@@ -247,25 +245,26 @@ bool KstFilterDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
             newScalar->writeLock(); // to match with plugin->writeLock()
             if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != newScalar) {
               plugin->inputScalars()[(*it)._name]->writeUnlock();
-            } 
+            }
             plugin->inputScalars().insert((*it)._name, newScalar);
           } else {
             s->writeLock(); // to match with plugin->writeLock()
             if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
               plugin->inputScalars()[(*it)._name]->writeUnlock();
-            } 
+            }
             plugin->inputScalars().insert((*it)._name, s);
           }
         } else {
           s->writeLock(); // to match with plugin->writeLock()
           if (plugin->inputScalars().contains((*it)._name) && plugin->inputScalars()[(*it)._name] != s) {
             plugin->inputScalars()[(*it)._name]->writeUnlock();
-          } 
+          }
           plugin->inputScalars().insert((*it)._name, s);
         }
       }
     }
   }
+  KST::stringList.lock().readUnlock();
   KST::scalarList.lock().readUnlock();
   KST::vectorList.lock().readUnlock();
 
@@ -278,7 +277,9 @@ bool KstFilterDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
 
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
     QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
-    assert(field);
+    if (!field) {
+      continue;
+    }
 
     QLineEdit *li = static_cast<QLineEdit*>(field);
 
@@ -300,8 +301,10 @@ bool KstFilterDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
           v = new KstVector;
           v->writeLock();
           plugin->outputVectors().insert((*it)._name, v);
+          KST::addVectorToList(v);
         }
         v->setTagName(nt);
+        v->setProvider(plugin.data());
       } else if (plugin->outputVectors()[(*it)._name]->tagName() != nt) {
         while (KST::vectorTagNameNotUnique(nt, false)) {
           nt += "'";
@@ -313,8 +316,10 @@ bool KstFilterDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
           v = new KstVector;
           v->writeLock();
           plugin->outputVectors().insert((*it)._name, v);
+          KST::addVectorToList(v);
         }
         v->setTagName(nt);
+        v->setProvider(plugin.data());
       }
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
@@ -329,6 +334,21 @@ bool KstFilterDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p)
         s->writeLock();
         s->setTagName(nt);
       }
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      if (!KST::vectorTagNameNotUnique(nt, false)) {
+        KstStringPtr s = plugin->outputStrings()[(*it)._name];
+        s->writeLock();
+        s->setTagName(nt);
+      } else if (plugin->outputStrings()[(*it)._name]->tagName() != nt) {
+        while (KST::vectorTagNameNotUnique(nt, false)) {
+          nt += "'";
+        }
+        KstStringPtr s = plugin->outputStrings()[(*it)._name];
+        s->writeLock();
+        s->setTagName(nt);
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
     }
   }
 
@@ -366,16 +386,18 @@ bool KstFilterDialogI::createCurve(KstPluginPtr plugin) {
 
     fit->setHasPoints(_curveAppearance->showPoints());
     fit->setHasLines(_curveAppearance->showLines());
+    fit->setHasBars(_curveAppearance->showBars());
     fit->setLineWidth(_curveAppearance->lineWidth());
     fit->setLineStyle(_curveAppearance->lineStyle());
     fit->Point.setType(_curveAppearance->pointType());
+    fit->setBarStyle(_curveAppearance->barStyle());
 
     //
     // add the curve to the plot...
     //
     KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_window));
     if (w && w->view()->findChild(_plotName)) {
-      Kst2DPlotPtr plot = dynamic_cast<Kst2DPlot*>((w->view()->findChild(_plotName)).data());
+      Kst2DPlotPtr plot = kst_cast<Kst2DPlot>(w->view()->findChild(_plotName));
       if (plot) {
         plot->addCurve(fit.data());
         w->view()->paint(P_DATA);
@@ -446,25 +468,34 @@ bool KstFilterDialogI::new_I() {
 
 
 void KstFilterDialogI::generateEntries(const QString& fixedVector, bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
-  QString scalarLabelTemplate, vectorLabelTemplate;
+  QString scalarLabelTemplate, vectorLabelTemplate, stringLabelTemplate;
 
   if (input) {
+    stringLabelTemplate = i18n("Input String - %1:");
     scalarLabelTemplate = i18n("Input Scalar - %1:");
     vectorLabelTemplate = i18n("Input Vector - %1:");
   } else {
+    stringLabelTemplate = i18n("Output String - %1:");
     scalarLabelTemplate = i18n("Output Scalar - %1:");
     vectorLabelTemplate = i18n("Output Vector - %1:");
   }
 
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
     QString labellabel;
+    bool string = false;
     bool scalar = false;
     bool fixed = false;
     switch ((*it)._type) {
+      case Plugin::Data::IOValue::PidType:
+        continue;
       case Plugin::Data::IOValue::FloatType:
         labellabel = scalarLabelTemplate.arg((*it)._name);
         scalar = true;
         break;
+      case Plugin::Data::IOValue::StringType:
+        labellabel = stringLabelTemplate.arg((*it)._name);
+        string = true;
+        break;
       case Plugin::Data::IOValue::TableType:
         if ((*it)._name == fixedVector) {
           fixed = true;
@@ -484,24 +515,44 @@ void KstFilterDialogI::generateEntries(const QString& fixedVector, bool input, i
 
     QLabel *label = new QLabel(labellabel, parent, input ? "Input label" : "Output label");
     QWidget *widget = 0L;
-
     if (input) {
       if (scalar) {
         ScalarSelector *w = new ScalarSelector(parent, (*it)._name.latin1());
         widget = w;
         connect(w->_scalar, SIGNAL(activated(const QString&)), this, SLOT(updateScalarTooltip(const QString&)));
         connect(widget, SIGNAL(newScalarCreated()), this, SIGNAL(modified()));
+        if (!(*it)._default.isEmpty()) {
+          w->_scalar->insertItem((*it)._default);
+          w->_scalar->setCurrentText((*it)._default);
+          //printf("default: |%s|\n", (*it)._default.latin1());
+        }
         KstScalarPtr p = *KST::scalarList.findTag(w->_scalar->currentText());
-        w->allowDirectEntry( true );
+        w->allowDirectEntry(true);
         if (p) {
           p->readLock();
           QToolTip::add(w->_scalar, QString::number(p->value()));
           p->readUnlock();
         }
+      } else if (string) {
+        StringSelector *w = new StringSelector(parent, (*it)._name.latin1());
+        widget = w;
+        connect(w->_string, SIGNAL(activated(const QString&)), this, SLOT(updateStringTooltip(const QString&)));
+        connect(widget, SIGNAL(newStringCreated()), this, SIGNAL(modified()));
+        if (!(*it)._default.isEmpty()) {
+          w->_string->insertItem((*it)._default);
+          w->_string->setCurrentText((*it)._default);
+        }
+        KstStringPtr p = *KST::stringList.findTag(w->_string->currentText());
+        w->allowDirectEntry(true);
+        if (p) {
+          p->readLock();
+          QToolTip::add(w->_string, p->value());
+          p->readUnlock();
+        }
       } else {
         if (fixed) {
           widget = new QLabel(parent, (*it)._name.latin1());
-          dynamic_cast<QLabel*>(widget)->setText(_yvector);
+          static_cast<QLabel*>(widget)->setText(_yvector);
         } else {
           widget = new VectorSelector(parent, (*it)._name.latin1());
           connect(widget, SIGNAL(newVectorCreated(const QString&)), this, SIGNAL(modified()));
@@ -550,7 +601,7 @@ void KstFilterDialogI::pluginChanged(int idx) {
     QGridLayout *grid = new QGridLayout(_frameWidget, 4+variables, 2);
     grid->setMargin(6);
     grid->setSpacing(5);
-    grid->addWidget(new QLabel(i18n("Plugin Name:"), _frameWidget), cnt, 0);
+    grid->addWidget(new QLabel(i18n("Plugin name:"), _frameWidget), cnt, 0);
     grid->addWidget(new QLabel(pluginData._readableName, _frameWidget), cnt, 1);
     grid->addWidget(new QLabel(i18n("Description:"), _frameWidget), ++cnt, 0);
     QLabel *lab = new QLabel(pluginData._description, _frameWidget);
@@ -592,11 +643,9 @@ void KstFilterDialogI::pluginChanged(int idx) {
 
 void KstFilterDialogI::showPluginManager() {
   PluginManager *pm = new PluginManager(this, "Plugin Manager");
-  if (pm) {
-    pm->exec();
-    delete pm;
-    updatePluginList();
-  }
+  pm->exec();
+  delete pm;
+  updatePluginList();
 }
 
 
@@ -614,6 +663,20 @@ void KstFilterDialogI::updateScalarTooltip(const QString& n) {
 }
 
 
+void KstFilterDialogI::updateStringTooltip(const QString& n) {
+  KstStringPtr s = *KST::stringList.findTag(n);
+  QWidget *w = const_cast<QWidget*>(static_cast<const QWidget*>(sender()));
+  if (s) {
+    s->readLock();
+    QToolTip::remove(w);
+    QToolTip::add(w, s->value());
+    s->readUnlock();
+  } else {
+    QToolTip::remove(w);
+  }
+}
+
+
 #include "kstfilterdialog_i.moc"
 
 // vim: ts=2 sw=2 et
