@@ -17,49 +17,26 @@
  ***************************************************************************/
 
 #include <assert.h>
-#include <stdlib.h>
 
 // include files for Qt
-#include <qcheckbox.h>
-#include <qcombobox.h>
-#include <qlabel.h>
-#include <qlayout.h>
 #include <qlineedit.h>
-#include <qlistview.h>
-#include <qmessagebox.h>
-#include <qobjectlist.h>
-#include <qpushbutton.h>
-#include <qspinbox.h>
-#include <qstring.h>
-#include <qstringlist.h>
-#include <qtimer.h>
 #include <qtooltip.h>
 #include <qwhatsthis.h>
-#include <qwidget.h>
 #include <qregexp.h>
 
 // include files for KDE
-#include <kcolorbutton.h>
-#include <kcolordialog.h>
 #include <kdebug.h>
-#include <klocale.h>
-#include <kmdimainfrm.h>
 #include <kmessagebox.h>
-#include <kstaticdeleter.h>
 
 // application specific includes
 #include "curveappearancewidget.h"
-#include "kst.h"
-#include "kst2dplot.h"
-#include "kstdatacollection.h"
-#include "kstdoc.h"
 #include "kstfitdialog_i.h"
 #include "kstvcurve.h"
 #include "kstviewwindow.h"
 #include "plugincollection.h"
-#include "plugin.h"
 #include "pluginmanager.h"
 #include "scalarselector.h"
+#include "stringselector.h"
 #include "vectorselector.h"
 
 KstFitDialogI *KstFitDialogI::_inst = 0L;
@@ -106,16 +83,6 @@ void KstFitDialogI::OK() {
 }
 
 
-void KstFitDialogI::close() {
-  QDialog::close();
-}
-
-
-void KstFitDialogI::reject() {
-  QDialog::reject();
-}
-
-
 void KstFitDialogI::show_setCurve(const QString& strCurve,
                                   const QString& strPlotName,
                                   const QString& strWindow) {
@@ -130,9 +97,9 @@ void KstFitDialogI::show_setCurve(const QString& strCurve,
   _strCurve    = strCurve;
   if (curves.findTag(strCurve) != curves.end()) {
     curve = *(curves.findTag(strCurve));
-    _xvector = curve->getXVTag();
-    _yvector = curve->getYVTag();
-    _evector = curve->getYETag();
+    _xvector = curve->xVTag();
+    _yvector = curve->yVTag();
+    _evector = curve->yETag();
   }
 
   if ( _xvector=="<None>" ) return;
@@ -185,6 +152,7 @@ void KstFitDialogI::updatePluginList() {
 void KstFitDialogI::update() {
   QString selectedVector;
   QString selectedScalar;
+  QString selectedString;
   KstSharedPtr<Plugin> pPtr = PluginCollection::self()->plugin(_pluginList[PluginCombo->currentItem()]);
   if (pPtr) {
     const QValueList<Plugin::Data::IOValue>& itable = pPtr->data()._inputs;
@@ -195,6 +163,14 @@ void KstFitDialogI::update() {
           VectorSelector *vs = static_cast<VectorSelector*>(field);
           vs->update();
         }
+      } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+        QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+        if (field) {
+          StringSelector *ss = static_cast<StringSelector*>(field);
+          ss->update();
+        }
+      } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+        // Nothing
       } else {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
         if (field) {
@@ -211,6 +187,7 @@ void KstFitDialogI::fixupLayout() {
   _frameWidget->resize(_frameWidget->sizeHint());
   _pluginFrame->resize(_pluginFrame->sizeHint());
   resize(sizeHint());
+  setMinimumSize(size());
   updateGeometry();
 }
 
@@ -220,6 +197,7 @@ bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
 
   KST::vectorList.lock().readLock();
   KST::scalarList.lock().readLock();
+  KST::stringList.lock().readLock();
   const QValueList<Plugin::Data::IOValue>& itable = p->data()._inputs;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
@@ -259,6 +237,30 @@ bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
           plugin->inputVectors().insert((*it)._name, v);
         }
       }
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      QObject *field = _frameWidget->child((*it)._name.latin1(), "StringSelector");
+      if (field) {
+        StringSelector *ss = static_cast<StringSelector*>(field);
+        KstStringPtr s = *KST::stringList.findTag(ss->selectedString());
+        if (s == *KST::stringList.end()) {
+          QString val = ss->_string->currentText();
+
+          KstStringPtr newString = new KstString(ss->_string->currentText(), val, true, false);
+          newString->writeLock(); // to match with plugin->writeLock()
+          if (plugin->inputStrings().contains((*it)._name)) {
+            plugin->inputStrings()[(*it)._name]->writeUnlock();
+          }
+          plugin->inputStrings().insert((*it)._name, newString);
+        } else {
+          s->writeLock(); // to match with plugin->writeLock()
+          if (plugin->inputStrings().contains((*it)._name)) {
+            plugin->inputStrings()[(*it)._name]->writeUnlock();
+          }
+          plugin->inputStrings().insert((*it)._name, s);
+        }
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
     } else {
       QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
       if (field) {
@@ -292,6 +294,7 @@ bool KstFitDialogI::saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
       }
     }
   }
+  KST::stringList.lock().readUnlock();
   KST::scalarList.lock().readUnlock();
   KST::vectorList.lock().readUnlock();
 
@@ -304,7 +307,9 @@ bool KstFitDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
 
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
     QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
-    assert(field);
+    if (!field) {
+      continue;
+    }
 
     QLineEdit *li = static_cast<QLineEdit*>(field);
 
@@ -319,13 +324,28 @@ bool KstFitDialogI::saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p) {
     QString nt = li->text();
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
-        plugin->outputVectors()[(*it)._name]->setTagName(nt);
+        KstVectorPtr v = plugin->outputVectors()[(*it)._name];
+        v->setTagName(nt);
+        v->setProvider(plugin.data());
       } else if (plugin->outputVectors()[(*it)._name]->tagName() != nt) {
         while (KST::vectorTagNameNotUnique(nt, false)) {
           nt += "'";
         }
-        plugin->outputVectors()[(*it)._name]->setTagName(nt);
+        KstVectorPtr v = plugin->outputVectors()[(*it)._name];
+        v->setTagName(nt);
+        v->setProvider(plugin.data());
       }
+    } else if ((*it)._type == Plugin::Data::IOValue::StringType) {
+      if (!KST::vectorTagNameNotUnique(nt, false)) {
+        plugin->outputStrings()[(*it)._name]->setTagName(nt);
+      } else if (plugin->outputStrings()[(*it)._name]->tagName() != nt) {
+        while (KST::vectorTagNameNotUnique(nt, false)) {
+          nt += "'";
+        }
+        plugin->outputStrings()[(*it)._name]->setTagName(nt);
+      }
+    } else if ((*it)._type == Plugin::Data::IOValue::PidType) {
+      // Nothing
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
         plugin->outputScalars()[(*it)._name]->setTagName(nt);
@@ -349,12 +369,11 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
   KstLabel* label;
   KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
   KstVCurvePtr fit;
-  QString c_name;
   QString strLabel;
   bool bRetVal = false;
 
   // Generate new fit name
-  c_name = KST::suggestCurveName(plugin->tagName());
+  QString c_name = KST::suggestCurveName(plugin->tagName());
 
   if (plugin->inputVectors().find("X Array") != plugin->inputVectors().end()) {
     xVector = plugin->inputVectors()["X Array"];
@@ -371,9 +390,11 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
 
       fit->setHasPoints(_curveAppearance->showPoints());
       fit->setHasLines(_curveAppearance->showLines());
+      fit->setHasBars(_curveAppearance->showBars());
       fit->setLineWidth(_curveAppearance->lineWidth());
       fit->setLineStyle(_curveAppearance->lineStyle());
       fit->Point.setType(_curveAppearance->pointType());
+      fit->setBarStyle(_curveAppearance->barStyle());
 
       //
       // create the necessary label...
@@ -382,13 +403,13 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
                  arg(_strCurve.replace( '_',  "\\_" ).remove( QRegExp( "^[C][0-9]{1,2}\\-" ) )).
                  arg(plugin->plugin()->data()._readableName);
 
-       KstVectorPtr vectorParam = plugin->outputVectors()["Parameters"];
-      if( vectorParam ) {
+      KstVectorPtr vectorParam = plugin->outputVectors()["Parameters"];
+      if (vectorParam) {
         QString strParamName;
 
-        for( int i=0; i<vectorParam->length(); i++ ) {
+        for (int i = 0; i < vectorParam->length(); i++) {
           strParamName = plugin->plugin()->parameterName(i);
-          if( !strParamName.isEmpty() ) {
+          if (!strParamName.isEmpty()) {
             if (plugin->outputScalars().contains(strParamName)) {
               KstScalarPtr scalar = plugin->outputScalars()[strParamName];
 
@@ -405,7 +426,7 @@ bool KstFitDialogI::createCurve(KstPluginPtr plugin) {
       //
       KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_strWindow));
       if (w && w->view()->findChild(_strPlotName)) {
-        plot = dynamic_cast<Kst2DPlot*>((w->view()->findChild(_strPlotName)).data());
+        plot = kst_cast<Kst2DPlot>(w->view()->findChild(_strPlotName));
         if (plot) {
           label = new KstLabel(strLabel, LxTy, 0.0, 0.05, 0.05);
           plot->addLabel(label);
@@ -432,7 +453,7 @@ bool KstFitDialogI::new_I() {
   if (tagName != newFitPluginString) {
     if (KST::dataTagNameNotUnique(tagName, true, this)) {
       _tagName->setFocus();
-      return( false );
+      return false;
     }
   }
 
@@ -492,26 +513,35 @@ bool KstFitDialogI::new_I() {
 
 
 void KstFitDialogI::generateEntries(bool isWeighted, bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table) {
-  QString scalarLabelTemplate, vectorLabelTemplate;
+  QString scalarLabelTemplate, vectorLabelTemplate, stringLabelTemplate;
   int iInputVector = 0;
 
   if (input) {
+    stringLabelTemplate = i18n("Input String - %1:");
     scalarLabelTemplate = i18n("Input Scalar - %1:");
     vectorLabelTemplate = i18n("Input Vector - %1:");
   } else {
+    stringLabelTemplate = i18n("Output String - %1:");
     scalarLabelTemplate = i18n("Output Scalar - %1:");
     vectorLabelTemplate = i18n("Output Vector - %1:");
   }
 
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = table.begin(); it != table.end(); ++it) {
     QString labellabel;
+    bool string = false;
     bool scalar = false;
     bool fixed = false;
     switch ((*it)._type) {
+      case Plugin::Data::IOValue::PidType:
+        continue;
       case Plugin::Data::IOValue::FloatType:
         labellabel = scalarLabelTemplate.arg((*it)._name);
         scalar = true;
         break;
+      case Plugin::Data::IOValue::StringType:
+        labellabel = stringLabelTemplate.arg((*it)._name);
+        string = true;
+        break;
       case Plugin::Data::IOValue::TableType:
         if (input && ( iInputVector < 2 || ( isWeighted && iInputVector < 3 ) ) ) {
           fixed = true;
@@ -538,23 +568,41 @@ void KstFitDialogI::generateEntries(bool isWeighted, bool input, int& cnt, QWidg
         widget = w;
         connect(w->_scalar, SIGNAL(activated(const QString&)), this, SLOT(updateScalarTooltip(const QString&)));
         connect(widget, SIGNAL(newScalarCreated()), this, SIGNAL(modified()));
+        if (!(*it)._default.isEmpty()) {
+          w->_scalar->insertItem((*it)._default);
+          w->_scalar->setCurrentText((*it)._default);
+        }
         KstScalarPtr p = *KST::scalarList.findTag(w->_scalar->currentText());
         w->allowDirectEntry( true );
         if (p) {
           QToolTip::add(w->_scalar, QString::number(p->value()));
         }
+      } else if (string) {
+        StringSelector *w = new StringSelector(parent, (*it)._name.latin1());
+        widget = w;
+        connect(w->_string, SIGNAL(activated(const QString&)), this, SLOT(updateStringTooltip(const QString&)));
+        connect(widget, SIGNAL(newStringCreated()), this, SIGNAL(modified()));
+        if (!(*it)._default.isEmpty()) {
+          w->_string->insertItem((*it)._default);
+          w->_string->setCurrentText((*it)._default);
+        }
+        KstStringPtr p = *KST::stringList.findTag(w->_string->currentText());
+        w->allowDirectEntry( true );
+        if (p) {
+          QToolTip::add(w->_string, p->value());
+        }
       } else {
         if (fixed) {
           widget = new QLabel(parent, (*it)._name.latin1());
           switch (iInputVector) {
             case 0:
-              dynamic_cast<QLabel*>(widget)->setText(_xvector);
+              static_cast<QLabel*>(widget)->setText(_xvector);
               break;
             case 1:
-              dynamic_cast<QLabel*>(widget)->setText(_yvector);
+              static_cast<QLabel*>(widget)->setText(_yvector);
               break;
             case 2:
-              dynamic_cast<QLabel*>(widget)->setText(_evector);
+              static_cast<QLabel*>(widget)->setText(_evector);
               break;
           }
           iInputVector++;
@@ -606,7 +654,7 @@ void KstFitDialogI::pluginChanged(int idx) {
     QGridLayout *grid = new QGridLayout(_frameWidget, 4+variables, 2);
     grid->setMargin(6);
     grid->setSpacing(5);
-    grid->addWidget(new QLabel(i18n("Plugin Name:"), _frameWidget), cnt, 0);
+    grid->addWidget(new QLabel(i18n("Plugin name:"), _frameWidget), cnt, 0);
     grid->addWidget(new QLabel(pluginData._readableName, _frameWidget), cnt, 1);
     grid->addWidget(new QLabel(i18n("Description:"), _frameWidget), ++cnt, 0);
     QLabel *lab = new QLabel(pluginData._description, _frameWidget);
@@ -648,11 +696,9 @@ void KstFitDialogI::pluginChanged(int idx) {
 
 void KstFitDialogI::showPluginManager() {
   PluginManager *pm = new PluginManager(this, "Plugin Manager");
-  if (pm) {
-    pm->exec();
-    delete pm;
-    updatePluginList();
-  }
+  pm->exec();
+  delete pm;
+  updatePluginList();
 }
 
 
@@ -668,6 +714,18 @@ void KstFitDialogI::updateScalarTooltip(const QString& n) {
 }
 
 
+void KstFitDialogI::updateStringTooltip(const QString& n) {
+  KstStringPtr s = *KST::stringList.findTag(n);
+  QWidget *w = const_cast<QWidget*>(static_cast<const QWidget*>(sender()));
+  if (s) {
+    QToolTip::remove(w);
+    QToolTip::add(w, s->value());
+  } else {
+    QToolTip::remove(w);
+  }
+}
+
+
 #include "kstfitdialog_i.moc"
 
 // vim: ts=2 sw=2 et
