@@ -18,7 +18,6 @@
 #ifndef KSTFITDIALOGI_H
 #define KSTFITDIALOGI_H
 
-#include <qstringlist.h>
 #include "fitdialog.h"
 #include "kstplugin.h"
 
@@ -26,43 +25,42 @@ class KstDoc;
 
 class KstFitDialogI : public KstFitDialog {
   Q_OBJECT
-public:
-  KstFitDialogI(QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
-  virtual ~KstFitDialogI();
+  public:
+    KstFitDialogI(QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
+    virtual ~KstFitDialogI();
 
-  static KstFitDialogI *globalInstance();
+    static KstFitDialogI *globalInstance();
 
-public slots:
-  void update();
-  void show_setCurve(const QString& strCurve, const QString& strPlotName, const QString& strWindow);
-  bool new_I();
-  void updatePluginList();
-  void OK();
-  void Init();
-  void close();
-  void reject();
+  public slots:
+    void update();
+    void show_setCurve(const QString& strCurve, const QString& strPlotName, const QString& strWindow);
+    bool new_I();
+    void updatePluginList();
+    void OK();
+    void Init();
 
-private slots:
-  void pluginChanged(int);
-  void showPluginManager();
-  void fixupLayout();
-  void updateScalarTooltip(const QString& n);
+  private slots:
+    void pluginChanged(int);
+    void showPluginManager();
+    void fixupLayout();
+    void updateScalarTooltip(const QString& n);
+    void updateStringTooltip(const QString& n);
 
-private:
-  QStringList _pluginList;
-  QWidget *_frameWidget;
-  QString _xvector;
-  QString _yvector;
-  QString _evector;
-  QString _strWindow;
-  QString _strPlotName;
-  QString _strCurve;
-  static KstFitDialogI* _inst;
+  private:
+    QStringList _pluginList;
+    QWidget *_frameWidget;
+    QString _xvector;
+    QString _yvector;
+    QString _evector;
+    QString _strWindow;
+    QString _strPlotName;
+    QString _strCurve;
+    static KstFitDialogI* _inst;
 
-  bool createCurve(KstPluginPtr plugin);
-  bool saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p);
-  bool saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p);
-  void generateEntries(bool isWeighted, bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table);
+    bool createCurve(KstPluginPtr plugin);
+    bool saveInputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p);
+    bool saveOutputs(KstPluginPtr plugin, KstSharedPtr<Plugin> p);
+    void generateEntries(bool isWeighted, bool input, int& cnt, QWidget *parent, QGridLayout *grid, const QValueList<Plugin::Data::IOValue>& table);
 };
 
 #endif
