@@ -16,11 +16,8 @@
  ***************************************************************************/
 
 #include <qpainter.h>
-#include <qpalette.h>
 #include <qtable.h>
 #include "kstfittable.h"
-#include "kstrvector.h"
-#include "kstdatacollection.h"
 
 KstFitTable::KstFitTable( QWidget * parent, const char * name ) : QTable( parent, name ) {
   _pdParams = NULL;
