@@ -1,5 +1,5 @@
 /***************************************************************************
-                          ksthistogram.cpp: Power Spectra for KST
+                          ksthistogram.cpp: Histogram for KST
                              -------------------
     begin                : July 2002
     copyright            : (C) 2002 by C. Barth Netterfield
@@ -16,7 +16,6 @@
  ***************************************************************************/
 
 /** A class for handling histograms for kst
-
  *@author C. Barth Netterfield
  */
 
@@ -24,40 +23,36 @@
 #include <stdlib.h>
 #include <math.h>
 
-#include "kstdoc.h"
+#include <qstylesheet.h>
+
+#include <kglobal.h>
+#include <klocale.h>
+
 #include "kstdatacollection.h"
 #include "ksthsdialog_i.h"
 
-#include <klocale.h>
+static const QString& BINS = KGlobal::staticQString("B");
+static const QString& HIST = KGlobal::staticQString("H");
 
-static const QString BINS = "B";
-static const QString HIST = "H";
 KstHistogram::KstHistogram(const QString &in_tag, KstVectorPtr in_V,
                            double xmin_in, double xmax_in,
                            int in_n_bins,
-                           KstHsNormType in_norm_mode,
-                           const QColor &in_color)
-: KstBaseCurve() {
-  setHasPoints(false);
-  setHasLines(true);
-  setLineWidth(0);
-  setLineStyle(0);
+                           KstHsNormType in_norm_mode)
+: KstDataObject() {
+  setRealTimeAutoBin(false);
 
-  commonConstructor(in_tag, in_V, xmin_in, xmax_in, in_n_bins, in_norm_mode,
-                    in_color);
+  commonConstructor(in_tag, in_V, xmin_in, xmax_in, in_n_bins, in_norm_mode);
 }
 
 KstHistogram::KstHistogram(QDomElement &e)
-: KstBaseCurve(e) {
+: KstDataObject(e) {
   QString in_tag;
   KstVectorPtr in_V;
-  QColor in_color("magenta");
   double xmax_in=1, xmin_in=-1;
   int in_n_bins = 10;
   KstHsNormType in_norm_mode;
 
-  setHasPoints(false);
-  setHasLines(true);
+  setRealTimeAutoBin(false);
 
   in_norm_mode = KST_HS_NUMBER;
   /* parse the DOM tree */
@@ -74,14 +69,6 @@ KstHistogram::KstHistogram(QDomElement &e)
           in_V = *it;
         }
         KST::vectorList.lock().readUnlock();
-      } else if (e.tagName() == "color") {
-        in_color.setNamedColor(e.text());
-      } else if (e.tagName() == "hasLines") {
-        HasLines = (e.text() != "0");
-      } else if (e.tagName() == "hasPoints") {
-        HasPoints = (e.text() != "0");
-      } else if (e.tagName() == "pointType") {
-        Point.setType(e.text().toInt());
       } else if (e.tagName() == "NormMode") {
         if (e.text()=="NUMBER") in_norm_mode = KST_HS_NUMBER;
         else if (e.text()=="PERCENT") in_norm_mode = KST_HS_PERCENT;
@@ -93,13 +80,14 @@ KstHistogram::KstHistogram(QDomElement &e)
         xmax_in = e.text().toDouble();
       } else if (e.tagName() == "numBins") {
         in_n_bins = e.text().toInt();
+      } else if (e.tagName() == "realtimeautobin") {
+        _realTimeAutoBin = (e.text() != "0");
       }
     }
     n = n.nextSibling();
   }
 
-  commonConstructor(in_tag, in_V, xmin_in, xmax_in, in_n_bins, in_norm_mode,
-                    in_color);
+  commonConstructor(in_tag, in_V, xmin_in, xmax_in, in_n_bins, in_norm_mode);
 
 }
 
@@ -108,10 +96,8 @@ void KstHistogram::commonConstructor(const QString &in_tag, KstVectorPtr in_V,
                                      double xmin_in,
                                      double xmax_in,
                                      int in_n_bins,
-                                     KstHsNormType in_norm_mode,
-                                     const QColor &in_color) {
+                                     KstHsNormType in_norm_mode) {
   _typeString = i18n("Histogram");
-  NumUsed = 0;
 
   NormMode = in_norm_mode;
 
@@ -136,8 +122,6 @@ void KstHistogram::commonConstructor(const QString &in_tag, KstVectorPtr in_V,
   }
   Bins = new unsigned long[in_n_bins];
 
-  Color = in_color;
-
   KstVectorPtr iv = new KstVector(in_tag+"-bins", NBins);
   KST::addVectorToList(iv);
   _bVector = _outputVectors.insert(BINS, iv);
@@ -163,6 +147,7 @@ KstHistogram::~KstHistogram() {
 KstObject::UpdateType KstHistogram::update(int update_counter) {
   int i_bin, i_pt, ns;
   double y=0;
+  double MaxY;
 
   if (KstObject::checkUpdateCounter(update_counter))
     return NO_CHANGE;
@@ -171,6 +156,15 @@ KstObject::UpdateType KstHistogram::update(int update_counter) {
     V->update(update_counter);
   }
 
+  //do auto-binning if necessary
+  if (_realTimeAutoBin) {
+    int temp_NBins;
+    double temp_xMin, temp_xMax;
+    KstHistogram::AutoBin(V, &temp_NBins, &temp_xMax, &temp_xMin);
+    setNBins(temp_NBins);
+    setXRange(temp_xMin, temp_xMax);
+  }
+
   NS = 3*NBins+1;
   W = (MaxX - MinX)/double(NBins);
 
@@ -187,19 +181,13 @@ KstObject::UpdateType KstHistogram::update(int update_counter) {
     }
   }
 
-  MaxY = MinY = MeanY = 0;
-  MinPosY = ns;
+  MaxY = 0;
 
   for (i_bin=0; i_bin<NBins; i_bin++) {
     y=Bins[i_bin];
     if (y>MaxY) MaxY = y;
-    if ((y>0) && (y<MinPosY)) MinPosY = y;
-    MeanY += y;
   }
 
-  if (NBins>0) MeanY/=NBins;
-  else MeanY = 0;
-
   switch (NormMode) {
       case KST_HS_NUMBER:
         Normalization = 1.0;
@@ -220,14 +208,6 @@ KstObject::UpdateType KstHistogram::update(int update_counter) {
         Normalization = 1.0;
         break;
   }
-  MaxY *=Normalization;
-  MinPosY *=Normalization;
-  MeanY *= Normalization;
-
-  MeanX = (MaxX + MinX)/2.0;
-  if (MinX>0) MinPosX = MinX;
-  else if (MaxX>0) MinPosX = W/5.0;
-  else MinPosX = 1.0;
 
   double *bins =  (*_bVector)->value();
   double *hist =  (*_hVector)->value();
@@ -240,7 +220,7 @@ KstObject::UpdateType KstHistogram::update(int update_counter) {
   (*_bVector)->update(update_counter);
   (*_hVector)->update(update_counter);
 
-  return (UPDATE);
+  return UPDATE;
 }
 
 void KstHistogram::point(int i, double &x, double &y) {
@@ -254,7 +234,7 @@ void KstHistogram::point(int i, double &x, double &y) {
   }
 }
 
-int KstHistogram::getBins() const {
+int KstHistogram::nBins() const {
   return NBins;
 }
 
@@ -273,7 +253,6 @@ void KstHistogram::setXRange(double xmin_in, double xmax_in) {
 }
 
 void KstHistogram::setNBins(int in_n_bins) {
-
   delete[] Bins;
 
   NBins = in_n_bins;
@@ -287,7 +266,7 @@ void KstHistogram::setNBins(int in_n_bins) {
 
 }
 
-QString KstHistogram::getVTag() const {
+QString KstHistogram::vTag() const {
   return V->tagName();
 }
 
@@ -295,25 +274,25 @@ void KstHistogram::setVector(KstVectorPtr new_v) {
   V = new_v;
 }
 
-QString KstHistogram::getYLabel() const {
+QString KstHistogram::yLabel() const {
   switch (NormMode) {
-      case KST_HS_NUMBER:
-        return i18n("Number in Bin");
-        break;
-      case KST_HS_PERCENT:
-        return i18n("Percent in Bin");
-        break;
-      case KST_HS_FRACTION:
-        return i18n("Fraction in Bin");
-        break;
-      case KST_HS_MAX_ONE:
-        return i18n("Histogram");
-        break;
+    case KST_HS_NUMBER:
+      return i18n("Number in Bin");
+      break;
+    case KST_HS_PERCENT:
+      return i18n("Percent in Bin");
+      break;
+    case KST_HS_FRACTION:
+      return i18n("Fraction in Bin");
+      break;
+    case KST_HS_MAX_ONE:
+      return i18n("Histogram");
+      break;
   }
   return i18n("Histogram");
 }
 
-QString KstHistogram::getXLabel() const {
+QString KstHistogram::xLabel() const {
   return V->label();
 }
 
@@ -321,50 +300,49 @@ KstCurveType KstHistogram::type() const {
   return KST_HISTOGRAM;
 }
 
-void KstHistogram::save(QTextStream &ts) {
-  ts << " <histogram>" << endl;
-  ts << "  <tag>" << tagName() << "</tag>" << endl;
-  ts << "  <vectag>" << V->tagName() << "</vectag>" << endl;
+void KstHistogram::save(QTextStream &ts, const QString& indent) {
+  // FIXME: clean this up - all lower case nodes, maybe save points in the
+  // point class itself, etc
+  QString l2 = indent + "  ";
+  ts << indent << "<histogram>" << endl;
+  ts << l2 << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+  ts << l2 << "<vectag>" << QStyleSheet::escape(V->tagName()) << "</vectag>" << endl;
 
-  ts << "  <color>" << Color.name() << "</color>" << endl;
-  ts << "  <hasLines>" << HasLines << "</hasLines>" << endl;
-  ts << "  <hasPoints>" << HasPoints << "</hasPoints>" << endl;
-  ts << "  <pointType>" << Point.getType() << "</pointType>" << endl;
-
-  ts << "  <numBins>"  << NBins << "</numBins>" << endl;
-  ts << "  <minX>" << MinX << "</minX>" << endl;
-  ts << "  <maxX>" << MaxX << "</maxX>" << endl;
+  ts << l2 << "<numBins>"  << NBins << "</numBins>" << endl;
+  ts << l2 << "<realtimeautobin>" << _realTimeAutoBin << "</realtimeautobin>" << endl;
+  ts << l2 << "<minX>" << MinX << "</minX>" << endl;
+  ts << l2 << "<maxX>" << MaxX << "</maxX>" << endl;
   switch (NormMode) {
-  case KST_HS_NUMBER:
-    ts << "  <NormMode>NUMBER</NormMode>" << endl;
-    break;
-  case KST_HS_PERCENT:
-    ts << "  <NormMode>PERCENT</NormMode>" << endl;
-    break;
-  case KST_HS_FRACTION:
-    ts << "  <NormMode>FRACTION</NormMode>" << endl;
-    break;
-  case KST_HS_MAX_ONE:
-    ts << "  <NormMode>MAX_ONE</NormMode>" << endl;
-    break;
+    case KST_HS_NUMBER:
+      ts << l2 << "<NormMode>NUMBER</NormMode>" << endl;
+      break;
+    case KST_HS_PERCENT:
+      ts << l2 << "<NormMode>PERCENT</NormMode>" << endl;
+      break;
+    case KST_HS_FRACTION:
+      ts << l2 << "<NormMode>FRACTION</NormMode>" << endl;
+      break;
+    case KST_HS_MAX_ONE:
+      ts << l2 << "<NormMode>MAX_ONE</NormMode>" << endl;
+      break;
   }
-  ts << " </histogram>" << endl;
+  ts << indent << "</histogram>" << endl;
 }
 
-double KstHistogram::getVMax() const {
+double KstHistogram::vMax() const {
   return V->max();
 }
 
-double KstHistogram::getVMin() const {
+double KstHistogram::vMin() const {
   return V->min();
 }
 
-int KstHistogram::getVNumSamples() const {
+int KstHistogram::vNumSamples() const {
   return V->sampleCount();
 }
 
 QString KstHistogram::propertyString() const {
-  return i18n("Histogram: %1").arg(getVTag());
+  return i18n("Histogram: %1").arg(vTag());
 }
 
 void KstHistogram::_showDialog() {
@@ -403,4 +381,13 @@ bool KstHistogram::slaveVectorsUsed() const {
   return true;
 }
 
+void KstHistogram::setRealTimeAutoBin(bool autoBin) {
+  _realTimeAutoBin = autoBin;
+}
+
+bool KstHistogram::realTimeAutoBin() {
+  return _realTimeAutoBin;
+}
+
 // vim: ts=2 sw=2 et
+
