@@ -22,46 +22,42 @@
 #ifndef KSTHISTOGRAM_H
 #define KSTHISTOGRAM_H
 
-#include "kstpoint.h"
 #include "kstbasecurve.h"
-#include <qstring.h>
-#include <qcolor.h>
 
-typedef enum {KST_HS_NUMBER, KST_HS_PERCENT, KST_HS_FRACTION, KST_HS_MAX_ONE} KstHsNormType;
+enum KstHsNormType { KST_HS_NUMBER, KST_HS_PERCENT, KST_HS_FRACTION, KST_HS_MAX_ONE};
 
-class KstHistogram: public KstBaseCurve {
+class KstHistogram: public KstDataObject {
 public:
   KstHistogram(const QString &in_tag, KstVectorPtr in_V,
                double xmin_in, double xmax_in,
                int in_n_bins,
-               KstHsNormType new_norm_in,
-               const QColor &in_color);
+               KstHsNormType new_norm_in);
   KstHistogram(QDomElement &e);
   virtual ~KstHistogram();
 
   virtual UpdateType update(int update_counter = -1);
 
   virtual void point(int i, double &x1, double &y1);
-  virtual void save(QTextStream &ts);
+  virtual void save(QTextStream &ts, const QString& indent = QString::null);
   virtual QString propertyString() const;
 
-  int getBins() const;
+  int nBins() const;
   void setNBins(int in_n_bins);
 
   void setXRange(double xmin_in, double xmax_in);
 
-  QString getVTag() const;
+  QString vTag() const;
 
   void setVector(KstVectorPtr);
 
-  virtual QString getYLabel() const;
-  virtual QString getXLabel() const;
+  virtual QString yLabel() const;
+  virtual QString xLabel() const;
 
   virtual KstCurveType type() const;
 
-  double getVMax() const;
-  double getVMin() const;
-  int getVNumSamples() const;
+  double vMax() const;
+  double vMin() const;
+  int vNumSamples() const;
 
   bool isNormNum()        const { return NormMode == KST_HS_NUMBER; }
   void setIsNormNum()           { NormMode = KST_HS_NUMBER; }
@@ -78,10 +74,19 @@ public:
 
   virtual bool slaveVectorsUsed() const;
 
-  virtual QString getXVTag() const {return (*_bVector)->tagName();}
-  virtual QString getYVTag() const {return (*_hVector)->tagName();}
+  virtual QString xVTag() const {return (*_bVector)->tagName();}
+  virtual QString yVTag() const {return (*_hVector)->tagName();}
+
+  void setRealTimeAutoBin(bool autobin);
+  bool realTimeAutoBin();
+
+  KstVectorPtr vX() const { return *_bVector; }
+  KstVectorPtr vY() const { return *_hVector; }
+
 
 private:
+  double MaxX, MinX;
+  int NS;
   KstVectorPtr V;
 
   int NBins;
@@ -90,11 +95,13 @@ private:
 
   double W;
 
+  //real-time auto bin on every update
+  bool _realTimeAutoBin;
+
   void commonConstructor(const QString &in_tag, KstVectorPtr in_V,
                          double xmin_in, double xmax_in,
                          int in_n_bins,
-                         KstHsNormType in_norm,
-                         const QColor &in_color);
+                         KstHsNormType in_norm);
 
   KstHsNormType NormMode;
 
