@@ -18,32 +18,22 @@
 #include <assert.h>
 
 // include files for Qt
-#include <qfile.h>
 #include <qfileinfo.h>
 #include <qregexp.h>
 
 // include files for KDE
-#include <kmdimainfrm.h>
-#include <kprinter.h>
 
 // application specific includes
 #include "kst.h"
-#include "kst2dplot.h"
 #include "kstcolorsequence.h"
-#include "kstdatacollection.h"
-#include "kstdefaultnames.h"
 #include "kstdoc.h"
-#include "kstequationcurve.h"
-#include "ksteventmonitorentry.h"
+#include "kstequation.h"
 #include "ksthistogram.h"
 #include "kstiface_impl.h"
-#include "kstimage.h"
-#include "kstmatrix.h"
 #include "kstplugin.h"
-#include "kstpsdcurve.h"
-#include "kstscalar.h"
+#include "kstpsd.h"
+#include "kstrvector.h"
 #include "kstvcurve.h"
-#include "kstvector.h"
 #include "kstviewwindow.h"
 #include "plugincollection.h"
 
@@ -62,6 +52,12 @@ void KstIfaceImpl::showDataManager() {
 }
 
 
+QStringList KstIfaceImpl::stringList() {
+  KstReadLocker ml(&KST::stringList.lock());
+  return KST::stringList.tagNames();
+}
+
+
 QStringList KstIfaceImpl::scalarList() {
   KstReadLocker ml(&KST::scalarList.lock());
   return KST::scalarList.tagNames();
@@ -235,20 +231,19 @@ bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const
     etag += '\'';
   }
 
-  KstEquationCurvePtr eq;
-
-  eq = new KstEquationCurve(etag, equation, start, end, numSamples,
-                            color.isValid() ? color : QColor("darkBlue"));
+  KstEquationPtr eq = new KstEquation(etag, equation, start, end, numSamples);
 
   if (!eq->isValid()) {
     return false;
   }
 
+  KstVCurvePtr vc = new KstVCurve(etag + "-curve", eq->vX(), eq->vY(), 0L, 0L, color.isValid() ? color : KstColorSequence::next(plot->Curves,plot->backgroundColor()));
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(eq));
+  KST::dataObjectList.append(KstDataObjectPtr(vc));
   KST::dataObjectList.lock().writeUnlock();
 
-  plot->Curves.append(KstBaseCurvePtr(eq));
+  plot->Curves.append(KstBaseCurvePtr(vc));
 
   _doc->forceUpdate();
   _doc->setModified();
@@ -258,12 +253,12 @@ bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const
 
 
 bool KstIfaceImpl::plotEquation(double start, double end, int numSamples, const QString& equation, const QString& plotName) {
-  return plotEquation(start, end, numSamples, equation, plotName, KstColorSequence::next());
+  return plotEquation(start, end, numSamples, equation, plotName, QColor());
 }
 
 
 bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation, const QString& plotName) {
-  return plotEquation(xvector, equation, plotName, KstColorSequence::next());
+  return plotEquation(xvector, equation, plotName, QColor());
 }
 
 
@@ -331,20 +326,19 @@ bool KstIfaceImpl::plotEquation(const QString& xvector, const QString& equation,
     etag += '\'';
   }
 
-  KstEquationCurvePtr eq;
-
-  eq = new KstEquationCurve(etag, equation, v, true,
-                            color.isValid() ? color : QColor("darkBlue"));
+  KstEquationPtr eq = new KstEquation(etag, equation, v, true);
 
   if (!eq->isValid()) {
     return false;
   }
 
+  KstVCurvePtr vc = new KstVCurve(etag + "-curve", eq->vX(), eq->vY(), 0L, 0L, color.isValid() ? color : KstColorSequence::next(plot->Curves,plot->backgroundColor()));
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(eq));
+  KST::dataObjectList.append(KstDataObjectPtr(vc));
   KST::dataObjectList.lock().writeUnlock();
 
-  plot->Curves.append(KstBaseCurvePtr(eq));
+  plot->Curves.append(KstBaseCurvePtr(vc));
   
   _doc->forceUpdate();
   _doc->setModified();
@@ -487,6 +481,33 @@ int KstIfaceImpl::vectorSize(const QString& name) {
 }
 
 
+bool KstIfaceImpl::setString(const QString& name, const QString& value) {
+  KstReadLocker ml(&KST::stringList.lock());
+  KstStringList::Iterator it = KST::stringList.findTag(name);
+
+  if (it == KST::stringList.end()) {
+    return false;
+  }
+
+  KstWriteLocker l2(*it);
+  *(*it) = value;
+  return true;
+}
+
+
+const QString& KstIfaceImpl::string(const QString& name) {
+  KstReadLocker ml(&KST::stringList.lock());
+  KstStringList::Iterator it = KST::stringList.findTag(name);
+
+  if (it != KST::stringList.end()) {
+    KstReadLocker ml2(*it);
+    return (*it)->value();
+  }
+
+  return QString::null;
+}
+
+
 double KstIfaceImpl::vector(const QString& name, int index) {
   KstReadLocker ml(&KST::vectorList.lock());
   KstVectorList::Iterator it = KST::vectorList.findTag(name);
@@ -618,7 +639,7 @@ bool KstIfaceImpl::deletePlot(const QString& window, const QString& name) {
     KstViewObjectList objects = pTLV->findChildrenType<KstViewObject>(true);
 
     for (it = objects.begin(); it != objects.end(); ++it) {
-      KstViewObjectPtr object = dynamic_cast<KstViewObject*>((*it).data());
+      KstViewObjectPtr object = kst_cast<KstViewObject>(*it);
       if (object && object->tagName() == name) {
         pTLV->removeChild(object, true);
 
@@ -657,7 +678,7 @@ bool KstIfaceImpl::addCurveToPlot(KMdiChildView *win, const QString& plot, const
   KstViewWindow *w = dynamic_cast<KstViewWindow*>(win);
 
   if (w) {
-    KstTopLevelViewPtr view = dynamic_cast<KstTopLevelView*>(w->view().data());
+    KstTopLevelViewPtr view = kst_cast<KstTopLevelView>(w->view());
     if (view) {
       Kst2DPlotList plots = view->findChildrenType<Kst2DPlot>(true);
       if (plots.findTag(plot) != plots.end()) {
@@ -702,7 +723,7 @@ bool KstIfaceImpl::removeCurveFromPlot(KMdiChildView *win, const QString& plot,
   KstViewWindow *w = dynamic_cast<KstViewWindow*>(win);
 
   if (w) {
-    KstTopLevelViewPtr view = dynamic_cast<KstTopLevelView*>(w->view().data());
+    KstTopLevelViewPtr view = kst_cast<KstTopLevelView>(w->view());
     if (view) {
       Kst2DPlotList plots = view->findChildrenType<Kst2DPlot>(true);
       if (plots.findTag(plot) != plots.end()) {
@@ -835,7 +856,7 @@ void KstIfaceImpl::reloadVector(const QString& vector) {
   KstVectorList::Iterator v = KST::vectorList.findTag(vector);
   if (v != KST::vectorList.end()) {
     (*v)->writeLock();
-    KstRVector *r = dynamic_cast<KstRVector*>((*v).data());
+    KstRVectorPtr r = kst_cast<KstRVector>(*v);
     if (r) {
       r->reload();
     }
@@ -1032,7 +1053,6 @@ bool KstIfaceImpl::positionWindow(int topleftX, int topleftY, int width, int hei
 }
 
 bool KstIfaceImpl::maximizePlot(const QString& plotName) {
-  //find the plot
   KstApp *app = KstApp::inst();
   KMdiIterator<KMdiChildView*> *iter = app->createIterator();
   while (iter->currentItem()) {
@@ -1040,10 +1060,52 @@ bool KstIfaceImpl::maximizePlot(const QString& plotName) {
     KstViewWindow *viewwindow = dynamic_cast<KstViewWindow*>(childview);
     if (viewwindow) {
       Kst2DPlotList plotlist = viewwindow->view()->findChildrenType<Kst2DPlot>(false);
-      Kst2DPlotList::Iterator plot_iter=plotlist.findTag(plotName);
+      Kst2DPlotList::Iterator plot_iter = plotlist.findTag(plotName);
       if (plot_iter != plotlist.end()) {
         app->deleteIterator(iter);
-        (*plot_iter)->zoom(true);  //zoom it
+        (*plot_iter)->zoom(true);
+        return true;
+      }
+    }
+    iter->next();
+  }
+  app->deleteIterator(iter);
+  return false;
+}
+
+bool KstIfaceImpl::unmaximizePlot(const QString& plotName) {
+  KstApp *app = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *iter = app->createIterator();
+  while (iter->currentItem()) {
+    KMdiChildView *childview = iter->currentItem();
+    KstViewWindow *viewwindow = dynamic_cast<KstViewWindow*>(childview);
+    if (viewwindow) {
+      Kst2DPlotList plotlist = viewwindow->view()->findChildrenType<Kst2DPlot>(false);
+      Kst2DPlotList::Iterator plot_iter = plotlist.findTag(plotName);
+      if (plot_iter != plotlist.end()) {
+        app->deleteIterator(iter);
+        (*plot_iter)->zoom(false);
+        return true;
+      }
+    }
+    iter->next();
+  }
+  app->deleteIterator(iter);
+  return false;
+}
+
+bool KstIfaceImpl::toggleMaximizePlot(const QString& plotName) {
+  KstApp *app = KstApp::inst();
+  KMdiIterator<KMdiChildView*> *iter = app->createIterator();
+  while (iter->currentItem()) {
+    KMdiChildView *childview = iter->currentItem();
+    KstViewWindow *viewwindow = dynamic_cast<KstViewWindow*>(childview);
+    if (viewwindow) {
+      Kst2DPlotList plotlist = viewwindow->view()->findChildrenType<Kst2DPlot>(false);
+      Kst2DPlotList::Iterator plot_iter = plotlist.findTag(plotName);
+      if (plot_iter != plotlist.end()) {
+        app->deleteIterator(iter);
+        (*plot_iter)->zoomToggle();
         return true;
       }
     }
@@ -1082,11 +1144,11 @@ bool KstIfaceImpl::setPlotAxes(const QString& plotName,
 
 QString KstIfaceImpl::createHistogram(const QString& name,
 			       const QString& vector,
-  			       double min,
+             double min,
 			       double max,
 			       int numBins,
 			       int normalizationType,
-                               const QColor& colour) {
+             const QColor& color) {
 
   //get the vector
   KST::vectorList.lock().readLock();
@@ -1138,7 +1200,7 @@ QString KstIfaceImpl::createHistogram(const QString& name,
   }
 
   KstHistogramPtr histogram = new KstHistogram(htag, *iter, min, max, numBins,
-                               normtype, colour.isValid() ? colour : QColor("darkBlue"));
+                               normtype);
 
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(histogram));
@@ -1152,34 +1214,33 @@ QString KstIfaceImpl::createHistogram(const QString& name,
 
 QString KstIfaceImpl::createPowerSpectrum(const QString & name,
 				   const QString& vector,
-  				   bool appodize,
+  				 bool appodize,
 				   bool removeMean,
 				   int fftLength,
 				   const QString& rateUnits,
 				   double sampleRate,
 				   const QString& vectorUnits,
-                                   const QColor& colour) {
+           const QColor& color) {
   //get the vector
   KST::vectorList.lock().readLock();
   KstVectorList::Iterator iter = KST::vectorList.findTag(vector);
   KST::vectorList.lock().readUnlock();
 
   if (iter == KST::vectorList.end()) {
-    return QString();
+    return QString::null;
   }
 
   //suggest a name if not supplied
   QString pstag_end;
   if (name.isEmpty())
-    pstag_end = QString(vector);
+    pstag_end = vector;
   else
-    pstag_end = QString(name);
+    pstag_end = name;
 
   //count number of power spectra and make a unique name
-  KstPSDCurveList pslist = kstObjectSubList<KstDataObject,KstPSDCurve>(KST::dataObjectList);
+  KstPSDList pslist = kstObjectSubList<KstDataObject,KstPSD>(KST::dataObjectList);
   int i = pslist.count() + 1;
-  QString stringnum;
-  stringnum = stringnum.setNum(i);
+  QString stringnum = QString::number(i);
 
   QString pstag = "C" + stringnum + "-PSD_" + pstag_end;
 
@@ -1188,11 +1249,15 @@ QString KstIfaceImpl::createPowerSpectrum(const QString & name,
     pstag = "C" + stringnum + "-PSD_" + pstag_end;
   }
 
-  KstPSDCurvePtr powerspectrum = new KstPSDCurve(pstag, *iter, sampleRate, true, fftLength,
-                               appodize, removeMean, vectorUnits, rateUnits, colour.isValid() ? colour : QColor("darkBlue"));
+  KstPSDPtr powerspectrum = new KstPSD(pstag, *iter, sampleRate, true, fftLength,
+                               appodize, removeMean, vectorUnits, rateUnits);
+  
+  // FIXME: should we be plotting this?  Otherwise, remove color
+  //KstVCurvePtr vc = new KstVCurve(pstag + "-curve", *iter, powerspectrum->vY(), 0L, 0L, color.isValid() ? color : QColor("darkBlue"));
 
   KST::dataObjectList.lock().writeLock();
   KST::dataObjectList.append(KstDataObjectPtr(powerspectrum));
+  //KST::dataObjectList.append(KstDataObjectPtr(vc));
   KST::dataObjectList.lock().writeUnlock();
 
   _doc->forceUpdate();
@@ -1441,7 +1506,7 @@ QString KstIfaceImpl::createMatrix(const QString& name, const QString& vector, d
 
   //basic check to see if parameters are consistent with vector
   (*iter)->readLock();
-  if (!useMaxX && (*iter)->sampleCount() < xNumSteps * yNumSteps) {
+  if (!useMaxX && (*iter)->sampleCount() < (int)(xNumSteps * yNumSteps)) {
     (*iter)->readUnlock();
     return QString();
   }
@@ -1509,7 +1574,7 @@ QString KstIfaceImpl::createImage(const QString &name,
     }
     KPalette* pal = new KPalette(paletteName);
     matrix->readLock();
-    image = new KstImage(imgtag, matrix, lowerZ, upperZ, pal);
+    image = new KstImage(imgtag, matrix, lowerZ, upperZ, false, pal);
     matrix->readUnlock();
   } else if (imageType == 1) {
     //need a contourmap
@@ -1517,7 +1582,7 @@ QString KstIfaceImpl::createImage(const QString &name,
       return QString::null;
     }
     matrix->readLock();
-    image = new KstImage(imgtag, matrix, numContours, contourColor.isValid() ? contourColor : QColor("darkBlue"));
+    image = new KstImage(imgtag, matrix, numContours, contourColor.isValid() ? contourColor : QColor("darkBlue"), 0);
     matrix->readUnlock();
   } else if (imageType == 2) {
     //need both contourmap and colormap
@@ -1532,8 +1597,8 @@ QString KstIfaceImpl::createImage(const QString &name,
       return QString::null;
     }
     matrix->readLock();
-    image = new KstImage(imgtag, matrix, lowerZ, upperZ, pal,
-                         numContours, contourColor.isValid() ? contourColor : QColor("darkBlue"));
+    image = new KstImage(imgtag, matrix, lowerZ, upperZ, false, pal,
+                         numContours, contourColor.isValid() ? contourColor : QColor("darkBlue"), 0);
     matrix->readUnlock();
   } else {
     return QString::null;
