@@ -18,7 +18,7 @@
 #ifndef KSTLISTENUMS_H
 #define KSTLISTENUMS_H
 
-typedef enum {SLAVE_USED, NOT_CURVE, CURVE_DELETED, VECTOR_DELETED, VECTOR_USED, NOT_VECTOR, PLUGIN_DELETED, NOT_PLUGIN, OBJECT_DELETED, NOT_OBJECT } RemoveStatus;
+enum RemoveStatus { SLAVE_USED, NOT_CURVE, CURVE_DELETED, VECTOR_DELETED, VECTOR_USED, NOT_VECTOR, PLUGIN_DELETED, NOT_PLUGIN, OBJECT_DELETED, NOT_OBJECT };
 
 
 #endif
