@@ -16,12 +16,9 @@
  ***************************************************************************/
 
 #include <qpainter.h>
-#include <qpalette.h>
-#include <qtable.h>
-#include <qcolor.h>
-#include <klocale.h>
+
+
 #include "kstlogtable.h"
-#include "kstdebug.h"
 
 KstLogTable::KstLogTable(QWidget *parent, const char *name )
 : QTable(parent, name) {
