@@ -17,7 +17,6 @@
 
 #include "kstplotbase.h"
 
-#include <qpainter.h>
 
 KstPlotBase::KstPlotBase(const QString& type) : KstMetaPlot(type) {
 }
@@ -36,8 +35,8 @@ KstObject::UpdateType KstPlotBase::update() {
 }
 
 
-void KstPlotBase::save(QTextStream& ts) {
-  KstMetaPlot::save(ts);
+void KstPlotBase::save(QTextStream& ts, const QString& indent) {
+  KstMetaPlot::save(ts, indent);
 }
 
 
