@@ -18,11 +18,9 @@
 #include <assert.h>
 
 // include files for Qt
-#include <qpainter.h>
 
 // include files for KDE
 #include <klocale.h>
-#include <kpopupmenu.h>
 
 // application specific includes
 #include "kst.h"
@@ -36,6 +34,7 @@ KstPlotGroup::KstPlotGroup() : KstMetaPlot("KstPlotGroup") {
   _layoutActions |= Delete | Raise | Lower | RaiseToTop | LowerToBottom | Rename | MoveTo;
   setTagName(i18n("Plot Group %1").arg(++pgcount));
   _type = "plotgroup";
+  setBorderColor(Qt::blue);
 }
 
 
@@ -53,13 +52,22 @@ KstObject::UpdateType KstPlotGroup::update() {
 }
 
 
-bool KstPlotGroup::removeChild(KstViewObjectPtr obj, bool recursive) {
-  KstViewObjectList::Iterator it;
+void KstPlotGroup::setHasFocus(bool hasFocus) {
+  _hasFocus = hasFocus;
+  forEachChild<bool>(&KstViewObject::setHasFocus, hasFocus);
+}
 
+
+void KstPlotGroup::removeFocus(QPainter& p) {
+  forEachChild<QPainter&>(&KstViewObject::removeFocus, p);
+}
+
+
+bool KstPlotGroup::removeChild(KstViewObjectPtr obj, bool recursive) {
   if (KstViewObject::removeChild(obj, recursive)) {
     if (_children.count() > 1) {
       QRect gg = _children.first()->geometry();
-      for (it = _children.begin(); it != _children.end(); ++it) {
+      for (KstViewObjectList::Iterator it = _children.begin(); it != _children.end(); ++it) {
         gg |= (*it)->geometry();
       }
 
@@ -69,7 +77,9 @@ bool KstPlotGroup::removeChild(KstViewObjectPtr obj, bool recursive) {
         updateAspect();
       }
     } else {
-      flatten();
+      if (_parent) { // can be false if we are being deleted already
+        flatten();
+      }
     }
 
     return true;
@@ -92,17 +102,17 @@ void KstPlotGroup::flatten() {
 }
 
 
-void KstPlotGroup::save(QTextStream& ts) {
-  ts << "<" << type() << ">" << endl;
-  KstMetaPlot::save(ts);
-  ts << "</" << type() << ">" << endl;
+void KstPlotGroup::save(QTextStream& ts, const QString& indent) {
+  ts << indent << "<" << type() << ">" << endl;
+  KstMetaPlot::save(ts, indent + "  ");
+  ts << indent << "</" << type() << ">" << endl;
 }
 
 
-void KstPlotGroup::saveTag(QTextStream& ts) {
-  ts << "<" << type() << ">" << endl;
-  KstMetaPlot::saveTag(ts);
-  ts << "</" << type() << ">" << endl;
+void KstPlotGroup::saveTag(QTextStream& ts, const QString& indent) {
+  ts << indent << "<" << type() << ">" << endl;
+  KstMetaPlot::saveTag(ts, indent + "  ");
+  ts << indent << "</" << type() << ">" << endl;
 }
 
 
@@ -110,6 +120,12 @@ void KstPlotGroup::paint(KstPaintType type, QPainter& p) {
   QRegion clipRegion(geometry());
   QBrush brush(_backgroundColor);
 
+  if (KstApp::inst()->getZoomRadio() == KstApp::LAYOUT) {
+    setBorderWidth(3);
+  } else {
+    setBorderWidth(0);
+  }
+
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     clipRegion -= QRegion((*i)->geometry());
   }
