@@ -21,9 +21,6 @@
 #include "kstdataobject.h"
 #include "plugin.h"
 
-#include <qvaluelist.h>
-#include <qstring.h>
-#include <qdom.h>
 
 /*  Usage notes:
  *   - Output vectors are created internally, but may be renamed.
@@ -31,32 +28,46 @@
  */
 
 class KstPlugin : public KstDataObject {
-public:
-  KstPlugin();
-  KstPlugin(QDomElement &e);
-  virtual ~KstPlugin();
+  public:
+    KstPlugin();
+    KstPlugin(QDomElement &e);
+    virtual ~KstPlugin();
 
-  virtual UpdateType update(int update_counter);
+    virtual UpdateType update(int update_counter);
 
-  virtual void save(QTextStream &ts);
+    virtual void save(QTextStream &ts, const QString& indent = QString::null);
 
-  virtual bool slaveVectorsUsed() const;
-  virtual bool isValid() const;
+    virtual bool slaveVectorsUsed() const;
+    virtual bool isValid() const;
 
-  virtual bool setPlugin(KstSharedPtr<Plugin> plugin);
-  KstSharedPtr<Plugin> plugin() const;
+    virtual bool setPlugin(KstSharedPtr<Plugin> plugin);
+    KstSharedPtr<Plugin> plugin() const;
 
-  virtual QString propertyString() const;
+    virtual QString propertyString() const;
 
-protected:
-  static void countScalarsAndVectors(const QValueList<Plugin::Data::IOValue>& table, unsigned& scalars, unsigned& vectors);
-  virtual void _showDialog();
-  KstSharedPtr<Plugin> _plugin;
-  unsigned _inScalarCnt, _inArrayCnt, _outScalarCnt, _outArrayCnt;
-  void *_localData;
+    virtual const KstCurveHintList* curveHints() const;
 
-private:
-  void commonConstructor();
+    QString lastError() const;
+
+  protected:
+    static void countScalarsAndVectors(const QValueList<Plugin::Data::IOValue>& table, unsigned& scalars, unsigned& vectors);
+    virtual void _showDialog();
+    KstSharedPtr<Plugin> _plugin;
+    unsigned _inScalarCnt, _inArrayCnt, _inStringCnt, _outScalarCnt;
+    unsigned _inPid, _outArrayCnt, _outStringCnt;
+    void *_localData;
+    QString _lastError;
+
+    int *_inArrayLens, *_outArrayLens;
+    double *_inScalars, *_outScalars;
+    double **_inVectors, **_outVectors;
+    char **_inStrings;
+    char **_outStrings;
+
+  private:
+    void commonConstructor();
+    void allocateParameters();
+    void freeParameters();
 };
 
 typedef KstSharedPtr<KstPlugin> KstPluginPtr;
