@@ -18,39 +18,27 @@
 
 // include files for Qt
 #include <qcheckbox.h>
-#include <qlabel.h>
-#include <qlineedit.h>
-#include <qpushbutton.h>
 #include <qspinbox.h>
-#include <qstring.h>
-#include <qwidget.h>
 
 // include files for KDE
 #include <kcombobox.h>
 #include <kdebug.h>
-#include <klocale.h>
-#include <kmdimainfrm.h>
 #include <kmessagebox.h>
-#include <kstaticdeleter.h>
 
 // application specific inclues
 #include "fftoptionswidget.h"
 #include "curveappearancewidget.h"
 #include "curveplacementwidget.h"
-#include "kst2dplot.h"
-#include "kst.h"
-#include "kstdatacollection.h"
-#include "kstdoc.h"
 #include "kstpsddialog_i.h"
+#include "kstvcurve.h"
 #include "kstviewwindow.h"
-#include "kstvectordialog_i.h"
 #include "vectorselector.h"
 
 #define DIALOGTYPE KstPsdDialogI
 #define DTYPE "Power Spectrum"
 #include "dataobjectdialog.h"
 
-static const QString defaultTag = "<Auto Name>";
+static const QString& defaultTag = KGlobal::staticQString("<Auto Name>");
 
 KstPsdDialogI *KstPsdDialogI::_inst = 0L;
 static KStaticDeleter<KstPsdDialogI> _psInst;
@@ -73,8 +61,8 @@ KstPsdDialogI::~KstPsdDialogI() {
   DP = 0L;
 }
 
-KstPSDCurvePtr KstPsdDialogI::_getPtr(const QString &tagin) {
-  KstPSDCurveList c = kstObjectSubList<KstDataObject, KstPSDCurve>(KST::dataObjectList);
+KstPSDPtr KstPsdDialogI::_getPtr(const QString& tagin) {
+  KstPSDList c = kstObjectSubList<KstDataObject, KstPSD>(KST::dataObjectList);
   return *c.findTag(tagin);
 }
 
@@ -91,28 +79,26 @@ void KstPsdDialogI::_fillFieldsForEdit() {
 
   _tagName->setText(DP->tagName());
 
-  _vector->setSelection(DP->getVTag());
-
-  _curveAppearance->setValue(
-    DP->hasLines(), DP->hasPoints(), DP->getColor(),
-    DP->Point.getType(), DP->lineWidth(), DP->lineStyle());
+  _vector->setSelection(DP->vTag());
 
   /* set sample rate, Units, FFT len, and vector units */
-  _kstFFTOptions->FFTLen->setValue(DP->getLen());
-  _kstFFTOptions->SampRate->setText(QString::number(DP->getFreq()));
+  _kstFFTOptions->FFTLen->setValue(DP->len());
+  _kstFFTOptions->SampRate->setText(QString::number(DP->freq()));
 
   _kstFFTOptions->VectorUnits->setText(DP->VUnits);
   _kstFFTOptions->RateUnits->setText(DP->RUnits);
-  _kstFFTOptions->Apodize->setChecked(DP->getApodize());
-  _kstFFTOptions->RemoveMean->setChecked(DP->getRemoveMean());
-  _kstFFTOptions->Interleaved->setChecked(DP->getAverage());
+  _kstFFTOptions->Apodize->setChecked(DP->apodize());
+  _kstFFTOptions->RemoveMean->setChecked(DP->removeMean());
+  _kstFFTOptions->Interleaved->setChecked(DP->average());
 
   DP->readUnlock();
 
+  _curveAppearance->hide();
   _curvePlacement->hide();
 
   adjustSize();
   resize(minimumSizeHint());
+  setFixedHeight(height());
 }
 
 void KstPsdDialogI::_fillFieldsForNew() {
@@ -133,6 +119,7 @@ void KstPsdDialogI::_fillFieldsForNew() {
 
   adjustSize();
   resize(minimumSizeHint());
+  setFixedHeight(height());
 }
 
 void KstPsdDialogI::update() {
@@ -154,7 +141,7 @@ bool KstPsdDialogI::new_I() {
   }
 
   if (_vector->selectedVector().isEmpty()) {
-    KMessageBox::sorry(0L, i18n("New PSD not made: define vectors first."));
+    KMessageBox::sorry(this, i18n("New PSD not made: define vectors first."));
     return false;
   }
 
@@ -172,21 +159,23 @@ bool KstPsdDialogI::new_I() {
     return false;
   } else {
     p->readLock();
-    KstPSDCurvePtr curve = new KstPSDCurve(tag_name, p,
+    KstPSDPtr psd = new KstPSD(tag_name, p,
                             _kstFFTOptions->SampRate->text().toDouble(),
                             _kstFFTOptions->Interleaved->isChecked(),
                             _kstFFTOptions->FFTLen->text().toInt(),
                             _kstFFTOptions->Apodize->isChecked(),
                             _kstFFTOptions->RemoveMean->isChecked(),
                             _kstFFTOptions->VectorUnits->text(),
-                            _kstFFTOptions->RateUnits->text(),
-                            _curveAppearance->color());
+                            _kstFFTOptions->RateUnits->text());
 
-    curve->setHasPoints(_curveAppearance->showPoints());
-    curve->setHasLines(_curveAppearance->showLines());
-    curve->Point.setType(_curveAppearance->pointType());
-    curve->setLineWidth(_curveAppearance->lineWidth());
-    curve->setLineStyle(_curveAppearance->lineStyle());
+    KstVCurvePtr vc = new KstVCurve(tag_name + "-curve", psd->vX(), psd->vY(), 0L, 0L, _curveAppearance->color());
+    vc->setHasPoints(_curveAppearance->showPoints());
+    vc->setHasLines(_curveAppearance->showLines());
+    vc->setHasBars(_curveAppearance->showBars());
+    vc->Point.setType(_curveAppearance->pointType());
+    vc->setLineWidth(_curveAppearance->lineWidth());
+    vc->setLineStyle(_curveAppearance->lineStyle());
+    vc->setBarStyle(_curveAppearance->barStyle());
 
     Kst2DPlotPtr plot;
     KstViewWindow *w = dynamic_cast<KstViewWindow*>(KstApp::inst()->findWindow(_curvePlacement->_plotWindow->currentText()));
@@ -197,9 +186,9 @@ bool KstPsdDialogI::new_I() {
     if (w) {
       if (_curvePlacement->existingPlot()) {
         /* assign curve to plot */
-        plot = dynamic_cast<Kst2DPlot*>(w->view()->findChild(_curvePlacement->plotName()).data());
+        plot = kst_cast<Kst2DPlot>(w->view()->findChild(_curvePlacement->plotName()));
         if (plot) {
-          plot->addCurve(curve.data());
+          plot->addCurve(vc.data());
           w->view()->paint(P_DATA);
         }
       }
@@ -210,11 +199,11 @@ bool KstPsdDialogI::new_I() {
         if (_curvePlacement->reGrid()) {
           w->view()->cleanup(_curvePlacement->columns());
         }
-        plot = dynamic_cast<Kst2DPlot*>(w->view()->findChild(name).data());
+        plot = kst_cast<Kst2DPlot>(w->view()->findChild(name));
         if (plot) {
           _curvePlacement->update();
           _curvePlacement->setCurrentPlot(plot->tagName());
-          plot->addCurve(curve.data());
+          plot->addCurve(vc.data());
           plot->GenerateDefaultLabels();
           w->view()->paint(P_DATA);
         }
@@ -222,9 +211,11 @@ bool KstPsdDialogI::new_I() {
     }
     p->readUnlock();
     KST::dataObjectList.lock().writeLock();
-    KST::dataObjectList.append(curve.data());
+    KST::dataObjectList.append(psd.data());
+    KST::dataObjectList.append(vc.data());
     KST::dataObjectList.lock().writeUnlock();
-    curve = 0L;
+    psd = 0L;
+    vc = 0L;
     emit modified();
   }
   return true;
@@ -238,18 +229,12 @@ bool KstPsdDialogI::edit_I() {
   }
 
   DP->writeLock();
-  DP->setTagName(_tagName->text());
 
   KST::vectorList.lock().readLock();
-  DP->setVector(*KST::vectorList.findTag(_vector->selectedVector()));
+  DP->setVector(*KST::vectorList.findTag(_vector->selectedVector())); 
   KST::vectorList.lock().readUnlock();
 
-  DP->setColor(_curveAppearance->color());
-  DP->setHasPoints(_curveAppearance->showPoints());
-  DP->setHasLines(_curveAppearance->showLines());
-  DP->setLineWidth(_curveAppearance->lineWidth());
-  DP->setLineStyle(_curveAppearance->lineStyle());
-  DP->Point.setType(_curveAppearance->pointType());
+  DP->setTagName(_tagName->text());
 
   if (!_kstFFTOptions->checkValues()) {
     DP->writeUnlock();
@@ -266,6 +251,7 @@ bool KstPsdDialogI::edit_I() {
   DP->setRemoveMean(_kstFFTOptions->RemoveMean->isChecked());
   DP->setAverage(_kstFFTOptions->Interleaved->isChecked());
   DP->update();
+
   DP->writeUnlock();
 
   emit modified();
