@@ -16,17 +16,30 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <assert.h>
 #include <math.h>
 #include <stdlib.h>
 
+#include <qstylesheet.h>
+
 #include <kdebug.h>
 #include <klocale.h>
 
-#include "enodes.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
 #include "kstrvector.h"
-#include "rwlock.h"
+
+//
+// ReqNF <=0 means read from ReqF0 to end of File
+// ReqF0 < means start at EndOfFile-ReqNF.
+//
+// ReqNF      ReqF0      Action
+//  < 1        >=0       read from ReqF0 to end of file
+//  < 1        < 0       illegal: fixed in checkIntegrity
+//    1         ??       illegal: fixed in checkIntegrity
+//  > 1        < 0       read the last ReqNF frames from the file
+//  > 1        >=0       Read ReqNF frames starting at frame ReqF0
+//
 
 /** Create a KstRVector: raw data from a file */
 KstRVector::KstRVector(KstDataSourcePtr in_file, const QString &in_field,
@@ -72,7 +85,7 @@ KstRVector::KstRVector(QDomElement &e, const QString &o_file,
         in_n = e.text().toInt();
       } else if (e.tagName() == "skip") {
         in_skip = e.text().toInt();
-        in_DoSkip = true;
+        in_DoSkip = in_skip > 0;
       } else if (e.tagName() == "doAve") {
         in_DoAve = true;
         in_DoSkip = true;
@@ -108,7 +121,6 @@ void KstRVector::commonRVConstructor(KstDataSourcePtr in_file,
                                      int in_n, int in_skip, bool in_DoSkip,
                                      bool in_DoAve) {
   _numSamples = 0;
-  NumUsed = 0;
   _scalars["sum"]->setValue(0.0);
   _scalars["sumsquared"]->setValue(0.0);
   F0 = NF = 0; // nothing read yet
@@ -128,8 +140,6 @@ void KstRVector::commonRVConstructor(KstDataSourcePtr in_file,
     Skip = 1;
   }
 
-  // ReqNF <=0 means read to end of file
-  // ReqF0 < 0 means count back ReqNF from end of file
   if (ReqNF <= 0 && ReqF0 < 0) {
     ReqF0 = 0;
   }
@@ -166,9 +176,14 @@ void KstRVector::change(KstDataSourcePtr in_file, const QString &in_field,
   _field = in_field;
   setTagName(in_tag);
 
+  if (_file) {
+    _file->writeLock();
+  }
   reset();
-  // ReqNF <=0 means read to end of file
-  // ReqF0 < 0 means count back ReqNF from end of file
+  if (_file) {
+    _file->writeUnlock();
+  }
+
   if (ReqNF <= 0 && ReqF0 < 0) {
     ReqF0 = 0;
   }
@@ -180,13 +195,25 @@ void KstRVector::changeFile(KstDataSourcePtr in_file) {
     KstDebug::self()->log(i18n("Data file for vector %1 was not opened.").arg(tagName()), KstDebug::Warning);
   }
   _file = in_file;
+  if (_file) {
+    _file->writeLock();
+  }
   reset();
+  if (_file) {
+    _file->writeUnlock();
+  }
 }
 
 void KstRVector::changeFrames(int in_f0, int in_n,
                               int in_skip, bool in_DoSkip,
                               bool in_DoAve) {
+  if (_file) {
+    _file->writeLock();
+  }
   reset();
+  if (_file) {
+    _file->writeUnlock();
+  }
   Skip = in_skip;
   DoSkip = in_DoSkip;
   DoAve = in_DoAve;
@@ -197,8 +224,6 @@ void KstRVector::changeFrames(int in_f0, int in_n,
   ReqF0 = in_f0;
   ReqNF = in_n;
 
-  // ReqNF <=0 means read to end of file
-  // ReqF0 < 0 means count back ReqNF from end of file
   if (ReqNF <= 0 && ReqF0 < 0) {
     ReqF0 = 0;
   }
@@ -213,10 +238,6 @@ KstRVector::~KstRVector() {
   }
 }
 
-
-  // ReqNF <=0 means read to end of file
-  // ReqF0 < 0 means count back ReqNF from end of file
-
 bool KstRVector::readToEOF() const {
   return ReqNF <= 0;
 }
@@ -257,27 +278,38 @@ int KstRVector::reqStartFrame() const {
 }
 
 /** Save vector information */
-void KstRVector::save(QTextStream &ts) {
-  if (!_file) {
-    return;
-  }
-
-  ts << "  <tag>" << tagName() << "</tag>" << endl;
-  ts << "  <filename>" << _file->fileName() << "</filename>" << endl;
-  ts << "  <field>" << _field << "</field>" << endl;
-  ts << "  <start>" << ReqF0 << "</start>" << endl;
-  ts << "  <num>" << ReqNF << "</num>" << endl;
-  if (doSkip()) {
-    ts << "  <skip>" << Skip << "</skip>" << endl;
-    if (doAve()) {
-      ts << "  <doAve/>" << endl;
+void KstRVector::save(QTextStream &ts, const QString& indent, bool saveAbsolutePosition) {
+  if (_file) {    
+    ts << indent << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+    _file->readLock();
+    ts << indent << "<filename>" << QStyleSheet::escape(_file->fileName()) << "</filename>" << endl;
+    _file->readUnlock();
+    ts << indent << "<field>" << _field << "</field>" << endl;
+    if (saveAbsolutePosition) {
+      ts << indent << "<start>" << F0 << "</start>" << endl;
+      ts << indent << "<num>" << NF << "</num>" << endl;
+    } else {
+      ts << indent << "<start>" << ReqF0 << "</start>" << endl;
+      ts << indent << "<num>" << ReqNF << "</num>" << endl;
+    }
+    if (doSkip()) {
+      ts << indent << "<skip>" << Skip << "</skip>" << endl;
+      if (doAve()) {
+        ts << indent << "<doAve/>" << endl;
+      }
     }
   }
 }
 
 /** return the name of the file */
 QString KstRVector::filename() const {
-  return _file ? _file->fileName() : QString::null;
+  QString rc;
+  if (_file) {
+    _file->readLock();
+    rc = _file->fileName();
+    _file->readUnlock();
+  }
+  return rc;
 }
 
 /** return the field */
@@ -290,8 +322,14 @@ QString KstRVector::label() const {
   QString label;
 
   _field.toInt(&ok);
-  if (ok && _file && _file->fileType() == "ASCII") {
-    label = i18n("Column %1").arg(_field);
+  if (ok && _file) {
+    _file->readLock();
+    if (_file->fileType() == "ASCII") {
+      label = i18n("Column %1").arg(_field);
+    } else {
+      label = _field;
+    }
+    _file->readUnlock();
   } else {
     label = _field;
   }
@@ -299,8 +337,10 @@ QString KstRVector::label() const {
   return label;
 }
 
-void KstRVector::reset() {
-  SPF = _file ? _file->samplesPerFrame(_field) : SPF;
+void KstRVector::reset() { // must be called with a lock
+  if (_file) {
+    SPF = _file->samplesPerFrame(_field);
+  }
   F0 = NF = 0;
   resize(0);
   _numSamples = 0;
@@ -312,17 +352,20 @@ void KstRVector::checkIntegrity() {
     Skip = 1;
   }
 
-  /* if dirty, reset */
   if (_dirty) {
     reset();
   }
 
-  /* if it looks like we have a new file, reset */
+  //
+  // if it looks like we have a new file, reset
+  //
   if (_file && (SPF != _file->samplesPerFrame(_field) || _file->frameCount(_field) < NF)) {
     reset();
   }
 
+  //
   // check for illeagal NF and F0 values
+  //
   if (ReqNF < 1 && ReqF0 < 0) {
     ReqF0 = 0; // for this illegal case, read the whole file
   }
@@ -342,11 +385,18 @@ KstObject::UpdateType KstRVector::update(int update_counter) {
     return NO_CHANGE;
   }
 
-  return doUpdate();
+  if (_file) {
+    _file->writeLock();
+  }
+  KstObject::UpdateType rc = doUpdate();
+  if (_file) {
+    _file->writeUnlock();
+  }
+  
+  return rc;
 }
 
 KstObject::UpdateType KstRVector::doUpdate(bool force) {
-  KstObject::UpdateType  update;
   int i, j, k, shift, n_read=0;
   int ave_nread;
   int new_f0, new_nf, tmp_fn;
@@ -357,31 +407,24 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
     return NO_CHANGE;
   }
 
-  /*******************************/
-  /**** Set new_nf and new_f0 ****/
-  // ReqNF <=0 means read from ReqF0 to end of File
-  // ReqF0 < means start at EndOfFile-ReqNF.
-
-  // ReqNF      ReqF0      Action
-  //  < 1        >=0       read from ReqF0 to end of file
-  //  < 1        < 0       illegal: fixed in checkIntegrity
-  //    1         ??       illegal: fixed in checkIntegrity
-  //  > 1        < 0       read the last ReqNF frames from the file
-  //  > 1        >=0       Read ReqNF frames starting at frame ReqF0
+  //
+  // set new_nf and new_f0
+  //
+  int fc = _file->frameCount(_field);
   if (ReqNF < 1) { // read to end of file
     new_f0 = ReqF0;
-    new_nf = _file->frameCount(_field) - new_f0;
+    new_nf = fc - new_f0;
   } else if (ReqF0 < 0) { // count back from end of file
-    new_nf = _file->frameCount(_field);
+    new_nf = fc;
     if (new_nf > ReqNF) {
       new_nf = ReqNF;
     }
-    new_f0 = _file->frameCount(_field) - new_nf;
+    new_f0 = fc - new_nf;
   } else {
     new_f0 = ReqF0;
     new_nf = ReqNF;
-    if (new_f0 + new_nf > _file->frameCount(_field)) {
-      new_nf = _file->frameCount(_field) - new_f0;
+    if (new_f0 + new_nf > fc) {
+      new_nf = fc - new_f0;
     }
     if (new_nf <= 0) {
       new_nf = new_f0 = 0;
@@ -389,8 +432,9 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   }
 
   if (DoSkip) {
-    /*********************************/
-    /* change new_f0 and new_nf so they both lie on skip boundaries */
+    //
+    // change new_f0 and new_nf so they both lie on skip boundaries
+    //
     tmp_fn = ((new_f0+new_nf)/Skip) * Skip;
     new_f0 = (((new_f0-1)/Skip)+1) * Skip;
     new_nf = tmp_fn-new_f0;
@@ -400,8 +444,9 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
     return NO_CHANGE;
   }
 
-  /***********************************/
-  /**** Shift vector if necessary ****/
+  //
+  // shift vector if necessary
+  //
   if (new_f0 < F0 || new_f0 >= F0 + NF) { // No useful data around.
     reset();
   } else { // shift stuff rather than re-read
@@ -421,7 +466,9 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   }
 
   if (DoSkip) {
-    /**** Reallocate V if necessary ****/
+    //
+    // reallocate V if necessary
+    //
     if (new_nf / Skip != _size) {
       bool rc = resize(new_nf/Skip);
       if (!rc) {
@@ -435,8 +482,7 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
         /* enlarge AveReadBuf if necessary */
         if (N_AveReadBuf < Skip*SPF) {
           N_AveReadBuf = Skip*SPF;
-          AveReadBuf = static_cast<double*>
-            (realloc(AveReadBuf, N_AveReadBuf*sizeof(double)));
+          AveReadBuf = static_cast<double*>(realloc(AveReadBuf, N_AveReadBuf*sizeof(double)));
           if (!AveReadBuf) {
             // FIXME: handle failed resize
           }
@@ -454,7 +500,9 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
       }
     }
   } else {
-    /**** Reallocate V if necessary ****/
+    //
+    // reallocate V if necessary
+    //
     if ((new_nf - 1)*SPF + 1 != _size) {
       bool rc = resize((new_nf - 1)*SPF + 1);
       if (!rc) {
@@ -466,7 +514,9 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
       NF--; /* last frame read was only partially read... */
     }
 
-    /**** Read the new data from File ****/
+    //
+    // read the new data from file
+    //
     n_read = _file->readField(_v+NF*SPF, _field, new_f0 + NF, new_nf - NF - 1);
     n_read += _file->readField(_v+(new_nf-1)*SPF, _field, new_f0 + new_nf-1, -1);
   }
@@ -476,6 +526,7 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   F0 = new_f0;
   _numSamples += n_read;
 
+  //
   // if for some reason (eg, realtime reading an nfs mounted
   // dirfile) not all of the data was read, the data will never
   // be read; the samples will be filled in with the last data
@@ -483,8 +534,9 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
   // This is bad - I think it will be worthwhile
   // to add blocking w/ timeout to KstFile.
   // As a first fix, mount all nsf mounted dirfiles with "-o noac"
+  //
   _dirty = false;
-  if (_numSamples != _size) {
+  if (_numSamples != _size && !(_numSamples == 0 && _size == 1)) {
     _dirty = true;
     for (i = _numSamples; i < _size; i++) {
       _v[i] = _v[0];
@@ -498,9 +550,7 @@ KstObject::UpdateType KstRVector::doUpdate(bool force) {
     NumShifted = _size;
   }
 
-  update = KstVector::update(-1);
-  
-  return update;
+  return KstVector::update(-1);
 }
 
 
@@ -511,32 +561,53 @@ int KstRVector::samplesPerFrame() const {
 
 /** return true if it has a valid file and field, or false otherwise */
 bool KstRVector::isValid() const {
-  return _file && _file->isValidField(_field);
+  if (_file) {
+    _file->readLock();
+    bool rc = _file->isValidField(_field);
+    _file->readUnlock();
+    return rc;
+  }
+  return false;
 }
 
 int KstRVector::fileLength() const {
-  return _file ? _file->frameCount(_field) : 0;
+  if (_file) {
+    _file->readLock();
+    int rc = _file->frameCount(_field);
+    _file->readUnlock();
+
+    return rc;
+  }
+
+  return 0;
 }
 
 void KstRVector::reload() {
-  if (!_file) {
-    return;
-  }
-
-  if (_file->reset()) { // Try the efficient way first
-    reset();
-    return;
-  }
-
-  // The inefficient way
-  KstDataSourcePtr newsrc = KstDataSource::loadSource(_file->fileName(), _file->fileType());
-  if (newsrc) {
-    KST::dataSourceList.lock().writeLock();
-    KST::dataSourceList.remove(_file);
-    KST::dataSourceList.append(newsrc);
-    KST::dataSourceList.lock().writeUnlock();
-    _file = newsrc;
-    reset();
+  if (_file) {
+    _file->writeLock();
+    if (_file->reset()) {
+      //
+      // try the efficient way first
+      //
+      reset();
+    } else {
+      //
+      // the inefficient way
+      //
+      KstDataSourcePtr newsrc = KstDataSource::loadSource(_file->fileName(), _file->fileType());
+      assert(newsrc != _file);
+      if (newsrc) {
+        _file->writeUnlock();
+        KST::dataSourceList.lock().writeLock();
+        KST::dataSourceList.remove(_file);
+        _file = newsrc;
+        _file->writeLock();
+        KST::dataSourceList.append(_file);
+        KST::dataSourceList.lock().writeUnlock();
+        reset();
+      }
+    }
+    _file->writeUnlock();
   }
 }
 
