@@ -19,8 +19,6 @@
 #define KSTRVECTOR_H
 
 #include "kstvector.h"
-#include <qstring.h>
-#include <qdom.h>
 
 #include "kstdatasource.h"
 
@@ -83,7 +81,7 @@ public:
   int samplesPerFrame() const;
 
   /** Save vector information */
-  virtual void save(QTextStream &ts);
+  virtual void save(QTextStream &ts, const QString& indent = QString::null, bool saveAbsolutePosition = false);
 
   /** return the name of the file if a RVector - otherwise return "" */
   QString filename() const;
@@ -154,9 +152,9 @@ private:
   int N_AveReadBuf;
   double *AveReadBuf;
 
-  void reset();
+  void reset(); // must be called with a lock
 
-  void checkIntegrity();
+  void checkIntegrity(); // must be called with a lock
 };
 
 typedef KstSharedPtr<KstRVector> KstRVectorPtr;
