@@ -25,7 +25,7 @@ static int iAnonymousScalarCounter = 0;
 
 /** Create the base scalar */
 KstScalar::KstScalar(const QString& in_tag, double val, bool orphan, bool displayable, bool doLock)
-: KstObject(), _orphan(orphan), _displayable(displayable) {
+: KstObject(), _value(val), _orphan(orphan), _displayable(displayable) {
   QString _tag = in_tag;
   if (_tag.isEmpty()) {
     QString nt = i18n("Anonymous Scalar %1");
@@ -39,7 +39,6 @@ KstScalar::KstScalar(const QString& in_tag, double val, bool orphan, bool displa
     }
   }
   setTagName(_tag);
-  _value = val;
 
   // FIXME: passing in a lock variable indicates a design problem
   if (doLock) {
@@ -52,7 +51,8 @@ KstScalar::KstScalar(const QString& in_tag, double val, bool orphan, bool displa
 }
 
 
-KstScalar::KstScalar(QDomElement& e) : KstObject(), _orphan(false), _displayable(true) {
+KstScalar::KstScalar(QDomElement& e)
+: KstObject(), _orphan(false), _displayable(true) {
   QDomNode n = e.firstChild();
   bool ok;
   
@@ -61,18 +61,19 @@ KstScalar::KstScalar(QDomElement& e) : KstObject(), _orphan(false), _displayable
     if(!e.isNull()) {
       if (e.tagName() == "tag") {
         setTagName(e.text());
+      } else if (e.tagName() == "orphan") {
+        _orphan = true;
       } else if (e.tagName() == "value") {
         setValue(e.text().toDouble());
       }
     }
     n = n.nextSibling();
-
-    if (tagName().toDouble(&ok)==value()) {
-      if (ok) {
-        _displayable = false;
-      }
-    }
   }
+
+  if (tagName().toDouble(&ok) == value() && ok) {
+    _displayable = false;
+  }
+
   KST::scalarList.append(this);
 }
 
@@ -83,13 +84,16 @@ KstScalar::~KstScalar() {
 
 KstObject::UpdateType KstScalar::update(int updateCounter) {
   Q_UNUSED(updateCounter)
-    return NO_CHANGE;
+  return NO_CHANGE;
 }
 
 
-void KstScalar::save(QTextStream &ts) {
-  ts << "    <tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
-  ts << "    <value>" << value() << "</value>" << endl;
+void KstScalar::save(QTextStream &ts, const QString& indent) {
+  ts << indent << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+  if (_orphan) {
+    ts << indent << "<orphan/>" << endl;
+  }
+  ts << indent << "<value>" << value() << "</value>" << endl;
 }
 
 
