@@ -35,7 +35,7 @@ class KstScalar : public KstObject {
     const QString label() const { return QString::number(_value); }
 
     /** Save vector information */
-    virtual void save(QTextStream &ts);
+    virtual void save(QTextStream &ts, const QString& indent = QString::null);
 
     /** Update the vector.  Return true if there was new data. */
     virtual UpdateType update(int updateCounter = -1);
