@@ -16,10 +16,7 @@
  ***************************************************************************/
 
 #include <qpainter.h>
-#include <qpalette.h>
-#include <qtable.h>
 #include "kstscalartable.h"
-#include "kstrvector.h"
 #include "kstdatacollection.h"
 
 KstScalarTable::KstScalarTable( QWidget * parent, const char * name ) : QTable( parent, name ) {
