@@ -20,6 +20,8 @@
 
 #include <qcolor.h>
 
+#include "kstplotdefines.h"
+
 class KstSettings {
   public:
     KstSettings();
@@ -36,6 +38,21 @@ class KstSettings {
     QColor backgroundColor;
     QColor foregroundColor;
 
+    bool promptWindowClose : 1;
+    bool showQuickStart : 1;
+    bool xMajor : 1;
+    bool yMajor : 1;
+    bool xMinor : 1;
+    bool yMinor : 1;
+    bool majorGridColorDefault : 1;
+    bool minorGridColorDefault : 1;
+    bool xAxisInterpret : 1;
+
+    QColor majorColor;
+    QColor minorColor;
+    KstXAxisInterpretation xAxisInterpretation;
+    KstXAxisDisplay xAxisDisplay;
+    
   private:
     static KstSettings *_self;
 };
