@@ -16,39 +16,35 @@
  ***************************************************************************/
 
 #include <assert.h>
-#include <stdlib.h>
 
 // include files for Qt
-#include <qapplication.h>
 
 // include files for KDE
 #include <kdebug.h>
 #include <klocale.h>
-#include <kpopupmenu.h>
-#include <kprinter.h>
 
 // application specific includes
 #include "kst.h"
 #include "kstdoc.h"
 #include "kstplotgroup.h"
+#include "kstsettings.h"
 #include "ksttoplevelview.h"
-#include "kstviewwidget.h"
 
 KstTopLevelView::KstTopLevelView(QWidget *parent, const char *name, WFlags w)
 : KstViewObject("KstTopLevelView"), _w(new KstViewWidget(this, parent, name, w)) {
+  _onGrid = true;
+  setTagName(name);
   commonConstructor();
 }
 
 
 KstTopLevelView::KstTopLevelView(QDomElement& e, QWidget *parent, const char *name, WFlags w) : KstViewObject(e), _w(new KstViewWidget(this, parent, name, w)) {
   commonConstructor();
-
   loadChildren(e);
 }
 
 
 void KstTopLevelView::commonConstructor() {
-  _onGrid = true;
   _focusOn = false;
   setViewMode(_mode = DisplayMode);
   _pressDirection = -1;
@@ -67,9 +63,18 @@ KstObject::UpdateType KstTopLevelView::update() {
 }
 
 
-void KstTopLevelView::save(QTextStream& ts) {
+void KstTopLevelView::save(QTextStream& ts, const QString& indent) {
+  bool temp_onGrid;
+  if (_maximized) {
+    temp_onGrid = _prevOnGrid;
+  } else {
+    temp_onGrid = _onGrid;
+  }
+  if (temp_onGrid && _columns > 0) { // only if on a grid and we have contents
+    ts << indent << "<columns>" << _columns << "</columns>" << endl;
+  }
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    (*i)->saveTag(ts);
+    (*i)->saveTag(ts, indent);
   }
 }
 
@@ -77,6 +82,8 @@ void KstTopLevelView::save(QTextStream& ts) {
 void KstTopLevelView::paint(KstPaintType type, QPainter& p) {
   if (type != P_PRINT && type != P_EXPORT) {
     updateAlignment();
+  } else {
+    p.fillRect(geometry(), QBrush(KstSettings::globalSettings()->backgroundColor, SolidPattern));
   }
   QRegion clipRegion(geometry());
   p.setClipRegion(clipRegion);
@@ -89,15 +96,13 @@ void KstTopLevelView::resized(const QSize& size) {
 }
 
 void KstTopLevelView::updateAlignment() {
-  double x_max = 0,  x;
-  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    x = (*i)->xInternalAlignment();
-    if (x > x_max) {
-      x_max = x;
-    }
-  }
+  QRect plotRegion;
 
-  KST::alignment.setX(x_max);
+  KST::alignment.reset();  
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    (*i)->internalAlignment(plotRegion);
+    KST::alignment.setPosition((*i)->geometry(), plotRegion);
+  }
 }
 
 void KstTopLevelView::resize(const QSize& size) {
@@ -106,25 +111,19 @@ void KstTopLevelView::resize(const QSize& size) {
 }
 
 
-void KstTopLevelView::paint(KstPaintType type, bool erase) {
+void KstTopLevelView::paint(KstPaintType type) {
   QPainter p;
   p.begin(_w);
   p.setViewXForm(true);
-  if (erase) {
-    abort();
-    QBrush brush(_backgroundColor);
-    p.fillRect(geometry(), brush);
-  } else {
-    QRegion r(_w->geometry());
-    for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-      r = r.subtract(QRegion((*i)->geometry()));
-    }
-    p.setClipRegion(r);
-    _lastClipRegion = r;
-    QBrush brush(_backgroundColor);
-    p.fillRect(geometry(), brush);
-    p.setClipping(false);
+  QRegion r(_w->geometry());
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    r = r.subtract(QRegion((*i)->geometry()));
   }
+  p.setClipRegion(r);
+  _lastClipRegion = r;
+  QBrush brush(_backgroundColor);
+  p.fillRect(geometry(), brush);
+  p.setClipping(false);
   paint(type, p);
   p.end();
 }
@@ -183,6 +182,7 @@ void KstTopLevelView::setViewMode(ViewMode v) {
   if (_mode == LayoutMode && v != LayoutMode) {
     recursively<bool>(&KstViewObject::setSelected, false);
     clearFocus();
+    paint(P_PAINT);
   } else if (_mode == DisplayMode && v != DisplayMode) {
     recursively<bool>(&KstViewObject::zoom, false);
   }
@@ -409,7 +409,7 @@ static QRect newSize(const QRect& oldSize, int direction, const QPoint& pos) {
       g.setTopRight(save);
       break;
     default:
-      abort();
+      //abort();
       break;
   }
 
@@ -892,8 +892,6 @@ void KstTopLevelView::groupSelection() {
   QRect gg = _selectionList.first()->geometry();
 
   if (pg) {
-    pg->setBorderWidth(3);
-    pg->setBorderColor(Qt::blue);
     // First build the container
     for (it = _selectionList.begin(); it != _selectionList.end(); ++it) {
       gg |= (*it)->geometry();
@@ -929,9 +927,17 @@ void KstTopLevelView::cancelMouseOperations() {
 }
 
 
+bool KstTopLevelView::removeChild(KstViewObjectPtr obj, bool recursive) {
+  if (_pressTarget == obj) {
+    _pressTarget = 0L;
+  }
+
+  return KstViewObject::removeChild(obj, recursive);
+}
+
+
 void KstTopLevelView::cleanup(int cols) {
   KstViewObject::cleanup(cols);
-  _onGrid = true;
 }
 
 
@@ -982,8 +988,9 @@ QSize KstTopLevelView::averageChildSize() {
     widths += (*i)->geometry().width();
     heights += (*i)->geometry().height();
   }
-  return QSize(widths / _children.count(), heights / _children.count());
+  return _children.count() > 0 ? QSize(widths / _children.count(), heights / _children.count()) : QSize(0, 0);
 }
 
+
 #include "ksttoplevelview.moc"
 // vim: ts=2 sw=2 et
