@@ -19,8 +19,7 @@
 #define KSTTOPLEVELVIEW_H
 
 #include <qcursor.h>
-
-#include "kstviewobject.h"
+#include <qguardedptr.h>
 
 class KPopupMenu;
 class KstViewWidget;
@@ -34,7 +33,7 @@ class KstTopLevelView : public KstViewObject {
     virtual ~KstTopLevelView();
 
     virtual UpdateType update();
-    virtual void save(QTextStream& ts);
+    virtual void save(QTextStream& ts, const QString& indent = QString::null);
 
     void release(); // Release this from it's window/view.  When you call this,
                     // you'd better be deleting this object in the next line.
@@ -44,7 +43,7 @@ class KstTopLevelView : public KstViewObject {
     virtual void paint(KstPaintType type, QPainter& p);
 
     void updateAlignment();
-    void paint(KstPaintType type, bool erase = false);
+    void paint(KstPaintType type);
 
     void clearFocus();
     enum ViewMode { LayoutMode = 0, DisplayMode, Unknown = 15 };
@@ -60,6 +59,7 @@ class KstTopLevelView : public KstViewObject {
 
     template<class T> KstSharedPtr<T> createPlot(const QString& name);
 
+    virtual bool removeChild(KstViewObjectPtr obj, bool recursive = false);
     virtual void cleanup(int cols = -1);
 
     bool mouseGrabbed() const { return _mouseGrabbed; }
@@ -97,6 +97,7 @@ class KstTopLevelView : public KstViewObject {
 
   private:
     void commonConstructor();
+    // If no children, returns (0,0)
     QSize averageChildSize();
 
     QGuardedPtr<KstViewWidget> _w;
@@ -121,9 +122,14 @@ KstSharedPtr<T> KstTopLevelView::createPlot(const QString& name) {
   if (_onGrid) {
     // FIXME: make this more powerful, preserve columns
     appendChild(plot);
-    cleanup();
+    this->cleanup(-1); // GCC 2.95/ppc bug.  Don't touch!!!
   } else {
-    plot->resize(averageChildSize());
+    QSize sz = averageChildSize();
+    if (sz != QSize(0, 0)) {
+      plot->resize(sz);
+    } else {
+      plot->resize(size());
+    }
     // First look at the overall clip mask.  If there are gaps, take the
     // biggest one and use that location.
     QRegion r = _lastClipRegion;
