@@ -15,21 +15,22 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <qmessagebox.h>
 #include <klocale.h>
 #include <kdebug.h>
 
+#include <qstylesheet.h>
+
 #include "kstcolorsequence.h"
 #include "kstcurvedialog_i.h"
 #include "kstdatacollection.h"
 #include "kstdebug.h"
-#include "kstdoc.h"
+#include "kstrvector.h"
 #include "kstvcurve.h"
 
-static const QString COLOR_XVECTOR = "X";
-static const QString COLOR_YVECTOR = "Y";
-static const QString EXVECTOR = "EX";
-static const QString EYVECTOR = "EY";
+static const QString& COLOR_XVECTOR = KGlobal::staticQString("X");
+static const QString& COLOR_YVECTOR = KGlobal::staticQString("Y");
+static const QString& EXVECTOR = KGlobal::staticQString("EX");
+static const QString& EYVECTOR = KGlobal::staticQString("EY");
 
 KstVCurve::KstVCurve(const QString &in_tag, KstVectorPtr in_X, KstVectorPtr in_Y, KstVectorPtr in_EX, KstVectorPtr in_EY, const QColor &in_color)
 : KstBaseCurve() {
@@ -49,10 +50,15 @@ KstVCurve::KstVCurve(const QString &in_tag, KstVectorPtr in_X, KstVectorPtr in_Y
 KstVCurve::KstVCurve(QDomElement &e)
 : KstBaseCurve(e) {
   QString in_tag, xname, yname, exname, eyname;
-  QColor in_color(KstColorSequence::next());
+  // QColor in_color(KstColorSequence::next(-1));
+  QColor in_color("red"); // the above line is invalid.
 
   setHasPoints(false);
-  setHasLines(true);
+  setHasLines(false);
+  setHasBars(false);
+  setLineWidth(0);
+  setLineStyle(0);
+  setBarStyle(0);
 
   /* parse the DOM tree */
   QDomNode n = e.firstChild();
@@ -71,18 +77,23 @@ KstVCurve::KstVCurve(QDomElement &e)
         eyname = e.text();
       } else if (e.tagName() == "color") {
         in_color.setNamedColor(e.text());
+
+      // the following options are only needed to change from the default
       } else if (e.tagName() == "hasLines") {
         HasLines = (e.text() != "0");
       } else if (e.tagName() == "hasPoints") {
         HasPoints = (e.text() != "0");
+      } else if (e.tagName() == "hasBars") {
+        HasBars = (e.text() != "0");
       } else if (e.tagName() == "pointType") {
         Point.setType(e.text().toInt());
       } else if (e.tagName() == "lineWidth") {
         LineWidth = e.text().toInt();
       } else if (e.tagName() == "lineStyle") {
         LineStyle = e.text().toInt();
+      } else if (e.tagName() == "barStyle") {
+        BarStyle = e.text().toInt();
       }
-
     }
     n = n.nextSibling();
   }
@@ -96,7 +107,6 @@ KstVCurve::KstVCurve(QDomElement &e)
 
 void KstVCurve::commonConstructor(const QString &in_tag, const QColor &in_color) {
   _typeString = i18n("Curve");
-  NumUsed = 0;
   Color = in_color;
   setTagName(in_tag);
 }
@@ -178,7 +188,6 @@ KstObject::UpdateType KstVCurve::update(int update_counter) {
   if (MinPosX > MaxX) {
     MinPosX = 0;
   }
-
   MaxY = VY->max();
   MinY = VY->min();
   MeanY = VY->mean();
@@ -216,7 +225,7 @@ void KstVCurve::getEYPoint(int i, double &x, double &y, double &ey) {
   ey = EY->interpolate(i, NS);
 }
 
-QString KstVCurve::getXVTag() const{
+QString KstVCurve::xVTag() const{
   if (!VX.data()) {
     return QString::null;
   } else {
@@ -224,7 +233,7 @@ QString KstVCurve::getXVTag() const{
   }
 }
 
-QString KstVCurve::getYVTag() const{
+QString KstVCurve::yVTag() const{
   if (!VY.data()) {
     return QString::null;
   } else {
@@ -232,7 +241,7 @@ QString KstVCurve::getYVTag() const{
   }
 }
 
-QString KstVCurve::getXETag() const{
+QString KstVCurve::xETag() const{
   if (!EX.data()) {
     return "<None>";
   } else {
@@ -240,7 +249,7 @@ QString KstVCurve::getXETag() const{
   }
 }
 
-QString KstVCurve::getYETag() const {
+QString KstVCurve::yETag() const {
   if (!EY.data()) {
     return "<None>";
   } else {
@@ -257,24 +266,38 @@ bool KstVCurve::hasYError() const {
 }
 
 /** Save curve information */
-void KstVCurve::save(QTextStream &ts) {
-  ts << " <curve>" << endl;
-  ts << "  <tag>" << tagName() << "</tag>" << endl;
-  ts << "  <xvectag>" << VX->tagName() << "</xvectag>" << endl;
-  ts << "  <yvectag>" << VY->tagName() << "</yvectag>" << endl;
+void KstVCurve::save(QTextStream &ts, const QString& indent) {
+  QString l2 = indent + "  ";
+  ts << indent << "<curve>" << endl;
+  ts << l2 << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+  ts << l2 << "<xvectag>" << QStyleSheet::escape(VX->tagName()) << "</xvectag>" << endl;
+  ts << l2 << "<yvectag>" << QStyleSheet::escape(VY->tagName()) << "</yvectag>" << endl;
   if (EX.data()) {
-    ts << "  <exVectag>" << EX->tagName() << "</exVectag>" << endl;
+    ts << l2 << "<exVectag>" << QStyleSheet::escape(EX->tagName()) << "</exVectag>" << endl;
   }
   if (EY.data()) {
-    ts << "  <eyVectag>" << EY->tagName() << "</eyVectag>" << endl;
+    ts << l2 << "<eyVectag>" << QStyleSheet::escape(EY->tagName()) << "</eyVectag>" << endl;
   }
-  ts << "  <color>" << Color.name() << "</color>" << endl;
-  ts << "  <hasLines>" << HasLines << "</hasLines>" << endl;
-  ts << "  <hasPoints>" << HasPoints << "</hasPoints>" << endl;
-  ts << "  <pointType>" << Point.getType() << "</pointType>" << endl;
-  ts << "  <lineWidth>" << LineWidth << "</lineWidth>" << endl;
-  ts << "  <lineStyle>" << LineStyle << "</lineStyle>" << endl;
-  ts << " </curve>" << endl;
+  ts << l2 << "<color>" << Color.name() << "</color>" << endl;
+  if (HasLines) {
+    ts << l2 << "<hasLines/>" << endl;
+    ts << l2 << "<lineWidth>" << LineWidth << "</lineWidth>" << endl;
+    ts << l2 << "<lineStyle>" << LineStyle << "</lineStyle>" << endl;
+  }
+  if (HasPoints) {
+    ts << l2 << "<hasPoints/>" << endl;
+    ts << l2 << "<pointType>" << Point.type() << "</pointType>" << endl;
+  }
+  if (HasBars) {
+    ts << l2 << "<hasBars/>" << endl;
+    ts << l2 << "<barStyle>" << BarStyle << "</barStyle>" << endl;
+  }
+//   ts << l2 << "<hasLines>" << HasLines << "</hasLines>" << endl;
+//   ts << l2 << "<hasPoints>" << HasPoints << "</hasPoints>" << endl;
+//   ts << l2 << "<pointType>" << Point.type() << "</pointType>" << endl;
+//   ts << l2 << "<lineWidth>" << LineWidth << "</lineWidth>" << endl;
+//   ts << l2 << "<lineStyle>" << LineStyle << "</lineStyle>" << endl;
+  ts << indent << "</curve>" << endl;
 }
 
 void KstVCurve::setXVector(KstVectorPtr new_vx) {
@@ -293,15 +316,15 @@ void KstVCurve::setYError(KstVectorPtr new_ey) {
   EY = new_ey;
 }
 
-QString KstVCurve::getXLabel() const {
+QString KstVCurve::xLabel() const {
   return VX->label();
 }
 
-QString KstVCurve::getYLabel() const {
+QString KstVCurve::yLabel() const {
   return VY->label();
 }
 
-QString KstVCurve::getTopLabel() const {
+QString KstVCurve::topLabel() const {
   return VY->fileLabel();
 }
 
@@ -310,7 +333,7 @@ KstCurveType KstVCurve::type() const {
 }
 
 QString KstVCurve::propertyString() const {
-  return i18n("%1 vs %2").arg(getYVTag()).arg(getXVTag());
+  return i18n("%1 vs %2").arg(yVTag()).arg(xVTag());
 }
 
 void KstVCurve::_showDialog() {
@@ -322,4 +345,13 @@ int KstVCurve::samplesPerFrame() const {
   return rvp ? rvp->samplesPerFrame() : 1;
 }
 
+bool KstVCurve::uses(KstObjectPtr p) const {
+  bool rc = KstBaseCurve::uses(p);
+  KstVectorPtr v = kst_cast<KstVector>(p);
+  if (v) {
+    rc = rc || VX == v || VY == v || EX == v || EY == v;
+  }
+  return rc;
+}
+
 // vim: ts=2 sw=2 et
