@@ -16,55 +16,17 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <ctype.h>
+#include <assert.h>
 #include <math.h>
 #include <stdlib.h>
-#include <string.h>
-#include <time.h>
-#include <assert.h>
+
+#include <qdeepcopy.h>
+
 #include <kdebug.h>
 #include <klocale.h>
-#include "kstdoc.h"
 #include "kstdatacollection.h"
+#include "kstmath.h"
 #include "kstvector.h"
-#include "kstdebug.h"
-
-#ifdef __sun
-#include <ieeefp.h>
-#endif
-
-/*
-** For systems without NAN, this is a NAN in IEEE double format.
-** Code lifted from kde screensavers.
-*/
-#if !defined(NAN)
-static inline double nan__()
-{
-  static const unsigned int one = 1;
-  static const bool BigEndian = (*((unsigned char *) &one) == 0);
-
-  static const unsigned char be_nan_bytes[] = { 0x7f, 0xf8, 0, 0, 0, 0, 0, 0 };
-  static const unsigned char le_nan_bytes[] = { 0, 0, 0, 0, 0, 0, 0xf8, 0x7f };
-
-  return *( ( const double * )( BigEndian ? be_nan_bytes : le_nan_bytes ) );
-}
-#   define NAN (::nan__())
-#endif
-
-/*
-** Both Solaris and FreeBSD-current do weird things with the
-** isnan() defined in math.h - in particular on FreeBSD it
-** gets #undeffed by the C++ math routines later. Use the
-** std:: version in those cases.
-*/
-#ifdef isnan
-#define KST_ISNAN(a)	isnan(a)
-#elif defined(__APPLE__)
-#define KST_ISNAN(a)	(a == NAN ? 1 : 0)
-#else
-  // HUH? Ok let's get rid of the silliness here sometime.
-#define KST_ISNAN(a)	isnan(a)
-#endif
 
 static int anonymousVectorCounter = 0;
 
@@ -139,11 +101,13 @@ double KstVector::interpolate(int in_i, int ns_i) {
   int j;
 
   /** Limits checks - optional? **/
-  if (in_i < 0)
+  if (in_i < 0) {
     return _v[0];
+  }
 
-  if (in_i >= ns_i - 1)
+  if (in_i >= ns_i - 1) {
     return _v[_size - 1];
+  }
 
   /** speedup check **/
   if (ns_i == _size) { /* no extrapolating or decimating needed */
@@ -189,6 +153,8 @@ void KstVector::CreateScalars() {
     _scalars["max"]->_KShared_ref();
     _scalars.insert("min", new KstScalar(tagName() + "-Min"));
     _scalars["min"]->_KShared_ref();
+    _scalars.insert("last", new KstScalar(tagName() + "-Last"));
+    _scalars["last"]->_KShared_ref();
     _scalars.insert("mean", new KstScalar(tagName() + "-Mean"));
     _scalars["mean"]->_KShared_ref();
     _scalars.insert("sigma", new KstScalar(tagName() + "-Sigma"));
@@ -210,6 +176,7 @@ void KstVector::RenameScalars() {
   if (!_isScalarList) {
     _scalars["max"]->setTagName(tagName() + "-Max");
     _scalars["min"]->setTagName(tagName() + "-Min");
+    _scalars["last"]->setTagName(tagName() + "-Last");
     _scalars["mean"]->setTagName(tagName() + "-Mean");
     _scalars["sigma"]->setTagName(tagName() + "-Sigma");
     _scalars["rms"]->setTagName(tagName() + "-Rms");
@@ -237,9 +204,14 @@ void KstVector::updateScalars() {
   }
 }
 
-double* KstVector::realloced(double *memptr, int newSize) {
+double* KstVector::realloced(double *memptr, int newSize, bool bNew) {
   double *old = _v;
   _v = memptr;
+  if (bNew) {
+    NumNew = newSize;
+  } else {
+    NumNew = newSize - _size;
+  }
   _size = newSize;
   updateScalars();
   return old;
@@ -247,6 +219,7 @@ double* KstVector::realloced(double *memptr, int newSize) {
 
 
 void KstVector::zero() {
+  _ns_min = _ns_max = 0.0;
   for (int i = 0; i < _size; i++) {
     _v[i] = 0.0;
   }
@@ -282,14 +255,28 @@ bool KstVector::resize(int sz, bool reinit) {
 
 KstObject::UpdateType KstVector::update(int update_counter) {
   int i, i0;
-  double max, min, sum, sum2, minpos, v;
+  double max, min, sum, sum2, minpos, last, v;
   double last_v;
-  double dv2=0.0, dv, no_spike_max_dv;
+  double dv2 = 0.0, dv, no_spike_max_dv;
 
-  max = min = sum = sum2 = minpos = 0.0;
+  max = min = sum = sum2 = minpos = last = 0.0;
+  if (KstObject::checkUpdateCounter(update_counter)) {
+    return KstObject::NO_CHANGE;
+  }
 
-  if (KstObject::checkUpdateCounter(update_counter))
-    return NO_CHANGE;
+  if (update_counter != -1) {
+    KstObjectPtr prov = KstObjectPtr(_provider);
+    if (prov) {
+      prov->update(update_counter);
+      /*  Update anyway, it's never possible to get NO_CHANGE because of
+       *  the guard above
+      KstObject::UpdateType u = prov->update(update_counter);
+      if (u == KstObject::NO_CHANGE) {
+        return u;
+      }
+      */
+    }
+  }
 
   _nsum = 0;
   if (_size > 0) {
@@ -298,7 +285,6 @@ KstObject::UpdateType KstVector::update(int update_counter) {
     // FIXME: expensive!!  Can we somehow cache this at least?
     for (i = 0; i < _size && !finite(_v[i]); i++)
       /* Do Nothing */ ;
-
     if (i == _size) {
       if (!_isScalarList) {
         _scalars["sum"]->setValue(sum);
@@ -306,6 +292,7 @@ KstObject::UpdateType KstVector::update(int update_counter) {
         _scalars["max"]->setValue(max);
         _scalars["min"]->setValue(min);
         _scalars["minpos"]->setValue(minpos);
+        _scalars["last"]->setValue(last);
       }
       _ns_max = _ns_min = 0;
 
@@ -354,9 +341,9 @@ KstObject::UpdateType KstVector::update(int update_counter) {
           if (v < min) {
             min = v;
           }
-          if (v < minpos && v > 0) {
-              minpos = v;
-          }
+        }
+        if (v < minpos && v > 0) {
+            minpos = v;
         }
       } else {
         _is_rising = false;
@@ -367,6 +354,8 @@ KstObject::UpdateType KstVector::update(int update_counter) {
 
     _ns_max = _ns_min = last_v = _v[i0];
 
+    last = _v[_size-1];
+
     for (i = i0; i < _size; i++) {
       v = _v[i]; // get rid of redirections
       if (finite(v)) {
@@ -393,6 +382,7 @@ KstObject::UpdateType KstVector::update(int update_counter) {
       _scalars["max"]->setValue(max);
       _scalars["min"]->setValue(min);
       _scalars["minpos"]->setValue(minpos);
+      _scalars["last"]->setValue(last);
     }
 
     updateScalars();
@@ -403,10 +393,11 @@ KstObject::UpdateType KstVector::update(int update_counter) {
   return NO_CHANGE;
 }
 
-void KstVector::save(QTextStream &ts) {
-  Q_UNUSED(ts);
-
-assert(0);
+void KstVector::save(QTextStream &ts, const QString& indent, bool saveAbsolutePosition) {
+  Q_UNUSED(ts)
+  Q_UNUSED(indent)
+  Q_UNUSED(saveAbsolutePosition)
+  abort();
 }
 
 
@@ -525,5 +516,29 @@ KstVectorPtr KstVector::generateVector(double x0, double x1, int n, const QStrin
   return xv;
 }
 
+
+void KstVector::setProvider(KstObject* obj) {
+  _provider = obj;
+}
+
+
+bool KstVector::deleteDependents() {
+  KST::dataObjectList.lock().readLock();
+  KstDataObjectList dol = QDeepCopy<KstDataObjectList>(KST::dataObjectList);
+  KST::dataObjectList.lock().readUnlock();
+  for (KstDataObjectList::Iterator i = dol.begin(); i != dol.end(); ++i) {
+    bool user = (*i)->uses(this);
+    if (user) {
+      KstDataObjectPtr dop = *i;
+      KST::dataObjectList.lock().writeLock();
+      KST::dataObjectList.remove(dop);
+      KST::dataObjectList.lock().writeUnlock();
+      dop->deleteDependents();
+    }
+  } 
+
+  return true;
+}
+
 #include "kstvector.moc"
 // vim: et sw=2 ts=2
