@@ -17,7 +17,7 @@
 
 #include "kstvectordefaults.h"
 #include "kstrvector.h"
-#include "kstpsdcurve.h"
+#include "kstpsd.h"
 #include "kstdatacollection.h"
 #include "stdinsource.h"
 
@@ -97,7 +97,7 @@ void KstVectorDefaults::sync() {
     vl[j]->readLock();
     KstDataSourcePtr dsp = vl[j]->dataSource();
     vl[j]->readUnlock();
-    if (dsp && !dynamic_cast<KstStdinSource*>(dsp.data())) {
+    if (dsp && !kst_cast<KstStdinSource>(dsp)) {
       break;
     }
     --j;
@@ -114,19 +114,19 @@ void KstVectorDefaults::sync() {
     vl[j]->readUnlock();
   }
 
-  KstPSDCurveList pl = kstObjectSubList<KstDataObject, KstPSDCurve>(KST::dataObjectList);
+  KstPSDList pl = kstObjectSubList<KstDataObject, KstPSD>(KST::dataObjectList);
 
   j = pl.count() - 1;
   if (j >= 0) {
     pl[j]->readLock();
-    _psd_freq = pl[j]->getFreq();
-    _fft_len = pl[j]->getLen();
+    _psd_freq = pl[j]->freq();
+    _fft_len = pl[j]->len();
 
     _vUnits = pl[j]->VUnits;
     _rUnits = pl[j]->RUnits;
-    _apodize = pl[j]->getApodize();
-    _removeMean = pl[j]->getRemoveMean();
-    _psd_average = pl[j]->getAverage();
+    _apodize = pl[j]->apodize();
+    _removeMean = pl[j]->removeMean();
+    _psd_average = pl[j]->average();
 
     pl[j]->readUnlock();
   }
