@@ -16,27 +16,21 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <assert.h>
+
 #include <qcheckbox.h>
-#include <qlabel.h>
-#include <qlineedit.h>
-#include <qpushbutton.h>
 #include <qspinbox.h>
-#include <qstring.h>
-#include <qtoolbutton.h>
-#include <qwidget.h>
 
 #include <kcombobox.h>
-#include <kcompletion.h>
-#include <kfiledialog.h>
+#include <kdialogbase.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kstaticdeleter.h>
+#include <kurlcompletion.h>
 #include <kurlrequester.h>
 
 #include "datarangewidget.h"
 #include "kstdatacollection.h"
-#include "kstdoc.h"
-#include "kstrvector.h"
 #include "kstvectordefaults.h"
 #include "kstvectordialog_i.h"
 
@@ -54,28 +48,39 @@ KstVectorDialogI *KstVectorDialogI::globalInstance() {
   return _inst;
 }
 
+
 KstVectorDialogI::KstVectorDialogI(QWidget* parent, const char* name,
                                    bool modal, WFlags fl)
 : KstVectorDialog(parent, name, modal, fl) {
+  FileName->completionObject()->setDir(QDir::currentDirPath());
   Init();
 
   FileName->setMode(KFile::File | KFile::Directory | KFile::ExistingOnly
                     | KFile::LocalOnly);
   connect(FileName, SIGNAL(textChanged(const QString&)),
           this, SLOT(updateCompletion()));
+  connect(_configure, SIGNAL(clicked()), this, SLOT(configureSource()));
+  _configure->setEnabled(false);
   _fieldCompletion = Field->completionObject();
   Field->setAutoDeleteCompletionObject(true);
+  setFixedHeight(height());
+  _configWidget = 0L;
 }
 
+
 KstVectorDialogI::~KstVectorDialogI() {
+  delete _configWidget;
+  _configWidget = 0L;
   DP = 0L;
 }
 
+
 KstRVectorPtr KstVectorDialogI::_getPtr(const QString &tagin) {
   KstRVectorList v = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   return *v.findTag(tagin);
 }
 
+
 void KstVectorDialogI::updateCompletion() {
   QString current_text = Field->currentText();
   Field->clear();
@@ -85,18 +90,27 @@ void KstVectorDialogI::updateCompletion() {
   KstDataSourcePtr ds = *KST::dataSourceList.findFileName(FileName->url());
   KST::dataSourceList.lock().readUnlock();
 
+  delete _configWidget; // FIXME: very inefficient!!!!
+  _configWidget = 0L;
   QStringList list;
   if (ds) {
     ds->readLock();
     list = ds->fieldList();
     Field->setEditable(!ds->fieldListIsComplete());
+    _configWidget = ds->configWidget();
     ds->readUnlock();
   } else {
-    list = KstDataSource::fieldListForSource(FileName->url());
-    // FIXME find a better solution here
-    Field->setEditable(true);
+    QString type;
+    bool complete = false;
+    list = KstDataSource::fieldListForSource(FileName->url(), QString::null, &type, &complete);
+    Field->setEditable(!complete);
+    if (!list.isEmpty() && !type.isEmpty()) {
+      _configWidget = KstDataSource::configWidgetForSource(FileName->url(), type);
+    }
   }
 
+  _configure->setEnabled(_configWidget);
+
   _fieldCompletion = Field->completionObject();
 
   Field->insertStringList(list);
@@ -109,6 +123,7 @@ void KstVectorDialogI::updateCompletion() {
   }
 }
 
+
 void KstVectorDialogI::_fillFieldsForEdit() {
   if (DP == 0L) {
     return; // shouldn't be needed
@@ -123,8 +138,8 @@ void KstVectorDialogI::_fillFieldsForEdit() {
   if (_fieldCompletion) {
     _fieldCompletion->clear();
   }
-  KstDataSourcePtr tf;
   {
+    KstDataSourcePtr tf;
     KST::dataSourceList.lock().readLock();
     KstDataSourceList::Iterator it = KST::dataSourceList.findFileName(DP->filename());
     if (it != KST::dataSourceList.end()) {
@@ -174,6 +189,7 @@ void KstVectorDialogI::_fillFieldsForEdit() {
   DP->readUnlock();
 }
 
+
 void KstVectorDialogI::_fillFieldsForNew() {
   KstRVectorList vectorList = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
 
@@ -197,10 +213,12 @@ void KstVectorDialogI::_fillFieldsForNew() {
   Field->setFocus();
 }
 
+
 void KstVectorDialogI::update() {
   /* no longer does anything (?) */
 }
 
+
 bool KstVectorDialogI::new_I() {
   KstDataSourcePtr file;
   KstRVectorList vectorList = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
@@ -227,12 +245,12 @@ bool KstVectorDialogI::new_I() {
       file = KstDataSource::loadSource(FileName->url());
       if (!file || !file->isValid()) {
         KST::dataSourceList.lock().writeUnlock();
-        KMessageBox::sorry(0L, i18n("The file could not be loaded."));
+        KMessageBox::sorry(this, i18n("The file could not be loaded."));
         return false;
       }
       if (file->isEmpty()) {
         KST::dataSourceList.lock().writeUnlock();
-        KMessageBox::sorry(0L, i18n("The file does not contain data."));
+        KMessageBox::sorry(this, i18n("The file does not contain data."));
         return false;
       }
       KST::dataSourceList.append(file);
@@ -244,7 +262,7 @@ bool KstVectorDialogI::new_I() {
   file->readLock();
   if (!file->isValidField(Field->currentText())) {
     file->readUnlock();
-    KMessageBox::sorry(0L, i18n("The requested field is not defined for the requested file."));
+    KMessageBox::sorry(this, i18n("The requested field is not defined for the requested file."));
     return false;
   }
   file->readUnlock();
@@ -268,6 +286,7 @@ bool KstVectorDialogI::new_I() {
   return true;
 }
 
+
 bool KstVectorDialogI::edit_I() {
   KstDataSourcePtr file;
 
@@ -291,12 +310,12 @@ bool KstVectorDialogI::edit_I() {
       file = KstDataSource::loadSource(FileName->url());
       if (!file || !file->isValid()) {
         KST::dataSourceList.lock().writeUnlock();
-        KMessageBox::sorry(0L, i18n("The file could not be opened."));
+        KMessageBox::sorry(this, i18n("The file could not be opened."));
         return false;
       }
       if (file->isEmpty()) {
         KST::dataSourceList.lock().writeUnlock();
-        KMessageBox::sorry(0L, i18n("The file does not contain data."));
+        KMessageBox::sorry(this, i18n("The file does not contain data."));
         return false;
       }
       KST::dataSourceList.append(file);
@@ -308,7 +327,7 @@ bool KstVectorDialogI::edit_I() {
   file->readLock();
   if (!file->isValidField(Field->currentText())) {
     file->readUnlock();
-    KMessageBox::sorry(0L, i18n("The requested field is not defined for "
+    KMessageBox::sorry(this, i18n("The requested field is not defined for "
                                 "the requested file\n"));
     return false;
   }
@@ -332,5 +351,20 @@ bool KstVectorDialogI::edit_I() {
   return true;
 }
 
+
+void KstVectorDialogI::configureSource() {
+  assert(_configWidget);
+  KDialogBase *dlg = new KDialogBase(this, "Data Config Dialog", true, i18n("Configure Data Source"));
+  connect(dlg, SIGNAL(okClicked()), _configWidget, SLOT(save()));
+  connect(dlg, SIGNAL(applyClicked()), _configWidget, SLOT(save()));
+  _configWidget->reparent(dlg, QPoint(0, 0));
+  dlg->setMainWidget(_configWidget);
+  dlg->exec();
+  _configWidget->reparent(0L, QPoint(0, 0));
+  dlg->setMainWidget(0L);
+  delete dlg;
+}
+
+
 #include "kstvectordialog_i.moc"
 // vim: sw=2 ts=2 et
