@@ -16,10 +16,8 @@
  ***************************************************************************/
 
 #include <qpainter.h>
-#include <qpalette.h>
 #include <qtable.h>
 #include "kstvectortable.h"
-#include "kstrvector.h"
 #include "kstdatacollection.h"
 
 KstVectorTable::KstVectorTable( QWidget * parent, const char * name ) : QTable( parent, name ) {
