@@ -17,23 +17,16 @@
 
 #include <assert.h>
 #include <math.h>
-#include <stdio.h>
+#include <stdlib.h>
 
 // include files for Qt
-#include <qmessagebox.h>
-#include <qpainter.h>
-#include <qpen.h>
+#include <qstylesheet.h>
 
 // include files for KDE
 #include <kdatastream.h>
 #include <kdebug.h>
-#include <klineeditdlg.h>
-#include <klocale.h>
-#include <kmdimainfrm.h>
-#include <kpopupmenu.h>
 
 // application specific includes
-#include "kst2dplot.h"
 #include "kst.h"
 #include "kstdoc.h"
 #include "kstplotgroup.h"
@@ -43,6 +36,7 @@
 #include "kstviewwindow.h"
 
 KstViewObject::KstViewObject(const QString& type) : KstObject(), _geom(0, 0, 1, 1), _type(type) {
+  _parent = 0L;
   _standardActions = 0;
   _layoutActions = 0;
   _maximized = false;
@@ -59,6 +53,7 @@ KstViewObject::KstViewObject(const QString& type) : KstObject(), _geom(0, 0, 1,
 KstViewObject::KstViewObject(QDomElement& e) : KstObject() {
   _foregroundColor = KstSettings::globalSettings()->foregroundColor;
   _backgroundColor = KstSettings::globalSettings()->backgroundColor;
+  _parent = 0L;
   load(e);
 }
 
@@ -68,7 +63,7 @@ void KstViewObject::load(QDomElement& e) {
   _standardActions = 0;
   _layoutActions = 0;
   _maximized = false;
-  _onGrid = true;
+  _onGrid = false;
   _columns = 0;
   _focus = false;
   _selected = false;
@@ -105,8 +100,8 @@ void KstViewObject::loadChildren(QDomElement& e) {
   QDomNode n = e.firstChild();
 
   while (!n.isNull()) {
-    QDomElement el = n.toElement(); // try to convert the node to an element.
-    if (!el.isNull()) { // the node was really an element.
+    QDomElement el = n.toElement();
+    if (!el.isNull()) {
       if (el.tagName() == "plot") {
         QString in_tag = el.text();
         if (KstApp::inst()->plotHolderWhileOpeningDocument().count(in_tag) > 0) {
@@ -133,7 +128,7 @@ KstObject::UpdateType KstViewObject::update(int) {
 }
 
 
-void KstViewObject::save(QTextStream& ts) {
+void KstViewObject::save(QTextStream& ts, const QString& indent) {
   KstAspectRatio aspect;
 
   if (_maximized) {
@@ -142,21 +137,19 @@ void KstViewObject::save(QTextStream& ts) {
     aspect = _aspect;
   }
 
-  ts << "  <tag>" << tagName() << "</tag>" << endl;
-  ts << "  <aspect x=\"" << aspect.x <<
-               "\" y=\"" << aspect.y <<
-               "\" w=\"" << aspect.w <<
-               "\" h=\"" << aspect.h << "\" />" << endl;
-  if (_onGrid && _columns > 0) { // only if on a grid and we have contents
-    ts << "  <columns>" << _columns << "</columns>" << endl;
-  }
+  ts << indent << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+  ts << indent << "<aspect x=\"" << aspect.x <<
+    "\" y=\"" << aspect.y <<
+    "\" w=\"" << aspect.w <<
+    "\" h=\"" << aspect.h << "\" />" << endl;
+
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    (*i)->save(ts);
+    (*i)->save(ts, indent + "  ");
   }
 }
 
 
-void KstViewObject::saveTag(QTextStream& ts) {
+void KstViewObject::saveTag(QTextStream& ts, const QString& indent) {
   KstAspectRatio aspect;
 
   if (_maximized) {
@@ -165,13 +158,13 @@ void KstViewObject::saveTag(QTextStream& ts) {
     aspect = _aspect;
   }
 
-  ts << "  <tag>" << tagName() << "</tag>" << endl;
-  ts << "  <aspect x=\"" << aspect.x <<
-               "\" y=\"" << aspect.y <<
-               "\" w=\"" << aspect.w <<
-               "\" h=\"" << aspect.h << "\" />" << endl;
+  ts << indent << "<tag>" << QStyleSheet::escape(tagName()) << "</tag>" << endl;
+  ts << indent << "<aspect x=\"" << aspect.x <<
+    "\" y=\"" << aspect.y <<
+    "\" w=\"" << aspect.w <<
+    "\" h=\"" << aspect.h << "\" />" << endl;
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    (*i)->saveTag(ts);
+    (*i)->saveTag(ts, indent + "  ");
   }
 }
 
@@ -198,20 +191,38 @@ void KstViewObject::writeUnlock() const {
 void KstViewObject::paint(KstPaintType type, QPainter& p) {
   bool hadClipping = p.hasClipping();
   QRegion oldRegion = p.clipRegion();
+  bool maximized = false;
 
+  //
+  // handle the case where we have maximized plots
+  //
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    // Compute the clip reg - this can't be stored if we want to keep the code
-    // generic, but it's slow, so it's a bit of a conundrum
-    QRegion clipRegion = oldRegion;
-    KstViewObjectList::Iterator j = i;
-    for (++j; j != _children.end(); ++j) {
-      clipRegion -= QRegion((*j)->geometry());
+    if ((*i)->_maximized) {
+      (*i)->_lastClipRegion = (*i)->geometry();
+      (*i)->paint(type, p);
+      maximized = true;
+      break;
     }
-    (*i)->_lastClipRegion = clipRegion;
-    p.setClipRegion(clipRegion);
-    (*i)->paint(type, p);
   }
 
+  if (!maximized) {
+    for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+      // Compute the clip reg - this can't be stored if we want to keep the code
+      // generic, but it's slow, so it's a bit of a conundrum
+      QRegion clipRegion = oldRegion;
+      KstViewObjectList::Iterator j = i;
+      for (++j; j != _children.end(); ++j) {
+        clipRegion -= QRegion((*j)->geometry());
+      }
+      (*i)->_lastClipRegion = clipRegion;
+      p.setClipRegion(clipRegion);
+      (*i)->paint(type, p);
+      if ((*i)->_maximized) {
+        break;
+      }
+    }
+  }
+  
   p.setClipRegion(oldRegion);
 
   if (_focus) {
@@ -282,17 +293,15 @@ void KstViewObject::prependChild(KstViewObjectPtr obj, bool keepAspect) {
 
 
 bool KstViewObject::removeChild(KstViewObjectPtr obj, bool recursive) {
-  uint rc = _children.remove(obj);
-  if (rc == 0 && recursive) {
+  bool rc = _children.remove(obj) > 0;
+  if (recursive) {
     for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-      if ((*i)->removeChild(obj, true)) {
-        obj->_parent = 0L;
-        return true;
-      }
+      rc = rc && (*i)->removeChild(obj, true);
     }
   }
 
-  return rc > 0;
+  obj->_parent = 0L;
+  return rc;
 }
 
 
@@ -331,6 +340,8 @@ KstViewObjectList& KstViewObject::children() {
 
 
 void KstViewObject::resize(const QSize& size) {
+  assert(size.width() > 0);
+  assert(size.height() > 0);
   _geom.setSize(size);
   updateAspect();
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
@@ -338,6 +349,7 @@ void KstViewObject::resize(const QSize& size) {
   }
 }
 
+
 void KstViewObject::resizeForPrint(const QSize& size) {
   _geomOld = _geom;
   _geom.setSize(size);
@@ -346,6 +358,7 @@ void KstViewObject::resizeForPrint(const QSize& size) {
   }
 }
 
+
 void KstViewObject::revertForPrint() {
   _geom = _geomOld;
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
@@ -353,6 +366,7 @@ void KstViewObject::revertForPrint() {
   }
 }
 
+
 void KstViewObject::resizeFromAspect(double x, double y, double w, double h) {
   _aspect.x = x;
   _aspect.y = y;
@@ -366,10 +380,15 @@ QSize KstViewObject::size() const {
   return _geom.size();
 }
 
-double KstViewObject::xInternalAlignment() {
-    return ( 0 );
+
+void KstViewObject::internalAlignment(QRect& plotRegion) {
+  plotRegion.setLeft(0);
+  plotRegion.setRight(0);
+  plotRegion.setTop(0);
+  plotRegion.setLeft(0);
 }
 
+
 void KstViewObject::parentResized() {
   updateFromAspect();
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
@@ -377,6 +396,7 @@ void KstViewObject::parentResized() {
   }
 }
 
+
 void KstViewObject::parentResizedForPrint() {
   _geomOld = _geom;
   updateFromAspect();
@@ -385,6 +405,7 @@ void KstViewObject::parentResizedForPrint() {
   }
 }
 
+
 void KstViewObject::parentRevertedForPrint() {
   _geom = _geomOld;
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
@@ -392,73 +413,92 @@ void KstViewObject::parentRevertedForPrint() {
   }
 }
 
+
 void KstViewObject::setBackgroundColor(const QColor& color) {
   _backgroundColor = color;
 }
 
+
 QColor KstViewObject::backgroundColor() const {
   return _backgroundColor;
 }
 
+
 void KstViewObject::setForegroundColor(const QColor& color) {
   _foregroundColor = color;
 }
 
+
 QColor KstViewObject::foregroundColor() const {
   return _foregroundColor;
 }
 
+
 int KstViewObject::columns() const {
   return _columns;
 }
 
+
 void KstViewObject::setColumns(int cols) {
   _columns = cols;
 }
 
+
 bool KstViewObject::onGrid() const {
   return _onGrid;
 }
 
+
 void KstViewObject::setOnGrid(bool on_grid) {
   _onGrid = on_grid;
 }
 
+
 void KstViewObject::cleanup(int cols) {
   if (_children.count() < 1) {
     return;
   }
-  if (cols <= 0) {
-    cols = int(sqrt(_children.count()));
-  }
 
-  _onGrid = true;
-  _columns = cols;
+  if (!_onGrid) {
+    if (cols <= 0) {
+      cols = int(sqrt(_children.count()));
+    }
+    _onGrid = true;
+    _columns = QMAX(1, cols);
+  } else {
+    if (cols > 0) {
+      _columns = cols;
+    } else if (_columns <= 0) {
+      _columns = QMAX(1, int(sqrt(_children.count())));
+    }
+  }
 
   int pos = 0;
   int x = 0;
   int y = 0;
-  int w = _geom.width() / cols;
-  int lastRow = _children.count() / cols;
-  int h = _geom.height() / (lastRow + (_children.count() % cols > 0 ? 1 : 0));
+  int w = _geom.width() / _columns;
+  int lastRow = _children.count() / _columns;
+  int h = _geom.height() / (lastRow + (_children.count() % _columns > 0 ? 1 : 0));
 
+  //kdDebug() << "cleanup with w=" << w << " and h=" << h << endl;
+  //kdDebug() << "columns=" << _columns  << endl;
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     QSize sz(w, h);
     QPoint pt(x, y);
     // Adjust the last column to be sure that we don't spill over
-    if (pos % cols == cols - 1) {
+    if (pos % _columns == _columns - 1) {
       sz.setWidth(_geom.width() - x);
     }
 
     // Adjust the last row to be sure that we don't spill over
-    if ((pos + 1) / cols > lastRow) {
+    if ((pos + 1) / _columns > lastRow) {
       sz.setHeight(_geom.height() - y);
     }
 
     (*i)->move(pt);
     (*i)->resize(sz);
 
-    if (++pos % cols == 0) {
+    if (++pos % _columns == 0) {
       x = 0;
       y += h;
     } else {
@@ -473,13 +513,14 @@ QPoint KstViewObject::position() const {
 }
 
 
-void KstViewObject::parentMoved( ) {
+void KstViewObject::parentMoved(const QPoint& offset) {
   updateFromAspect();
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    (*i)->parentMoved();
+    (*i)->parentMoved(offset);
   }
 }
 
+
 void KstViewObject::move(const QPoint& pos) {
   QPoint offset = pos - _geom.topLeft();
 
@@ -487,7 +528,7 @@ void KstViewObject::move(const QPoint& pos) {
     _geom.moveTopLeft(pos);
     updateAspectPos( );
     for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-      (*i)->parentMoved( );
+      (*i)->parentMoved(offset);
     }
   }
 }
@@ -513,7 +554,7 @@ void KstViewObject::recursively(void (KstViewObject::*method)(), bool self) {
 }
 
 
-KstViewObjectPtr KstViewObject::findChild(const QString& name) {
+KstViewObjectPtr KstViewObject::findChild(const QString& name, bool recursive) {
   if (tagName() == name || _children.isEmpty()) {
     return KstViewObjectPtr();
   }
@@ -524,9 +565,11 @@ KstViewObjectPtr KstViewObject::findChild(const QString& name) {
     if ((*i)->tagName() == name) {
       return *i;
     }
-    rc = (*i)->findChild(name);
-    if (rc) {
-      return rc;
+    if (recursive) {
+      rc = (*i)->findChild(name, recursive);
+      if (rc) {
+        return rc;
+      }
     }
     if (i == _children.begin()) {
       break;
@@ -538,21 +581,28 @@ KstViewObjectPtr KstViewObject::findChild(const QString& name) {
 
 
 KstViewObjectPtr KstViewObject::findChild(const QPoint& pos) {
+  KstViewObjectPtr obj;
+
   if (!_geom.contains(pos) || _children.isEmpty()) {
-    return KstViewObjectPtr();
+    return obj;
   }
 
   KstViewObjectList::Iterator i = _children.end();
   for (--i; ; --i) {
     if (QRect((*i)->position(), (*i)->size()).contains(pos)) {
-      return *i;
+      if (obj == 0L || (*i)->_maximized) {
+        obj = *i;
+        if ((*i)->_maximized) {
+          break;
+        }
+      }
     }
     if (i == _children.begin()) {
       break;
     }
   }
 
-  return KstViewObjectPtr();
+  return obj;
 }
 
 
@@ -560,14 +610,17 @@ const QRect& KstViewObject::geometry() const {
   return _geom;
 }
 
+
 QRect KstViewObject::contentsRect() const {
   return _geom;
 }
 
+
 void KstViewObject::setContentsRect(QRect& rect) {
   _geom = rect;
 }
 
+
 bool KstViewObject::popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topParent) {
   Q_UNUSED(pos)
   Q_UNUSED(topParent)
@@ -666,7 +719,7 @@ bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstView
     KMdiIterator<KMdiChildView*> *it = KstApp::inst()->createIterator();
     while (it->currentItem()) {
       KstViewWindow *c = dynamic_cast<KstViewWindow*>(it->currentItem());
-      KstTopLevelViewPtr tlv = dynamic_cast<KstTopLevelView*>(topParent.data());
+      KstTopLevelViewPtr tlv = kst_cast<KstTopLevelView>(topParent);
 
       if (c && (!tlv || tlv != c->view())) {
         hasEntry = true;
@@ -691,9 +744,15 @@ bool KstViewObject::layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstView
 void KstViewObject::deleteObject() {
   // FIXME: eliminate _parent!!
   KstApp::inst()->document()->setModified();
+  KstViewObjectPtr vop(this);
   if (_parent) {
     _parent->removeChild(this);
+    _parent = 0L;
   }
+  while (!_children.isEmpty()) {
+    removeChild(_children.first());
+  }
+  vop = 0L; // basically "delete this;"
   QTimer::singleShot(0, KstApp::inst(), SLOT(updateDialogs()));
 }
 
@@ -771,15 +830,15 @@ void KstViewObject::lower() {
 void KstViewObject::moveTo(int id) {
   // FIXME: eliminate _parent!!
   KstApp::inst()->document()->setModified();
-  QString strWindow = _moveToMap[id];
-  KMdiChildView*  pWindow;
-  KstViewWindow*  pView;
+  QString windowName = _moveToMap[id];
+  KMdiChildView *mdiChild;
+  KstViewWindow *window;
 
-  if (!strWindow.isEmpty()) {
-    pWindow = KstApp::inst()->findWindow(strWindow);
-    if (pWindow) {
-      pView = dynamic_cast<KstViewWindow*>(pWindow);
-      if (pView) {
+  if (!windowName.isEmpty()) {
+    mdiChild = KstApp::inst()->findWindow(windowName);
+    if (mdiChild) {
+      window = dynamic_cast<KstViewWindow*>(mdiChild);
+      if (window) {
         if (_parent) {
           KstViewObjectPtr t = this;
           KstViewObjectList::Iterator it = _parent->_children.find(t);
@@ -787,8 +846,8 @@ void KstViewObject::moveTo(int id) {
           if (it != _parent->_children.end()) {
             _parent->_children.remove(it);
 
-            pView->view()->appendChild(t,true);
-            pView->view()->paint(P_PAINT);
+            window->view()->appendChild(t, true);
+            window->view()->paint(P_PAINT);
           }
         }
       }
@@ -819,6 +878,7 @@ void KstViewObject::updateAspectPos() {
   }
 }
 
+
 void KstViewObject::updateAspectSize() {
   // FIXME: eliminate _parent!!
   if (_parent) {
@@ -853,15 +913,8 @@ void KstViewObject::setSelected(bool selected) {
 
 
 void KstViewObject::zoom(bool zoom) {
-  if (_maximized && !zoom) {
-    _maximized = false;
-    _aspect = _aspectOldZoomedObject;
-    updateFromAspect();
-  } else if (!_maximized && zoom) {
-    _maximized = true;
-    _aspectOldZoomedObject = _aspect;
-    resizeFromAspect(0.0, 0.0, 1.0, 1.0);
-    raiseToTop();
+  if (_maximized != zoom) {
+    zoomToggle();
   }
 }
 
@@ -874,6 +927,7 @@ void KstViewObject::zoomToggle() {
       _parent->zoomToggle();
     }
     updateFromAspect();
+    setOnGrid(_prevOnGrid);
   } else {
     _maximized = true;
     _aspectOldZoomedObject = _aspect;
@@ -881,7 +935,8 @@ void KstViewObject::zoomToggle() {
       _parent->zoomToggle();
     }
     resizeFromAspect(0.0, 0.0, 1.0, 1.0);
-    raiseToTop();
+    _prevOnGrid = onGrid();
+    setOnGrid(false);
   }
 
   for (KstViewObjectList::Iterator it = _children.begin(); it != _children.end(); ++it) {
@@ -975,7 +1030,7 @@ void KstViewObject::readBinary(QDataStream& str) {
     KstViewObjectPtr o = KstViewObjectFactory::self()->createA(type);
     if (o.data()) {
       str >> o;
-      appendChild( o, true );
+      appendChild(o, true);
     } else {
       abort();
       // FIXME: can't decode this one!  How to recover?
@@ -994,6 +1049,16 @@ bool KstViewObject::mouseHandler() const {
 }
 
 
+void KstViewObject::setHasFocus(bool hasFocus) {
+  _hasFocus = hasFocus;
+}
+
+
+void KstViewObject::removeFocus(QPainter& p) {
+  Q_UNUSED(p)
+}
+
+
 void KstViewObject::mouseMoveEvent(QWidget *view, QMouseEvent *e) {
   Q_UNUSED(view)
   Q_UNUSED(e)
@@ -1054,6 +1119,7 @@ void KstViewObject::selectAll() {
   }
 }
 
+
 void KstViewObject::unselectAll() {
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
     (*i)->setSelected(false);
