@@ -18,13 +18,8 @@
 #ifndef KSTVIEWOBJECT_H
 #define KSTVIEWOBJECT_H
 
-#include <assert.h>
-#include <stdlib.h>
-
 #include <qcolor.h>
 #include <qdom.h>
-#include <qpoint.h>
-#include <qsize.h>
 
 #include <kdeversion.h>
 #if KDE_VERSION >= KDE_MAKE_VERSION(3,2,0)
@@ -45,7 +40,7 @@ typedef KstViewObjectPtr (*KstViewObjectFactoryMethod)();
 // P_PAINT: Neither 'data' nor 'plot' needs to change
 // P_PLOT: data didn't change
 // P_DATA: data changed
-typedef enum { P_PAINT, P_DATA, P_PLOT, P_ZOOM, P_PRINT, P_EXPORT } KstPaintType;
+enum KstPaintType { P_PAINT, P_DATA, P_PLOT, P_ZOOM, P_PRINT, P_EXPORT };
 
 struct KstAspectRatio {
   KstAspectRatio() : x(0.0), y(0.0), w(0.0), h(0.0) {}
@@ -69,8 +64,8 @@ class KstViewObject : public KstObject {
     virtual ~KstViewObject();
 
     virtual UpdateType update(int = -1);
-    virtual void save(QTextStream& ts);
-    virtual void saveTag(QTextStream& ts);
+    virtual void save(QTextStream& ts, const QString& indent = QString::null);
+    virtual void saveTag(QTextStream& ts, const QString& indent = QString::null);
     virtual void loadChildren(QDomElement& e);
 
     // If cols <= 0, optimal is chosen automatically
@@ -85,7 +80,7 @@ class KstViewObject : public KstObject {
     virtual void revertForPrint();
     virtual void resizeFromAspect(double x, double y, double w, double h);
     virtual QSize size() const;
-    virtual double xInternalAlignment();
+    virtual void internalAlignment(QRect& plotRegion);
     virtual QPoint position() const;
     virtual const QRect& geometry() const;
     virtual QRect contentsRect() const;
@@ -95,6 +90,8 @@ class KstViewObject : public KstObject {
     // Draw a focus highlight
     virtual void setFocus(bool focus);
     virtual bool focused() const;
+    virtual void setHasFocus(bool hasFocus);
+    virtual void removeFocus(QPainter& p);
 
     virtual void readLock() const;
     virtual void readUnlock() const;
@@ -111,9 +108,8 @@ class KstViewObject : public KstObject {
 
     // can't be const due to KstViewObjectPtr?
     KstViewObjectPtr findChild(const QPoint& pos);
-    KstViewObjectPtr findChild(const QString& name);
+    KstViewObjectPtr findChild(const QString& name, bool recursive = true);
     bool contains(KstViewObjectPtr child);
-    template<class T> KstViewObjectPtr findChildType(const QPoint& pos);
     template<class T> KstObjectList<KstSharedPtr<T> > findChildrenType(bool recursive = false);
 
     virtual void setBackgroundColor(const QColor& color);
@@ -127,6 +123,9 @@ class KstViewObject : public KstObject {
     void recursively(void (KstViewObject::*)(), bool self = false);
     template<class T, class U> void recursively(void (U::*)(T), T, bool self = false);
     template<class T, class U> void forEachChild(void (U::*)(T), T, bool self = false);
+    // GCC bug: compiler crashes if forEachChild is used here too
+    // note also: self=false
+    template<class U> void forEachChild2(void (U::*)());
 
     virtual bool popupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topParent);
     virtual bool layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topParent);
@@ -145,7 +144,7 @@ class KstViewObject : public KstObject {
                            Edit =         1024,
                            Zoom =         2048,
                            Pause =        4096,
-                           MoveTo =       8192,
+                           MoveTo =       8192
                         };
 
     virtual void drawFocusRect(QPainter& p);
@@ -156,7 +155,7 @@ class KstViewObject : public KstObject {
     virtual void setSelected(bool selected);
     void selectAll();
     void unselectAll();
-
+    
     virtual void recursivelyQuery(bool (KstViewObject::*method)() const, KstViewObjectList& list, bool matchRecurse = false);
 
     virtual void detach(); // remove from its parent
@@ -185,12 +184,13 @@ class KstViewObject : public KstObject {
     virtual void updateAspectSize();
     virtual void updateAspectPos();
     virtual void updateAspect();
+    virtual void zoomToggle();
 
   protected slots:
     virtual void parentResized();
     virtual void parentResizedForPrint();
     virtual void parentRevertedForPrint();
-    virtual void parentMoved();
+    virtual void parentMoved(const QPoint& offset);
 
     /***********  Actions ************/
     virtual void deleteObject();
@@ -200,7 +200,6 @@ class KstViewObject : public KstObject {
     virtual void lower();
     virtual void moveTo(int);
     virtual void rename();
-    virtual void zoomToggle();
     virtual void pauseToggle();
 
   protected:
@@ -212,12 +211,15 @@ class KstViewObject : public KstObject {
     QRect _geom;
     QRect _geomOld;
     QColor _backgroundColor, _foregroundColor; // FIXME: use QColorGroup
+    bool _hasFocus : 1;
     bool _focus : 1;
     bool _selected : 1;
     bool _onGrid : 1;
+    bool _prevOnGrid : 1;
     bool _maximized : 1;
     int _columns : 6; // "64 columns ought to be enough for anyone"
-    KstViewObjectPtr _parent; // FIXME: this is bad and should be removed ASAP
+    KstViewObject *_parent; // danger!!
+    //KstViewObjectPtr _parent; // FIXME: this is bad and should be removed ASAP
                               //        It was introduced as a temporary hack
                               //        but is no longer needed as events now
                               //        pass the view pointer along.  Also use
@@ -232,6 +234,17 @@ class KstViewObject : public KstObject {
 };
 
 
+template<class U>
+void KstViewObject::forEachChild2(void (U::*method)()) {
+  for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
+    U *it = kst_cast<U>(*i);
+    if (it) {
+      (it->*method)();
+    }
+  }
+}
+
+
 template<class T, class U>
 void KstViewObject::forEachChild(void (U::*method)(T), T arg, bool self) {
   if (self) {
@@ -241,7 +254,7 @@ void KstViewObject::forEachChild(void (U::*method)(T), T arg, bool self) {
     }
   }
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    U *it = dynamic_cast<U*>((*i).data());
+    U *it = kst_cast<U>(*i);
     if (it) {
       (it->*method)(arg);
     }
@@ -267,7 +280,7 @@ template<class T>
 KstObjectList<KstSharedPtr<T> > KstViewObject::findChildrenType(bool recursive) {
   KstObjectList<KstSharedPtr<T> > rc;
   for (KstViewObjectList::Iterator i = _children.begin(); i != _children.end(); ++i) {
-    T *o = dynamic_cast<T*>((*i).data());
+    T *o = kst_cast<T>(*i);
     if (o) {
       rc.append(o);
     }
@@ -280,13 +293,5 @@ KstObjectList<KstSharedPtr<T> > KstViewObject::findChildrenType(bool recursive)
   return rc;
 }
 
-
-template<class T>
-KstViewObjectPtr KstViewObject::findChildType(const QPoint& pos) {
-  // FIXME
-  abort();
-  return KstViewObjectPtr();
-}
-
 #endif
 // vim: ts=2 sw=2 et
