@@ -26,7 +26,7 @@ class KstViewVectorsDialogI : public KstViewVectorsDialog {
   public:
     KstViewVectorsDialogI(QWidget* parent = 0,
                         const char* name = 0,
-                        bool modal = FALSE, WFlags fl = 0 );
+                        bool modal = false, WFlags fl = 0 );
     virtual ~KstViewVectorsDialogI();
     KstVectorTable* tableVectors;
     
@@ -34,16 +34,12 @@ class KstViewVectorsDialogI : public KstViewVectorsDialog {
     void updateViewVectorsDialog();
     void updateViewVectorsDialog(const QString& strVector);
     void showViewVectorsDialog();
-    void updateDefaults(int index=0);
+    void updateDefaults(int index = 0);
 
   protected slots:
     virtual void languageChange();  
     virtual void vectorChanged(const QString& str);
-  
-  private slots:
-    
-  signals:
 };
 
-
-#endif // KSTVIEWVECTORSDIALOGI_H
+#endif
+// vim: ts=2 sw=2 et
