@@ -16,20 +16,12 @@
  ***************************************************************************/
 
 #include <assert.h>
-#include <stdlib.h>
 
-#include <qapplication.h>
-#include <qdragobject.h>
-#include <qevent.h>
-#include <qmessagebox.h>
-#include <qtimer.h>
 
 #include <kdebug.h>
-#include <kpopupmenu.h>
 
 #include "kst.h"
 #include "kstplotdrag.h"
-#include "ksttoplevelview.h"
 #include "kstviewwidget.h"
 
 KstViewWidget::KstViewWidget(KstTopLevelViewPtr view, QWidget *parent, const char *name, WFlags w)
@@ -125,28 +117,34 @@ KstViewObjectPtr KstViewWidget::findChildFor(const QPoint& pos) {
 // Optimize: cache the previous child here to make it faster?  How to invalidate?
 void KstViewWidget::mouseMoveEvent(QMouseEvent *e) {
   if (_view->viewMode() == KstTopLevelView::DisplayMode) {
+    //
+    // display mode...
+    //
     KstViewObjectPtr vo;
+
     if (_view->mouseGrabbed()) {
       vo = _view->mouseGrabber();
     } else {
       vo = findChildFor(e->pos());
     }
+
     if (vo) {
       //kdDebug() << "Found mouse handler " << vo->tagName() << endl;
       vo->mouseMoveEvent(this, e);
     } else {
       setCursor(QCursor(Qt::ArrowCursor));
     }
-    return;
-  }
-
-  // Layout mode
-  if ((e->state() & Qt::MouseButtonMask) == 0) {
-    _view->updateFocus(e->pos());
-    e->accept();
-  } else if (e->state() & Qt::LeftButton) {
-    _view->pressMove(e->pos(), e->state() & Qt::ShiftButton);
-    e->accept();
+  } else {
+    //
+    // layout mode...
+    //
+    if ((e->state() & Qt::MouseButtonMask) == 0) {
+      _view->updateFocus(e->pos());
+      e->accept();
+    } else if (e->state() & Qt::LeftButton) {
+      _view->pressMove(e->pos(), e->state() & Qt::ShiftButton);
+      e->accept();
+    }
   }
 }
 
@@ -273,7 +271,8 @@ void KstViewWidget::paint() {
 
 // Note: e can be null
 void KstViewWidget::paintEvent(QPaintEvent *e) {
-  if ( e ) {  // Resize/expose/etc triggered by X11
+  assert(this);
+  if (e) {  // Resize/expose/etc triggered by X11
     _view->paint(P_PAINT);
   } else { // explicitly forced paint in the code
     _view->paint(P_ZOOM);
