@@ -1,8 +1,8 @@
 /***************************************************************************
                               kstviewwindow.cpp
                              -------------------
-    begin                : Tue Aug 22 13:46:13 CST 2000
-    copyright            : (C) 2000 by Barth Netterfield
+    begin                : April 2004
+    copyright            : (C) 2004 by The University of Toronto
     email                :
  ***************************************************************************/
 
@@ -15,43 +15,19 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <stdlib.h>
-#include <stdio.h>
-
 // include files for Qt
-#include <qevent.h>
-#include <qdir.h>
-#include <qpainter.h>
 #include <qpaintdevicemetrics.h>
-#include <qtimer.h>
-#include <qpushbutton.h>
-#include <qmessagebox.h>
-#include <qbuffer.h>
+#include <qstylesheet.h>
 
 // include files for KDE
-#include <kaccel.h>
-#include <kconfig.h>
 #include <kdebug.h>
-#include <kiconloader.h>
-#include <klocale.h>
-#include <kmenubar.h>
 #include <kmessagebox.h>
 #include <kprinter.h>
-#include <kstandarddirs.h>
-#include <kstatusbar.h>
-#include <kstdaction.h>
-#include <kmdimainfrm.h>
 
 // application specific includes
-#include "kstviewwindow.h"
-#include "kstdebug.h"
 #include "kstdoc.h"
-#include "kstevents.h"
-#include "kstextension.h"
-#include "kstsettingsdlg.h"
-#include "pluginmanager.h"
-#include "ksttoplevelview.h"
-#include "kst.h"
+#include "kstsettings.h"
+#include "kstviewwindow.h"
 
 #define KST_STATUSBAR_FRAME 0
 #define KST_STATUSBAR_DATA 1
@@ -60,7 +36,7 @@
 KstViewWindow::KstViewWindow(QWidget *parent, const char* name)
 : KMdiChildView(QString::null, parent, name) {
   commonConstructor();
-  _view = new KstTopLevelView(this);
+  _view = new KstTopLevelView(this, name);
 }
 
 
@@ -141,13 +117,16 @@ void KstViewWindow::readProperties(KConfig* config) {
 
 
 void KstViewWindow::slotActivated(KMdiChildView*) {
-  if (KstApp::inst()->getZoomRadio() == KstApp::LAYOUT) {
-    if( view()->viewMode() == KstTopLevelView::DisplayMode ) {
-      view()->setViewMode( KstTopLevelView::LayoutMode );
-    }
-  } else {
-    if( view()->viewMode() == KstTopLevelView::LayoutMode ) {
-      view()->setViewMode( KstTopLevelView::DisplayMode );
+  // KDE bug: KMDIMainFrm close -> activate -> loops back from the destructor
+  if (KstApp::inst()) {
+    if (KstApp::inst()->getZoomRadio() == KstApp::LAYOUT) {
+      if( view()->viewMode() == KstTopLevelView::DisplayMode ) {
+        view()->setViewMode( KstTopLevelView::LayoutMode );
+      }
+    } else {
+      if( view()->viewMode() == KstTopLevelView::LayoutMode ) {
+        view()->setViewMode( KstTopLevelView::DisplayMode );
+      }
     }
   }
 }
@@ -245,30 +224,30 @@ void KstViewWindow::immediatePrintToPng(const QString &filename, const QSize& si
 }
 
 
-void KstViewWindow::save(QTextStream& ts) {
+void KstViewWindow::save(QTextStream& ts, const QString& indent) {
   QRect restoreGeom = restoreGeometry();
   QRect internalGeom = internalGeometry();
 
-  ts << "  <tag>" << caption() << "</tag>" << endl;
+  ts << indent << "<tag>" << QStyleSheet::escape(caption()) << "</tag>" << endl;
 
-  ts << "  <restore"  << " x=\"" << restoreGeom.x()
-                    << "\" y=\"" << restoreGeom.y()
-                    << "\" w=\"" << restoreGeom.width()
-                    << "\" h=\"" << restoreGeom.height() << "\" />" << endl;
+  ts << indent << "<restore"  << " x=\"" << restoreGeom.x()
+    << "\" y=\"" << restoreGeom.y()
+    << "\" w=\"" << restoreGeom.width()
+    << "\" h=\"" << restoreGeom.height() << "\" />" << endl;
 
-  ts << "  <internal" << " x=\"" << internalGeom.x()
-                    << "\" y=\"" << internalGeom.y()
-                    << "\" w=\"" << internalGeom.width()
-                    << "\" h=\"" << internalGeom.height() << "\" />" << endl;
+  ts << indent << "<internal" << " x=\"" << internalGeom.x()
+    << "\" y=\"" << internalGeom.y()
+    << "\" w=\"" << internalGeom.width()
+    << "\" h=\"" << internalGeom.height() << "\" />" << endl;
 
   if (isMinimized()) {
-    ts << "  <minimized/>" << endl;
+    ts << indent << "<minimized/>" << endl;
   }
   if (isMaximized()) {
-    ts << "  <maximized/>" << endl;
+    ts << indent << "<maximized/>" << endl;
   }
 
-  view()->save(ts);
+  view()->save(ts, indent + "  ");
 }
 
 
@@ -276,5 +255,26 @@ KstTopLevelViewPtr KstViewWindow::view() const {
   return _view;
 }
 
+void KstViewWindow::setCaption(const QString& szCaption) {
+  KMdiChildView::setCaption(szCaption);
+  _view->setTagName(szCaption);
+}
+
+void KstViewWindow::closeEvent(QCloseEvent *e) {
+  if (KstSettings::globalSettings()->promptWindowClose) {
+    if (view()->children().size()) {
+      if (KMessageBox::warningYesNo(0L, i18n("Are you sure you want to close window '%1'?\nClosing a window deletes all plots in the window.").arg(caption())) == KMessageBox::Yes) {
+        KMdiChildView::closeEvent(e);
+      } else {
+        e->ignore();
+      }
+    } else {
+      KMdiChildView::closeEvent(e);
+    }
+  } else {
+    KMdiChildView::closeEvent(e);
+  }
+}
+
 #include "kstviewwindow.moc"
 // vim: ts=2 sw=2 et
