@@ -1,8 +1,8 @@
 /***************************************************************************
                                kstviewwindow.h
                              -------------------
-    begin                : Tue Aug 22 13:46:13 CST 2000
-    copyright            : (C) 2000 by Barth Netterfield
+    begin                : Apr 2004
+    copyright            : (C) 2004 by The University of Toronto
     email                :
  ***************************************************************************/
 
@@ -21,21 +21,12 @@
 #include <config.h>
 
 // include files for KDE
-#include <kaccel.h>
-#include <kaction.h>
-#include <kapplication.h>
 #include <klineeditdlg.h>
-#include <kmainwindow.h>
-#include <kmdichildview.h>
 #include <klocale.h>
 
 #include "kst.h"
-#include "kst2dplot.h"
 #include "kstdatacollection.h"
 #include "kstdefaultnames.h"
-#include "kstdataobject.h"
-#include "ksttoplevelview.h"
-#include "testview.h"
 
 class QLabel;
 
@@ -51,8 +42,9 @@ class KstViewWindow : public KMdiChildView {
     /** pause the updating of data */
     void setPaused(bool paused);
     void togglePaused();
-    void save(QTextStream& ts);
+    void save(QTextStream& ts, const QString& indent = QString::null);
     KstTopLevelViewPtr view() const;
+    virtual void setCaption(const QString& szCaption);
 
   protected:
     /** saves the window properties for each open window during session
@@ -67,6 +59,12 @@ class KstViewWindow : public KMdiChildView {
      * @see KMainWindow#readProperties */
     virtual void readProperties(KConfig *cfg);
 
+    /**
+    * Ignores the event and calls KMdiMainFrm::childWindowCloseRequest instead. This is because the
+    * mainframe has control over the views. Therefore the MDI view has to request the mainframe for a close.
+    */
+    virtual void closeEvent(QCloseEvent *e);
+
   private slots:
     // Hack to update KStdActions
     void updateActions();
@@ -132,7 +130,7 @@ QString KstViewWindow::createPlot(const QString& suggestedName, bool prompt) {
     bool ok = false;
     name = KLineEditDlg::getText(i18n("Enter a name for the new plot:"), name, &ok, 0L);
     if (!ok) {
-      return QString();
+      return QString::null;
     }
     //check the name
     duplicate = true;
@@ -151,7 +149,7 @@ QString KstViewWindow::createPlot(const QString& suggestedName, bool prompt) {
             name = KLineEditDlg::getText(i18n("Enter a name for the new plot:"), name, &ok, 0L);
             if (!ok) {
               app->deleteIterator(iter);
-              return QString();
+              return QString::null;
             }
           }
         }
