@@ -429,7 +429,7 @@ void KMdiMainFrm::addWindow( KMdiChildView* pWnd, int flags, int index)
     //      const QPixmap& wndIcon = pWnd->icon() ? *(pWnd->icon()) : QPixmap();
 
     m_documentTabWidget->insertTab(pWnd, pWnd->icon() ? *(pWnd->icon()) : QPixmap(),pWnd->tabCaption(), index);
-    
+
     /*
        connect(pWnd,SIGNAL(iconOrCaptionUdpated(QWidget*,QPixmap,const QString&)),
        m_documentTabWidget,SLOT(updateView(QWidget*,QPixmap,const QString&)));
@@ -1092,8 +1092,8 @@ void KMdiMainFrm::closeAllViews()
   QValueListIterator<KMdiChildView *> childIt;
   for (childIt = children.begin(); childIt != children.end(); ++childIt)
   {
-    (*childIt)->close(); 
-  } 
+    (*childIt)->close();
+  }
 }
 
 
@@ -2298,20 +2298,20 @@ void KMdiMainFrm::fillWindowMenu()
   }
   m_pWindowMenu->insertSeparator();
   if (!bTabPageMode && !bIDEAlMode) {
-    int placMenuId = m_pWindowMenu->insertItem(i18n("&Tile..."), m_pPlacingMenu);
+    int placMenuId = m_pWindowMenu->insertItem(i18n("&Tile"), m_pPlacingMenu);
     m_pPlacingMenu->clear();
     m_pPlacingMenu->insertItem(i18n("Ca&scade Windows"), m_pMdi,SLOT(cascadeWindows()));
     m_pPlacingMenu->insertItem(i18n("Cascade &Maximized"), m_pMdi,SLOT(cascadeMaximized()));
     m_pPlacingMenu->insertItem(i18n("Expand &Vertically"), m_pMdi,SLOT(expandVertical()));
     m_pPlacingMenu->insertItem(i18n("Expand &Horizontally"), m_pMdi,SLOT(expandHorizontal()));
-    m_pPlacingMenu->insertItem(i18n("Tile &Non-overlapped"), m_pMdi,SLOT(tileAnodine()));
+    m_pPlacingMenu->insertItem(i18n("Tile &Non-Overlapped"), m_pMdi,SLOT(tileAnodine()));
     m_pPlacingMenu->insertItem(i18n("Tile Overla&pped"), m_pMdi,SLOT(tilePragma()));
     m_pPlacingMenu->insertItem(i18n("Tile V&ertically"), m_pMdi,SLOT(tileVertically()));
     if (m_mdiMode == KMdi::ToplevelMode) {
       m_pWindowMenu->setItemEnabled(placMenuId, false);
     }
     m_pWindowMenu->insertSeparator();
-    int dockUndockId = m_pWindowMenu->insertItem(i18n("&Dock/Undock..."), m_pDockMenu);
+    int dockUndockId = m_pWindowMenu->insertItem(i18n("&Dock/Undock"), m_pDockMenu);
     m_pDockMenu->clear();
     m_pWindowMenu->insertSeparator();
     if (bNoViewOpened) {
@@ -2528,13 +2528,13 @@ void KMdiMainFrm::removeFromActiveDockList(KMdiDockContainer* td) {
       break;
     }
   }
-  /*    
+  /*
         if (d->activeDockPriority[0]==0) {
         if (d->focusList) d->focusList->restore();
         delete d->focusList;
         d->focusList=0;
         }
-   */    
+   */
 }
 
 void KMdiMainFrm::prevToolViewInDock() {
