@@ -1,5 +1,5 @@
 /***************************************************************************
-                          main.cpp  -  description
+                                  main.cpp 
                              -------------------
     begin                : Tue Aug 22 13:46:13 CST 2000
     copyright            : (C) 2000 by Barth Netterfield
@@ -17,26 +17,22 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <iostream>
 
-#include <kdebug.h>
-#include <kcmdlineargs.h>
 #include <kaboutdata.h>
+#include <kcmdlineargs.h>
+#include <kdebug.h>
 #include <kimageio.h>
-#include <klocale.h>
-#include <kmdimainfrm.h>
 
 #include "kst.h"
-#include "kst2dplot.h"
 #include "kstcolorsequence.h"
 #include "kstdatacollection.h"
 #include "kstdefaultnames.h"
 #include "kstdoc.h"
-#include "kstequationcurve.h"
+#include "kstequation.h"
 #include "ksthistogram.h"
 #include "kstlegend.h"
-#include "kstpsdcurve.h"
-#include "ksttoplevelview.h"
+#include "kstpsd.h"
+#include "kstrvector.h"
 #include "kstvcurve.h"
 #include "kstversion.h"
 #include "plugincollection.h"
@@ -98,18 +94,18 @@ struct InType {
 /*        check command line options for simple usage errors    */
 /*                                                              */
 /****************************************************************/
-void CheckForCMDErrors(KCmdLineArgs *args) {
+static void CheckForCMDErrors(KCmdLineArgs *args) {
   bool nOK;
 
   if (args->getOption("n").toInt(&nOK) == 0) {
-    std::cerr << I18N_NOOP("error: exiting because '-n 0' requests vectors with no data in them\n");
-    exit(0);
+    kdError() << i18n("Exiting because '-n 0' requests vectors with no data in them.") << endl;
+    exit(0); // FIXME: can cause crashes
   }
   if (args->getOption("n").toInt(&nOK)>0) {
     if (args->getOption("n").toInt(&nOK) <
         2*args->getOption("s").toInt(&nOK)) {
-      std::cerr << "error: requested data skipping would leave vector with fewer than two points\n";
-      exit(0);
+      kdError() << i18n("Requested data skipping would leave vector with fewer than two points.") << endl;
+      exit(0); // FIXME: can cause crashes
     }
   }
 }
@@ -120,8 +116,7 @@ void CheckForCMDErrors(KCmdLineArgs *args) {
 /*        fill the in struct from the command line args         */
 /*                                                              */
 /****************************************************************/
-void SetCMDOptions(KCmdLineArgs *args, struct InType &in,
-                   int n_y) {
+static void SetCMDOptions(KCmdLineArgs *args, InType &in, int n_y) {
   bool nOK;
   in.n = args->getOption("n").toInt(&nOK);
   in.f = args->getOption("f").toInt(&nOK);
@@ -173,10 +168,18 @@ void SetCMDOptions(KCmdLineArgs *args, struct InType &in,
 /*        create the plots                                      */
 /*                                                              */
 /****************************************************************/
-void CreatePlots(struct InType &in, KstTopLevelViewPtr tlv) {
+static void CreatePlots(InType &in, KstTopLevelViewPtr tlv) {
+  KstApp *kst = KstApp::inst();
+  QString creatingPlots = i18n("Creating plots");
   Kst2DPlotPtr plot;
   int i_plot;
   int size_boost;
+  int count;
+  int handled;
+  
+  kst->slotUpdateProgress( 0, 0, creatingPlots );
+  count = in.n_plots;
+  handled = 0;
 
   for (i_plot=0; i_plot < in.n_plots; i_plot++) {
     plot = new Kst2DPlot(KST::suggestPlotName());
@@ -189,9 +192,14 @@ void CreatePlots(struct InType &in, KstTopLevelViewPtr tlv) {
                             double(i_plot/in.n_cols)/double(in.n_rows),
                             1.0/double(in.n_cols), 1.0/double(in.n_rows));
 
+    handled++;
+    kst->slotUpdateProgress( count, handled, creatingPlots );
   }
+
   tlv->setColumns(in.n_cols);
   tlv->setOnGrid(true);
+
+  kst->slotUpdateProgress( 0, 0, QString::null );
 }
 
 /******************************************************************/
@@ -200,7 +208,7 @@ void CreatePlots(struct InType &in, KstTopLevelViewPtr tlv) {
 /*        start:end:Nsamp                                         */
 /*                                                                */
 /******************************************************************/
-void SetEqXRanges(QString xeqS, double *min, double *max, int *n, bool *ok) {
+static void SetEqXRanges(QString xeqS, double *min, double *max, int *n, bool *ok) {
   *ok = true;
   QStringList fields = QStringList::split( QChar(':'), xeqS );
   if (fields.count()!=3) {
@@ -228,31 +236,24 @@ void SetEqXRanges(QString xeqS, double *min, double *max, int *n, bool *ok) {
 /*        if a vector with the same field and file exists, use it */
 /*                                                                */
 /******************************************************************/
-KstRVector *GetOrCreateVector(QString field, KstDataSourcePtr file, struct InType &in) {
+static KstRVector *GetOrCreateVector(const QString& field, KstDataSourcePtr file, InType &in) {
   int i_v = 0, n_v;
   KstRVector *V;
-  KstRVectorList vectorList =
-    kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+  KstRVectorList vectorList = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
 
   n_v = vectorList.count();
 
   for (i_v=0; i_v<n_v; i_v++) {
     V = *vectorList.at(i_v);
-    if (V->field() == field) {
-      if (V->filename() == file->fileName()) {
-        return (V);
-      }
+    if (V->field() == field && V->filename() == file->fileName()) {
+      return V;
     }
   }
-  V = new KstRVector(file, field,
-
-  QString("V") + QString::number(1 + i_v++)
-                          + "-" + field,
-                          in.f, in.n, in.skip, in.doskip, in.doave);
+  V = new KstRVector(file, field, QString("V") + QString::number(1 + i_v++)
+       + "-" + field, in.f, in.n, in.skip, in.doskip, in.doave);
   if (!V->isValid()) {
-    kdError() << I18N_NOOP("Error: Invalid field: ") << V->field()
-              << endl << I18N_NOOP("File: ") << V->filename() << endl;
-    exit(0); // fixme: this can cause crashes
+    kdError() << i18n("Invalid field: %1 in file %2").arg(V->field()).arg(V->filename()) << endl;
+    exit(0); // FIXME: this can cause crashes
   }
 
   KST::addVectorToList(V);
@@ -265,22 +266,18 @@ KstRVector *GetOrCreateVector(QString field, KstDataSourcePtr file, struct InTyp
 /*        Create vectors that don't exist.                      */
 /*                                                              */
 /****************************************************************/
-bool NoVectorEq(QString eq) {
-  if (eq.find("[")>=0)
-    return false;
-  else
-    return true;
+static bool NoVectorEq(const QString& eq) {
+  return eq.find('[') < 0;
 }
 
-void ProcessEq(QString &eq, KstDataSourcePtr file,
-               struct InType &in, bool *ok) {
+static void ProcessEq(QString &eq, KstDataSourcePtr file, InType &in, bool *ok) {
   QString field;
   int i0=0, i1=0;
   KstVector *v;
   *ok = true;
 
-  while ((i0 = eq.find("[", i0))>=0) {
-    i1 = eq.find("]", i0);
+  while ((i0 = eq.find('[', i0))>=0) {
+    i1 = eq.find(']', i0);
     if (i1>0) {
       field = eq.mid(i0+1, i1-i0-1);
       v = GetOrCreateVector(field, file, in);
@@ -290,6 +287,11 @@ void ProcessEq(QString &eq, KstDataSourcePtr file,
   }
 }
 
+
+static void exitHelper(void) {
+  KstApp::doubleCheckCleanup();
+}
+
 /****************************************************************/
 /*                                                              */
 /*        main for kst.  mostly command line handling           */
@@ -299,11 +301,11 @@ int main(int argc, char *argv[]) {
   int i_file, i_v, i_curve;
   int i_plot;
 
-  KAboutData aboutData( "kst", I18N_NOOP("Kst"),
-                        KSTVERSION, description, KAboutData::License_GPL,
-                        I18N_NOOP("(c) 2000-2004 Barth Netterfield"),
-                        0,
-                        "http://extragear.kde.org/apps/kst.php");
+  KAboutData aboutData("kst", I18N_NOOP("Kst"),
+                       KSTVERSION, description, KAboutData::License_GPL,
+                       I18N_NOOP("(c) 2000-2004 Barth Netterfield"),
+                       0,
+                       "http://extragear.kde.org/apps/kst.php");
   aboutData.addAuthor("Barth Netterfield",
                       I18N_NOOP("Original author and maintainer."),
                       "netterfield@astro.utoronto.ca",
@@ -312,6 +314,14 @@ int main(int argc, char *argv[]) {
                       I18N_NOOP("Developed for the University of Toronto."),
                       "info@staikos.net",
                       "http://www.staikos.net/");
+  aboutData.addAuthor("Sumus Technology Limited",
+                      I18N_NOOP("Developed for the University of British Columbia"),
+                      "info@sumusltd.com",
+                      "http://www.sumusltd.com/");
+  aboutData.addAuthor("Rick Chern",
+                      I18N_NOOP("University of British Columbia"),
+                      "",
+                      "");
 
   KCmdLineArgs::init( argc, argv, &aboutData );
   KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.
@@ -319,56 +329,74 @@ int main(int argc, char *argv[]) {
   KApplication app;
   KImageIO::registerFormats();
 
+  KstApp::initialize();
+
+  atexit(exitHelper);
+
   if (app.isRestored()) {
     RESTORE(KstApp)
   } else {
     KstApp *kst = new KstApp;
-
-    struct InType in;
+    InType in;
     QColor color;
+    KstDataSourcePtr file;
     QCStringList ycolList;
     QCStringList yEqList;
     QCStringList psdList;
     QCStringList hsList;
     QCStringList errorList;
     unsigned int i_ycol;
-    QCStringList::Iterator psd;
-    QCStringList::Iterator hs;
+    QCStringList::Iterator hs_string;
     QCStringList::Iterator eq_i;
+    bool showQuickStart = false;
     bool nOK;
+    int n_y = 0;
+    int n_eq = 0;
 
-    /* temp variables: these all get stuck into list objects */
-    KstDataSourcePtr file;
-    KstRVectorPtr xvector;
-    KstRVectorPtr yvector;
-    KstRVectorPtr evector;
-    KstVCurvePtr curve;
-    KstPSDCurvePtr psdcurve;
-    KstEquationCurvePtr eqcurve;
-    KstHistogramPtr hscurve;
-    Kst2DPlotPtr plot;
-
-    int n_y, n_eq=0;
-
-    /* Parse command line args */
     KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
     CheckForCMDErrors(args);
 
+    QString printfile = args->getOption("print");
+    QString pngfile = args->getOption("png");
+    bool print_and_exit = false;
+
+    if (printfile != "<none>") {
+      print_and_exit = true;
+    }
+    if (pngfile != "<none>") {
+      print_and_exit = true;
+    }
+
     // Initialise the plugin loader and collection.
     PluginCollection::self();
 
+    if (!print_and_exit) {
+      app.setMainWidget(kst);
+      kst->show();
+    }
+
+    //
     // get Y axis columns
+    //
     ycolList = args->getOptionList("y");
     yEqList = args->getOptionList("ye");
     psdList = args->getOptionList("p");
     hsList = args->getOptionList("h");
     errorList = args->getOptionList("e");
 
+    //
     // y axis or PSD specified, so the files are data files, not kst files.
-    n_y = ycolList.count() + psdList.count() + hsList.count()
-      + yEqList.count();
+    //
+    n_y = ycolList.count() + psdList.count() + hsList.count() + yEqList.count();
     if (n_y > 0) {
+      QString creatingEquations = i18n("Creating equations");
+      QString creatingCurves = i18n("Creating curves");
+      QString creatingPlots = i18n("Creating plots");
+      int count;
+      int handled;
+      
+      kst->slotUpdateProgress( 0, 0, QString::null );
 
       SetCMDOptions(args, in, n_y);
 
@@ -383,65 +411,77 @@ int main(int argc, char *argv[]) {
       }
 
       if (!tlv) {
-        exit(0);
+        exit(0); // FIXME: can cause crashes
       }
 
       CreatePlots(in, tlv);
-      Kst2DPlotList plist =
-        kstObjectSubList<KstViewObject, Kst2DPlot>(tlv->children());
+      Kst2DPlotList plist = kstObjectSubList<KstViewObject, Kst2DPlot>(tlv->children());
 
       i_plot = 0;
-      plot = *plist.at(i_plot);
+      Kst2DPlotPtr plot = *plist.at(i_plot);
 
-      /* make stand alone equations if there are no files */
-      if (args->count()<1) {
+      //
+      // make stand alone equations if there are no files
+      //
+      if (args->count() < 1) {
         if (!yEqList.isEmpty()) {
           QString eqS;
           double max, min;
           int n;
           bool xeq;
+          
           SetEqXRanges(args->getOption("xe"), &min, &max, &n, &xeq);
           if (xeq) {
+            count = yEqList.size();
+            handled = 0;
+            kst->slotUpdateProgress( count, handled, creatingEquations );
+
             for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
               eqS = *eq_i;
               if (NoVectorEq(eqS)) {
-                eqcurve =
-                  new KstEquationCurve(QString("E")+QString::number(n_eq+1)+
-                                       "-" + eqS,
-                                       eqS,
-                                       min, max, n, KstColorSequence::next());
+                KstEquationPtr eq = new KstEquation(QString("E") +QString::number(n_eq+1)+ "-" + eqS, eqS, min, max, n);
+                KstVCurvePtr vc = new KstVCurve(eq->tagName() + "-curve", eq->vX(), eq->vY(), 0L, 0L, KstColorSequence::next(plot->Curves,plot->backgroundColor()));
 
                 KST::dataObjectList.lock().writeLock();
-                KST::dataObjectList.append(eqcurve.data());
+                KST::dataObjectList.append(eq.data());
+                KST::dataObjectList.append(vc.data());
                 KST::dataObjectList.lock().writeUnlock();
-                plot->addCurve(eqcurve.data());
+                plot->addCurve(vc.data());
 
                 if (in.sep_plots) {
                   i_plot++;
                   if (i_plot <in.n_plots) plot = *plist.at(i_plot);
                 }
               }
+
+              handled++;
+              kst->slotUpdateProgress( count, handled, creatingEquations );            
             }
           }
         }
       }
 
-      /* Make the requested curves for each data file */
+      //
+      // make the requested curves for each data file
+      //
+      count = args->count();
+      handled = 0;
+      kst->slotUpdateProgress( count, handled, creatingCurves );
+
       for (i_curve = i_v = 0, i_file = 0; i_file < args->count(); i_file++) {
-        /* Make the file */
+        //
+        // make the file
+        //
         file = KstDataSource::loadSource(args->arg(i_file));
 
         if (!file) {
-          std::cerr << I18N_NOOP("Error: No data in file: ")
-	       << args->arg(i_file) << "\n";
+          kdError() << i18n("No data in file: %1").arg(args->arg(i_file)) << endl;
           delete kst;
           exit(0);
         }
 
         if (!file->isValid() || file->isEmpty()) {
-          std::cerr << I18N_NOOP("No data in file: ")
-            << args->arg(i_file)
-            << ".  Trying to continue...\n";
+          kdError() << i18n("No data in file %1.  Trying to continue...").arg(args->arg(i_file)) << endl;
           // The file might get data later!
         }
 
@@ -449,16 +489,29 @@ int main(int argc, char *argv[]) {
         KST::dataSourceList.append(file);
         KST::dataObjectList.lock().writeUnlock();
 
+        KstRVectorPtr yvector;
+        KstRVectorPtr evector;
+        KstVCurvePtr curve;
+        KstPSDPtr psd;
+        KstHistogramPtr hs;
+        KstRVectorPtr xvector;
+
         if (!ycolList.isEmpty()) { // if there are some xy plots
-          /* make the x axis vector */
+          //
+          // make the x axis vector
+          //
           xvector = GetOrCreateVector(args->getOption("x"), file, in);
 
-          /* make the y axis vectors */
+          //
+          // make the y axis vectors
+          //
           for (i_ycol = 0; i_ycol < ycolList.count(); ++i_ycol ) {
             yvector = GetOrCreateVector(*(ycolList.at(i_ycol)), file, in);
 
-            /* make the curves */
-            color = KstColorSequence::next();
+            //
+            // make the curves
+            //
+            color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
             curve = new KstVCurve(QString("C") + QString::number(1+i_curve++)
                                   + "-" + yvector->field(),
                                   KstVectorPtr(xvector), KstVectorPtr(yvector),
@@ -486,6 +539,7 @@ int main(int argc, char *argv[]) {
             } // end (if they are separate plots)
           } // next y col
         } // end (if there are some xy plots)
+
         if (!yEqList.isEmpty()) {
           QString eqS;
           double max, min;
@@ -496,26 +550,25 @@ int main(int argc, char *argv[]) {
           for (eq_i = yEqList.begin(); eq_i != yEqList.end(); ++eq_i) {
             eqS = *eq_i;
             ProcessEq(eqS, file, in, &eq_ok);
+            KstEquationPtr eq;
             if (xeq) {
-              eqcurve =
-                new KstEquationCurve(QString("E")+QString::number(n_eq+1)+
-                    "-" + eqS,
-                    eqS,
-                    min,max,n, KstColorSequence::next());
+              eq = new KstEquation(QString("E")+QString::number(n_eq+1)+
+                    "-" + eqS, eqS, min,max,n);
             } else {
-              if (xvector==NULL)
+              if (!xvector)
                 xvector = GetOrCreateVector(args->getOption("x"), file, in);
 
-              eqcurve =
-                new KstEquationCurve(QString("E")+QString::number(n_eq+1)+eqS,
+              eq = new KstEquation(QString("E")+QString::number(n_eq+1)+eqS,
                     eqS,
                     KstVectorPtr(xvector),
-                    true, KstColorSequence::next());
+                    true);
             }
+            KstVCurvePtr vc = new KstVCurve(eq->tagName() + "-curve", eq->vX(), eq->vY(), 0L, 0L, KstColorSequence::next(plot->Curves,plot->backgroundColor()));
             KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(eqcurve.data());
+            KST::dataObjectList.append(eq.data());
+            KST::dataObjectList.append(vc.data());
             KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(eqcurve.data());
+            plot->addCurve(vc.data());
 
             if (in.sep_plots) {
               i_plot++;
@@ -525,26 +578,30 @@ int main(int argc, char *argv[]) {
             }
           }
         }
+
         if (psdList.count() > 0) { // if there are some psd plots
           KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-          for(psd = psdList.begin(); psd != psdList.end(); ++psd ) {
+          for (QCStringList::ConstIterator it = psdList.begin(); it != psdList.end(); ++it) {
 
-            yvector = GetOrCreateVector(*psd, file, in);
+            yvector = GetOrCreateVector(*it, file, in);
 
-            color = KstColorSequence::next();
+            color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
 
-            psdcurve = new KstPSDCurve(
-                QString("P") + QString::number(1+i_curve++) + "-" + yvector->field(),
+            psd = new KstPSD( QString("P") + QString::number(1+i_curve++) + "-" + yvector->field(),
                 KstVectorPtr(yvector), in.rate, true, in.len,
-                true, true, in.VUnits, in.RUnits, color);
+                true, true, in.VUnits, in.RUnits);
+            KstVCurvePtr vc = new KstVCurve(psd->tagName() + "-curve",
+					    psd->vX(), psd->vY(),
+					    0L, 0L, color);
             if (in.has_points) {
-              psdcurve->setHasPoints(true);
-              psdcurve->setHasLines(false);
+              vc->setHasPoints(true);
+              vc->setHasLines(false);
             }
             KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(psdcurve.data());
+            KST::dataObjectList.append(psd.data());
+            KST::dataObjectList.append(vc.data());
             KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(psdcurve.data());
+            plot->addCurve(vc.data());
 
             if (in.sep_plots) {
               i_plot++;
@@ -554,28 +611,39 @@ int main(int argc, char *argv[]) {
             }
           } // next psd
         } // end (if there are some psds)
-        if (hsList.count()>0) { // if there are some histograms
+
+        if (hsList.count() > 0) { // if there are some histograms
           double max, min;
           int N;
 
           KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
-          for (hs = hsList.begin(); hs != hsList.end(); ++hs ) {
-            yvector = GetOrCreateVector(*hs, file, in);
+          for (hs_string = hsList.begin();
+	       hs_string != hsList.end(); ++hs_string ) {
+            yvector = GetOrCreateVector(*hs_string, file, in);
 
-            color = KstColorSequence::next();
+            color = KstColorSequence::next(plot->Curves,plot->backgroundColor());
 
             KstHistogram::AutoBin(KstVectorPtr(yvector), &N, &max, &min);
 
-            hscurve = new KstHistogram(QString("H") +
+            hs = new KstHistogram(QString("H") +
                 QString::number(1+i_curve++)
                 + "-" + yvector->field(),
                 KstVectorPtr(yvector), min, max, N,
-                KST_HS_NUMBER,
-                color);
+                KST_HS_NUMBER);
+            KstVCurvePtr vc = new KstVCurve(hs->tagName() + "-curve",
+					    hs->vX(), hs->vY(),
+					    0L, 0L, color);
+
+            vc->setHasPoints(false);
+            vc->setHasLines(false);
+            vc->setHasBars(true);
+            vc->setBarStyle(1);
+
             KST::dataObjectList.lock().writeLock();
-            KST::dataObjectList.append(hscurve.data());
+            KST::dataObjectList.append(hs.data());
+            KST::dataObjectList.append(vc.data());
             KST::dataObjectList.lock().writeUnlock();
-            plot->addCurve(hscurve.data());
+            plot->addCurve(vc.data());
 
             if (in.sep_plots) {
               i_plot++;
@@ -585,7 +653,14 @@ int main(int argc, char *argv[]) {
             }
           } // next histogram
         } // end (if there are some histograms)
+
+        handled++;
+        kst->slotUpdateProgress( count, handled, creatingCurves );
       } // next data file
+
+      count = in.n_plots;
+      handled = 0;
+      kst->slotUpdateProgress( count, handled, creatingPlots );
       for (i_plot = 0; i_plot < in.n_plots; i_plot++) {
         plot = *plist.at(i_plot);
         plot->GenerateDefaultLabels();
@@ -593,54 +668,50 @@ int main(int argc, char *argv[]) {
           plot->Legend->setShow(true);
           plot->Legend->setFront(true);
         }
+        handled++;
+        kst->slotUpdateProgress( count, handled, creatingPlots );
       }
-    } else if (args->count() > 0) {
-      /* open a kst file */
-      /* some of the options can be overridden */
+
+      kst->slotUpdateProgress( 0, 0, QString::null );
+    } else if (args->count() > 0) { // open a kst file
+      // some of the options can be overridden
       kst->openDocumentFile(args->arg(0),
-          args->getOption("F"), // override FileName
-          // override number of frames
-          args->getOption("n").toInt(&nOK),
-          // override starting frame
-          args->getOption("f").toInt(&nOK),
-          // override skip
-          args->getOption("s").toInt(&nOK),
-          // add averaging
-          args->isSet("a"));
+          args->getOption("F"),             // override FileName
+          args->getOption("n").toInt(&nOK), // override number of frames
+          args->getOption("f").toInt(&nOK), // override starting frame
+          args->getOption("s").toInt(&nOK), // override skip
+          args->isSet("a"));                // add averaging
     } else {
       //kst->openDocumentFile();
+      showQuickStart = true;
     }
 
-    QString printfile = args->getOption("print");
-    QString pngfile = args->getOption("png");
-    bool print_and_exit = false;
-
     if (printfile != "<none>") {
-      args->clear();
       kst->forceUpdate();
       kst->immediatePrintToFile(printfile);
-      print_and_exit = true;
     }
 
     if (pngfile != "<none>") {
-      args->clear();
       kst->forceUpdate();
       kst->immediatePrintToPng(pngfile);
-      print_and_exit = true;
     }
 
     if (print_and_exit) {
+      args->clear();
       delete kst;
       exit(0);
     } else {
       args->clear();
-      app.setMainWidget(kst);
-      kst->show();
       kst->updateDialogs();
       kst->document()->setModified(false);
+      if (showQuickStart) {
+        kst->showQuickStartDialog();
+      }
     }
 
+    //
     // LEAVE THIS HERE - causes crashes otherwise!
+    //
     int rc = app.exec();
     delete kst;
     return rc;
