@@ -17,20 +17,15 @@
 
 
 #include "kstdebug.h"
-#include "plugin.h"
 #include "plugincollection.h"
 #include "pluginloader.h"
 #include "pluginxmlparser.h"
 
 #include <kdebug.h>
-#include <kglobal.h>
-#include <klibloader.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
-#include <kstaticdeleter.h>
 
 #include <qdir.h>
-#include <qfileinfo.h>
 #include <qregexp.h>
 
 PluginCollection *PluginCollection::_self = 0L;
@@ -155,7 +150,7 @@ void PluginCollection::unloadAllPlugins() {
 
 
 KstSharedPtr<Plugin> PluginCollection::plugin(const QString& name) {
-  if (!_plugins.contains(name)) {
+  if (!_plugins.contains(name) || _plugins[name] == 0L) {
     if (!_installedPluginNames.contains(name)) {
       rescan();
     }
@@ -226,9 +221,11 @@ void PluginCollection::scanPlugins() {
 
     QFileInfoListIterator fit(*list);
     QFileInfo *fi;
-
+    int status;
+    
     while ((fi = fit.current()) != 0L) {
-      if (_parser->parseFile(*it + fi->fileName()) == 0) {
+      status = _parser->parseFile(*it + fi->fileName());
+      if (status == 0) {
         // dupe? - prefer earlier installations
         if (_installedPluginNames.contains(_parser->data()._name)) {
           ++fit;
@@ -243,7 +240,7 @@ void PluginCollection::scanPlugins() {
           backup.remove(_parser->data()._name);
         }
       } else {
-        KstDebug::self()->log(i18n("Error parsing XML file '%1'; skipping.").arg(*it + fi->fileName()), KstDebug::Warning);
+        KstDebug::self()->log(i18n("Error [%2] parsing XML file '%1'; skipping.").arg(*it + fi->fileName()).arg(status), KstDebug::Warning);
       }
       ++fit;
     }
