@@ -23,9 +23,6 @@
 #include <kdebug.h>
 #include <klibloader.h>
 #include <klocale.h>
-#include <kstaticdeleter.h>
-#include <qfile.h>
-#include <qregexp.h>
 
 
 PluginLoader *PluginLoader::_self = 0L;
@@ -70,9 +67,16 @@ Plugin *PluginLoader::loadPlugin(const QString& xmlfile, const QString& object)
     return 0L;
   }
 
-  plug->_symbol = plug->_lib->symbol(plug->_data._name.latin1());
-  if (plug->_lib->hasSymbol("parameterName")) {
-    plug->_parameterName = plug->_lib->symbol("parameterName");
+  if (plug->_lib->hasSymbol(plug->_data._name.latin1())) {
+    plug->_symbol = plug->_lib->symbol(plug->_data._name.latin1());
+  }
+
+  if (plug->_lib->hasSymbol("freeLocalData")) {
+    plug->_freeSymbol = plug->_lib->symbol("freeLocalData");
+  }
+
+  if (plug->_lib->hasSymbol("errorCode")) {
+    plug->_errorSymbol = plug->_lib->symbol("errorCode");
   }
 
   if (!plug->_symbol) {
@@ -81,6 +85,10 @@ Plugin *PluginLoader::loadPlugin(const QString& xmlfile, const QString& object)
     return 0L;
   }
 
+  if (plug->_lib->hasSymbol("parameterName")) {
+    plug->_parameterName = plug->_lib->symbol("parameterName");
+  }
+
   plug->_xmlFile = xmlfile;
   plug->_soFile = object;
 
