@@ -23,32 +23,32 @@
 class KTempFile;
 
 class KstStdinSource : public KstDataSource {
-public:
-  KstStdinSource();
+  public:
+    KstStdinSource(KConfig *cfg);
 
-  virtual ~KstStdinSource();
+    virtual ~KstStdinSource();
 
-  virtual KstObject::UpdateType update(int = -1);
+    virtual KstObject::UpdateType update(int = -1);
 
-  virtual int readField(double *v, const QString &field, int s, int n);
+    virtual int readField(double *v, const QString &field, int s, int n);
 
-  virtual bool isValidField(const QString &field) const;
+    virtual bool isValidField(const QString &field) const;
 
-  virtual int samplesPerFrame(const QString &field);
+    virtual int samplesPerFrame(const QString &field);
 
-  virtual int frameCount(const QString& field = QString::null) const;
+    virtual int frameCount(const QString& field = QString::null) const;
 
-  virtual QString fileType() const;
+    virtual QString fileType() const;
 
-  virtual void save(QTextStream &ts);
+    virtual void save(QTextStream &ts, const QString& indent = QString::null);
 
-  virtual bool isValid() const;
+    virtual bool isValid() const;
 
-  virtual bool isEmpty() const;
+    virtual bool isEmpty() const;
 
-private:
-  KstDataSourcePtr _src;
-  KTempFile *_file;
+  private:
+    KstDataSourcePtr _src;
+    KTempFile *_file;
 };
 
 
