@@ -79,32 +79,36 @@ void UpdateThread::run() {
 #if UPDATEDEBUG > 1
       kdDebug() << "Update resulted in: TRUE!" << endl;
 #endif
-      // Wait for UI thread to finish events.  This is a
-      // a hack to keep update thread from getting ahead of the UI thread,
-      // which would block UI thread by filling it with un-processable requests.
-      bool do_wait = false;
-      // Race warning: Syncronization of updating() is not assured, but updating() will always
-      // return a valid answer which was true 'close' to when we asked.  This will safely keep
-      // the update thread from over filling the UI thread.
-      while (_doc->updating()) {  // wait for the UI to finish old events
-        do_wait = true;
-        usleep(1000); // 1 ms on 2.6 kernel.  NOTE: on 2.4 kernel, the minimum usleep is 10ms,
-                      // So this will run more coarsly on 2.4.  Should be OK though.
-      }
-      if (do_wait) { // one extra usleep in case something was pending.
-        usleep(1000);
-      }
-      while (_doc->updating()) { // check again... not strictly needed given implicit repaint below,
-                                 // but it should just return false, so no harm done.
-        usleep(1000);
-      }
-
       if (gotData) {
-        QApplication::postEvent(_doc, new ThreadEvent(ThreadEvent::UpdateDataDialogs));
+        QApplication::postEvent(_doc, 
+                       new ThreadEvent(ThreadEvent::UpdateDataDialogs));
         // this event also triggers an implicit repaint
       } else {
         QApplication::postEvent(_doc, new ThreadEvent(ThreadEvent::Repaint));
       }
+      // Wait for UI thread to finish events.  If we don't wait
+      // 1: the UI thread could get flooded with events
+      // 2: the update thread could change vectors during a paint, causing
+      //    inconsistent curves to be plotted ie, the X vector updated 
+      //    and the Y vector not yet updated...
+
+      // Race warning: Syncronization of updating() is not assured,
+      // but updating() will always return a valid answer which was
+      // true 'close' to when we asked.  This will safely keep the
+      // update thread from over filling the UI thread.  The usleeps
+      // will hopefully give the UI thread a chance to set itself...
+
+      usleep(1000); // 1 ms on 2.6 kernel. 10ms on 2.4 kernel
+
+      while (_doc->updating()) {  // wait for the UI to finish old events
+        usleep(1000); 
+      }
+      usleep(1000);
+      // check again... not strictly needed given implicit repaint below,
+      // but it should just return false, so no harm done.
+      while (_doc->updating()) { 
+        usleep(1000);
+      }
     }
   }
 
@@ -115,7 +119,6 @@ void UpdateThread::run() {
 bool UpdateThread::doUpdates(bool force, bool *gotData) {
   KstObject::UpdateType tU, U = KstObject::NO_CHANGE;
   unsigned i;
-  int lastFrame = -1;
 
   if (gotData) {
     *gotData = false;
@@ -137,25 +140,30 @@ bool UpdateThread::doUpdates(bool force, bool *gotData) {
 #endif
 
   // Update the files
-  KST::dataSourceList.lock().readLock();
-  unsigned cnt = KST::dataSourceList.count();
-  for (i = 0; i < cnt; ++i) {
-    KstDataSourcePtr dsp = KST::dataSourceList[i];
+  if (!_paused) { // don't update even if paused && force
+    KST::dataSourceList.lock().readLock();
+    unsigned cnt = KST::dataSourceList.count();
+    for (i = 0; i < cnt; ++i) {
+      KstDataSourcePtr dsp = KST::dataSourceList[i];
 
-    dsp->update();
+      dsp->writeLock();
+      dsp->update();
+      dsp->writeUnlock();
 
-    if (_done || (_paused && !force)) {
-      KST::dataSourceList.lock().readUnlock();
+      if (_done) {
+        KST::dataSourceList.lock().readUnlock();
 #if UPDATEDEBUG > 1
-      kdDebug() << "1 Returning from scan with U=" << (int)U << endl;
+        kdDebug() << "1 Returning from scan with U=" << (int)U << endl;
 #endif
-      return false;
+        return false;
+      }
     }
+    KST::dataSourceList.lock().readUnlock();
   }
-  KST::dataSourceList.lock().readUnlock();
 
   { // scope to destroy the temporary list
-    KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
+    KstRVectorList rvl = 
+      kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
 
     unsigned cnt = rvl.count();
 
@@ -175,8 +183,6 @@ bool UpdateThread::doUpdates(bool force, bool *gotData) {
           return U == KstObject::UPDATE;
         }
 
-        lastFrame = rv->numFrames() + rv->startFrame();
-
         if (U == KstObject::NO_CHANGE) {
           U = KstObject::UPDATE;
         }
@@ -192,12 +198,7 @@ bool UpdateThread::doUpdates(bool force, bool *gotData) {
     }
   }
 
-  if (lastFrame > 0) {
-    // FIXME: change to an event
-    //emit newFrameMsg(lastFrame);
-  }
-
-  if (U == KstObject::NO_CHANGE) { // no need to update further
+  if ((U == KstObject::NO_CHANGE) && (!force)) { // no need to update further
     return false;
   }
 
