@@ -31,10 +31,10 @@ bool PluginEditor::isModified()
 void PluginEditor::init()
 {
     _doc = 0L;
-    setModified(false);    
+    setModified(false);
     listView1->setDefaultRenameAction(QListView::Accept);
     _savedFileName = QString::null;
-    initListView();    
+    initListView();
 }
 
 void PluginEditor::fileNew()
@@ -42,14 +42,14 @@ void PluginEditor::fileNew()
     if (isModified()) {
 	switch (QMessageBox::information( this, QString::null, tr("This file has been modified. Do you wish to save it?"),  tr("&Yes"), tr("&No"), tr("&Cancel"))){
 	case 0: // yes
-	    PluginEditor::fileSave();	 
+	    PluginEditor::fileSave();
 	    break;
 	case 1: // no
 	    break;
 	case 2: // cancel
 	    return;
 	    break;
-	default:	    
+	default:
 	    break;
 	}
     }
@@ -67,18 +67,18 @@ void PluginEditor::closeEvent( QCloseEvent* e )
     if (isModified()) {
 	switch (QMessageBox::information( this, QString::null, tr("This file has been modified. Do you wish to save it?"), tr("&Yes"), tr("&No"), tr("&Cancel"))) {
 	    case 0: // yes
-		PluginEditor::fileSave();	 
+		PluginEditor::fileSave();
 		break;
 	    case 1: // no
 		break;
 	    case 2: // cancel
 		return;
 		break;
-	    default:	    
+	    default:
 		break;
 	}
     }
-    
+
     switch (QMessageBox::information( this, QString::null, tr("Are you sure you want to quit?"), tr("&Yes"), tr("&No"), tr("&Cancel"))){
 	case 0: // yes
 	    delete _doc;
@@ -125,13 +125,13 @@ void PluginEditor::fileOpen()
     QFile file(filename);
 
     if (!file.open(IO_ReadOnly)) {
-	QMessageBox::information(this, QString::null, tr("Error: Could Not Open File!"),  tr("&Ok"));
+	QMessageBox::information(this, QString::null, tr("Error: Could Not Open File!"),  tr("&OK"));
 	file.close();
 	return;
     }
 
     if (!_doc->setContent(&file)) {
-	QMessageBox::information(this, QString::null, tr("Error: Could Not Parse XML!"), tr("&Ok"));
+	QMessageBox::information(this, QString::null, tr("Error: Could Not Parse XML!"), tr("&OK"));
 	file.close();
 	_savedFileName = filename;
 	return;
@@ -148,7 +148,7 @@ void PluginEditor::fileOpen()
 
     while (!node.isNull()) {
 	// handles INTRO tags
-	if (node.isElement() && node.nodeName() == "intro") {	
+	if (node.isElement() && node.nodeName() == "intro") {
 	    QDomNode child = node.firstChild();
 
 	    while (!child.isNull()) {
@@ -176,7 +176,7 @@ void PluginEditor::fileOpen()
 		}
 
 		child = child.nextSibling();
-	    }		   
+	    }
 	}
 
 	// handles INTERFACE tag
@@ -235,7 +235,7 @@ void PluginEditor::fileOpen()
 
 			}
 
-			subchild = subchild.nextSibling();   
+			subchild = subchild.nextSibling();
 		    }
 		}
 
@@ -268,7 +268,7 @@ void PluginEditor::fileOpen()
 
 		    type->setText(1, child.nodeName());
 		    type->setRenameEnabled(1, true);
-		}	    
+		}
 		child = child.nextSibling();
 	    }
 	}
@@ -286,7 +286,7 @@ void PluginEditor::fileSave()
     if (_savedFileName.isEmpty()) {
 	fileSaveAs();
     }
-    
+
     // _savedFileName can change in fileSaveAs()
     if (_savedFileName.isEmpty()) {
 	return;
@@ -296,67 +296,67 @@ void PluginEditor::fileSave()
 
     if (file.open(IO_WriteOnly)){
 	QTextStream out (&file);
-	
+
 	// sequentially write output file
-	
+
 	delete _doc;
 	_doc = new QDomDocument;
 	QDomElement root = _doc->createElement("Module");
 	_doc->appendChild(root);
 	QDomElement intro = _doc->createElement("intro");
 	root.appendChild(intro);
-	
+
 	// write INTRO
-	
+
 	QListViewItem *iter = listView1->findItem("intro", 0, Qt::ExactMatch);
 	iter = iter->firstChild();
 	while (iter) {
 	    QDomElement element = _doc->createElement(iter->text(0));
-	    
+
 	    if (iter->text(0) == "version") {
 		QListViewItem *version = iter->firstChild();
-		
+
 		if (version->text(1).length() > 0) {
 		    element.setAttribute(version->text(0), version->text(1));
 		}
-		
+
 		version = version->nextSibling();
-		
+
 		if (version->text(1).length() > 0) {
 		    element.setAttribute(version->text(0), version->text(1));
 		}
 	    }
-	    
+
 	    if (iter->text(1).length() > 0) {
 		if (iter->text(0) == "modulename" || iter->text(0) == "author") {
 		    element.setAttribute("name", iter->text(1));
 		}
-	
+
 		if (iter->text(0) == "description") {
 		    element.setAttribute("text", iter->text(1));
 		}
-		
+
 		if (iter->text(0) == "state") {
-		    element.setAttribute("devstate", iter->text(1));		
+		    element.setAttribute("devstate", iter->text(1));
 		}
 	    }
-		
+
 	    if (element.attributes().length() > 0) {
 		intro.appendChild(element);
 	    }
 
 	    iter = iter->nextSibling();
 	}
-	
-	// Write INTERFACE	
+
+	// Write INTERFACE
 	QDomElement interface = _doc->createElement("interface");
-	root.appendChild(interface);	
-	createTags("input", &interface);	
+	root.appendChild(interface);
+	createTags("input", &interface);
 	createTags("output", &interface);
-	
+
 	// Write PARALIST
-	createTags("paralist", &root);		
-	
+	createTags("paralist", &root);
+
 	root.save(out, 1);
     }
 
@@ -386,7 +386,7 @@ void PluginEditor::filePrint()
 
 void PluginEditor::fileExit()
 {
-    
+
 }
 
 
@@ -412,7 +412,7 @@ void PluginEditor::entryAdd()
 {
     QListViewItem *cur = listView1->currentItem();
     QListViewItem *parent = cur;
- 
+
     while (parent->parent() != 0) {
 	parent = parent->parent();
     }
@@ -442,7 +442,7 @@ void PluginEditor::entryAdd()
 		    newInterface, "helptext");
 
 	    helptext->setRenameEnabled(1, true);
- 
+
 	    QListViewItem *desc = new QListViewItem(
 		    newInterface, "descr");
 
@@ -455,8 +455,8 @@ void PluginEditor::entryAdd()
 
 	    listView1->setSelected(newInterface, true);
 	    newInterface->setOpen(true);
-	    newInterface->startRename(0);		
-	    
+	    newInterface->startRename(0);
+
     } else if (cur->text(0) == "paralist") {
 	QListViewItem *newParalist = new QListViewItem(
 		cur, "new param", QString::null);
@@ -490,18 +490,18 @@ void PluginEditor::removeEntry()
     if (p && (p->text(0) != "intro" && p->text(0) != "interface" && p->text(0) != "version")) {
 	delete listView1->currentItem();
     }
-    
+
     setModified(true);
 }
 
 void PluginEditor::initListView()
 {
-    listView1->clear();    
-    
+    listView1->clear();
+
     QListViewItem *root = new QListViewItem(listView1, "paralist", QString::null);
     root->setOpen(true);
     listView1->insertItem(root);
-    
+
     root = new QListViewItem(listView1, "interface", QString::null);
     root->setOpen(true);
     QListViewItem *child = new QListViewItem(root, "input", QString::null);
@@ -530,7 +530,7 @@ void PluginEditor::initListView()
     child = new QListViewItem(root, "documentation", QString::null);
     child->setRenameEnabled(1,true);
     listView1->insertItem(root);
-     
+
     listView1->setSelected(listView1->firstChild(), true);
 }
 
@@ -539,7 +539,7 @@ void PluginEditor::renameOccured( QListViewItem *item, int col )
 {
   Q_UNUSED( item )
   Q_UNUSED( col )
-  
+
   setModified(true);
 }
 
@@ -547,19 +547,19 @@ void PluginEditor::renameOccured( QListViewItem *item, int col )
 void PluginEditor::createTags( const QString & outerTag, QDomElement *parent )
 {
 	QListViewItem *iter = listView1->findItem(outerTag, 0, Qt::ExactMatch)->firstChild();
-	
+
 	QDomElement outerElement = _doc->createElement(outerTag);
 	parent->appendChild(outerElement);
-	
+
 	while (iter) {
 	    QDomElement element = _doc->createElement(QString::null);
-	    element.setAttribute("name", iter->text(0));   
+	    element.setAttribute("name", iter->text(0));
 	    QListViewItem *child = iter->firstChild();
-	    
+
 	    while (child) {
 		if (child->text(0) == "type") {
 		    if (child->text(1).contains(':', false) > 0) {
-			QStringList list = QStringList::split(':', child->text(1),FALSE);		
+			QStringList list = QStringList::split(':', child->text(1),FALSE);
 			QStringList::Iterator it = list.begin();
 			element.setTagName(*it);
 			element.setAttribute("type",*(++it));
@@ -571,8 +571,8 @@ void PluginEditor::createTags( const QString & outerTag, QDomElement *parent )
 		}
 		child = child->nextSibling();
 	    }
-	    
+
 	    outerElement.appendChild(element);
 	    iter = iter->nextSibling();
-	}	    
+	}
 }
