@@ -4,11 +4,10 @@
  *  Released under the terms of the GPL.
  */
 
-#include <stdlib.h>
-#include <math.h>
 #include <gsl/gsl_fit.h>
 #include "../common.h"
 
+extern "C" int parameterName(int iIndex, char** pName);
 extern "C" int kstfit_linear_unweighted(const double *const inArrays[], const int inArrayLens[],
 		const double inScalars[],
 		double *outArrays[], int outArrayLens[],
@@ -19,9 +18,12 @@ int kstfit_linear_unweighted(const double *const inArrays[], const int inArrayLe
 		double *outArrays[], int outArrayLens[],
 		double outScalars[])
 {
+  KST_UNUSED(inScalars)
+
   int i = 0;
   int	iLength;
   int iReturn = -1;
+  double* pInputs[3];
   double c0 = 0.0;
   double c1 = 0.0;
   double cov00 = 0.0;
@@ -31,13 +33,13 @@ int kstfit_linear_unweighted(const double *const inArrays[], const int inArrayLe
   double y;
   double yErr;
 
-  if( precursor( inArrayLens, outArrays, outArrayLens, &iLength, false, true, 1 ) ) {
-    if( !gsl_fit_linear( inArrays[XVALUES], 1, inArrays[YVALUES], 1, iLength, &c0, &c1, &cov00, &cov01, &cov11, &dSumSq ) ) {
+  if( precursor( inArrays, inArrayLens, outArrays, outArrayLens, &iLength, false, true, 2, pInputs ) ) {
+    if( !gsl_fit_linear( pInputs[XVALUES], 1, pInputs[YVALUES], 1, iLength, &c0, &c1, &cov00, &cov01, &cov11, &dSumSq ) ) {
         
       for( i=0; i<iLength; i++ ) {
-        gsl_fit_linear_est( inArrays[XVALUES][i], c0, c1, cov00, cov01, cov11, &y, &yErr );
+        gsl_fit_linear_est( pInputs[XVALUES][i], c0, c1, cov00, cov01, cov11, &y, &yErr );
         outArrays[YFIT][i] = y;
-        outArrays[YRESIDUALS][i] = inArrays[YVALUES][i] - y;
+        outArrays[YRESIDUALS][i] = pInputs[YVALUES][i] - y;
         outArrays[Y_LOW_VALS][i] = y - yErr;
         outArrays[Y_HGH_VALS][i] = y + yErr;
       }
@@ -53,6 +55,25 @@ int kstfit_linear_unweighted(const double *const inArrays[], const int inArrayLe
       iReturn = 0;
     } 
   }
+
+  postcursor( inArrays, false, pInputs );
   
   return iReturn;
 }
+
+int parameterName(int iIndex, char** pName) {
+  int iRetVal = 0;
+
+  switch (iIndex) {
+    case 0:
+      *pName = strdup("Intercept");
+      iRetVal = 1;
+      break;
+    case 1:
+      *pName = strdup("Gradient");
+      iRetVal = 1;
+      break;
+  }
+
+  return iRetVal;
+}
