@@ -39,14 +39,13 @@ int kstfit_linear_weighted(
   int	iLength;
   int iReturn = -1;
   
-  if (inArrayLens[YVALUES] >= 2 && inArrayLens[XVALUES] >= 2) {
+  if (inArrayLens[YVALUES] >= 2 &&
+      inArrayLens[XVALUES] >= 2 &&
+      inArrayLens[WEIGHTS] >= 2) {
     iLength = inArrayLens[YVALUES];
-    if( inArrayLens[XVALUES] < iLength ) {
+    if( inArrayLens[XVALUES] > iLength ) {
       iLength = inArrayLens[XVALUES];
     }
-    if( inArrayLens[WEIGHTS] < iLength ) {
-      iLength = inArrayLens[WEIGHTS];
-    }
         
     //
     // first do some sanity checks...
@@ -124,10 +123,10 @@ int kstfit_linear_weighted(
                     // fill in the matrices and vectors...
                     //
                     for( i=0; i<iLength; i++ ) {
-                      gsl_vector_set( pVectorY, i, inArrays[YVALUES][i] );
-                      gsl_vector_set( pVectorWeights, i, inArrays[WEIGHTS][i] );
+                      gsl_vector_set( pVectorY, i, interpolate(i, iLength, inArrays[YVALUES], inArrayLens[YVALUES]));
+                      gsl_vector_set( pVectorWeights, i, interpolate(i, iLength, inArrays[WEIGHTS], inArrayLens[WEIGHTS]));
                       for( j=0; j<iNumParams; j++ ) {
-                        dX = calculate_matrix_entry( inArrays[XVALUES][i], j );
+                        dX = calculate_matrix_entry( interpolate(i, iLength, inArrays[XVALUES], inArrayLens[XVALUES]), j );
                         gsl_matrix_set( pMatrixX, i, j, dX );
                       }
                     }
@@ -151,7 +150,7 @@ int kstfit_linear_weighted(
                                 gsl_vector_get( pVectorParameters, j );
                         }
                         outArrays[YFIT][i] = dY;
-                        outArrays[YRESIDUALS][i] = inArrays[YVALUES][i] - dY;
+                        outArrays[YRESIDUALS][i] = interpolate(i, iLength, inArrays[YVALUES], inArrayLens[YVALUES]) - dY;
                       }
                       
                       for( i=0; i<iNumParams; i++ ) {
