@@ -27,6 +27,7 @@ double calculate_matrix_entry( double dX, int iPos ) {
   return dY;
 }
 
+extern "C" int parameterName(int iIndex, char** pName);
 extern "C" int kstfit_sinusoid_weighted(
   const double *const inArrays[], 
   const int inArrayLens[],
@@ -57,3 +58,19 @@ int kstfit_sinusoid_weighted(
   
   return iRetVal;
 }
+
+int parameterName(int iIndex, char** pName) {
+  char name[50];
+
+  if( iIndex == 0 ) {
+    *pName = strdup( "Mean" );
+  } else if (iIndex % 2 == 1 ) {
+    sprintf( name, " cos(%d 2PI x/P)", ( iIndex + 1 ) / 2 );
+    *pName = strdup( name );
+  } else {
+    sprintf( name, "-sin(%d 2PI x/P)", ( iIndex + 0 ) / 2 );
+    *pName = strdup( name );
+  }
+
+  return 1;
+}
