@@ -8,12 +8,12 @@
 #include <math.h>
 #include "../filters.h"
 
-extern "C" int butterworth_bandstop(const double *const inArrays[], const int inArrayLens[],
+extern "C" int filter_bandstop(const double *const inArrays[], const int inArrayLens[],
 		const double inScalars[],
 		double *outArrays[], int outArrayLens[],
 		double outScalars[]);
 
-int butterworth_bandstop(const double *const inArrays[], const int inArrayLens[],
+int filter_bandstop(const double *const inArrays[], const int inArrayLens[],
 		const double inScalars[],
 		double *outArrays[], int outArrayLens[],
 		double outScalars[])
@@ -32,10 +32,18 @@ int butterworth_bandstop(const double *const inArrays[], const int inArrayLens[]
 
 double filter_calculate( double dFreqValue, const double inScalars[] ) {
   double dValue;
-  
-  dValue  = dFreqValue * inScalars[2];
-  dValue /= ( (dFreqValue * dFreqValue) - (inScalars[1] * inScalars[1]) );
-  dValue  = 1.0 / ( 1.0 + pow( dValue, 2.0 * (double)inScalars[0] ) ); 
-  
+
+  int order2 = int(2.0 * inScalars[0]);
+  double lowpass  = inScalars[1] + 0.5*inScalars[2];
+  double highpass = inScalars[1] - 0.5*inScalars[2];
+
+  if( dFreqValue > 0.0 ) {
+    dValue = 1.0 / ( 1.0 + gsl_pow_int( dFreqValue / lowpass, order2) );
+    dValue *= 1.0 / ( 1.0 + gsl_pow_int( highpass / dFreqValue, order2) );
+    dValue = 1.0 - dValue;
+  } else {
+    dValue = 0.0;
+  }
+
   return dValue;
 }
